/*
 * Created on May 2, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.bridge;

import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.List;

import javax.xml.bind.JAXBException;

import org.msda.components.manager.object.ObjectRemoteAccess;
import org.msda.components.utils.ComponentThread;
import org.msda.framework.FrameworkMain;
import org.msda.msd.Hop;
import org.msda.msd.InformationComponent;
import org.msda.msd.MSDMessage;
import org.msda.msd.UbisecID;
import org.msda.util.InfoType;
import org.msda.util.JaxMeUtil;
import org.msda.util.PropagationUtil;
import org.msda.util.comm.NetworkManager;

/**
 * This class represents a thread that can manage a remote service access between two networks, <br>
 *   - either between this bridge and a manager of the destined network, <br>
 *   - or between this bridge and another bridge if the destined network is farer than one hop.<br> 
 * <p>
 * NOTE AGNES : the optimization with remote access between two bridges has to be done
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 */
public class BridgeThreadRemoteAccess extends ComponentThread {
    
    private MSDBridgePool bridgePool;
    private JaxMeUtil jaxme;
    private NetworkManager networkManager;
    
    /**
     * Constructor.
     * 
     * @param bridgePool
     */
    public BridgeThreadRemoteAccess( MSDBridgePool bridgePool ) {
        this.bridgePool = bridgePool;
        this.jaxme = new JaxMeUtil();
        this.networkManager = new NetworkManager();
        this.start();
    }
    
    
    /**
     * @see org.msda.components.utils.ComponentThread#terminate()
     */
    public void terminate() {
        // TODO Auto-generated method stub
    }
    

    /**
     * 
     * @see java.lang.Runnable#run()
     */
    public void run() {
        
        while( active ) {
            
            try { 
                ObjectRemoteAccess object = (ObjectRemoteAccess)this.bridgePool.getQueueRemoteAccess().get();
                
                super.isBusy = true;
                MSDMessage message = object.getMessage();
                UbisecID srcNetworkID = object.getNetworkID();
                Socket socketIn = object.getSocket();
                //String IPAddress = object.getIPAddress();
                
                if ( InfoType.Message.REMOTE_ACCESS == message.getType().intValue() ) {
                    
                    // 2 cases 
                    int index = PropagationUtil.getInstance().getNetworkFromDest( srcNetworkID, message.getBackwardPath().getHops() );
                    
                    // 
                    // TODO - optimization with direct connection between bridges
                    //
                    UbisecID destID = PropagationUtil.getInstance().getSrcNetwork( index, message.getBackwardPath().getHops() );
                    if ( index==0 ) {
                        // manager
                        //System.out.println("should sent to a manager"); 
                    } else {
                        // bridge
                        //System.out.println("should sent to a bridge");
                    }
                    
                    List propagation = message.getBackwardPath().getHops();
                    
                    // for now always the manager...
                    InformationComponent infoRes = this.bridgePool.getLocalMgr().getManager( destID.getName() );
                    
                    // FOR NOW : removing the hop is done in the manager if size > 1
                    if ( propagation.size()==1 ) {
                        Hop removedHop = (Hop)message.getBackwardPath().getHops().remove( index );
                        message.getContent().getSvcAccess().setSrcNetID( removedHop.getDestNetID() );
                    }
                    
                    byte[] msg = jaxme.toString( message ).getBytes();
                    int port = infoRes.getManager().getRemoteAccessPort().intValue();
                    
                    Socket socketOut = networkManager.sendTo( msg, InetAddress.getByName(infoRes.getIPAddr()), port );
                    
                    byte[] res =  networkManager.getReceivedData( socketOut );
                    networkManager.sendTo(  res, socketIn );
                } 
                
            } catch ( JAXBException e ) {
            	FrameworkMain.msdaOutput.notifyException(e);
            } catch (UnknownHostException e) {
            	FrameworkMain.msdaOutput.notifyException(e);
            }  catch (NullPointerException e) {
            	FrameworkMain.msdaOutput.notifyException(e);
                System.exit( 1 );
            } catch (InterruptedException e) {
            }
            
            // reinit
            super.isBusy = false;
        }
    }
}
