/*
 * Created on May 2, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.bridge;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

import javax.xml.bind.JAXBException;

import org.msda.components.manager.object.ObjectRemoteAccess;
import org.msda.components.utils.ComponentThread;
import org.msda.msd.MSDMessage;
import org.msda.msd.UbisecID;
import org.msda.msd.impl.UbisecIDImpl;
import org.msda.util.JaxMeUtil;
import org.msda.util.comm.NetworkManager;

/**
 * Thread that listens to the remote access port of the bridge for one network interface.<p>
 *
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 */
public class BridgeThreadRemoteAccessListener extends ComponentThread {

    protected ServerSocket serverSocket;
    private MSDBridgePool bridgePool;
    private UbisecID networkID;
    
    private int port;
    private JaxMeUtil jaxme;
    private NetworkManager networkManager;

    private String IPAddress;
    
    /**
     * Listener to a network interface
     * 
     * @param bridgePool	
     * @param port			bridge remote access port for that network interface
     * @param networkName	network name of that network interface
     * @param IPAddress		bridge IP address for that network interface
     */
    public BridgeThreadRemoteAccessListener(MSDBridgePool bridgePool, int port, String networkName, String IPAddress) {
        
        super();

        this.bridgePool = bridgePool;
        this.jaxme = new JaxMeUtil();
        this.networkManager = new NetworkManager();
      
        this.IPAddress = IPAddress;
        this.networkID = new UbisecIDImpl();
        this.networkID.setName( networkName );
        this.port = port;
        
        try {
            this.serverSocket = new ServerSocket ( port );
        } catch (IOException e) {
            e.printStackTrace();
        }
        
        this.start();
    }
    
    public void terminate() {
        try {
            this.serverSocket.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    public void run() {
        
        while(active) {
            
            try {
                Socket socket = serverSocket.accept();
            	byte[] res = networkManager.getReceivedData( socket, false );
                MSDMessage mess = jaxme.toMSDMessage( res );
                
                ObjectRemoteAccess object = new ObjectRemoteAccess();
                object.setMessage( mess );
                object.setNetworkID( networkID );
                object.setSocket( socket );
                object.setIPAddress( this.IPAddress );
                
                bridgePool.getQueueRemoteAccess().add( object );
                
            } catch( JAXBException e ) {
            	e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
