/*
 * Created on May 3, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.bridge;


import java.util.List;

import org.msda.components.core.MSDDaemon;
import org.msda.components.manager.client.BlockingQueue;
import org.msda.components.utils.LocalComponentsMgr;
import org.msda.framework.FrameworkOutput;
import org.msda.msd.NetworkConfig;
import org.msda.util.comm.NetworkUtils;

/**
 * This class allows the different pools of threads to be managed by the msd bridge.
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class MSDBridgePool {

    // Here the number of thread corresponds to the number of network interface + 1 (for the manager)  
    // At the beginning there is only one thread, then
    // for each network interface that is activated, a thread is created
    
	private BlockingQueue queueRemoteAccess;
	private BridgeThreadRemoteAccess[] poolRemoteAccess;
	private BridgeThreadRemoteAccessListener[] poolThreadRemoteAccessListener;
	private FrameworkOutput board;
	private LocalComponentsMgr localMgr;
	private List networks;
	
	/**
	 * 
	 * @param board
	 * @param localMgr
	 * @param networks
	 * @param daemon
	 */
	public MSDBridgePool( LocalComponentsMgr localMgr, List networks, MSDDaemon daemon ) {
	    
		this.board = org.msda.framework.FrameworkMain.msdaOutput;
	    this.localMgr = localMgr;
	    
	    //	  Case of the remote access
		this.queueRemoteAccess = new BlockingQueue();
	    this.poolRemoteAccess = null;
	    
	    poolThreadRemoteAccessListener = new BridgeThreadRemoteAccessListener[ networks.size() ];
	    for( int i=0; i<poolThreadRemoteAccessListener.length; i++) {
	        NetworkConfig config = ((NetworkConfig)networks.get(i));
	    	int remoteAccessPort = config.getBridgeRemoteAccessPort().intValue();
	    	String networkName = config.getName();
	    	String ipAddress = NetworkUtils.getIPAddress( config.getInterface() );
	    	poolThreadRemoteAccessListener[i] = new BridgeThreadRemoteAccessListener( this, remoteAccessPort, networkName, ipAddress);
	    }
	    
	    this.networks = networks;
	}
    
    /**
     * Returns the list of NetworkConfig
     * 
     * @return Returns the list of NetworkConfig
     */
	public List getNetworks()  {
		return networks;
	}

	/**
	 * Returns the LocalComponentsMgr.
	 * 
	 * @return Returns the LocalComponentsMgr.
	 */
	public LocalComponentsMgr getLocalMgr() {
		return localMgr;
	}
	
	/**
     * Adds a new thread for remote access in the pool.
     */
    public void createNewRemoteAccessThread() {
        
        // TODO - modify should be called each time a localComponent is added
        //        without the disappearing of another one
        // This method is called each time a new network interface is activated
        
        if ( poolRemoteAccess==null ) {
            this.poolRemoteAccess = new BridgeThreadRemoteAccess[1];
            this.poolRemoteAccess[0] = new BridgeThreadRemoteAccess( this );
            
        } else {
	        // Increases the pool
	        BridgeThreadRemoteAccess[] tmp = poolRemoteAccess;
	        poolRemoteAccess = new BridgeThreadRemoteAccess[ tmp.length+1 ];
	        poolRemoteAccess[0] = new BridgeThreadRemoteAccess( this );
	        System.arraycopy( tmp, 0, poolRemoteAccess, 1, tmp.length );
        }
    }
    
    /**
     * Returns the queueRemoteAccess
     * 
     * @return Returns the queueRemoteAccess
     */
    public BlockingQueue getQueueRemoteAccess() {
        return queueRemoteAccess;
    }
    
    /**
     * Returns the poolRemoteAccess
     * 
     * @return Returns the poolRemoteAccess
     */
    public BridgeThreadRemoteAccess[] getPoolRemoteAccess() {
        return poolRemoteAccess;
    }
}
