/*
 * Created on Dec 13, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.core;

import java.math.BigInteger;
import java.util.List;
import java.util.Vector;

import org.msda.components.interfaces.CommunicatorCallback;
import org.msda.components.interfaces.CommunicatorInterface;
import org.msda.components.interfaces.MSDABridgeInterface;
import org.msda.components.interfaces.MSDAManagerInterface;
import org.msda.components.interfaces.MSDANetworkInterface;
import org.msda.context.NodeProfile;
import org.msda.msd.ContextInformationTable;
import org.msda.msd.ContextRulesTree;
import org.msda.msd.ControlMessage;
import org.msda.msd.InformationComponent;
import org.msda.msd.MSDDescription;
import org.msda.msd.MSDMessage;
import org.msda.msd.MSDRequest;
import org.msda.msd.ManagerConfig;
import org.msda.msd.MessageContent;
import org.msda.msd.NetworkConfig;
import org.msda.msd.UbisecID;
import org.msda.msd.impl.ControlMessageImpl;
import org.msda.msd.impl.InformationComponentImpl;
import org.msda.msd.impl.MSDMessageImpl;
import org.msda.msd.impl.ManagerConfigImpl;
import org.msda.msd.impl.MessageContentImpl;
import org.msda.msd.impl.UbisecIDImpl;
import org.msda.util.InfoType;
import org.msda.util.MSDException;
import org.msda.util.MSDUtil;
import org.msda.util.ParamsUtil;
import org.msda.util.comm.NetworkUtils;
import org.msda.util.security.*;

import secureMulticast.client.LKHClient;

/*
 * @author raverdy
 */
public class NetworkMonitor implements CommunicatorCallback {
	private NodeProfile profile = new NodeProfile();
	//
	private class NetworkDevice {
		boolean gotBeacon;
		InformationComponent info;
	}
	Vector networkDeviceList;
	
	private NetworkConfig netConfig=null;
	private String IPAddress ="";
	private String controlGroupAddress ="";
	private int controlPort;
	private String dataGroupAddress ="";
	private String netname ="";
	private int dataPort;
	private String ifname ="";
	private UbisecID netId;
	private int currentState;
	private float benefit;
	//
	public boolean agreeManager;
	private boolean isManager;
	//
	private MSDMessage beaconDevice;
	private MSDMessage beaconMgr;
	private MSDMessage beaconBridge;
	private MSDMessage existMgr;
	private MSDMessage activateBridge;
	
	private boolean doCheckBeaconLoop;
	private boolean doSendBeaconLoop;
	private boolean beaconManager;
	private Thread 	sendBeaconThread;
	private Thread 	checkBeaconThread;
	private InformationComponent currentManager;
	
	private CommunicatorInterface communicator;
	private MSDAManagerInterface managerInterface;
	private MSDABridgeInterface bridgeInterface;
	private MSDANetworkInterface networkInterface;
	
	private InformationComponent infoComponent;
	
	private MSDDaemon daemon;
	private LKHClient lkhclient;
	
	/**
	 * 
	 * @param netConfig
	 * @param agreeManager
	 * @param managerInterface
	 * @param bridgeInterface
	 * @param benefit
	 */
	public NetworkMonitor ( MSDDaemon daemon, MSDANetworkInterface networkInterface, NetworkConfig netConfig, boolean agreeManager, MSDAManagerInterface managerInterface, MSDABridgeInterface bridgeInterface)
	{
		//
		this.daemon = daemon;
		this.netConfig = netConfig;
		this.netname = netConfig.getName();
		this.ifname = netConfig.getInterface();
		this.controlGroupAddress = netConfig.getControlGroup();
		this.controlPort = netConfig.getControlPort().intValue();
		this.dataGroupAddress = netConfig.getDataGroup();
		this.dataPort = netConfig.getDataPort().intValue();
		//
		this.agreeManager = agreeManager;
		this.IPAddress = NetworkUtils.getIPAddress(ifname);
		this.netId = new UbisecIDImpl();
		this.netId.setName(netConfig.getName());
		//
		this.managerInterface = managerInterface;
		this.networkInterface = networkInterface;
		this.bridgeInterface = bridgeInterface;
		this.currentState = InfoType.STATE_START;
		
		this.benefit = profile.getBenefit();
		
		this.networkDeviceList = new Vector();
		//
		InitMessages();
	}
	
	
	/**
	 * Starts the daemon and wait for the resolution of the initial state of the device 
	 *
	 */
	private void start ()
	{
		try {
			this.communicator = networkInterface.registerCallback(netId, ifname, controlGroupAddress, controlPort, this);
		} 
		catch (MSDException e) {
			//board.info("Could not register with the control multicast group " + ifname + " " + controlGroupAddress + ":" + controlPort + ", no need to continue");
			e.printStackTrace();
			System.exit(-1);
		}
		waitInitialState();
	}	
	
	/**
	 *	Initializes the mnessages for the beacons 
	 *
	 */
	private void InitMessages ()
	{
		MessageContent messageContent;
		ControlMessage controlMessage;
		ManagerConfig managerConfig;
		//
		//
		beaconMgr = new MSDMessageImpl();
		messageContent = new MessageContentImpl();
		infoComponent = new InformationComponentImpl();
		controlMessage = new ControlMessageImpl ();
		//
		infoComponent.setCompID(daemon.getStackId());
		if (daemon.getAgreeBridge()) {
			infoComponent.setAgreeBridge(BigInteger.valueOf(1));
		}
		else {
			infoComponent.setAgreeBridge(BigInteger.valueOf(0));
		}
		
		//certificate = new byte[0];
		//infoComponent.setCertificate(certificate);
		
		infoComponent.setIPAddr(IPAddress);
		infoComponent.setNetName(netConfig.getName());
		if (agreeManager)
		{
			managerConfig = new ManagerConfigImpl();
			managerConfig.setName(managerInterface.getName());
			managerConfig.setNetworkName(managerInterface.getNetworkName());
			managerConfig.setRemoteAccessPort(BigInteger.valueOf( managerInterface.getRemoteAccessPort() ));
			infoComponent.setManager(managerConfig);		
			
			if (daemon.isContextEnabled())
			{
				//board.info("Add network context information");
				ContextInformationTable ctx = daemon.getManager().getContextManager().getLastNetworkCtx();
				if ( ctx!=null ) 
					infoComponent.setCtxInfo(ctx);
				
				ContextRulesTree rules =  daemon.getManager().getContextManager().getLastNetworkRules();
				if ( rules!=null )
					infoComponent.setCtxRules(rules);
			}				
		}
		else
		{
			infoComponent.setManager(null);				
		}
		infoComponent.getActvNets().add(netConfig);
		infoComponent.setBenefit(profile.getBenefit());
		
		controlMessage.setInfoComp(infoComponent);
		messageContent.setControl(controlMessage);
		beaconMgr.setContent(messageContent);
		beaconMgr.setType(BigInteger.valueOf(InfoType.Message.BEACON_MANAGER));
		//
		existMgr = new MSDMessageImpl();
		existMgr.setType(BigInteger.valueOf(InfoType.Message.EXIST_MANAGER));
		//
		beaconBridge = new MSDMessageImpl();
		beaconBridge.setContent(messageContent);
		beaconBridge.setType(BigInteger.valueOf(InfoType.Message.BEACON_BRIDGE));
		//
		beaconDevice = new MSDMessageImpl();
		beaconDevice.setContent(messageContent);
		beaconDevice.setType(BigInteger.valueOf(InfoType.Message.BEACON_DEVICE));
		//
		activateBridge = new MSDMessageImpl();
		activateBridge.setType(BigInteger.valueOf(InfoType.Message.BRIDGE_ACTIVATION));
	}
	
	
	/**
	 * Resolve the initial state of the device
	 *
	 */
	private void waitInitialState()
	{
		// First send the initial request for the msd manager and waits 5 secs
		// Then create beacon loops (sending and checking)
		// Then send the request for the manager beacon
		
		checkBeaconThread = checkBeaconLoop();				
		sendBeaconThread = sendBeaconLoop();
		isManager = false;
		this.sendRequestManagerBeacon();
		try {
			synchronized (this) {
				this.wait (InfoType.STARTUP_TIMEOUT);
			}
			// if no message recieved from the msd manager
			// and agree to be a manager, then activate manager
			// otherwise become a simple device
			if (currentState==InfoType.STATE_START) {
				if (this.agreeManager) {
					currentState = InfoType.STATE_MANAGER;
					this.managerInterface.startManager();
					isManager = true;
					// optimization. the manager directly adds itself to the list of devices it manages
					this.managerInterface.updateInfoComponent(InfoType.Component.DEVICE, beaconMgr.getContent().getControl().getInfoComp());
				}
				else {
					//	Started as device
					isManager = false;
					currentState = InfoType.STATE_DEVICE;
				}
			}
			// activate beacon loops (send/check)
			checkBeaconThread.start();
			sendBeaconThread.start();
		}
		catch (InterruptedException e) {
			// beacon came ealry, no need to be an MSDA manager
		}
	}
	
	/**
	 * 
	 *
	 */
	//		void checkBridges ()
	//		{
	//		synchronized (devList) {
	//		DeviceInfo tmpDevice;
	//		for (int x=0; x<devList.size(); x++)
	//		{
	//		tmpDevice = (DeviceInfo)devList.get(x);
	//		if (tmpDevice.agreeBridge && tmpDevice.activeBridge)
	//		if (tmpDevice.beaconReceived == false)
	//		{
	//		// XXX here I should remove the device entry/unregister the bridge
	//		tmpDevice.activeBridge = false;
	//		}
	//		}
	//		}
	//		}
	
	
	/**
	 * 
	 *
	 */
	//		void unmarkBridgesBeacon ()
	//		{
	//		synchronized (devList) {
	//		DeviceInfo tmpDevice;
	//		for (int x=0; x<devList.size(); x++)
	//		{
	//		tmpDevice = devList.get(x);
	//		if (tmpDevice.agreeBridge && tmpDevice.activeBridge)
	//		tmpDevice.beaconReceived = false;
	//		}
	//		}
	//		}
	
	private boolean updateDeviceBeacon ( InformationComponent info ) 
	{
		NetworkDevice tmpDev, newDev;
		synchronized (this) {
			for (int x=0; x<networkDeviceList.size(); x++)  {	
				tmpDev = (NetworkDevice) networkDeviceList.get(x);
				if (tmpDev.info.getCompID().getName().equals(info.getCompID().getName())) {
					tmpDev.gotBeacon = true;
					return true;
				}
			}
			newDev = new NetworkDevice();
			newDev.gotBeacon = true;
			newDev.info = info;
			networkDeviceList.add(newDev);
		}
		return false;
	}
	
	
	/**
	 * 
	 * @return
	 */
	public Thread sendBeaconLoop ()
	{
		try
		{
			Thread socketThread = new Thread( new Runnable() 
					{
				public void run() 
				{	
					doSendBeaconLoop = true;
					while (doSendBeaconLoop)
					{
						try {
							// wait for the beacon sending timeout
							// - then if manager, send the manager beacon
							// - then, if agree to be bridge
							//		for each regoistered network, if active=Bridge then send the bridge beacon on this group
							synchronized (sendBeaconThread) {
								sendBeaconThread.wait (InfoType.BEACON_SEND_TIMEOUT);
							}
							
							if (currentState == InfoType.STATE_MANAGER) {
								//board.info (netname+"> sending manager beacon");
								sendBeaconManager();
							}
							
							if (daemon.getAgreeBridge()) {
								if (bridgeInterface.isActiveInterface (netConfig)) 
								{
									//board.info (netname+"> sending bridge beacon");
									sendBeaconBridge ();
								}
							}
							//								XXX PGR commented for now
							//								sendBeaconDevice ();
						}
						catch (InterruptedException e) {
							// maybe a request for the manager beacon has been sent, restart the loop
						}
					}
					// exit the thread loop...
					//board.info ("exit doSendBeaconLoop");
				}
					});	
			return socketThread;   
		}
		catch (Exception e)
		{
			//board.info("Error when creating MSDDaemon sendBeaconLoop Thread");
			e.printStackTrace();
			System.exit(-1);
		}
		return null;
	}
	
	
	/**
	 * Check the list of beacons received and remove the components that have 
	 * not sent a beacon recently
	 * @return
	 */
	public Thread checkBeaconLoop ()
	{
		try
		{
			Thread socketThread = new Thread( new Runnable() 
					{
				public void run() 
				{	
					NetworkDevice tmpDev;
					doCheckBeaconLoop = true;
					while (doCheckBeaconLoop)
					{
						// first uncheck all notifications for received beacons
						// - a device or bridge waits for the manager beacon
						// - a manager waits for the device and bridge beacons
						// then wait for the beacon timeout and then check for beacons that have not been received
						// if manager disappeared and agree to be manager, become manager
						// if manager disappeared and agree to be bridge, desactivate interface
						beaconManager = false;
						// need to be synch with the incoming control messages !!!
						synchronized (this) {
							for (int x=0; x<networkDeviceList.size(); x++)  {	
								tmpDev = (NetworkDevice) networkDeviceList.get(x);
								tmpDev.gotBeacon = false;
							}
						}
						
						try {
							// wait for the beacons to be received
							synchronized (checkBeaconThread) {
								checkBeaconThread.wait (InfoType.BEACON_RECEIVE_TIMEOUT);
							}
							
							
							// board.info ("check for missing devices (out of " + networkDeviceList.size() + " devices)");
							// check the list of devices in the network and checks for device that have not sent their beacon
							for (int x=0; x<networkDeviceList.size(); x++) {	
								tmpDev = (NetworkDevice) networkDeviceList.get(x);
								if (tmpDev.gotBeacon==false) {
									
									// if component is a manager and the missing device is a bridge, update the GUI
									if ( (currentState == InfoType.STATE_MANAGER) &&
											tmpDev.info.getAgreeBridge().equals(BigInteger.valueOf(1)) )
									{
										managerInterface.removeInfoComponent(InfoType.Component.BRIDGE, tmpDev.info);
									}
									
									// if the component is a bridge and the device is a manager, update the gui
									if ( (daemon.getAgreeBridge()) &&
											currentManager != null &&
											currentManager.getCompID().getName().equals(tmpDev.info.getCompID().getName()))
									{
										bridgeInterface.desactivateInterface(netConfig);
										bridgeInterface.removeInfoComponent(InfoType.Component.MANAGER, tmpDev.info);
									}
									networkDeviceList.remove(x);
									x = 0;
								}
							}
							
							if (beaconManager == false) {
								if (currentManager != null) {
									currentManager = null;
								}
								if (agreeManager) {
									// - if component is potential manager then becomes one (conflicts are resolved later)
									// board.info (netname + "> Become MSD manager for network " + netname );
									currentState = InfoType.STATE_MANAGER;
									currentManager = null;
									isManager = true;
									managerInterface.startManager();
								}
							}
							
							//								// ATTENTION some parts need to be synch with the incoming control messages !!!
							//								switch (currentState) {
							//								// if this device is a regular device, just checks for the dissappearnce of the MSDA manager
							//								case InfoType.STATE_DEVICE:
							//									synchronized (this) {
							//										if (beaconManager == false) {
							//											if (currentManager != null) {
							//												// MSDA Manager disappeared....
							//												// - if component is bridge, then desactivate the interface (can be reactivated by the next manager)
							//												// - and notify the bridge about this disappearance (update GUI)
							//												board.info (netname + "> MSD Manager disapeared for network " + netname );
							//												if (agreeBridge)
							//												{
							//													bridgeInterface.desactivateInterface(netConfig);
							//													bridgeInterface.removeInfoComponent(InfoType.Component.MANAGER, currentManager);
							//												}
							//												currentManager = null;
							//											}
							//											if (agreeManager) {
							//												// - if component is potential manager then becomes one (conflicts are resolved later)
							//												board.info (netname + "> Become MSD manager for network " + netname );
							//												currentState = InfoType.STATE_MANAGER;
							//												currentManager = null;
							//												isManager = true;
							//												managerInterface.startManager();
							//											}
							//										}
							//									}
							//								break;
							//									
							//								// if the device is the manager, checks for the dissapearance of bridges (and update the GUI)
							//								case InfoType.STATE_MANAGER:
							//									board.info ("check for missing devices (out of " + networkDeviceList.size() + " devices)");
							//									synchronized (this) {
							//										for (int x=0; x<networkDeviceList.size(); x++) {	
							//											tmpDev = (NetworkDevice) networkDeviceList.get(x);
							//											if (tmpDev.gotBeacon==false) {
							//												board.info ("> Beacon for device " + tmpDev.info.getCompID().getName() + " not received, device removed");
							//												if (tmpDev.info.getAgreeBridge().equals(BigInteger.valueOf(1)))
							//												{
							//													managerInterface.removeInfoComponent(InfoType.Component.BRIDGE, tmpDev.info);
							//												}
							//												networkDeviceList.remove(x);
							//												x = 0;
							//											}
							//										}
							//									}
							//									break;
							//								}
							
							
							
						}
						catch (InterruptedException e) 
						{
							//board.info ("Unexpected interruption");
						}
					}
					// exit the thread loop...
					//board.info ("exit doSendBeaconLoop");
				}
					});	
			return socketThread;   
		}
		catch (Exception e)
		{
			//board.info("Error when creating MSDDaemon sendBeaconLoop Thread");
			e.printStackTrace();
			System.exit(-1);
		}
		return null;
	}
	
	
	
	
	
	/**
	 * 
	 * @param mess
	 */
	public void sendControlMessage (MSDMessage mess)
	{
		try
		{
			communicator.sendMSDMessage(mess);
		}
		catch (MSDException ex)
		{
			//board.info("Cannot send control message");
			ex.printStackTrace();
			return;
		}
		return;
	}
	
	/**
	 * 
	 *
	 */
	public void sendActivateBridge(InformationComponent info)
	{
		MessageContent content = new MessageContentImpl();
		ControlMessage control = new ControlMessageImpl();
		activateBridge.setContent(null);
		control.setInfoComp(info);
		content.setControl(control);
		activateBridge.setContent(content);
		sendControlMessage (activateBridge);
	}
	
	/**
	 * to send network context information
	 *
	 */
	public void sendBeaconManager ()
	{		
		InformationComponent infoComponent = beaconMgr.getContent().getControl().getInfoComp();
		if (daemon.isContextEnabled())
		{
			ContextInformationTable netCtx = daemon.getManager().getContextManager().getLastNetworkCtx();
			if(netCtx!=null){
				infoComponent.setCtxInfo(netCtx);
				//			    board.info("[MSDDeamon]: ///////////////////// NetworkContext //////////////////////");	
				//			    board.info("[MSDDeamon]: CreationTime " + netCtx.getCreationTime());
				//			    board.info("[MSDDeamon]: NumServices " + netCtx.getNumServices());
				//			    board.info("[MSDDeamon]: NumActiveServices " + netCtx.getNumActiveServices());
				//			    board.info("[MSDDeamon]: NumUsers " + netCtx.getNumUsers());
			}
			else {
				//board.info("sendBeaconManager ::NetworkContext is null");
			}
		}			
		//			board.info("> send control message: beacon manager");
		sendControlMessage (beaconMgr);
		
	}
	
	/**
	 * 
	 *
	 */
	public void sendRequestManagerBeacon()
	{
		//board.info("> send control message: request manager beacon");
		sendControlMessage (existMgr);
	}
	
	/**
	 * 
	 */
	public void  sendBeaconDevice()
	{
		//board.info("> send control message: beacon device");
		sendControlMessage (beaconDevice);			
	}
	
	/**
	 * 
	 */
	public void  sendBeaconBridge()
	{
		//board.info("> send control message: beacon bridge");
		
		// WARNING commented this out...not clear if this is required...
		// ADLC- adds the list of active networks
		//		ControlMessage message = beaconBridge.getContent().getControl();
		//		InformationComponent information = message.getInfoComp();
		//		if ( information.getActvNets().size() != bridgeNetworks.size() ) {
		//			information.getActvNets().clear();
		//			information.getActvNets().addAll( bridgeNetworks );
		//		}
		
		sendControlMessage (beaconBridge);			
	}
	
	
	/* (non-Javadoc)
	 * @see org.msda.components.CommunicatorCallback#newDescription(org.msda.msd.MSDDescription)
	 */
	public void newDescription(MSDDescription description) {
		// not usefull
	}
	
	/* (non-Javadoc)
	 * @see org.msda.components.CommunicatorCallback#newRequest(org.msda.msd.MSDRequest)
	 */
	public void newRequest(MSDRequest request) {
		// not usefull
	}
	
	/* (non-Javadoc)
	 * @see org.msda.components.CommunicatorCallback#responseRequest(org.msda.msd.MSDMessage)
	 */
	public void responseRequest(MSDMessage message) {
		// not usefull
	}
	
	/* (non-Javadoc)
	 * @see org.msda.components.CommunicatorCallback#controlMessage(org.msda.msd.MSDMessage)
	 */
	public void controlMessage(MSDMessage message) {
		int messageType = message.getType().intValue();
		
		// If a manager beacon is received
		//  - if manager myself and different, only the one with best benefit (smaller id) stays)
		//  - if in start mode, register the manager ID and become a simple device (must also send info component aftwerwards)
		//  - if simple device, restart the manager beacon
		switch (messageType)
		{
		case InfoType.Message.BEACON_MANAGER:
			processIncomingBeaconManager (message);
		break;
		case InfoType.Message.EXIST_MANAGER:
			processIncomingExistManager (message);
		break;
		case InfoType.Message.BEACON_DEVICE:
			processIncomingBeaconDevice (message);
		break;
		case InfoType.Message.BEACON_BRIDGE:
			processIncomingBeaconBridge (message);
		break;
		case InfoType.Message.INFO_COMPONENT:
			processIncomingInfoComponent (message);
		break;
		case InfoType.Message.BRIDGE_ACTIVATION:
			processIncomingBridgeActivation (message);
		break;
		case InfoType.Message.BRIDGE_DESACTIVATION:
			processIncomingBridgeDesactivation (message);
		break;
		case InfoType.Message.GET_INFO_COMPONENT:
			processIncomingInfoComponent(message);
		break;
		case InfoType.Message.INVALID:
		case InfoType.Message.REMOTE_ACCESS:
			//case InfoType.Message.REMOTE_ACCESS_ERROR:
		case InfoType.Message.REMOTE_ACCESS_OK:
		case InfoType.Message.REQUEST_RESPONSE:
		case InfoType.Message.SERVICE_REQUEST:
		case InfoType.Message.SERVICE_UPDATE:
		default:
			//board.info ("MISSING: received an untreaded control message of type " + messageType);
		}
	}
	
	/**
	 * 
	 * @param message
	 */
	private void processIncomingBridgeDesactivation (MSDMessage message)
	{
		InformationComponent targetDevice = message.getContent().getControl().getInfoComp();
		if (MSDUtil.getInstance().compareID(targetDevice.getCompID(), daemon.getStackId() ))
		{
			//board.info ("incoming control message : DESACTIVATE BRIDGE" + message.getSender().getName() + " on net " + netname + "(" + ifname + ")" );
			if (daemon.getAgreeBridge() && bridgeInterface!=null)
			{
				bridgeInterface.desactivateInterface(this.netConfig);
			}
		}
	}
	
	/**
	 * 
	 * @param message
	 */
	private void processIncomingBridgeActivation (MSDMessage message)
	{
		InformationComponent targetDevice = message.getContent().getControl().getInfoComp();
		if (MSDUtil.getInstance().compareID(targetDevice.getCompID(), daemon.getStackId() ))
		{
			//board.info ("incoming control message : ACTIVATE BRIDGE" + message.getSender().getName() + " on net " + netname + "(" + ifname + ")" );
			if (daemon.getAgreeBridge() && bridgeInterface!=null)
			{
				bridgeInterface.activateInterface(this.netConfig);
			}
			
		}
	}
	
	
	/**
	 * 
	 * @param message
	 */
	private void processIncomingInfoComponent (MSDMessage message)
	{
		//board.info ("incoming control message : INFO COMPONENT  (not processed yet)" + message.getSender().getName() + " on net " + netname + "(" + ifname + ")" );
		// FOR NOW
		// if receive info component and is manager and info component of bridge
		// callback msd manager to add a bridge info
		//			InformationComponent tmpComp = message.getContent().getControl().getInfoComponent();
		//			if (agreeManager && isManager)
		//			{
		//			managerInterface.updateInfoComponent (InfoType.Component.DEVICE, tmpComp);
		//			}
		//			if (agreeBridge)
		//			{
		//			bridgeInterface.updateInfoComponent (InfoType.Component.DEVICE, tmpComp);
		//			}
	}
	
	/**
	 * 
	 * @param message
	 */
	private void processIncomingExistManager (MSDMessage message)
	{
		//board.info ("incoming control message : EXIST MANAGER" + message.getSender().getName() + " on net " + netname + "(" + ifname + ")" );
		// received a request for the MSD manager
		// - if msd manager, then wake up the beacon send thread so that the
		// manager beacon will be sent
		switch (currentState)
		{
		case InfoType.STATE_MANAGER:
			synchronized (sendBeaconThread) {
				sendBeaconThread.notify();
			}
		break;
		}
	}
	
	
	
	/**
	 * 
	 * @param message
	 */
	private void processIncomingBeaconManager (MSDMessage message) 
	{
		boolean sendBeaconDevice;
		
		// board.info ("incoming control message : BEACON MANAGER " + message.getSender().getName() + " on net " + netname + "(" + ifname + ")" );
		InformationComponent beaconComponent = message.getContent().getControl().getInfoComp();
		updateDeviceBeacon ( beaconComponent );
		
		switch (currentState)
		{
		// if a device in startin mode receive th msd manager beacon, it becomes 
		// an active device, initialize its manager related member variables
		// awake the other threads and send the device beacon (only sent once for now)
		// this beacon contains information on wether the device agrees to be an msd bridge
		// if so, it may be elected.
		case InfoType.STATE_START:
			// if starting and manager exist, go into simple device mode
			String managerIP, netif;
			
			synchronized (this) {
				currentState = InfoType.STATE_DEVICE;
				beaconManager = true;
				isManager = false;
				currentManager = beaconComponent;
				this.notifyAll();
			}
			
			managerIP = beaconComponent.getIPAddr();
			netif = daemon.getNetworkInterface(beaconComponent.getNetName());
		
			// Create the client of the LKH tree
			if ( InfoType.ACTIVE_SECURITY ) {
				lkhclient = ClientLKH.createLKHClient (daemon.startup.getNetworks(), managerIP, beaconComponent.getNetName());
			}
			
			if (agreeManager)
			{
				//board.info ("update info compo. manager");
				managerInterface.updateInfoComponent (InfoType.Component.MANAGER, beaconComponent);
			}
			if ( daemon.getAgreeBridge() )
			{
				//board.info ("update info compo. bridge");
				bridgeInterface.updateInfoComponent(InfoType.Component.MANAGER, beaconComponent);
			}
			this.sendBeaconDevice();
		break;
		
		// if the device is already active, then it reinit the manager beacon (still present)
		// and checks if the beacon is for the same manager, a new one, or if the manager was not initialized
		// if a new MSD manager, then the device sends its info component information
		case InfoType.STATE_DEVICE:
			// in DEVICE mode, manager ok
			sendBeaconDevice = false;
		synchronized (this) {
			beaconManager = true;
			isManager = false;
			if ( (currentManager != null) && (MSDUtil.getInstance().compareID (beaconComponent.getCompID(), currentManager.getCompID())) )
			{
				// same manager do nothing
			}
			else
			{
				// NEW MSD MANAGER
				if (agreeManager)
				{
					// update info compo. manager
					managerInterface.updateInfoComponent (InfoType.Component.MANAGER, beaconComponent);
				}
				if ( daemon.getAgreeBridge() )
				{
					// update info compo. bridge
					bridgeInterface.desactivateInterface(netConfig);
					//							XXX PGR								if (currentManager != null) bridgeInterface.removeInfoComponent(InfoType.Component.MANAGER, currentManager);
					bridgeInterface.updateInfoComponent(InfoType.Component.MANAGER, beaconComponent);
				}
				currentManager = beaconComponent;
				//board.info ("   - and send my device beacon (to be elected if necessary)");
				sendBeaconDevice = true;
			}
		}
		if (sendBeaconDevice)
			this.sendBeaconDevice();
		break;
		
		
		// if already a manager, and the id of the beacon is different than mine
		// then there is a conflict that needs to be resolved.
		// for now, 
		case InfoType.STATE_MANAGER:
			// if also manager, then one of the two should suspend...
			//	board.info ("in MANAGER mode also...");
			if (MSDUtil.getInstance().compareID (beaconComponent.getCompID(), daemon.getStackId() )) {
				//	board.info ("...its me, no problem");						
			}
			else 
			{
				// need to decide which one stays as a manager. 
				// simple comparison only based on benefit, should have a fallbck mechanism 
				// for now the benefit is the time (in millisec) when the app was started.
				float otherBenefit = beaconComponent.getBenefit();
				//
				synchronized (this) {
					if (this.benefit<otherBenefit) {
						//board.info (" - conflict: become a DEVICE");
						currentState = InfoType.STATE_DEVICE;
						beaconManager = true;
						isManager = false;
						this.managerInterface.stopManager();
						if ( daemon.getAgreeBridge() ) {
							//board.info ("   - (bridge) first desactivate my old interface");
							bridgeInterface.desactivateInterface(netConfig);
							//								XXX PGR									bridgeInterface.removeManager(netConfig, currentManager);
							bridgeInterface.updateInfoComponent(InfoType.Component.MANAGER, beaconComponent);
						}
						currentManager = beaconComponent;
					}
					else 
					{
						//board.info ("- conflict: stay MANAGER");
						currentManager = null;
						isManager = true;
					}
				}
			}
		}
	}
	
	
	/**
	 * 
	 * @param message
	 */
	private void processIncomingBeaconBridge (MSDMessage message)
	{
		// received a bridge beacon
		// - if bridge already exist, restart the counter, otherwise add the bridge
		// board.info ("incoming control message : BEACON BRIDGE " + message.getSender().getName() + " on net " + netname + "(" + ifname + ")" );
		InformationComponent beaconComponent = message.getContent().getControl().getInfoComp();
		updateDeviceBeacon ( beaconComponent );
		
		if (agreeManager && isManager)
		{
			//board.info ("update info compo. bridge");
			managerInterface.updateInfoComponent (InfoType.Component.BRIDGE, beaconComponent);
		}
		if ( daemon.getAgreeBridge() )
		{
			//board.info ("update info compo. bridge");
			bridgeInterface.updateInfoComponent (InfoType.Component.BRIDGE, beaconComponent);
		}
	}
	
	/**
	 * 
	 * @param message
	 */
	private void processIncomingBeaconDevice (MSDMessage message)
	{
		// FOR NOW
		// if receive info component and is manager and info component of bridge
		// callback msd manager to add a bridge info
		// board.info ("incoming control message : BEACON DEVICE " + message.getSender().getName() + " on net " + netname + "(" + ifname + ")"  + agreeManager + " " + isManager);
		InformationComponent beaconComponent = message.getContent().getControl().getInfoComp();
		updateDeviceBeacon ( beaconComponent );
		
		if (agreeManager && isManager)
		{
			//board.info ("update info compo. manager");
			managerInterface.updateInfoComponent (InfoType.Component.DEVICE, beaconComponent);
		}
		if ( daemon.getAgreeBridge() )
		{
			//board.info ("update info compo. bridge");
			bridgeInterface.updateInfoComponent (InfoType.Component.DEVICE, beaconComponent);
		}
	}
}
