/*
 * Created on Aug 27, 2004
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.interfaces;

import org.msda.msd.*;

/**
 * This callback interface is used by classes that register with an 
 * MSDA group to receive MSDA messages (e.g., MSDA bridges and MSDA managers).<br>
 * NOTE: The sender of a message receives its message (in the listenner thread) and should
 * therefore filter it if required.
 *  
 * @author raverdy 
 */

public interface CommunicatorCallback 
{
	
	/**
	 * Receives an incoming MSD description from the MSD group for processing.<br>
	 * 
	 * @param description : description received from the MSD group. 
	 */
	
	public void newDescription(MSDDescription description);

	/**
	 * Receives an incoming MSD request from the MSD group for processing.<br>
	 * 
	 * @param request : request received from the MSD group. 
	 */
	public void newRequest(MSDRequest request);
	
	/**
	 * Receives an incoming MSD message from the MSD group for processing.<br>
	 * The incoming MSD message contains a response to a discovery request.
	 * 
	 * @param message : message received from the MSD group
	 */
	public void responseRequest(MSDMessage message);
	
	
	/**
	 * Receives an incoming MSD message of type InformationComponent.<br>
	 * 
	 * @param message : message received from the MSD group
	 */
	public void controlMessage( MSDMessage message ); 
}
