/*
 * Created on Aug 13, 2004
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.interfaces;

import org.msda.msd.*;
import org.msda.util.*;


/**
 * This interface is used to configure the MSDA bridge and get status information.
 * 
 * @author raverdy
 */
public interface MSDABridgeInterface { 


	/**
	 * Creates an entry for a new network to connect to. <br>
	 * In theory, all manager/bridges communication in different networks should use the same default multicast group and 
	 * port but for testing reasons (not enough computers), the group and port can be specified.
	 * This method needs to be synchronized as it manipulates the interface list.
	 *
	 * @param networkConfig that contains <p>
	 *  		netname 	name of the network (for id purposes) <br>
	 * 			ifname		name of nthe network interface to use (eth0...) <br>
	 * 			groupAddress	multicast group address to use for communicating with the MSDA manager <br>
	 * 			groupPort 	port number to use for communicating with the msd manager 
	 *  
	 * @return true if the interface could be added, false otherwise
	 */
    public boolean activateInterface( NetworkConfig networkConfig );

    /**
     * Desactivates the network interface given in the <code>networkConfig</code>
     * 
     * @param networkConfig	The network interface
     * @return
     */
    public boolean desactivateInterface( NetworkConfig networkConfig );

	/**
	 * Sets the bridging status for network interface
	 * 
	 * @param netId the network id of the network to setup
	 * @param activateIn true if bridge should accept incoming messages from this network, false otherwise
	 * @param activateOut true if bridge should propagate outgoing messages to this network, false otherwise
	 */
	public void setBridgingStatus ( UbisecID netId, boolean activateIn, boolean activateOut ) throws MSDException;

	/**
	 * Returns the bridging status for incoming packets on the specified network
	 * 
	 * @param netId the network to check
	 * @return true if incoming messages accepted, false otherwise
	 */	
	public boolean getIncomingBridgingStatus ( UbisecID netId ) throws MSDException;
	
	/**
	 * Returns the bridging status for outgoing packets on the specified network
	 * 
	 * @param netId the network to check
	 * @return true if messages propagated to this network, false otherwise
	 */
	public boolean getOutgoingBridgingStatus ( UbisecID netId ) throws MSDException;
	
	/**
	 * Returns if the network interface identified by <code>networkConfig</code> is active or not. 
	 * 
	 * @param networkConfig
	 * @return	true if the network interface is active, false otherwise
	 */
	public boolean isActiveInterface (NetworkConfig networkConfig);
	
	/**
	 *  Updates the cache of local components based on the received information about the other component.
	 * 
	 * @param typeComponent	The component type
	 * @param info			The information about the component
	 */
	public void	updateInfoComponent (int typeComponent, InformationComponent info);
	
	/**
	 * 
	 * @param typeComponent
	 * @param info
	 */
	public void removeInfoComponent(int typeComponent, InformationComponent info);

}
