/*
 * Created on Aug 12, 2004
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.interfaces;


import org.msda.context.MSDContextManager;
import org.msda.msd.*;

/**
 * This interface is used by the boostrapping method to configure the MSD manager.
 * 
 * @author raverdy
 *
 */
public interface MSDAManagerInterface { 

	/**
	 * Activates the MSD manager
	 * 
	 * @return true if the MSD manager is active without problem, false otherwise
	 */
	public boolean startManager();
	
	/**
	 * Desactivates the MSD manager (e.g., when the daemon/device detects another
	 * MSD Manager in the network).
	 * 
	 * @return 
	 */
	public boolean stopManager();

	/**
	 * Terminates the MSD Manager
	 */
	public void terminate();

	/**
	 * Starts a specific SD plugin that is dynamically loaded. 
	 * 
	 * @param pluginName 		the name of the plugin to load and start
	 * @param pluginCreator		the name of the plugin creator
	 * @param pluginClassName	the name of the java class to load
	 * @param args 				the arguments to start the plugin
	 * @param httpServerPort	server port for the plugin
	 * @param directMSDPort		server port for direct (socket) connection to the manager
	 * @return true if the plugin is started, false otherwise
	 */
	public boolean startPlugin (String pluginName, String pluginCreator, String pluginClassName, String[] args, int httpServerPort, int directMSDPort );
	
	/**
	 * Stops a specific SD plugin
	 * 
	 * @return true if stopped, false otherwise 
	 */
	public boolean stopPlugin ();
	
	/**
	 * TODO: check if relevant in the msd manager 
	 * 
	 * @return 
	 */
	public boolean startTransformer ();
	
	/**
	 * TODO: check if relevant in the msd manager 
	 * 
	 * @return
	 */
	public boolean stopTransformer ();
	
	/**
	 * Returns the unique ID of the MSD manager (UbisecID)
	 * 
	 * @return the MSD manager UbisecID
	 */
	public UbisecID	getMSDManagerID ();
	
	/**
	 * Sets the unique ID of the MSD manager (UbisecID)
	 * 
	 * @param id the UbisecID of the MSD manager
	 */
	public void	setMSDManagerID (UbisecID id);
	
	
	/**
	 * Returns the manager name 
	 * 
	 * @return	Returns the manager name 
	 */
	public String getName();
	
	/**
	 * Returns the network name of the manager
	 * 
	 * @return	Returns the network name of the manager
	 */
	public String getNetworkName();
	
	/**
	 * Returns the remote access port of the manager
	 * 
	 * @return	Returns the remote access port of the manager
	 */
	public int getRemoteAccessPort();
	
	
	/**
	 * Returns the ContextManager
	 * 
	 * @return Returns the ContextManager
	 */
	public MSDContextManager getContextManager();
	
	/**
	 * Updates the cache of local components based on the received information about the other component.<br>
	 * In a specific case, the manager can activate the election of a bridge.
	 * 
	 * @param typeComponent		The component type (DEVICE, MANAGER, BRIDGE)
	 * @param info				The information about the component
	 */
	public void	updateInfoComponent (int typeComponent, InformationComponent info);

	/**
	 * 
	 * @param typeComponent
	 * @param info
	 */
	public void removeInfoComponent(int typeComponent, InformationComponent info);

}
