/*
 * Created on Aug 27, 2004
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.interfaces;

import org.msda.msd.*;
import org.msda.util.*;

/**
 * This interface is used to register a client callback with a msd group, and indirectly create msd groups.
 * 
 * @author raverdy 
 *
 */
public interface MSDANetworkInterface {  
	/**
	 * Creates an entry for communicating with a msd group (msd manager and bridges) and returns the interface to this msd group.
	 * With the usual parameters (ip address and port) the caller should provide a callback interface.
	 * 
	 * @param netId unique ID of the MSD component (MSND Manager or MSD Bridge) that is added to messages sent to the network
	 * @param ifname network interface name to use for connecting to the network
	 * @param groupAddress multicast group address to use for connecting to the network
	 * @param groupPort port to use for connecting to the network
	 * @param callback a CommunicatorCallback interface used for forwarding messages received from the network
	 * @return the proxy interface for sending meta descriptions/requests on the network
	 * @throws MSDException
	 */
	public CommunicatorInterface registerCallback (UbisecID netId, String ifname, String groupAddress, int groupPort, CommunicatorCallback callback) throws MSDException;
}
