/*
 * Created on Aug 16, 2004
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.interfaces;

import java.util.*;

import org.msda.msd.*;
import org.msda.util.*;

/**
 * This interface, implemented by each SD plugin, is used to intialize the plugin, and control it.
 * WARNING the name this class (i.e., the full path) should be the same as the one used in the MSDA Manager (dynamic classloader)
 * @author raverdy
 */
public interface PluginInterface {  

	/**
	 * Initializes the plugin mode
	 * 
	 * @param mode the mode (active or passive) the plugin should run in
	 */
	public void setMode ( int mode );

	/**
	 * Initializes the plugin info
	 * 
	 * @param name 				the plugin name
	 * @param creator 			the plugin creator
	 * @param httpServerPort	the HTTP server port to receive SOAP messages
	 * @param directMSDPort		to register the direct MSD service in its domain 
	 */
	public void setPluginInfo ( String name, String creator, int httpServerPort, int directMSDPort );
	
	/**
	 * Sets a parameter for the plugin.
	 * <p>
	 * TODO also add a method to clean up the info...
	 * 
	 * @param name the name of the parameter
	 * @param value the value of the parameter
	 */
	public void setPluginParam ( String name, String value );
	
	/**
	 * Registers the msd manager by providing its plugin callback interface in order to receive msd descriptions from the SD plugin
	 * 
	 * @param callbckInterface the callback interface of the msd manager
	 * @param coreInterface the interface to the msd core component
	 * @throws MSDException throws an exception if the registration could not be performed
	 */
	public void registerManager ( PluginCallback callbckInterface,  RequestInterface msdInterface )throws MSDException;
	
	/**
	 * Registers a transformer
	 * <p>
	 * TODO check if this method is necessary
	 * 
	 * @param transformerClassName the interface to communicate with the transformer
	 */
	public void registerTransformer ( String transformerClassName );
	
	/**
	 * Activates the SD plugin
	 *
	 */
	public void activate();	

	/**
	 * Terminates the SD plugin
	 *
	 */
	public void terminate();	
	

	/**
	 * Set the nofification mode of the plugin as either active(sends notification) or passive (manager needs to send request)
	 * 
	 * @param mode true if plugin has to send nofication for all new services
	 * @return true if the plugin supports the requested mode (either sends notification or process msdrequests)
	 */	
	public boolean setNotify ( boolean mode );
	
	/**
	 * Returns the notification mode of the plugin
	 * 
	 * @return true if the plugin sends notification or false if it process msdrequests
	 */
	public boolean doNotify ();	
	
	/**
	 * Evaluates an msd request and returns the list of matching services (msd descriptions)
	 * 
	 * @param request the msd request to evaluate
	 * @return the list of matching services
	 */
	public Vector evaluateRequest ( MSDRequest request );
	
	/**
	 * Sends the SOAP message to the service endpoint (URL).
	 * 
	 * @param sOAPMessage	the SOAP message
	 * @param serviceEP		the service endpoint
	 * @return
	 */
	public String sendToService(String sOAPMessage, String serviceEP);
	
	//
	// Needed by the GUI
	//
	
	/**
	 * Returns a string containing plugin information
	 * 
	 * @return String containing the plugin info
	 */
	public String getPluginInfo();
	
	/**
	 * Returns the plugin name
	 * 
	 * @return String	The plugin name
	 */
	public String getName();
	
}
