/*
 * Created on 12 mai 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.interfaces;

import org.msda.msd.MSDDescription;
import org.msda.msd.MSDRequest;

/**
 * This interface, implemented by the MSD manager and the plugins, is used by specific SD plugins to 
 * forward to the MSD manager the client request. <br>
 * Theses requests can be a :<br>
 * 	- service discovery <br>
 *  - service access
 * 
 * @author delachap
 *
 */
public interface RequestInterface {
    
    
    /**
     * Performs a service discovery.
     * 
     * @param request	The client request.
     * 
     * @return	Returns all the MSD Descriptions that satisfies the MSD Request. 
     */
	public MSDDescription[] getServices(MSDRequest request);
	
	/**
	 * Performs a service access (local or remote) from the manager of the client's network 
	 * to the manager of the service's network.
	 * 
	 * @param soapMessage	The access as a soap message
	 * @param descId		The MSD description ID of the service to access
	 * @param epService		The endPointReference of the service, i.e. its IP address in its network
	 * 
	 * @return 	The soap message response (as a String) of the service
	 */
	public String serviceAccess(String soapMessage, String descId, String epService);
	
}
