/*
 * Created on May 2, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.manager;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

import javax.xml.bind.JAXBException;

import org.msda.components.manager.object.ObjectRemoteAccess;
import org.msda.components.utils.ComponentThread;
import org.msda.framework.FrameworkOutput;
import org.msda.msd.MSDMessage;
import org.msda.msd.UbisecID;
import org.msda.msd.impl.UbisecIDImpl;
import org.msda.util.JaxMeUtil;
import org.msda.util.comm.NetworkManager;

/**
 * Thread that listens to the <RemoteAccessPort>
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 */
public class ThreadRemoteAccessListener extends ComponentThread {

    private FrameworkOutput board;
    private MSDManagerPool managerPool;
    
    private int port;
    private JaxMeUtil jaxme;
    private NetworkManager networkManager;
    
    protected ServerSocket serverSocket;
    
    /**
     * Constructor.
     * 
     * @param port  		the remote access port
     * @param managerPool
     */
    public ThreadRemoteAccessListener( int port, MSDManagerPool managerPool ) {
        
        super();
		this.board = org.msda.framework.FrameworkMain.msdaOutput;
        this.managerPool = managerPool;
        this.port = port;
        this.jaxme = new JaxMeUtil();
        this.networkManager = new NetworkManager();
        
        board.debug( "Listener for remote access on port "+port );
        
        try {
            this.serverSocket = new ServerSocket ( port );
            
        } catch (IOException e) {
            e.printStackTrace();
        }
        
        this.start();
    }
    
    public void terminate() {
        try {
            this.serverSocket.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    public void run() {
        
        while(active) {
            
            try {
                Socket socket = serverSocket.accept();
                
            	byte[] res = networkManager.getReceivedData( socket, false );
                MSDMessage mess = jaxme.toMSDMessage( res );
                
                ObjectRemoteAccess object = new ObjectRemoteAccess();
                object.setMessage( mess );
                UbisecID networkId = new UbisecIDImpl();
                networkId.setName( managerPool.getNetworkName() );
                object.setNetworkID( networkId );
                object.setSocket( socket );
                
                managerPool.getBlockingQueue().add( object );
                
            } catch (JAXBException e) {
                board.notifyError("ThreadRemoteAccessListener::run : Could not get a connection on the server socket. " + e.getMessage() );
                board.notifyException (e);
            } catch( IOException e ) {
                board.notifyError("ThreadRemoteAccessListener::run : Could not get a connection on the server socket. " + e.getMessage() );
                board.notifyException (e);
            }
        }
    }
}
