/*
 * Created on Apr 21, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.manager.client;

import java.util.Vector;

/**
 * A queue used to block a thread.
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 */
public class BlockingQueue {
	// Vector is synchronized by default
    private Vector queue;
    private boolean closed;
    
    /**
     * Initializes a new queue.
     *
     */
    public BlockingQueue() {
        this.queue = new Vector();
        this.closed = false;
    }
    
    /**
     * Closes the queue.
     */
    public void close() {
        this.closed = true;
        synchronized(this) {
            notifyAll();
        }
    }
    
    /**
     * @return true if the queue is closed, false otherwise.
     */
    public boolean isClosed() {
        return this.closed;
    }
    
    /**
     * @return true if the queue is empty, false otherwise.
     */
    public boolean isEmpty() {
        return this.queue.isEmpty();
    }
    
    /**
     * When a new Object is added to the vector, a notify wakes one of the available thread.
     * 
     * @param object	The object added to the queue.
     */
    public synchronized void add( Object object ) {
            this.queue.add( object );
            notifyAll();
    }
    
    /**
     * The available threads are waiting until <code>notify()</code> wakes up one of them.
     * 
     * @return the older Object added in the queue  
     * @throws InterruptedException
     */
    public synchronized Object get() throws InterruptedException {
        
        while( !closed && isEmpty() ) {
            wait();
        }
        
        if ( closed && isEmpty() )
            return null;
        
        // return the older element of the vector
        return this.queue.remove( 0 );
    }
}
