/*
 * Created on Sep 20, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.manager.client;

import java.io.IOException;
import java.math.BigInteger;
import java.net.Socket;

import javax.xml.bind.JAXBException;

import org.msda.msd.MSDDescription;
import org.msda.msd.MSDMessage;
import org.msda.msd.MSDRequest;
import org.msda.msd.MessageContent;
import org.msda.msd.impl.MSDMessageImpl;
import org.msda.msd.impl.MessageContentImpl;
import org.msda.util.InfoType;
import org.msda.util.JaxMeUtil;

/**
 * A client connection managed by the ClientConnectionMgr
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class ClientConnection extends Thread {

    private Socket socket;
    private ClientConnectionMgr mgr;
    private JaxMeUtil jaxme;
    
    private boolean loop;
    
    /**
     * A client connection.
     * 
     * @param mgr	The client connection manager
     */
    public ClientConnection( ClientConnectionMgr mgr ) {
        this.mgr = mgr;
        this.loop = true;
        this.socket = null;
        this.jaxme = new JaxMeUtil();
        
        this.start();
    }
    
    public void run() {
        
        while( loop ) {
            
            try {
                socket = (Socket)mgr.getQueueConnection().get();
                
                byte[] buf = new byte[InfoType.MSG_BUFFER_SIZE];
                
                int size = socket.getInputStream().read( buf );
                if ( size < 0) 
                    continue;
                
                // Resize the byte[] to keep only the used bytes
                byte[] tmp = buf; 
                buf = new byte[size];
                System.arraycopy(tmp, 0, buf, 0, size);
                
                //board.info( new String(buf));
                
                MSDMessage mess = jaxme.toMSDMessage( buf );
                
                switch( mess.getType().intValue() ) {
                
                case InfoType.Message.CLIENT_NEW_REQUEST:
                case InfoType.Message.CLIENT_RUNNING_REQUEST:
                    
                    // 
                    MSDRequest request = mess.getContent().getRequest();
                    MSDDescription[] results = mgr.getManager().getServices( request );
                    
                    // creating the response
                    MSDMessage response = new MSDMessageImpl();
                    response.setSender( mgr.getManager().getMSDManagerID() );
                    response.setType( BigInteger.valueOf( InfoType.Message.CLIENT_RESPONSE ) );
                    
                    if ( results!=null && results.length>0 ) {
                        MessageContent content = new MessageContentImpl();
                        response.setContent( content );
                        for (int i=0; i<results.length;i++) 
                            response.getContent().getResults().add( results[i] );
                    } 
                    
                    String res = jaxme.toString( response );
                    
                    // sending it back to the client
                    socket.getOutputStream().write( res.getBytes() );
                    break;
                }
                
            } catch (InterruptedException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } catch (JAXBException e) {
                e.printStackTrace();
            }
        }
        
        // reinit
        this.loop = true;
        this.socket = null;
    }
}
