/*
 * Created on Sep 20, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.manager.client;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

import org.msda.components.manager.MSDManager;

/**
 * Thread that manages the socket connections.
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class ClientConnectionMgr extends Thread {

    private static final int MAX_CLIENTS = 10;
    
    private BlockingQueue queueConnection;
    private ServerSocket serverSocket;
    private ClientConnection[] arrayClients;
    private MSDManager manager;
    
    private boolean loop;
    
    /**
     * Manager of client connections
     * 
     * @param manager	The MSD manager
     * @param port		The port for socket connections
     */
    public ClientConnectionMgr( MSDManager manager, int port ) {
        
        try {
            this.serverSocket = new ServerSocket(  port );
        } catch (IOException e) {
            e.printStackTrace();
            return;
        }
        
        this.loop = true;
        this.manager = manager;
        this.queueConnection = new BlockingQueue();
        
        arrayClients = new ClientConnection[MAX_CLIENTS];
        for( int i=0; i<arrayClients.length; i++)
            arrayClients[i] = new ClientConnection( this ); 
        
        this.start();
    }
    
    
    public void run() {
        
        while( loop ) {
            try {
                Socket socket = serverSocket.accept();
                this.queueConnection.add( socket );
                
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    
    /**
     * Returns the queue containing the socket connection 
     * 
     * @return Returns the queueConnection.
     */
    public BlockingQueue getQueueConnection() {
        return queueConnection;
    }
    
    /**
     * Returns the MSD manager
     * 
     * @return Returns the MSD manager.
     */
    public MSDManager getManager() {
        return manager;
    }
}
