/*
 * Created on Apr 19, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.manager.task;


import org.msda.components.manager.MSDManagerPool;
import org.msda.components.manager.object.Object4Queue;
import org.msda.components.manager.object.ObjectLocalRequest;
import org.msda.framework.FrameworkOutput;
import org.msda.msd.MSDRequest;
import org.msda.util.MSDException;

/**
 *  Task that manages local client requests.<br>
 * 
 *  @author raverdy, delachap - INRIA ARLES
 */
public class TaskLocalRequest extends TaskAbstract {

    private ObjectLocalRequest object;
    private FrameworkOutput board;
    private MSDManagerPool managerPool;

    /**
     * TaskLocalRequest.
     * 
     * @param msdManagerPool
     */
    public TaskLocalRequest( MSDManagerPool managerPool ) {
        
        this.managerPool = managerPool;
		this.board = org.msda.framework.FrameworkMain.msdaOutput;
        this.object = null;
    }
    

    /**
     * @see org.msda.components.manager.task.TaskAbstract#setObject(org.msda.components.manager.Object4Queue)
     */
    public void setObject(Object4Queue object) {
        this.object = (ObjectLocalRequest)object;
    }
    
    
    /**
     * @see org.msda.components.manager.task.TaskAbstract#terminate()
     */
    public void terminate() {
        
        synchronized( object ) {
	        object.notify();
	    }
        
        this.managerPool = null;
        this.board = null;
        this.object = null;
    }
    
    /**
     * @see org.msda.components.manager.task.TaskAbstract#run()
     */
    public void run () 
    {	      
        board.debug( "Starting TaskLocalRequest ..." );
        
        MSDRequest request = object.getRequest();
        boolean exist = object.isExistPendingRequest();
        if ( !exist ) {
            
            // 1 - Looks for compatible cache remote description. 
            try {
                // If there is nothing in the cache of remote descriptions 
                // send the request for propagation
                managerPool.getCommunicator().sendMSDRequest(request);
            } catch (MSDException e2) {
                board.info( "TaskLocalRequest:: Couldn't send a msd request on the network :"+e2.getMessage() );
            }
            
            // 2 - then, sends the request to the first plugin
            managerPool.evaluateRequest( request, false );
            
            // 3 - finally, looks for compatible cache local description 
            managerPool.getDescManager().evaluateRequest( managerPool.getMSDManager(), managerPool.getBlockingQueue(), request, true, false );
            
            board.debug("TaskLocalRequest:: End of the request "+System.currentTimeMillis());
        
        } else {      
            // existing request
            
//          1 - Looks for compatible cache remote description. 
            managerPool.getDescManager().evaluateRequest( managerPool.getMSDManager(), managerPool.getBlockingQueue(), request, false, false );
           
//          2 - looks for compatible cache local description 
            managerPool.getDescManager().evaluateRequest( managerPool.getMSDManager(), managerPool.getBlockingQueue(), request, true, false );
        }
    }
    
}
