/*
 * Created on May 20, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.manager.task;


import java.io.IOException;
import java.math.BigInteger;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Iterator;
import java.util.List;

import javax.xml.bind.JAXBException;

import org.msda.components.manager.MSDManagerPool;
import org.msda.components.manager.object.*;
import org.msda.framework.FrameworkOutput;
import org.msda.msd.*;
import org.msda.msd.impl.*;
import org.msda.util.InfoType;
import org.msda.util.JaxMeUtil;
import org.msda.util.PropagationUtil;
import org.msda.util.comm.NetworkManager;

/**
 * Task that manages remote service access.<br>
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 */
public class TaskRemoteAccess extends TaskAbstract {
    
    private MSDManagerPool managerPool;
    private FrameworkOutput board;
    private JaxMeUtil jaxme;
    private NetworkManager networkManager;
    
    private ObjectPlugin objectPlugin;
    private ObjectRemoteAccess object;
    
    /**
     * TaskRemoteAccess.
     * 
     * @param managerPool
     */
    public TaskRemoteAccess( MSDManagerPool managerPool ) {
        
        this.managerPool = managerPool;
		this.board = org.msda.framework.FrameworkMain.msdaOutput;
        this.jaxme = new JaxMeUtil();
        this.networkManager = new NetworkManager();
        this.objectPlugin = null;
        this.object = null;
    }
    
    /**
     * @see org.msda.components.manager.task.TaskAbstract#setObject(org.msda.components.manager.Object4Queue)
     */
    public void setObject(Object4Queue object) {
        this.object = (ObjectRemoteAccess)object;
    }
    
    /**
     * @see org.msda.components.manager.task.TaskAbstract#terminate()
     */
    public void terminate() {
        
        this.managerPool = null;
        this.board = null;
        this.jaxme = null;
        this.networkManager = null;
        this.objectPlugin = null;
        this.object = null;
    }
    
    /**
     * @see org.msda.components.manager.task.TaskAbstract#run()
     */
    public void run() {
        
        if (board!=null) board.debug( "Starting TaskRemoteAccess..." );
        
        try { 
            MSDMessage message = object.getMessage();
            Socket socketIn = object.getSocket();
            String pluginName = message.getContent().getSvcAccess().getPluginName();
            UbisecID srcNetworkID = object.getNetworkID();
            
            board.debug( "the message come from "+srcNetworkID.getName() );
            
            PropagationInformation pi = message.getBackwardPath();
            if (pi==null) { // can't happen...
                board.notifyError( "There is no propagation information associated to this message." );
                return;
            }
            
            List propagation = pi.getHops();
            if ( !propagation.isEmpty() ) {
                
                //
                // Forwards it to the bridge...
                //
                int index = PropagationUtil.getInstance().getNetworkFromSrc( srcNetworkID, propagation );
                board.debug(" index="+index);
                try {
                    board.debug(" message.getBackwardPath().getHops().size() ="+propagation.size());
                    Hop previousHop = (Hop)propagation.remove( index );
                    board.debug(" message.getBackwardPath().getHops().size() ="+propagation.size());
                    
                    Hop hop = (Hop)propagation.get( index );
                    InformationComponent infoRes = this.managerPool.getLocalMgr().getBridge( hop.getSrcNetID().getName(), hop.getDestNetID().getName() );
                    
                    byte[] msg = jaxme.toString( message ).getBytes();
                    
                    board.debug(" Case manager to bridge : is going to send..." );
                    // search the bridge with destIPAddress
                    NetworkConfig netConfig = this.managerPool.getLocalMgr().getNetworkConfig( infoRes, hop.getDestNetID().getName() );
                    int port = netConfig.getBridgeRemoteAccessPort().intValue();
                    
                    Socket socketOut = networkManager.sendTo( msg, InetAddress.getByName(infoRes.getIPAddr()), port );
                    
                    board.debug(" Case manager to bridge : done.. on port "+port+" "+infoRes.getIPAddr() );
                    
                    UbisecID networkID = new UbisecIDImpl();
                    networkID.setName(managerPool.getNetworkName());
                    
                    board.debug("Wait for incoming response...");
                    
                    byte[] res = networkManager.getReceivedData( socketOut );
                    
                    board.debug(" Case  manager to bridge : send them back to " + srcNetworkID.getName());

                    board.debug( "Waiting for response" );
                    		
                    networkManager.sendTo(  res, socketIn );
                    
                    board.debug(" Case  manager to bridge : send back done ");
                    
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
                
            } else {
                //
                //  Access to the plugin
                //
                board.debug("Case manager to service...");
                
                objectPlugin = new ObjectPlugin();
                objectPlugin.setEpService( message.getContent().getSvcAccess().getEPSvc() );
                objectPlugin.setSoapMessage( message.getContent().getSvcAccess().getSOAPMess() );
                
                // search the index of the thread that manages the plugin
                board.info( "ThreadRemoteAccess:: pluginName="+pluginName );
                Integer index = (Integer)managerPool.getPluginIndex().get( pluginName );
                board.info( "ThreadRemoteAccess:: index"+index );
                if ( index==null ) {
                    // TODO - erase that part...
                    board.debug( "ThreadRemoteAccess:: what follows is not correct - TO ERASE LATER");
                    board.debug( "ThreadRemoteAccess:: search as a substring "+pluginName );
                    Iterator it = managerPool.getPluginIndex().keySet().iterator();
                    while( it.hasNext() ) {
                        String key = (String)it.next();
                        //board.info(" key = "+key );
                        if ( key.toUpperCase().indexOf( pluginName.toUpperCase() ) > 0 ) {
                            board.info( "ThreadRemoteAccess:: found "+key);
                            pluginName = key;
                            index = (Integer)managerPool.getPluginIndex().get( pluginName );
                            break;
                        }
                    }
                }
                int i = index.intValue();
                objectPlugin.setIndex( i );
                managerPool.getTaskPlugins()[i].setObject( objectPlugin );
                managerPool.getTaskPlugins()[i].run();
                String soapResponse = managerPool.getTaskPlugins()[i].getResponse();
                
                // creates the response message
                MSDMessage response = new MSDMessageImpl();
                UbisecID networkID = new UbisecIDImpl();
                networkID.setName( managerPool.getNetworkName() );
                response.setSender( networkID );
                response.setType( BigInteger.valueOf( InfoType.Message.REMOTE_ACCESS_OK ) );
                response.setRespCode( message.getRespCode() );
                
                ServiceAccess serviceAccess = new ServiceAccessImpl();
                serviceAccess.setSrcNetID( networkID );
                serviceAccess.setSOAPMess( soapResponse );
                serviceAccess.setEPSvc( objectPlugin.getEpService() );
                serviceAccess.setPluginName( pluginName ); // useful ??
                
                MessageContent content = new MessageContentImpl();
                content.setSvcAccess( serviceAccess );
                response.setContent( content );
                
                srcNetworkID = message.getContent().getSvcAccess().getSrcNetID();
                
                byte[] msg = jaxme.toString( response ).getBytes();
                networkManager.sendTo(  msg, socketIn );
            }
            
        } catch( JAXBException e) { 
            board.notifyException( e );
        }     
    }
}
