/*
 * Created on May 11, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.manager.task;

import java.math.BigInteger;
import java.util.List;


import org.msda.components.interfaces.CommunicatorInterface;
import org.msda.components.manager.MSDManager;
import org.msda.components.manager.MSDManagerPool;
import org.msda.components.manager.object.Object4Queue;
import org.msda.components.manager.object.ObjectRemoteRequest;
import org.msda.components.utils.LocalComponentsMgr;
import org.msda.framework.FrameworkOutput;
import org.msda.msd.*;
import org.msda.msd.impl.*;
import org.msda.util.*;


/**
 * Task used to forward results of remote client requests.<br>
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 */
public class TaskRemoteRequest4Result extends TaskAbstract {
    private FrameworkOutput board;
    private CommunicatorInterface communicator;
    private LocalComponentsMgr localMgr;
    private ObjectRemoteRequest object;
    
    /**
     * TaskRemoteRequest4Result.
     * 
     * @param msdManagerPool
     */
    public TaskRemoteRequest4Result( MSDManagerPool msdManagerPool ) {
		this.board = org.msda.framework.FrameworkMain.msdaOutput;
        this.communicator = msdManagerPool.getCommunicator();
        this.localMgr = msdManagerPool.getLocalMgr();
        this.object = null;
    }
    
    /**
     * @see org.msda.components.manager.task.TaskAbstract#setObject(org.msda.components.manager.Object4Queue)
     */
    public void setObject(Object4Queue object) {
        this.object = (ObjectRemoteRequest)object;
    }
    
    /**
     * @see org.msda.components.manager.task.TaskAbstract#terminate()
     */
    public void terminate() {
        
        this.board = null;
        this.communicator = null;
        this.localMgr = null;
        this.object = null;
    }
    
    /**
     * @see org.msda.components.manager.task.TaskAbstract#run()
     */
    public void run() {
        
        int requestId = object.getRequestId();
        MSDDescription description = object.getDescription();
        MSDRequest request = object.getRequest();
        
        // update the propagation list of the MSDDescription
        addRequestPropagation( description, request );
        
        Param param = new ParamImpl();
        param.setType( MSDManager.FIELD_CLIENT_ID );
        param.setValue( Integer.toString( request.getClientID().intValue() ) );
        MetaInformation meta = new MetaInformationImpl();
        meta.getParams().add( param );
        description.setMetaInfo( meta );
        
        // MSDManager.FIELD_REQUEST_ID is put in the ResponseCode field
//        param = new ParamImpl();
//        param.setName( MSDManager.FIELD_REQUEST_ID );
//        param.setType( Integer.toString( request.getRequestID().intValue() ) );
//        description.getMetaInformation().getParams().add( param );
        
        // create the MSDMessage
        MessageContent content = new MessageContentImpl();
        content.setDesc( description );
        
        MSDMessage mess = new MSDMessageImpl();
        mess.setType( BigInteger.valueOf( InfoType.Message.REQUEST_RESPONSE ) );
        
        mess.setRespCode( BigInteger.valueOf( requestId ) );
        mess.setContent( content );
        
        // copy the propagation information
        mess.setBackwardPath( description.getPropaInfo() );
        
        try {
            if ( mess.getBackwardPath().getHops().isEmpty() ) {
                board.debug(" REMOTE REQUEST - list hops empty ! ");
                
            } else {
                
                Hop hopDest = (Hop)mess.getBackwardPath().getHops().get(0);
                board.debug(" REMOTE REQUEST - hop(0) ="+hopDest.getSrcNetID().getName()+" to "+hopDest.getDestNetID().getName());
                
                communicator.sendResponse(mess);
            }
            
        } catch (MSDException e1) {
            board.notifyError( "TaskRemoteRequest::run : Error while sending a RequestResponse "+e1.getMessage() );
        }
    }
    
    
    /**
	 * Adds the reverse path of the request to the description as propagation information.
	 * 
	 * @param description	The description to update
	 * @param request		The request containing the path
	 */
	private void addRequestPropagation( MSDDescription description, MSDRequest request ) {

	    List hops = request.getPropaInfo().getHops();
	    PropagationInformation propagation = new PropagationInformationImpl();
	    
	    // Create the reverse path
	    for(int i=0;i<hops.size();i++) {
	        Hop hop = (Hop)hops.get(i);
	        Hop newHop = new HopImpl();
	        newHop.setBridgeID( hop.getBridgeID() );// false info if the bridge disappears...
	        newHop.setSrcNetID( hop.getDestNetID() );
	        newHop.setDestNetID( hop.getSrcNetID() );
	        newHop.setSecurity( hop.getSecurity() );
	        newHop.setTime( hop.getTime() ); // wrong info... 
	        
	        // Warning
	        // some fields ( BridgeID and Time ) will be modified in the MSDBridge
	        
	        propagation.getHops().add( newHop );
	    }
	    description.setPropaInfo( propagation );
	}
}
