/*
 * Created on May 2, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.manager.task;


import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;

import javax.xml.bind.JAXBException;

import org.msda.components.manager.MSDManagerPool;
import org.msda.components.manager.object.Object4Queue;
import org.msda.components.manager.object.ObjectPlugin;
import org.msda.components.manager.object.ObjectServiceAccess;
import org.msda.framework.FrameworkOutput;
import org.msda.msd.Hop;
import org.msda.msd.InformationComponent;
import org.msda.msd.MSDMessage;
import org.msda.msd.NetworkConfig;
import org.msda.msd.PropagationInformation;
import org.msda.msd.impl.PropagationInformationImpl;
import org.msda.util.InfoType;
import org.msda.util.JaxMeUtil;
import org.msda.util.PropagationUtil;
import org.msda.util.comm.NetworkManager;

/**
 * Task that can manage a service access asked by a local client. <br>
 *    - remote service access <br>
 *    - local service access <br>
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 */
public class TaskServiceAccess extends TaskAbstract {
    
    private MSDManagerPool managerPool;
    private FrameworkOutput board;
    private JaxMeUtil jaxme;
    private NetworkManager networkManager;
    
    private MSDMessage message;
    private int clientId;
    private ObjectPlugin objectPlugin;
    private ObjectServiceAccess object;
    
    /**
     * TaskServiceAccess.
     * 
     * @param managerPool
     */
    public TaskServiceAccess( MSDManagerPool managerPool ) {
        
        this.managerPool = managerPool;
		this.board = org.msda.framework.FrameworkMain.msdaOutput;
        this.jaxme = new JaxMeUtil();
        this.networkManager = new NetworkManager();
        
        this.message = null;
        this.clientId = -1;
        this.objectPlugin = null;
        this.object = null;
    }
    
    /**
     * @see org.msda.components.manager.task.TaskAbstract#setObject(org.msda.components.manager.Object4Queue)
     */
    public void setObject(Object4Queue object) {
        this.object = (ObjectServiceAccess)object;
    }
    
    /**
     * @see org.msda.components.manager.task.TaskAbstract#terminate()
     */
    public void terminate() {
        
        if (objectPlugin!=null)
        	synchronized (objectPlugin) {
        		objectPlugin.notify();
			}
    
        synchronized( object ) {
            object.notify();
        }
        
        this.managerPool = null;
        this.board = null;
        this.jaxme = null;
        this.networkManager = null;
        
        this.message = null;
        this.clientId = -1;
        this.objectPlugin = null;
        this.object = null;
    }
    
    /**
     * @see org.msda.components.manager.task.TaskAbstract#run()
     */
    public void run() {
        
        message = object.getMessage();
        clientId = message.getRespCode().intValue();
        
        String pluginName = message.getContent().getSvcAccess().getPluginName();
    	if ( InfoType.Message.REMOTE_ACCESS == message.getType().intValue() &&  object.getHops()!=null) {
        
    		board.debug( "TaskServiceAccess:: Case First manager to Bridge " );
    		
//    		 get the src and dest network ids
            
            Hop hop = PropagationUtil.getInstance().getFirstHop( object.getHops() ); 
            if ( hop==null ) 
                board.debug( "Hop not found " );
                
            InformationComponent info = managerPool.getLocalMgr().getBridge( hop.getDestNetID().getName(), hop.getSrcNetID().getName() );
    		if ( info == null )
    			board.debug( "Bridge not found with "+hop.getDestNetID().getName() + " "+hop.getSrcNetID().getName() );
    		
    		NetworkConfig netConfig = managerPool.getLocalMgr().getNetworkConfig( info, hop.getDestNetID().getName() );
    		int port = netConfig.getBridgeRemoteAccessPort().intValue();
    		board.debug(" info IPAddress ="+info.getIPAddr()+"  port "+port );
    		
    		
    		try {
    		    // TODO - remove backward path, put a # of hop instead...
    		    PropagationInformation pi = new PropagationInformationImpl();
    		    pi.getHops().addAll( object.getHops() );
    		    message.setBackwardPath( pi );
    		    byte[] msg = jaxme.toString( message ).getBytes();
    		
    			Socket socket = networkManager.sendTo(  msg, InetAddress.getByName( info.getIPAddr()), port );
				
				byte[] res =  networkManager.getReceivedData( socket );
				MSDMessage response = jaxme.toMSDMessage( res );
				
				object.setResponse( response.getContent().getSvcAccess().getSOAPMess() );
			
    		} catch( IOException e) {
    		    e.printStackTrace();
    		} catch( JAXBException e ) {
    			e.printStackTrace();
			} 

        } else {
            //
            // Case LOCAL SERVICE ACCESS
            //
            board.info( "TaskServiceAccess:: LOCAL SERVICE ACCESS from this manager to the plugin" );
            
            objectPlugin = new ObjectPlugin();
            objectPlugin.setEpService( message.getContent().getSvcAccess().getEPSvc() );
            objectPlugin.setSoapMessage( message.getContent().getSvcAccess().getSOAPMess() );
            
            // search the index of the thread that manages the plugin
            Integer index = (Integer)managerPool.getPluginIndex().get( pluginName );
            if ( index==null ) {

                // There is an error
                object.setResponse( InfoType.SOAP_PLUGIN_ERROR );
                
            } else {
                // Wake up the threadPlugin
                objectPlugin.setIndex( index.intValue() );
                
                int i = index.intValue();
                managerPool.getTaskPlugins()[i].setObject( objectPlugin );
                managerPool.getTaskPlugins()[i].run();
                
                String response = managerPool.getTaskPlugins()[ index.intValue() ].getResponse();
                object.setResponse( response );
            }
        }
      
        synchronized( object ) {
            object.notify();
        }
    }
}
