/*
 * Created on Jul 4, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.manager.task;

import org.msda.components.manager.MSDManagerPool;
import org.msda.components.manager.object.Object4Queue;
import org.msda.components.utils.ComponentThread;

/**
 * Thread to manage and create new tasks.
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 */
public class ThreadTask extends ComponentThread {
	
	private MSDManagerPool msdManagerPool;
	private TaskAbstract task;
	
	/**
	 * Constructor.
	 * @param msdManagerPool
	 */
	public ThreadTask( MSDManagerPool msdManagerPool ) {
		
		this.msdManagerPool = msdManagerPool;
		
		this.start();
	}
	
    /**
     * @see org.msda.components.utils.ComponentThread#terminate()
     */
    public void terminate() {
        
		super.close();
		
		if ( super.isBusy )
			task.terminate();
	}
	
    /**
     * @see java.lang.Runnable#run()
     */
    public void run() {
		
		while( active ) {
			try {
				Object4Queue object = (Object4Queue)msdManagerPool.getBlockingQueue().get();
				
				super.isBusy = true;
				switch( object.getType() ) {
				
				case Object4Queue.LOCAL_REQUEST :
				    task = new TaskLocalRequest( msdManagerPool );
					task.setObject( object );
					task.run();
					break;
				
				case Object4Queue.REMOTE_REQUEST :
					task = new TaskRemoteRequest( msdManagerPool );
					task.setObject( object );
					task.run();
					break;
				
					// directly called in the TaskPlugin, but needed because of TaskRemoteRequest
				case Object4Queue.REMOTE_REQUEST4RESULT :
					task = new TaskRemoteRequest4Result( msdManagerPool );
					task.setObject( object );
					task.run();
					break;
				
				case Object4Queue.SERVICE_ACCESS :
					task = new TaskServiceAccess( msdManagerPool );
					task.setObject( object );
					task.run();
					break;
				
				case Object4Queue.REMOTE_ACCESS :
					task = new TaskRemoteAccess( msdManagerPool );
					task.setObject( object );
					task.run();
					break;
				
				case Object4Queue.PLUGIN:
					int index = object.getIndex();
					msdManagerPool.getTaskPlugins()[index].setObject( object );
					msdManagerPool.getTaskPlugins()[index].run();
					break;
				
				default:
					//error
					super.isBusy = false;
				}
				// reinit the thread
				super.isBusy = false;
				
			} catch (InterruptedException e) {
			}
		}
	}
}
