/*
 * Created on March 31, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.plugins.http;


import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Calendar;

import http.*;
import http.msg.*;

/**
 * @author rafik CHIBOUT
 *
 **/
public class HTTPServer {
	// default HTTP port for the server
	public int HTTP_PORT=8088;
	
	private HTTPListener listener = null;
	private HttpParser parser = new HttpParser();
	private boolean started = false;
	private ServerSocket socServer;
	
	/**
	 * @return Returns the hTTP_PORT.
	 */
	
	public int getHTTPPORT() {
		return HTTP_PORT;
	}
	
	/**
	 * @param http_port The hTTP_PORT to set.
	 */
	
	public void setHTTPPORT(int http_port) {
		HTTP_PORT = http_port;
	}
	
	/**
	 * @return Returns the listener.
	 */
	
	public HTTPListener getListener()
	{
		return listener;
	}
	
	public void setListener(HTTPListener httpListener)
	{
		listener = httpListener;
	}
	
	public boolean start()
	{
		try
		{
			socServer = new ServerSocket(this.getHTTPPORT());
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
			return false;
		}
		try
		{
			socServer.setSoTimeout(4000);
			socServer.setReuseAddress(true);
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		
		Thread deamon = listenProcess();
		started = true;
		deamon.start();
		return true;
	}
	
	
	public void stop()
	{
		started = false;
	}
	
	
	/**
	 * @return Returns the started.
	 */
	
	public boolean isStarted() {
		return started;
	}
	
	
	public httpHolder getReceivedMsg()
	{
		httpHolder forReturn = new httpHolder();
		
		Socket soc = null;
		try
		{
			soc = socServer.accept();
		}
		catch(Exception ex)
		{	
			return null;
		}
		
		try
		{
			InputStream socInput = soc.getInputStream();
			OutputStream socOutput = soc.getOutputStream();
			HttpMsg httpMessage = parser.parse(socInput);
			if (httpMessage == null) return null;
			forReturn.setHTTPMessage(httpMessage);
			forReturn.setOutputStream(socOutput);
			//board.info("A HTTP Message is received");
			return forReturn;	
		}
		
		catch(Exception ex)
		{
			//board.info("Error at HttpServer.getReceivedMsg()");
			ex.printStackTrace();
		}
		return null;
	}
	
	
	public Thread listenProcess() {
		try
		{
			Thread listenProcessThread = 
				new Thread( new Runnable() {
					public void run() 
					{	
						while (isStarted())
						{
							try
							{
								//board.info("HTTPServer is waiting for a new message");
								httpHolder message = getReceivedMsg();
								if (message!=null)
								{	
									// start a thread ...
									//board.info("HTTPServer receive a new message");
									MessageProcess process = new MessageProcess(message, listener);
									//board.info("HTTPServer give the new message to the MessageProcess");
									//listener.newHttpMessage(message);
								}
							}
							catch (Exception ex)
							{
								//board.info("Error at HttpServer.listenProcess.run()");
								ex.printStackTrace();
							}
						}
					}
				});
			return listenProcessThread;
		}
		catch (Exception ex)
		{
			//board.info("Error at create HttpServer.listenProcess Thread");
		}
		return null;
	}
	
	public static boolean sendHttpResponse(OutputStream out, byte[] content) throws IOException
	{
		HttpResponse response = new HttpResponse();
		response.setStatusCode(HttpStatus.OK);
		Calendar currentDate = Calendar.getInstance();
		currentDate.setTimeInMillis(System.currentTimeMillis());
		response.setDate(currentDate);
		response.setContentType("application/x-www-form-urlencoded");
		response.setVersion("1.0");
		
		
		response.setContent(content);
		//if (out == null) board.info("The out stream is null");
		//if (response.toString() == null ) board.info("the reponse to string is null");
		//board.info("http response = "+response.toString());
		out.write(response.toString().getBytes());
		//board.info("HTTPServer sendHttpResponse: The reponse is sent");
		out.flush();
		out.close();
		return true;	
	}
}
