/*
 * Created on Apr 22, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.plugins.msdaservice;

import java.io.*;
import com.siemens.upnp.*;
import com.siemens.upnp.devicehost.*;


/**
 * Direct MSD service for UPnP.
 * This class registers the UPnP service for direct MSDA access (plain XML messages over socket).
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 */
public class MSDServiceUPnP_Direct extends HostedDevice {
	//
	private HostedService service;
	
	/**
	 * 
	 * @param description
	 * @throws FileNotFoundException
	 * @throws InvalidDescriptionException
	 */
	public MSDServiceUPnP_Direct(File description) throws FileNotFoundException, InvalidDescriptionException 
	{
		super(description);
		initDevice();
	}
	
	/**
	 * 
	 * @param ctrUrl
	 */
	public MSDServiceUPnP_Direct(String ctrUrl)
	{
		createDevice(ctrUrl);
		initDevice();
	}
	
	/**
	 * 
	 * @param ctrUrl
	 */
	public void createDevice(String ctrUrl)
	{
		// required attributes
		this.setDeviceType("urn:schemas-upnp-org:device:direct_msd_device:1");
		this.setFriendlyName("MSD");
		this.setManufacturer("INRIA Arles Project");
		this.setModelName("SDP");
		this.setUDN("uuid:directmsd1");
		this.setPresentationUrl(this.getUrlBase()+"/presentation.html");
		// recommend attributes
		this.setModelDescription("DirectMSDServiceUPnP");
		this.setModelNumber("0.10");
		this.setSerialNumber("02021978");
		// optionals attributes
		this.setManufacturerURL("http://www-rocq.inria.fr/arles");
		this.setModelURL("http://www-rocq.inria.fr/arles");
		this.setUPC("Samples");
		
		//icons
		Icon icon = new Icon("icon.gif");
		icon.setMimeType("image/gif");
		icon.setHeight(30);
		icon.setWidth(30);
		icon.setColorDepth(8);
		this.addIcon(icon);
		
		//embadded Devices (create embadded devices here)
		
		// services
		service = this.createService();
		service.setServiceType("urn:schemas-upnp-org:service:directmsdservice:1:0");
		service.setServiceID("urn:upnp-org:directmsdservice:1");
		service.setControlURL(ctrUrl);
		
		HostedStateVariable stateVariableName = service.createStateVariable("direct", "string","no", "no", true);
		service.addStateVariable(stateVariableName);
		
		this.addService(service);
	}
	
	/**
	 * 
	 *
	 */
	public void initDevice()
	{
		this.setRootDevice(true);
	}
	
}
