/*
 * Created on Apr 22, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.plugins.msdaservice;

import java.io.*;
import com.siemens.upnp.*;
import com.siemens.upnp.devicehost.*;


/**
 * (SOAP) MSD service for UPnP.
 * This class registers the UPnP service for SOAP-based MSDA access (plain XML messages over socket).
 
 * @author rafik.chibout@inria.fr - INRIA ARLES
 */
public class MSDServiceUPnP_SOAP extends HostedDevice {
	//
	private HostedService service;
	
	/**
	 * 
	 * @param description
	 * @throws FileNotFoundException
	 * @throws InvalidDescriptionException
	 */
	public MSDServiceUPnP_SOAP(File description) throws FileNotFoundException, InvalidDescriptionException 
	{
		super(description);
		initDevice();
	}
	
	/**
	 * 
	 * @param ctrUrl
	 */
	public MSDServiceUPnP_SOAP(String ctrUrl)
	{
		createDevice(ctrUrl);
		initDevice();
	}
	
	/**
	 * 
	 * @param ctrUrl
	 */
	public void createDevice(String ctrUrl)
	{
		// required attributes
		this.setDeviceType("urn:schemas-upnp-org:device:msd_device:1");
		this.setFriendlyName("MSDA");
		this.setManufacturer("INRIA Arles Project");
		this.setModelName("SDP");
		this.setUDN("uuid:ubisecmsd1");
		this.setPresentationUrl(this.getUrlBase()+"/presentation.html");
		// recommend attributes
		this.setModelDescription("MSDServiceUPnP");
		this.setModelNumber("0.90");
		this.setSerialNumber("1234567890");
		// optionals attributes
		this.setManufacturerURL("http://www-rocq.inria.fr/arles");
		this.setModelURL("http://www-rocq.inria.fr/arles");
		this.setUPC("Samples");
		
		//icons
		Icon icon = new Icon("icon.gif");
		icon.setMimeType("image/gif");
		icon.setHeight(30);
		icon.setWidth(30);
		icon.setColorDepth(8);
		this.addIcon(icon);
		
		//embadded Devices (create embadded devices here)
		
		// services
		service = this.createService();
		service.setServiceType("urn:schemas-upnp-org:service:msdservice:1:0");
		service.setServiceID("urn:upnp-org:msdservice:1");
		service.setControlURL(ctrUrl);
		
		HostedStateVariable stateVariableName = service.createStateVariable("msd", "string","nomsd", "nomsd", true);
		service.addStateVariable(stateVariableName);
		
		
		// action setName
		//		Argument argumentIn  = new Argument("msd", Argument.IN, false, stateVarabileName.getName());
		//		Argument argumentOut = new Argument("msd", Argument.OUT, true, stateVarabileName.getName());
		//		Action getInfoAction = service.createAction("getMSD");
		//		getInfoAction.addArgument(argumentIn);
		//		getInfoAction.addArgument(argumentOut);
		//		service.addAction(getInfoAction);
		// action getName
		//		Argument argumentIn2  = new Argument("msd", Argument.IN, false, stateVarabileName.getName());
		//		Argument argumentOut2 = new Argument("imageURL", Argument.OUT, true, stateVarabileName.getName());
		//		Action getImageAction = service.createAction("getImage");
		//		getImageAction.addArgument(argumentIn2);
		//		getImageAction.addArgument(argumentOut2);
		//		service.addAction(getImageAction);
		this.addService(service);
		//		board.info("****************Device Decription**********************");
		//		board.info(this.getDeviceDescription());
		//		board.info("*************************************************");
		
		//		board.info("****************Service Decription**********************");
		//		Enumeration services = this.getServices().elements();
		//		if (services.hasMoreElements()) 
		//		{
		//			HostedService serviceTmp = (HostedService)services.nextElement();
		//			board.info(serviceTmp.getDescription());	
		//		}
		//		board.info("*************************************************");
		
		
	}
	
	/**
	 * 
	 *
	 */
	public void initDevice()
	{
		this.setRootDevice(true);
		//UPnPPresentationPageHandler pageHandler = new SampleDevice1PresentationPageHandler();
		//this.setPresentationPageHandler(pageHandler);
	}
	
}
