/*
 * Created on May 2, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.components.utils;

/**
 * This abstract class represents a thread
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public abstract class ComponentThread extends Thread {
	
	/**
	 * Uses to verify if the thread is busy or free...
	 */
	protected boolean isBusy;
	
	/**
	 * Uses to kill the thread if it is set to false
	 */
	protected boolean active;
	
	/**
	 * Default constructor. <br>
	 * Does not start automatically this thread.
	 */
	public ComponentThread() {
		
		active = true;
		isBusy = false;
	}
	
	/**
	 * Returns true if the thread is busy, false otherwise
	 * 
	 * @return Returns true if the thread is busy, false otherwise
	 */
	public boolean isBusy() {
		return isBusy;
	}
	
	/**
	 * Returns true if the thread is active, false otherwise
	 * 
	 * @return Returns true if the thread is active, false otherwise
	 */
	public boolean isActive() {
		return active;
	}
	
	/**
	 * Stops and ends the thread.
	 */
	public void close() {
		active = false;
	}
	
	abstract public void terminate();
	abstract public void run();	
}
