/*
 * Created on Aug 4, 2005
 * 
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr<br>
 *          Rafik.Chibout@inria.fr
 * 
 * Copyright INRIA 2005
 */
package org.msda.context;


import java.util.Calendar;

import org.msda.msd.CombinationNode;
import org.msda.msd.ComparisonNode;
import org.msda.msd.ContextElement;
import org.msda.msd.ContextInformationTable;
import org.msda.msd.ContextNode;
import org.msda.msd.ContextRules;
import org.msda.msd.ContextRulesTree;
import org.msda.msd.MSDRequest;
import org.msda.msd.TypeValue;

/**
 * Performs the context matching utilized by the whole system.<br>
 * It includes two types of matching:<br>
 * 1. Matching of user request and service description <br>
 * 2. Matching of user request and network context <br>
 * 
 * @author Oriana Riva, HIIT - August 2005
 * 
 */

public class ContextMatcher {
    
    private ContextInformationTable sCtx = null;
    private MSDRequest msdReq = null;
    private ContextRulesTree rulesCtx_dest = null;
    private ContextInformationTable netCtx_dest = null;
    private ContextInformationTable svcCtx = null;
    
    /**
     * Constructor
     * 
     * @param msdReq	MSD request
     * @param sCtx		service context information
     */
    public ContextMatcher(MSDRequest msdReq, ContextInformationTable sCtx){
        this.sCtx = sCtx;
        this.msdReq = msdReq;
    }
    
    /**
     * Constructor
     * 
     * @param msdReq
     * @param netCtx_src
     * @param netCtx_dest
     */
    public ContextMatcher(MSDRequest msdReq, ContextInformationTable netCtx_dest, ContextRulesTree rulesCtx_dest){
        this( msdReq, netCtx_dest, null, rulesCtx_dest );
    }
    
    public ContextMatcher(MSDRequest msdReq, ContextInformationTable netCtx_dest, ContextInformationTable svcCtx, ContextRulesTree rulesCtx_dest){
        this.netCtx_dest = netCtx_dest;
        this.svcCtx = svcCtx;
        this.rulesCtx_dest = rulesCtx_dest;
        this.msdReq = msdReq;
    }
    
    /**
     * to match the user request (i.e., contextRules)
     * with the service description
     * 
     * @return boolean
     */
    public boolean matchUserService(){
        
        if ( msdReq==null || sCtx==null || msdReq.getCtxRules()==null ||  msdReq.getCtxRules().getCRSvc()==null ) {
            print("rules or table CI null");
            return true;
        }
        
        ContextRules rules = msdReq.getCtxRules();
        ContextNode root = rules.getCRSvc().getRoot();
        
        if(root.getCi()!=null){  
            CombinationNode combNode = root.getCi();
            String operation = combNode.getOp();
            ContextNode ctxNode1 = combNode.getCN1();
            ContextNode ctxNode2 = combNode.getCN2();
            return processCombination(ctxNode1, ctxNode2, operation, sCtx, null);
        }        
        
        ComparisonNode compaNode = root.getCa();
        boolean result = processComparison(compaNode, sCtx, null);  
        print("result is " + result);
        return result;
    }
    
  
    /**
     * to match a user request with the network context
     * 
     * @return boolean
     */
    public boolean matchRequestNetwork(){
        
        if ( rulesCtx_dest==null || (netCtx_dest==null && this.svcCtx==null) || msdReq==null ) {
            print("rules or table CI null");
            return true;
        }
        
        ContextNode root = rulesCtx_dest.getRoot();
        
        if(root.getCi()!=null) {  
            CombinationNode combNode = root.getCi();
            String operation = combNode.getOp();
            ContextNode ctxNode1 = combNode.getCN1();
            ContextNode ctxNode2 = combNode.getCN2();
            return processCombination(ctxNode1, ctxNode2, operation, this.netCtx_dest, this.svcCtx );            
        }        
        
        ComparisonNode compaNode = root.getCa();
        boolean result = processComparison(compaNode, this.netCtx_dest, this.svcCtx);  
        print("result is " + result);
        return result;
        
    }
    
    
     /**
      * to process a comparison NAME OP VALUE
      * 
      * @param compariNode
      * @return boolean
      */  
    public boolean processComparison(ComparisonNode compariNode, ContextInformationTable netCtx, ContextInformationTable svcCtx){
                
        String name = compariNode.getName();            
        TypeValue value = getContextByName(name, netCtx);   
        
        if ( value==null )
            value = getContextByName(name, svcCtx);   
        
        if ( value==null )
            return false;
        
        // 0 is true
        // 1 is false  
        String strValue = "";
        int intValue = 0;
        boolean boolValue = false;
        Calendar timeValue = null;
        String operation = compariNode.getOp();        
        int type = -1;
        
        print("processing the comparison: " + name );
        //print("processing the comparison: " + operation);
        //print("processing the comparison: " + value );
        
        if(compariNode.getVal().getStr()!= null){
            strValue = compariNode.getVal().getStr();
            //print("the value is the string " + strValue);
            type = 0;            
        }
        else if (compariNode.getVal().getInt()!= null) {
            intValue = compariNode.getVal().getInt().intValue();
            //print("the value is an int "+intValue);
            type = 1;
        }
        else if (compariNode.getVal().getBool()!=null) {
            if ( compariNode.getVal().getBool().intValue()!=0 )
                boolValue = true;
            //print("the value is the boolean " + boolValue);
            type = 2;
        }
        else if (compariNode.getVal().getTime()!=null) {
            timeValue = compariNode.getVal().getTime();
           // print("the value is the time " + timeValue);
            type = 3;
        }
        else{
            print("error");
        }
         
        if(value!=null){            
            switch (type) {
            case 0:
                //print("process Operation case 0 " + strValue + " ," + name + ", " + value.getStr());  
                boolean result2 = processOperation(operation, value.getStr(), strValue);     
                print("result is " + result2);
                return result2;
            case 1:
                print("process Operation case 1 " + intValue  + " ," + name + ", " + value.getInt().intValue());  
                return processOperation(operation, value.getInt().intValue(), intValue );
                
            case 2:
                //print("process Operation case 2 " + boolValue  + " ," + name + ", " + value.getBool().intValue());
                boolean boolTmp = false;
                if ( value.getBool().intValue()!=0 )
                    boolTmp = true;
                return processOperation(operation, boolTmp, boolValue ); 
                
            case 3:
                // Time...
                //print("process Operation case 3 " + timeValue  + " ," + name + ", " + value.getTime());
                return processOperation(operation, value.getTime(), timeValue ); 
            default:
                print("ERROR: no opeator is set");
            	return false;
            }            
        }
        return false;
        
    }
    
    /**
     * to process an operation between int
     * four types of operation are supported: <, >, !=, =
     * 
     * @param operation
     * @param name
     * @param value
     * @return boolean
     */
    public boolean processOperation(String operation, int value1, int value2){
        
        if (operation.equals("G")) {
            print("it's >"+value1+" < "+value2);
            if(value1>value2)
                return true;
            return false;           
        }        
        else if(operation.equals("L")){            
            print("it's  "+value1+" < "+value2);
            if(value1<value2)
                return true;
            return false;  
        }        
        else if(operation.equals("=")){
            print("it's =");
            if(value1==value2)
                return true;
            return false;  
        }           
        else if(operation.equals("!=")){
            print("it's !=");
            if(value1!=value2)
                return true;
            return false;                         
        }
        else{
            print("invalid operator");
            return false;
        }
    }
    
    public boolean processOperation(String operation, Calendar value1, Calendar value2){
        
        if (operation.equals("G")) {
            print("it's >");
            if(value1.getTime().after( value2.getTime() ))
                return true;
            return false;           
        }        
        else if(operation.equals("L")){            
            print("it's < "+value1+" < "+value2);
            if(value1.getTime().before( value2.getTime() ))
                return true;
            return false;  
        }        
        else if(operation.equals("=")){
            print("it's =");
            if(value1.getTime().equals(value2.getTime()))
                return true;
            return false;  
        }           
        else if(operation.equals("!=")){
            print("it's !=");
            if(!value1.getTime().equals(value2.getTime()))
                return true;
            return false;                         
        }
        else{
            print("invalid operator");
            return false;
        }
    }
    
    /**
     * to process an operation between strings (=, !=)
     * 
     * @param operation
     * @param name
     * @param value
     * @return boolean
     */
    public boolean processOperation(String operation, String value1, String value2){
        
        if (operation.equals("=")){           
            if(value1.equals(value2)){
                print("it's equal");
                return true;
            }
            return false;           
        }
        else if(operation.equals("!=")){            
            if(value1.equals(value2)){
                print("it's equal");
                return false;
            }
            return true;
        }
        else{
            print("invalid operator");
            return false;
        }
    }
    
    /**
     * to process an operation between boolean (=, !=)
     * 
     * @param operation
     * @param name
     * @param value
     * @return boolean
     */
    public boolean processOperation(String operation, boolean value1, boolean value2){
        
        if (operation.equals("=")){           
            if(value1 && value2 ){
                print("it's equal");
                return true;
            }
            return false;           
        }
        else if(operation.equals("!=")){            
            if(value1 && value2){
                print("it's equal");
                return false;
            }
            return true;
        }
        else{
            print("invalid operator");
            return false;
        }
    }
    
    
    /**
     * to process a combination of two context nodes
     * two types of combination are supported: OR and AND
     * 
     * @param ctxNode1
     * @param ctxNode2
     * @param operation
     * @return
     */
    public boolean processCombination(ContextNode ctxNode1, ContextNode ctxNode2, String operation, ContextInformationTable netCtx, ContextInformationTable svcCtx){
        
        if(!operation.equals("AND")&&(!operation.equals("OR"))){
            print("invalid operator for combination");
            return false;
        }
        
        if((ctxNode1.getCi()!=null)&&(ctxNode2.getCi()!=null)){
            print("both nodes are combinations");
            CombinationNode combNode1 = ctxNode1.getCi();
            String operation1 = combNode1.getOp();
            ContextNode ctxN1 = combNode1.getCN1();
            ContextNode ctxN2 = combNode1.getCN2();
            
            CombinationNode combNode2 = ctxNode2.getCi();
            String operation2 = combNode2.getOp();
            ContextNode ctxN11 = combNode2.getCN1();
            ContextNode ctxN22 = combNode2.getCN2();
            
            boolean r1 = processCombination(ctxN1, ctxN2, operation1, netCtx, svcCtx);
            boolean r2 = processCombination(ctxN11, ctxN22, operation2, netCtx, svcCtx);
            if(operation.equals("AND"))
                return (r1 && r2);
            return (r1 || r2);
        }
        else if((ctxNode1.getCi()==null)&&(ctxNode2.getCi()!=null)){
            
            print("1 is comparison and 2 is combination");
            CombinationNode combNode2 = ctxNode2.getCi();
            String operation2 = combNode2.getOp();
            ContextNode ctxN1 = combNode2.getCN1();
            ContextNode ctxN2 = combNode2.getCN2();
            
            ComparisonNode compaNode1 = ctxNode1.getCa();
            print("it's comparison");
            boolean result = processComparison(compaNode1,netCtx, svcCtx);  
            print("result is " + result);
            
            boolean r = processCombination(ctxN1, ctxN2, operation2, netCtx, svcCtx);
            if(operation.equals("AND"))
                return (r && result);
            return (r || result);
        }
        
        else if((ctxNode1.getCi()!=null)&&(ctxNode2.getCi()==null)){
            
            print("2 is comparison and 1 is combination");
            CombinationNode combNode1 = ctxNode1.getCi();
            String operation1 = combNode1.getOp();
            ContextNode ctxN1 = combNode1.getCN1();
            ContextNode ctxN2 = combNode1.getCN2();
            
            ComparisonNode compaNode1 = ctxNode2.getCa();
            print("it's comparison");
            boolean result = processComparison(compaNode1, netCtx, svcCtx);  
            print("result is " + result);
            
            boolean r = processCombination(ctxN1, ctxN2, operation1, netCtx, svcCtx);
            if(operation.equals("AND"))
                return (r && result);
            return (r || result);
        }
        
        else{
            print("1 is comparison and 2 is comparison");
            ComparisonNode compaNode1 = ctxNode1.getCa();
            boolean result1 = processComparison(compaNode1, netCtx, svcCtx);  
            print("result1 is " + result1);
            
            ComparisonNode compaNode2 = ctxNode2.getCa();
            boolean result2 = processComparison(compaNode2, netCtx, svcCtx);  
            print("result2 is " + result2);
            
            if(operation.equals("AND"))
                return (result1 && result2);
            return (result1 || result2);            
        }
    }
    
    
    
   /**
    * to get a context parameter by specifying name
    * 
    * @param name 
    * @return String context value
    */
    public TypeValue getContextByName(String name, ContextInformationTable ctx){
        
        if ( ctx==null )
            return null;
        
        ContextElement elt = ContextUtil.getInstance().getContextElement( name, ctx.getCITable() );
        if ( elt==null )
            return null;
        return elt.getVal();
    }
    
    /**
     * printing function
     * 
     * @param o
     */
    public void print(String o){
        //board.info("[ContextMatcher ]: " + o);
    }

} 

	