/*
 * Created on Oct 4, 2005
 * 
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr
 * Preliminary version, please do not distribute
 * 
 * Copyright INRIA 2005
 */
package org.msda.context;

import java.math.BigInteger;
import java.util.Calendar;
import java.util.List;

import org.msda.msd.ContextElement;
import org.msda.msd.TypeValue;
import org.msda.msd.impl.ContextElementImpl;
import org.msda.msd.impl.TypeValueImpl;

/**
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class ContextUtil {

    private static ContextUtil instance = new ContextUtil();
    
    //	private constructor to prevent from creating instances of this class     
    private ContextUtil() {}
    
    /**
     * Returns a static instance of ParamsUtil
     * 
     * @return Returns a static instance of ParamsUtil
     */
    public static ContextUtil getInstance() {
        return ContextUtil.instance;
    }
    
//    public Object getValue( TypeValue value ) {
//        
//    }
    
    /**
     * Gest the TypeValue corresponding to a context field name in a list of ContextElement
     * 
     * @param name	the context field name
     * @param list	the list of ContextElement
     * @return
     */
    public ContextElement getContextElement( String name, List list ) {
        
        for ( int i=0;i<list.size(); i++) {
            ContextElement elt = (ContextElement)list.get(i);
            if ( elt.getName().equals( name ) )
                return elt;
        }
        
        return null;
    }
    
    /**
     * Returns the index list of the context element corresponding to the context name
     * 
     * @param name	The context name
     * @param list	The list of ContextElement
     * @return	The index of the context element in the list, -1 if not found
     */
    public int getIndex( String name, List list ) {
       
        for ( int i=0;i<list.size(); i++) {
            ContextElement elt = (ContextElement)list.get(i);
            if ( elt.getName().equals( name ) )
                return i;
        }
        return -1;
    }
    
    /**
     * Returns a ContextElement fills with name and value
     * 
     * @param name	the context element name
     * @param value the context element value
     * 
     * @return The new context element
     */
    public ContextElement createCtxElement( String name, String value ) {
        
        ContextElement elt = new  ContextElementImpl();
        elt.setName( name );
        TypeValue val = new TypeValueImpl();
        val.setStr( value );
        elt.setVal( val );
        //elt.setType( BigInteger.valueOf( InfoType.Context.TYPE_STRING ) );
        return elt;
    }
    
    /**
     * Returns a ContextElement fills with name and value
     * 
     * @param name	the context element name
     * @param value the context element value
     * 
     * @return The new context element
     */
    public ContextElement createCtxElement( String name, int value ) {
        
        ContextElement elt = new  ContextElementImpl();
        elt.setName( name );
        TypeValue val = new TypeValueImpl();
        val.setInt( BigInteger.valueOf( value ) );
        elt.setVal( val );
        //elt.setType( BigInteger.valueOf( InfoType.Context.TYPE_INT ) );
        
        return elt;
    }
    
    /**
     * Returns a ContextElement fills with name and value
     * 
     * @param name	the context element name
     * @param value the context element value
     * 
     * @return The new context element
     */
    public ContextElement createCtxElement( String name, boolean value ) {
        
        ContextElement elt = new  ContextElementImpl();
        elt.setName( name );
        TypeValue val = new TypeValueImpl();
        if ( value )
            val.setBool( BigInteger.valueOf(1) );
        else 
            val.setBool( BigInteger.valueOf(0) );
        elt.setVal( val );
        //elt.setType( BigInteger.valueOf( InfoType.Context.TYPE_BOOLEAN ) );
        
        return elt;
    }
    
    /**
     * Returns a ContextElement fills with name and value
     * 
     * @param name	the context element name
     * @param value the context element value
     * 
     * @return The new context element
     */
    public ContextElement createCtxElement( String name, Calendar value ) {
        
        ContextElement elt = new  ContextElementImpl();
        elt.setName( name );
        TypeValue val = new TypeValueImpl();
        val.setTime( value );
        elt.setVal( val );
        //elt.setType( BigInteger.valueOf( InfoType.Context.TYPE_TIME ) );
        return elt;
    }
}
