/*
 * Created on August 3, 2005
 * 
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr
 * Preliminary version, please do not distribute
 * 
 * Copyright INRIA 2005
 */
package org.msda.context;

import java.math.BigInteger;
import java.util.Calendar;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import org.msda.msd.ContextElement;
import org.msda.msd.ContextInformation;
import org.msda.msd.ContextInformationTable;
import org.msda.msd.ContextRules;
import org.msda.msd.ContextRulesTree;
import org.msda.msd.NetCtxStartup;
import org.msda.msd.impl.NetCtxStartupImpl;
import org.msda.util.JaxMeUtil;


/**
 * The MSDContextManager performs the context management <br>
 * It stores:<br>
 * 	- context information of active users along with
 *    ContextRules specified in their profiles<br>
 *  - context information of services available in the 
 *    network where the MSDManger resides<br>
 *  - context information of the network in which
 *    the MSDManager resides.<br>
 * 
 *  @author Oriana Riva, INRIA - August 2005
 */

public class MSDContextManager {
    
    private static int MAX_NET = 3;
    private static int MAX_SER = 3;
    
    private boolean firstTime = false;
    
    
    // containts the set of current available services in the network
    // where the ContextManager resides
    // with history
    private Hashtable servicesCtx = new Hashtable(); 
    
    // contains the contexts of the users currently active in the network
    // where the ContextManager resides 
    // without history
    private Hashtable activeUsersCtx = new Hashtable();
    
    // contains the contextRules of the users currently active
    // in the network where the ContextManager resides
    private Hashtable activeUsersCtxRules = new Hashtable();
    
    // contains info about the network where the ContextManager resides
    // with history
    private Vector networkCtx = new Vector(); 
    
    private Vector rulesCtx = new Vector();
    
    // network ID
    private String netID;
    
    
    
    /**
     * constructor
     */
    public MSDContextManager(){
        //print("instantiate MSDContextManager");
    }
    
    
    /**
     * used to add a new user to the users record
     * (context information and context rules are stored)
     * 
     * @param String user_id
     * @param UserContext _userCtx
     * @param ContextRules _userRules
     */
    public void addUser(String user_id, ContextInformationTable _userCtx, ContextRulesTree _userRules){
        //print("add user " + user_id);        
        //print("user rules " + _userRules.toString());        
        //print("added user rules");
        if(!activeUsersCtx.containsKey(user_id)){
            //print("it does not contain the user");
            List list = getLastNetworkCtx().getCITable();
            int i = ContextUtil.getInstance().getIndex( NetworkTemplate.NUM_USERS, list );
            
            if ( i<0 ) {
	            ContextElement elt = ContextUtil.getInstance().createCtxElement( NetworkTemplate.NUM_USERS, 1 );
	            list.add( elt );
            } else {
                ContextElement elt = (ContextElement)list.get(i);
                BigInteger big = elt.getVal().getInt().add( BigInteger.ONE );
                elt.getVal().setInt( big );
                list.set( i, elt );
            }
        }
        activeUsersCtx.put(user_id, _userCtx);
        activeUsersCtxRules.put(user_id, _userRules);
    }
    
    /**
     * Loads the network context from a configuration file
     * 
     * @param filename
     */
    public void setNetworkContextFile( String filename ) {
        
        JaxMeUtil jaxme = new JaxMeUtil();
	    NetCtxStartup ctxStartup = new NetCtxStartupImpl();
	    ctxStartup = (NetCtxStartup)jaxme.loadConfigFile( filename );
	    
	    ContextInformation contextNet = ctxStartup.getCI();
	    ContextRules contextRules = ctxStartup.getCR();
	    
	    ContextInformationTable ciNet = contextNet.getCINet();
	    ContextElement elt = ContextUtil.getInstance().createCtxElement( NetworkTemplate.CREATION_TIME, Calendar.getInstance() );
	    ciNet.getCITable().add( elt );
	    
	    if(networkCtx.size()==MAX_NET){
	        for (int i = 0; i < networkCtx.size(); i++) 
	            networkCtx.add(i, networkCtx.elementAt(i+1));                
	        
	        networkCtx.addElement(ciNet);
	        if ( contextRules!=null )
	            rulesCtx.addElement(contextRules.getCRNet());
	    }
	    else{
	        networkCtx.addElement(ciNet);
	        if ( contextRules!=null )
	            rulesCtx.addElement(contextRules.getCRNet());
	    }   
	    
	    
	    
//	    try {
//            String res = jaxme.toString( contextNet );
//            board.info( res );
//        } catch (JAXBException e) {
//            e.printStackTrace();
//        }
    }
    
    /**
     * invoke to add a new service to the service records
     * 
     * @param ServiceContext _serviceCtx
     */
    public void addService(ServiceContext _serviceCtx){    
        print("the contex of the service is " + _serviceCtx.toString());    
        
        if(servicesCtx.isEmpty()){
            //print("the service does not exist and the service vector is empty");
            Vector ser = new Vector();
            ser.addElement(_serviceCtx);
            servicesCtx.put(_serviceCtx.getServiceID(),ser);
            //getLastNetworkCtx().setNumServices(getLastNetworkCtx().getNumServices().add(BigInteger.ONE));
        }
        else{
            for (int i = 0; i < servicesCtx.size(); i++) {
                //print("for " + servicesCtx.size());
                if(servicesCtx.containsKey(_serviceCtx.getServiceID())){
                    //print("the services exists already");
                    Vector services = (Vector)servicesCtx.get(_serviceCtx.getServiceID());
                    if(services.size()==MAX_SER){
                        //print("the vector is full");
                        for (int j = 0; j < services.size(); j++) {
                            services.add(j, services.elementAt(j+1));                
                        }
                    }
                    services.addElement(_serviceCtx);
                    servicesCtx.put(_serviceCtx.getServiceID(),services);
                    //print("size now is " + servicesCtx.size());
                    
                    List list = getLastNetworkCtx().getCITable();
                    int index = ContextUtil.getInstance().getIndex( NetworkTemplate.NUM_USERS, list );
                    if ( index<0 ) {
        	            ContextElement elt = ContextUtil.getInstance().createCtxElement( NetworkTemplate.NUM_USERS, 1 );
        	            list.add( elt );
                    } else {
                        ContextElement elt = (ContextElement)list.get(index);
                        BigInteger big = elt.getVal().getInt().add( BigInteger.ONE );
                        elt.getVal().setInt( big );
                        list.set( index, elt );
                    }
                    break;                    
                }
                else{
                    //print("the service does not exist");
                    Vector ser = new Vector();
                    ser.addElement(_serviceCtx);
                    servicesCtx.put(_serviceCtx.getServiceID(),ser);
                    //print("size now " + servicesCtx.size());
                    break;
                }
            }      
        }
    }
    

    
    /**
     * @param String serviceID
     * @return ServiceContext
     */
//    public ServiceContext getLastServiceCtx(String serviceID){  
//        
//        if(servicesCtx.containsKey(serviceID)){
//            Vector ser = (Vector)servicesCtx.get(serviceID);
//            return (ServiceContext)ser.lastElement();
//        }
//        return null;
//    }
    
    
    /**
     * Gets the last context characterization of the network
     * 
     * @return ContextInformationTable
     */
    public ContextInformationTable getLastNetworkCtx(){          
        if ( !networkCtx.isEmpty() )
            return (ContextInformationTable) networkCtx.lastElement();
        return null;
    }
    
    /**
     * Gets the last context rules tree of the network
     * 
     * @return	ContextRulesTree
     */
    public ContextRulesTree getLastNetworkRules() {
        if ( !rulesCtx.isEmpty() )
            return (ContextRulesTree) rulesCtx.lastElement();
        return null;
    }
    
    
    /*
     * print function
     */
    private static boolean debug = true;    
    public static void print(String s){
        if (debug){
//            board.info("[MSDContextManager: ]" + s);
        }       
    }
}
