package org.msda.context;

/**
 * @author Oriana Riva, INRIA - August 2005
 * 
 * Collection of parameters that characterize 
 * the context of a printer.
 *  
 * Document sources: PrintEnhanced:1 Service Template Version 1.01
 * for UPnP, May 4, 2005
 */

public class PrinterTemplate {

    /*
     * Private constructor to prevent instantiation.
     */
    private PrinterTemplate(){        
    }
    
    // List of profile parameters
    
    public static final String COLOR_SUPPORTED = "ColorSupported"; 
    
    public static final String CHAR_REP_SUPPORTED = "CharRepSupported";
    
    public static final String CRITICAL_ATTRIBUTES_SUPPORTED = "CriticalAttributesSupported";
    // this includes values for Text-layout, Image-layout, Image-orientation, Pfd-fidelity, Font-family, Font-size 
   
    
    // List of context parameters specific to the current job the printer is handling
    
    public static final String MEDIA_SIZE = "MediaSize";
    // medium size name and dimension that the Printer Service uses for all sheets of the job
  
    public static final String MEDIA_TYPE = "MediaType";
    // medium type that the Printer uses for all impressions of the job
    // Example values are stationery, transparency, envelope, labels, photographic, etc.
    
    public static final String NUMBER_UP = "NumberUp";
    // Examples: one page per side, two page per side, four pages per side
    
    public static final String ORIENTATION_REQUESTED = "OrientationRequested";
    // Possible values are portrait, landscape, reverse-landscape, reverse-portrait, device-setting
    
    public static final String PAGE_MARGINS= "PageMargins"; 
    // four margins size
    
    public static final String PRINTER_STATE= "PrinterState"; 
    
    public static final String PRINTER_STATE_REASONS= "PrinterStateReasons"; 
    
    public static final String SIDES = "Sides";

    public static final String DOCUMENT_FORMAT = "DocumentFormat"; 
    // supported values are listed in the <AllowedValueList>
    
    public static final String COPIES = "Copies"; // number of copies to be printed for the job

    
}
