package org.msda.context;

/**
 * @author Oriana Riva, INRIA - August 2005
 * 
 * Collection of parameters that characterize 
 * the context of a service.
 *  
 * Document sources: PrintEnhanced:1 Service Template Version 1.01
 * for UPnP, May 4, 2005; Dimming V1.0 Template Version 1.0
 * for UPnP, November 23, 2003; Scan:1.0 Service Template Version 1.01
 * for UPnP, September 11, 2002
 * 
 */

public class ServiceTemplate {

    /*
     * Private constructor to prevent instantiation.
     */
    private ServiceTemplate(){        
    }
    
    // List of profile parameters
    
    public static final String DEVICE_ID = "DeviceId";  //not in context info
    
    public static final String SERVICE_LOCATION = "ServiceLocation";  //not in context info
    
    public static final String SERVICE_NAME = "ServiceName";  //not in context info
    
    
    // List of context parameters
    
    public static final String SERVICE_STATE = "ServiceState";
    // Possible values are idle, processing, stopped, reserved, notReady, pending, finishing
    
    public static final String SERVICE_STATE_REASONS = "ServiceStateReasons";
    // information about why the Service is in the current state
    // Possible values are none, media-jam, paused, door-open, media-empty, marker-failure, etc.
    
    
    
    
    public static final String INTERNET_CONNECT_STATE = "InternetConnectState";
    // tells the client whether the Service currently has a connection to the Internet
    // Possible values are unknown, connected, not-connected
    
    public static final String JOB_ID = "JobId";
    
    public static final String JOB_ID_LIST = "JobIdList";
    
    public static final String CONTENT_COMPLETE_LIST = "ContentCompleteList";
    // list of all jobs in the JobIdList that are content complete
    
    public static final String JOB_NAME = "JobName";
    
    public static final String JOB_ORIGINATING_USER_NAME = "JobOriginatingUserName";
    
    public static final String DESTINATION = "Destination";
    // For printers, it's the data sink, i.e., the URI to which the Control Point is to send the HTTP Post operation for the job
    // For scannners, it's the URL where the device will put the image
    
    public static final String SERVICE_QUALITY = "ServiceQuality";
    // service quality requested for the job 
    // Possible values are draft, normal, high, device-setting, etc.  
    
    public static final String JOB_ABORT_STATE = "JobAbortState";
    // holds the terminating state of the job most recently aborted by the Printer
    
    public static final String JOB_END_STATE = "JobEndState";
    // holds the terminating state of the job most recently removed from the JobIdList
    // Possible values are aborted, successful, canceled 
    
    public static final String JOB_ABORT_REASON = "JobAbortReason"; 
    // FailureCode
    // Possible values are hardware-error, external-access-uri-not-found, external-access-object-failure,
    // external-access-http-error, copies, sides, number-up, orientation-requested, timeout-reached, destination-not-reachable etc.
    
    public static final String COST = "cost";
    // Service cost

    
    
    
}
