package org.msda.context;

/**
 * @author Oriana Riva, INRIA - August 2005
 * 
 * Collection of parameters that characterize 
 * the context of a user. In our case the user context 
 * mostly consists of items characterizing the device of the user.
 *  
 * Document sources: FIPA Device Ontology Specification XC00091C, May 2005
 * 
 */
public class UserTemplate {

    /*
     * Private constructor to prevent instantiation.
     */
    private UserTemplate(){        
    }
    
    
    // device profile characteristics
    
    public static final String NAME = "name";
    // e.g., SmartPhone
    
    public static final String VENDOR = "vendor";
    // e.g., Smartphone Ltd.
    
    public static final String VERSION = "version";
    // e.g., xyz
    
    public static final String INFO = "info"; 
    // information about the device
    
    public static final String TYPE = "type";
    // examples are 3G phones, PDA, etc.
    // e.g., mobile-phone, PDA, GPS
    
    public static final String HW_PROPERTIES = "hw-properties";
    
    public static final String SW_PROPERTIES = "sw-properties";
    
    public static final String CONNECTION = "connection";
    // the type of connection the device uses
    //e.g., Bluetooth, Infrared Data, High Speed Circuit Switched Data
    
    public static final String UI = "ui";
    // user interfaces that the device offers
    
    public static final String MEMORY = "memory";
    // amount of memory that the device has
    // e.g., 8MB, 3856KB
    
    public static final String CPU = "cpu";
    // type of cpu the device has
    // e.g., 64-bit ARM9-based RISC
    
    public static final String CONNECTION_INFORMATION = "connection_info"; 

    public static final String SCREEN = "screen";
    
    public static final String AUDIO_INPUT = "audio-input";
    // e.g., true
    
    public static final String AUDIO_OUTPUT = "audio-output";
    // e.g., true
    
    public static final String WIDTH = "width";
    // e.g., 500
    
    public static final String HEIGHT = "height";
    // e.g., 800
        
    public static final String RESOLUTION = "resolution";  
    // e.g., 1024 X 768 pixels
    
    public static final String COLOR = "color";
    // e.g., true
    
    public static final String GRAPHICS = "graphics";
    // it's true if the device is capable of displaying graphics 
    // e.g., true
    
    public static final String OS = "os";
    // e.g., SmartOS abc
    
    public static final String MIDDLEWARE_PLATFORM = "middleware-platform";
    // e.g., SmartMessages
    
    
    // device context characteristics
    
    public static final String MEMORY_AVAILABLE = "memory-available";
    
    public static final String CPU_STATUS = "cpu-status";
    
    public static final String CONNECTION_ACTIVE = "connection-active";
    
    public static final String CONNECTION_QOS = "qos_info";
    
    public static final String AUDIO_INPUT_ACTIVE = "audio-input-active";
    
    public static final String AUDIO_OUTPUT_ACTIVE = "audio-output-active";
    
   
    // fiels name of type String
    
    public static final String DEVICE_NAME = "deviceName";
    
    
}


