/*
 * Created on Aug 16, 2004
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.framework;

import java.io.*;
import java.net.*;
import java.util.Enumeration;
import javax.xml.bind.*;
import org.xml.sax.*; 

import org.msda.components.core.MSDDaemon;
import org.msda.gui.FrameworkGUI;
import org.msda.gui.MSDElectionGUI;
import org.msda.msd.*;
import org.msda.msd.impl.*;




/**
 * @author raverdy, delachap
 *
 */
public class FrameworkMain extends Thread {
	
	public static FrameworkOutput msdaOutput;
	public static FrameworkGUI msdaGUI;
	
	
	/**
	 * 
	 * @param args
	 */
	public static void main (String[] args) {
		MSDElectionGUI guiElec;
		MSDDaemon daemon ;
		UbisecID devId;
		PlatformStartup startup;
		
		//
		System.out.println ("JVM is " + System.getProperty("java.version") );
		System.out.println ("---------------------------" );

		if (System.getProperty("java.version").startsWith("1.3")) {
			System.out.println ("Cannot access network interfaces");
			System.out.println ("---------------------------" );
		}
		else
		{
			DisplayNetworkInterfaces ();
		}
		
		// First check that a filename has been provided
		if (args.length == 0)
		{
			System.out.println("usage: FrameworkMain configfile.xml");
			return;
		}
		else {
			try {
				URL cwd = new File(System.getProperty("user.dir")).toURL();
				System.out.println("CWD: " + cwd );
				System.out.println ("---------------------------" );
			}
			catch (MalformedURLException e)
			{
			}
		}

		System.out.println("Config file is " + args[0] );
		System.out.println ("---------------------------" );

		// load the configuration file and initialize the dashboard
		startup = LoadConfigFile (args[0]);
		if (startup == null)
			return;

		// crrating the outpout board
		msdaOutput = new FrameworkOutput (startup);
		System.out.println ("---------------------------" );

		// check for the gui parameter
		msdaGUI = new FrameworkGUI(); 
		
		// now process the config file 
		try 
		{
			guiElec = new MSDElectionGUI();
			devId = new UbisecIDImpl();
			devId.setName(startup.getDevice().getName());
			
			msdaOutput.info("Start MSDDeamon");				
			
			guiElec.setVisible(true);
			daemon = new MSDDaemon(startup, guiElec);
			daemon.start();
			guiElec.close();
			
			// 	Start the GUI -					
			msdaOutput.info ("Start the MSDA GUI...." );
			guiElec.createBridgeGUI( daemon.getStackBridge(), daemon.isAgreeBridge(), startup.getDevice() );
			msdaGUI.setGuiBridge( guiElec.getBridgeGUI() );
			if ( daemon.getManager()!=null )
				guiElec.createManagerGUI( daemon.getManager(), daemon.getManager().isActive(), startup.getDevice() );
			else 
				guiElec.createManagerGUI( null, false, startup.getDevice() );
			msdaGUI.setGuiManager( guiElec.getManagerGUI() );
			if ( daemon.getManager()!=null )
				daemon.getManager().getThreadDescriptionManager().setGUINotification( guiElec.getManagerGUI() );
			
		}	
		catch (Exception e)
		{
			System.out.println("Could not initialize " + e);
			e.printStackTrace();
			return;
		}
	}
	

	/**
	 * 
	 *
	 */
	private static void DisplayNetworkInterfaces ()
	{
		NetworkInterface netinterface;
		Enumeration ints;
		// Display the network interfaces...
		System.out.println ("Network interfaces are :");
		try {
			ints = NetworkInterface.getNetworkInterfaces();
			while( ints.hasMoreElements() )
			{
				netinterface =  (NetworkInterface)ints.nextElement();
				if (netinterface.getInetAddresses() != null)
					System.out.println ("  - [" + netinterface.getName() + "]  [" + netinterface.getInetAddresses().nextElement().toString() + "] " + netinterface.getDisplayName() );
			}
		}
		catch (SocketException  e) {		    
			e.printStackTrace();
		}
		System.out.println ("---------------------------" );
	}

	/**
	 * 
	 * @param filename
	 * @return
	 */
	public static PlatformStartup LoadConfigFile (String filename)
	{
		File inputFile;
		InputSource source;
		PlatformStartup startup = new PlatformStartupImpl();
		JAXBContext ctx;
		Unmarshaller u;
		
		//board.info ("loading config file " + filename);
		try {
			inputFile = new File(filename);
			source = new InputSource(new FileInputStream(inputFile));
			source.setSystemId(inputFile.toURL().toString());
			ctx = JAXBContext.newInstance("org.msda.msd");
			u = ctx.createUnmarshaller();
			startup = (PlatformStartup)u.unmarshal(source);			
		}
		catch ( IOException e )
		{
			System.out.println ("error when reading config file " + e);
			e.printStackTrace();
			return null;
		}
		catch ( JAXBException e )
		{
			System.out.println ("error when reading config file " + e);
			e.printStackTrace();
			return null;
		}
		return startup;
	}
}
