/*
 * Created on May 17, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.framework;

import java.io.IOException;
import java.text.FieldPosition;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import org.apache.log4j.*;

import org.msda.gui.*;
import org.msda.msd.*;


/**
 * Class that defines a dashboard of the msd component.<p>
 * The statistical information are gathered in that class.
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class FrameworkOutput  {
    private Logger logger;
    private ManagerGUI guiManager;
    private BridgeGUI guiBridge;
    
    // Statistical information
    private long nbOfReceivedMessages;
    private long nbOfSendMessages;
    private long sumSizeOfMessages;
    private long maxSizeOfMessage;
    private long minSizeOfMessage;
    private long lastTime;
    private boolean collected;
    
    private int level;
    
    
	/**
	 * For now, set the upper level
	 * 
	 * @param startup
	 * @return
	 */
	public FrameworkOutput (PlatformStartup startup) 
	{
		BasicConfigurator.configure();

		String filename = startup.getDevice().getName();
		System.out.println ("new output log is " + filename + ".log"  );

		logger = Logger.getRootLogger();
		logger.removeAllAppenders();

		// generic layout
		String pattern = "%p %r [%t] %m \t\t\t\t\t\t%n";
		//String pattern = "%p %r [%t] %m      [%l]%n";
		//String pattern = "%d{HH:mm:ss,SSS} %p [%t] %m%n";
		//String pattern = "%d{HH:mm:ss,SSS} %p [%t] %m%n";
		//String pattern = "%m%n";
		PatternLayout patternLayout = new PatternLayout( pattern );
		SimpleDateFormat format = new SimpleDateFormat( "yyyyMMdd");
		StringBuffer buffer = format.format( Calendar.getInstance().getTime(), new StringBuffer(), new FieldPosition( NumberFormat.INTEGER_FIELD ) );
		String date = buffer.toString();
		
		// use a console appender 
		Appender appender;
		logger.addAppender( new ConsoleAppender(patternLayout));   
		
		// use the file appender
		try {
			appender = new FileAppender(patternLayout, filename+".log"  );
			 logger.addAppender( appender );
		} catch (IOException e1) {
			e1.printStackTrace();
		} finally {
		}

        this.nbOfReceivedMessages = 0;
        this.sumSizeOfMessages = 0;
        this.minSizeOfMessage = -1;
        this.maxSizeOfMessage = -1;
        this.lastTime = System.currentTimeMillis();
        this.collected = false;
	}

//	public FrameworkOutput (String filename) 
//	{
//		System.out.println ("new output log is " + filename+".log"  );
//
//		BasicConfigurator.configure();
//
//		String pattern = "%m%n";//"%d{HH:mm:ss,SSS} %p [%t] %m%n";
//		PatternLayout patternLayout = new PatternLayout( pattern );
//		SimpleDateFormat format = new SimpleDateFormat( "yyyyMMdd");
//		StringBuffer buffer = format.format( Calendar.getInstance().getTime(), new StringBuffer(), new FieldPosition( NumberFormat.INTEGER_FIELD ) );
//		String date = buffer.toString();
//		
//		logger = Logger.getLogger( filename + ".log" );
//		logger.removeAllAppenders();
//		Appender appender;
//		try {
//			appender = new FileAppender(patternLayout, filename+".log"   );
//			logger.addAppender( appender );
//		} catch (IOException e1) {
//			e1.printStackTrace();
//		} finally {
//			// use a console appender in all cases
//			logger.addAppender( new ConsoleAppender(patternLayout));   
//		}
//		logger.setLevel(Level.ALL);
//		
//        this.nbOfReceivedMessages = 0;
//        this.sumSizeOfMessages = 0;
//        this.minSizeOfMessage = -1;
//        this.maxSizeOfMessage = -1;
//        this.lastTime = System.currentTimeMillis();
//        this.collected = false;
//	}	


	public static FrameworkOutput commonBoard () 
	{
		return FrameworkMain.msdaOutput;
	}
	
    
    /**
     * Returns the logger.
     * 
     * @return Returns the logger.
     */
    
    public void warn( Object o ) {
    	logger.warn( o );
    }
    
    public void info( Object o ) {
    	logger.info( o );
    }
    
    public void debug( Object o ) {
       	logger.debug( o );
    }

    public void error( Object o ) {
    	logger.error( o );
    }

    // 
    // Statistical information
    //
    
    /**
     * @return Returns the nbOfMessages.
     */
    public long getNbOfReceivedMessages() {
        return nbOfReceivedMessages;
    }
    
    /**
     * TODO - Test it !
     * 
     * @return
     */
    public long getAvgSizeOfMessages() {
        
       if ( this.nbOfReceivedMessages > 0 )
           return Math.round( this.sumSizeOfMessages / this.nbOfReceivedMessages ) ;
       return 0;
    }
    
    /**
     * @return Returns the maxSizeOfMessage.
     */
    public long getMaxSizeOfMessage() {
        return maxSizeOfMessage;
    }
    /**
     * @return Returns the minSizeOfMessage.
     */
    public long getMinSizeOfMessage() {
        return minSizeOfMessage;
    }
    
    /**
     * @return Returns the collected.
     */
    public boolean isCollected() {
        return collected;
    }
    /**
     * @param collected The collected to set.
     */
    public void setCollected(boolean collected) {
        this.collected = collected;
    }
    
     
    public void notifyException (Exception e)
    {
        java.io.ByteArrayOutputStream out = new java.io.ByteArrayOutputStream() ;
        e.printStackTrace( new java.io.PrintStream( out ) );
        logger.error( out.toString() );
    }
     
    public void notifyError( Object o ) {
    	logger.error( o );
        if ( this.guiManager!=null ) {
            this.guiManager.notifyError( o );
        }
    }
    
}
