/*
 * Created on May 11, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.gui;


import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Menu;
import java.awt.Panel;
import java.util.Vector;

import org.msda.components.bridge.MSDBridge;
import org.msda.framework.FrameworkOutput;
import org.msda.gui.bridge.MenuDisplay;
import org.msda.gui.bridge.SplitPanel;
import org.msda.gui.bridge.ToolbarPanel;
import org.msda.msd.DeviceConfig;

/**
 * GUI that shows a Bridge :<br>
 *  - Select "Network interfaces" to see the network interfaces of the bridge <br>
 *  - Select "Managers" to see the managers <br>
 *  - Select "Traffic" to see the last messages that have been forwarded by the bridge
 *  - Select "Statistics" to see some statistical information 
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class BridgeGUI extends Panel implements GUINotification, GUIMessageNotification {

    public static final Color COLOR = new Color( 230,230,230 );
    public static final Color SPECIAL = new Color( 207,222,237 );
    public static final Color NEGATIVE_COLOR =  Color.BLACK;
    
    public static final int WIDTH = 210;
    public static final int HEIGHT = 210;
    
    private SplitPanel splitPanel;
    private ToolbarPanel toolbarPanel;
    private MenuDisplay menuDisplay;
    
    private DeviceConfig config;
    private MSDBridge bridge;
    
    public BridgeGUI( MSDBridge bridge, DeviceConfig config ) {

        setSize( WIDTH, HEIGHT ); 
        setBackground( BridgeGUI.COLOR );
    	
    	this.bridge = bridge;
    	this.config = config;
    	
    	toolbarPanel = new ToolbarPanel( this );
    	toolbarPanel.setBackground( SPECIAL );
        splitPanel = new SplitPanel();
        splitPanel.setBackground( SPECIAL );
        
        toolbarPanel.setSize( BridgeGUI.WIDTH, 60 );
        splitPanel.setSize( BridgeGUI.WIDTH, BridgeGUI.HEIGHT-80 );
    	
        menuDisplay = new MenuDisplay( this );
        
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.fill = GridBagConstraints.BOTH;
        gridbag.setConstraints( toolbarPanel, c);
        gridbag.setConstraints( splitPanel, c);
        
        setLayout( gridbag );
        add( toolbarPanel );
        add( splitPanel );
    }
    
    public Vector getDevice() {
        Vector v = new Vector();
        if ( config!=null )
            v.add( config );
        return v;
    }
    
    /**
     * @return Returns the menuDisplay.
     */
    public Menu getMenuDisplay() {
        return menuDisplay;
    }
    
    /**
     * @return Returns the toolbarPanel.
     */
    public ToolbarPanel getToolbarPanel() {
        return toolbarPanel;
    }
    /**
     * @return Returns the splitPanel.
     */
    public SplitPanel getSplitPanel() {
        return splitPanel;
    }
    
    /**
     * 
     * @return Returns the network interfaces
     */
    public Vector getNetworkInterfaces() {
        
        if ( bridge!=null )
            return bridge.getNetworkInterfacesVector();
        return new Vector();
    }
    
    /**
     * 
     * @return Returns the active managers
     */
    public Vector getManagers() {
        
        if ( bridge!=null && bridge.getLocalMgr()!=null ) {
            return bridge.getLocalMgr().getManagers();
        }
       return new Vector();
    }
    
    /**
     * 
     * @return Returns the board.
     */
    public FrameworkOutput getDashBoard() {
        
        return bridge.getBoard();
    }
    
    //
    // Notification
    //

    /*
     *  (non-Javadoc)
     * @see org.msda.gui.GUINotification#addManager()
     */
    public void addManager() {
        if ( MenuDisplay.MANAGERS.equals( menuDisplay.getActivePanel() ) ) {
            menuDisplay.actionManagers();
        }
    }

    /*
     *  (non-Javadoc)
     * @see org.msda.gui.GUINotification#removeManager(java.lang.String)
     */
    public void removeManager(String name) {
        if ( MenuDisplay.MANAGERS.equals( menuDisplay.getActivePanel() ) ) {
            menuDisplay.actionManagers();
        }
    }

    /*
     *  (non-Javadoc)
     * @see org.msda.gui.GUINotification#addBridge()
     */
    public void addBridge() {
        // nothing to do
    }
    
    /*
     *  (non-Javadoc)
     * @see org.msda.gui.GUINotification#removeBridge(java.lang.String)
     */
    public void removeBridge(String name) {
        // nothing to do
    }
    
    /**
     * @see org.msda.gui.GUINotification#addDescription(boolean)
     */
    public void addDescription(boolean local) {
        // TODO Auto-generated method stub
    }

    /**
     * @see org.msda.gui.GUINotification#removeDescription(boolean)
     */
    public void removeDescription(boolean local) {
        // TODO Auto-generated method stub
    }
    
    /**
     * @see org.msda.gui.GUIMessageNotification#notify(java.lang.Object)
     */
    public void notify(Object o) {
        // case of the bridge, just put it on the "traffic panel"
        menuDisplay.append( (String)o );
        if ( MenuDisplay.TRAFFIC.equals( menuDisplay.getActivePanel() ) ) {
            menuDisplay.actionTraffic();
        }
    }
    
    /**
     * @see org.msda.gui.GUIMessageNotification#notify(java.lang.Object, java.lang.String)
     */
    public void notify(Object o, String from) {
        // TODO - remove it ?
        notify( o );
    }
    
    /**
     * @see org.msda.gui.GUIMessageNotification#notifyError(java.lang.Object)
     */
    public void notifyError(Object error) {
        //TODO
    }
}
