/*
 * Created on Jun 7, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.gui;


import java.awt.Color;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.MenuBar;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import org.msda.components.bridge.MSDBridge;
import org.msda.components.manager.MSDManager;
import org.msda.msd.DeviceConfig;

/**
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class MSDElectionGUI extends Frame {

    public static final Color COLOR_RED = new Color( 220,30,30 );
    public static final Color COLOR_GREEN = new Color( 30,230,30 );
    
    public static final int WIDTH = 220;
    public static final int HEIGHT = 80;
    private TextField text;
    
    private ManagerGUI managerGUI;
    private BridgeGUI bridgeGUI;
    
    
    public MSDElectionGUI() {
        
        super( "MSD Device" );
        
        setSize( WIDTH, HEIGHT ); 
        
        setBackground( MSDElectionGUI.COLOR_RED );
        text = new TextField( " That device contains MSD." );
        text.setEditable(false);
        
//      add to the panel
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.BOTH;
        gridbag.setConstraints( text, c);
        
        setLayout(gridbag);
        add( text );
        
//      To close the application
        addWindowListener(new WindowAdapter() { 
            public void windowClosing(WindowEvent e) { 
                setVisible(false); 
                dispose(); 
                System.exit(0); 
            } 
        }); 
    }
    
    
    public void startElection() {

        setBackground( Color.yellow );        
        text.setText( " Processing election ..." );
        repaint();
    }
    
    public void endElection( String type ) {
        
        setBackground( MSDElectionGUI.COLOR_GREEN );   
        text.setText( " Election finished : "+type );
        repaint();
    }
    
    /**
     * @return Returns the bridgeGUI.
     */
    public BridgeGUI getBridgeGUI() {
        return bridgeGUI;
    }
    /**
     * @return Returns the managerGUI.
     */
    public ManagerGUI getManagerGUI() {
        return managerGUI;
    }
    
    public void createManagerGUI( MSDManager manager, boolean isActive, DeviceConfig config ) {
        
        this.removeAll();
        setSize( 420, 420 );
        setBackground( Color.WHITE );
        managerGUI = new ManagerGUI( manager, config );
        
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.RELATIVE;
        c.fill = GridBagConstraints.BOTH;
        Panel p1 = new Panel();
        p1.add( managerGUI );
        gridbag.setConstraints( p1, c);
        Panel p2 = new Panel();
        if ( bridgeGUI!=null ) {
            p2.add( bridgeGUI );
            gridbag.setConstraints( p2, c);
        }
        setLayout(gridbag);
        
        add( p1 );
        add( p2 );
        
        managerGUI.getMenuDisplay().setEnabled( isActive );
        
        MenuBar menuBar = new MenuBar();
        menuBar.add( managerGUI.getMenuDisplay() );
        if ( bridgeGUI!=null ) 
            menuBar.add( bridgeGUI.getMenuDisplay() );
                
        setMenuBar( menuBar );
        
        this.paintAll( this.getGraphics() );
        setVisible(true); 
    }
    
    public void createBridgeGUI( MSDBridge bridge, boolean isBridge, DeviceConfig config ) {
        
        this.removeAll();
        setSize( 420, 420 );
        bridgeGUI = new BridgeGUI( bridge, config );
        
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.RELATIVE;
        c.fill = GridBagConstraints.HORIZONTAL;
        Panel p1 = new Panel();
        if ( managerGUI!=null ) {
            p1.add( managerGUI );
            gridbag.setConstraints( p1, c);
        }
        Panel p2 = new Panel();
        p2.add( bridgeGUI );
        gridbag.setConstraints( p2, c);
        
        setLayout(gridbag);
        
        add( p1 );
        add( p2 );
        
        bridgeGUI.getMenuDisplay().setEnabled( isBridge );
        
        MenuBar menuBar = new MenuBar();
        if ( managerGUI!=null)
        menuBar.add( managerGUI.getMenuDisplay() );
        menuBar.add( bridgeGUI.getMenuDisplay() );
                
        setMenuBar( menuBar );
        
        this.paintAll( this.getGraphics() );
        setVisible(true); 
    }
    
    public void close() {
        setVisible(false); 
        dispose(); 
    }
    
}
