/*
 * Created on May 24, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.gui.bridge;

import java.awt.CheckboxMenuItem;
import java.awt.Menu;
import java.awt.MenuShortcut;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.util.Vector;

import org.msda.framework.FrameworkOutput;
import org.msda.gui.BridgeGUI;

/**
 * GUI class to create the menus.
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class MenuDisplay extends Menu {

    public static final String NETWORKS = "Network interfaces";
    public static final String MANAGERS = "Managers";
    public static final String TRAFFIC = "Traffic";
    public static final String STAT = "Statistics";
    public static final String DEVICE = "Device"; 
    
    private CheckboxMenuItem[] items;
    
    private Vector vnetworks;
    private Vector vmanagers;
    private Vector vstat;
    private Vector vtraffic;
    private Vector vdevice;
    private StringBuffer bufTraffic;
    
    private String active;
    private BridgeGUI gui;
    
    public MenuDisplay( BridgeGUI gui ) {
        
        super( "Bridge" );
        
        this.gui = gui;
        this.bufTraffic = new StringBuffer();
        
        MenuItemListener menuListener = new MenuItemListener();
        
        items = new CheckboxMenuItem[5];
        items[0] = new CheckboxMenuItem( NETWORKS );
        items[1] = new CheckboxMenuItem( MANAGERS );
        items[2] = new CheckboxMenuItem( TRAFFIC );
        items[3] = new CheckboxMenuItem( STAT );
        items[4] = new CheckboxMenuItem( DEVICE );
        
        items[0].setShortcut( new MenuShortcut( KeyEvent.VK_N ) );
        items[1].setShortcut( new MenuShortcut( KeyEvent.VK_M ) );
        items[2].setShortcut( new MenuShortcut( KeyEvent.VK_T ) );
        items[3].setShortcut( new MenuShortcut( KeyEvent.VK_S ) );
        items[4].setShortcut( new MenuShortcut( KeyEvent.VK_D ) );
        
        active = NETWORKS;
        
        for (int i=0; i<items.length; i++) {
            items[i].addItemListener( menuListener );
            add( items[i] );
        }
    }
    
    /**
     * 
     * @return Returns the name of the active panel
     */
    public String getActivePanel() {
        return active;
    }
    
    public void clearTraffic() {
        
        this.bufTraffic = new StringBuffer();
    }
    
    /**
     * Append a new message to the current trace of messages
     * 
     * @param newMessage
     */
    public void append( String newMessage ) {
        
        this.bufTraffic = this.bufTraffic.append( newMessage );
        
    }
    
    /**
     * Defines what happens when the "Network interfaces" button is pressed.
     *
     */
    public void actionNetworks() {
        
        vnetworks = gui.getSplitPanel().init( gui.getNetworkInterfaces(), vnetworks );
        
        active = NETWORKS;
        
        select( 0 );
        
    }
    
    /**
     * Defines what happens when the "Managers" button is pressed.
     *
     */
    public void actionManagers() {
        
        vmanagers = gui.getSplitPanel().init( gui.getManagers(), null );
        
        active = MANAGERS;
        
        select( 1 );
    }
    
    /**
     * Defines what happens when the "Traffic" button is pressed.
     *
     */
    public void actionTraffic() {
        
//        vtraffic = gui.getSplitPanel().init( vtraffic, vtraffic );
        
        gui.getSplitPanel().append( bufTraffic );
        
        active = TRAFFIC;
        
        select( 2 );
    }
    
    /**
     * Defines what happens when the "Stat" button is pressed.
     *
     */
    public void actionStat() {
        
        FrameworkOutput board = gui.getDashBoard();
        Vector v = new Vector();
        v.add( board );
        
        vstat = gui.getSplitPanel().init( v, vstat );
        
        active = STAT;
        
        select( 3 );
    }
    
    /**
     * Defines what happens when the "Device" button is pressed
     *
     */
    public void actionDevice() {
        
        vdevice = gui.getSplitPanel().init( gui.getDevice(), null );
                
        active = DEVICE;
        select( 3 );
    }
    
    /**
     * Selects the item.
     * 
     * @param index - its index
     */
    private void select( int index ) {
        
        for ( int i=0; i<items.length; i++ ) {
            if ( i!=index ) {
                items[i].setState( false );
            } else {
                items[i].setState( true );
            }
        }
    }
    
    /**
     * Inner class to manage the different actions associated with the menus
     * 
     * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
     *
     */
    class MenuItemListener implements ItemListener {
        
        public void itemStateChanged(ItemEvent e) {
            Object action = e.getItem();
            if ( NETWORKS.equals( action ) ) {
                actionNetworks();
            } else if ( MANAGERS.equals( action ) ) {
                actionManagers();
            } else if ( TRAFFIC.equals( action ) ) {
                actionTraffic();
            } else if ( STAT.equals( action ) ) {
                actionStat();
            } else if ( DEVICE.equals( action ) ) {
                actionDevice();
            }
            gui.getToolbarPanel().addTitle( active );
        }
    }
}
