/*
 * Created on May 17, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.gui.bridge;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.List;
import java.awt.Panel;
import java.awt.ScrollPane;
import java.awt.TextArea;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Vector;

import org.msda.framework.FrameworkOutput;
import org.msda.gui.BridgeGUI;
import org.msda.msd.DeviceConfig;
import org.msda.msd.InformationComponent;
import org.msda.msd.NetworkConfig;
import org.msda.msd.Param;

/**
 * The splitPanel composed of a List and a TextArea.<br>
 * The adding information about the list elements are displayed in the TextArea.
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class SplitPanel extends Panel {

    private ScrollPane scroll;
    private TextArea textArea;
    private ListItemListener actionList;
    
    private List list;
    private Vector vlist;
    
    public SplitPanel() {
        
        setBackground( BridgeGUI.COLOR );
        
        textArea = new TextArea(17, 20);
        textArea.setBackground( BridgeGUI.COLOR );
        
        list = new List();
        //list.add( "The bridge has been initialized..." );
        actionList = new ListItemListener();
        list.addItemListener( actionList );
        
        scroll = new ScrollPane();
        scroll.add( list );
        scroll.setSize( BridgeGUI.WIDTH-20, 60 );
        
        // add to the panel
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.fill = GridBagConstraints.HORIZONTAL;
        gridbag.setConstraints( scroll, c);
        
        c.fill = GridBagConstraints.BOTH;
        gridbag.setConstraints( textArea, c);
        
        setLayout(gridbag);
        
        add( scroll );
        add( textArea );
    }
    
    /**
     * Appends the new message to the textArea.
     * 
     * @param bufTraffic
     */
    public void append( StringBuffer bufTraffic ) {
     
        scroll.remove( 0 );
        list = new List();
        scroll.add( list );
        
        textArea.setText( bufTraffic.toString() );
        
        this.getParent().paintAll( this.getParent().getGraphics() );
    }
    
    /**
     * Initializes the List.
     * @param first
     * @param second
     */
    public Vector init( Vector first, Vector second ) {
        
        if ( first==null ) {
            
            
            list.removeAll();
    	    list.setEnabled(false);
            
    	    list.add( "List null..." );
    	    
    	    list.setEnabled(true);
    	    list.addItemListener( actionList );
            
            return second;
        }
        
        boolean fill = false;
        if ( second==null ) {
            second = new Vector();
            fill = true;
        }
        
        vlist = second;
        
        list.removeAll();
	    list.setEnabled(false);
        
        for( int i=0; i<first.size(); i++) {
            
            Object object = first.get(i);
            
            if ( object instanceof String ) {
                // traffic
                
//                newList.add( (String)object );
//                if ( fill )
//                    vlist.add( object );
            
            } else if ( object instanceof DeviceConfig ) {
                
                DeviceConfig config = (DeviceConfig)object;
                list.add( "Device "+config.getName() );
                
                if ( fill ) {
                    
                    StringBuffer buf = new StringBuffer();
	                buf = buf.append( "Name = ").append( config.getName() );
	                	     
	                if ( !config.getParams().isEmpty() ) {
	                    buf = buf.append( System.getProperty( "line.separator" ) ).append( System.getProperty( "line.separator" ) );
	                    java.util.List list = config.getParams();
	                    for (int j=0;j<list.size(); j++) {
	                        Param param = (Param)list.get(j);
	                        buf = buf.append( param.getType() ).append( " : " ).append( param.getValue() ).append( System.getProperty( "line.separator" ) );
	                    }
	                }
                    vlist.add( buf.toString() );
                }
            } else if ( object instanceof NetworkConfig )  {
                
                NetworkConfig config = (NetworkConfig)object;
                StringBuffer buf = new StringBuffer();
                buf = buf.append( config.getName() );//.append( " on " ).append( config.getName() );
                list.add( buf.toString() );
                
                if ( fill ) {
                    buf = new StringBuffer();
                    buf = buf.append( "Name = " ).append( config.getName() ).append( System.getProperty( "line.separator" ) )
                    	     .append( "Interface = " ).append( config.getInterface() ).append( System.getProperty( "line.separator" ) )
                    		 .append( "Control " ).append( System.getProperty( "line.separator" ) )
                    	     .append( " Group = " ).append( config.getControlGroup() ).append( System.getProperty( "line.separator" ) )
                    		 .append( " Port = " ).append( config.getControlPort() ).append( System.getProperty( "line.separator" ) )
                    		 .append( "Data " ).append( System.getProperty( "line.separator" ) )
                    		 .append( " Group = " ).append( config.getDataGroup() ).append( System.getProperty( "line.separator" ) )
                    		 .append( " Port = " ).append( config.getDataPort() );
                    
                    if ( !config.getParams().isEmpty() ) {
                        buf = buf.append( System.getProperty( "line.separator" ) ).append( System.getProperty( "line.separator" ) );
                        
                        for (int j=0;j<config.getParams().size(); j++) {
                            Param param = (Param)config.getParams().get(j);
                            buf = buf.append( param.getType() ).append( " : " ).append( param.getValue() ).append( System.getProperty( "line.separator" ) );
                        }
                    }
                    
                    vlist.add( buf.toString() );
                }
                
            } else if ( object instanceof InformationComponent ) {
                
                InformationComponent info = (InformationComponent)object;
                
                list.add( info.getCompID().getName() );
                
                if ( fill ) {
	                StringBuffer buf = new StringBuffer();
	                buf = buf.append( "stack ID = ").append( info.getCompID().getName() ).append( System.getProperty( "line.separator" ) )
	                	     .append( "IP address = " ).append( info.getIPAddr() ).append( System.getProperty( "line.separator" ) )
	                		 .append( "name = " ).append( info.getManager().getName() ).append( System.getProperty( "line.separator" ) )
	                		 .append( "network name = " ).append( info.getManager().getNetworkName() ).append( System.getProperty( "line.separator" ) )
	                		 .append( "access port = " ).append( info.getManager().getRemoteAccessPort() ).append( System.getProperty( "line.separator" ) )
	                		 ;
	                
	                if ( !info.getParams().isEmpty() ) {
	                    buf = buf.append( System.getProperty( "line.separator" ) ).append( System.getProperty( "line.separator" ) );
	                    java.util.List list = info.getParams();
	                    for (int j=0;j<list.size(); j++) {
	                        Param param = (Param)list.get(j);
	                        buf = buf.append( param.getType() ).append( " : " ).append( param.getValue() ).append( System.getProperty( "line.separator" ) );
	                    }
	                }
	                vlist.add( buf.toString() );
                }

                
            } else if ( object instanceof FrameworkOutput ) {
                
                FrameworkOutput board = (FrameworkOutput)object;
                if ( board.isCollected() ) {
	                
                    // Keep an history of the statistical information
	                
                    list.add( "Statistical information on "+System.currentTimeMillis() );
	                
	                StringBuffer buf = new StringBuffer();
	                buf = buf.append( "Size of messages in bytes" ).append( System.getProperty( "line.separator" ) )         
	                		 .append( "# messages = " ).append( board.getNbOfReceivedMessages() ).append( System.getProperty( "line.separator" ) )
	                	     .append( "average = " ).append( board.getAvgSizeOfMessages() ).append( System.getProperty( "line.separator" ) ) 
	                		 .append( "min = " ).append( board.getMinSizeOfMessage() ).append( System.getProperty( "line.separator" ) )
	                		 .append( "max = " ).append( board.getMaxSizeOfMessage() );
	                
	                vlist.add( buf.toString() );
	                board.setCollected( false );
                } else {
                    board.info("No statistical information collected...");
                }
                
            }
        }
        
	    list.setEnabled(true);
	    list.addItemListener( actionList );
	    
        if ( vlist.size() > 0) {
            textArea.setText( (String)vlist.get( vlist.size()-1 ) );
            list.select( list.getItemCount()-1 );
        } else {
            textArea.setText( "The list is empty..." );
        }

        this.getParent().paintAll( this.getParent().getGraphics() );

        return second;
    }
    
    
    /**
     * Class that defines the action when an element of the list is selected. 
     * 
     * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
     */
    class ListItemListener implements ItemListener {
        
        public void itemStateChanged(ItemEvent e) {
            
            // transform into a string
            textArea.setText( (String)vlist.get( list.getSelectedIndex() ) );
            textArea.repaint();
        }
        
    };
}
