/*
 * Created on May 24, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.gui.manager;

import java.awt.CheckboxMenuItem;
import java.awt.Menu;
import java.awt.MenuShortcut;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.util.Vector;

import org.msda.framework.FrameworkMain;
import org.msda.framework.FrameworkOutput;
import org.msda.gui.ManagerGUI;

/**
 * GUI class to create the menus.
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class MenuDisplay extends Menu {
    
    public static final String DEVICE = "Device";
    public static final String PLUGINS = "Plugins";
    public static final String DESCRIPTIONS = "Local descriptions";
    public static final String DESCRIPTIONS_REMOTE = "Remote descriptions"; 
    public static final String BRIDGES = "Bridges";
    public static final String STAT = "Statistics";
    
    private CheckboxMenuItem[] items;
    
    private Vector vplugins;
    private Vector vdescriptionsLocal;
    private Vector vdescriptionsRemote;
    private Vector vbridges;
    private Vector vstat;
    private Vector vdevice;
    
    private String active;
    private ManagerGUI gui;
    
    public MenuDisplay( ManagerGUI gui ) {
        
        super( "Manager" );
        
        this.gui = gui;
        
        MenuItemListener menuListener = new MenuItemListener();
        
        items = new CheckboxMenuItem[6];
        items[0] = new CheckboxMenuItem( PLUGINS );
        items[1] = new CheckboxMenuItem( BRIDGES );
        items[2] = new CheckboxMenuItem( STAT );
        items[3] = new CheckboxMenuItem( DEVICE );
        items[4] = new CheckboxMenuItem( DESCRIPTIONS );
        items[5] = new CheckboxMenuItem( DESCRIPTIONS_REMOTE );
        
        items[0].setShortcut( new MenuShortcut( KeyEvent.VK_P ) );
        items[1].setShortcut( new MenuShortcut( KeyEvent.VK_B ) );
        items[2].setShortcut( new MenuShortcut( KeyEvent.VK_S ) );
        items[3].setShortcut( new MenuShortcut( KeyEvent.VK_D ) );
        items[4].setShortcut( new MenuShortcut( KeyEvent.VK_L ) );
        items[5].setShortcut( new MenuShortcut( KeyEvent.VK_R ) );
        
        active = PLUGINS;
        
        for (int i=0; i<items.length; i++) {
            
            items[i].addItemListener( menuListener );
            add( items[i] );
        }
    }
    
    /**
     * Set to null the vector containing the list of cached descriptions.<br>
     * Allows the splitPanel to be reinitialized.
     */
    public void setToNullDescriptions( boolean local ) {
        if ( local )
            this.vdescriptionsLocal = null;
        else 
            this.vdescriptionsRemote = null;
    }
    
    /**
     * 
     * @return The name of the active splitPanel.
     */
    public String getActivePanel() {
        return active;
    }
    
    
    /**
     * Defines what happens when the "Plugins" button is pressed.
     *
     */
    public void actionPlugins() {
        
        if ( vplugins==null ) {
            vplugins = gui.getSplitPanel().init( gui.getPlugins(), null );
        } else {
            vplugins = gui.getSplitPanel().init( gui.getPlugins(), vplugins );
        }
        active = PLUGINS;
        select( 0 );
    }
    
    /**
     * Defines what happens when the "Description" button is pressed.
     *
     */
    public void actionDescriptionsLocal() {
        
        vdescriptionsLocal = gui.getSplitPanel().init( gui.getLocalDescriptions(), null );
        active = DESCRIPTIONS;
        select( 4 );
    }
    
    /**
     * Defines what happens when the "Remote descriptions" button is pressed.
     *
     */
    public void actionDescriptionsRemote() {
        
        vdescriptionsRemote = gui.getSplitPanel().init( gui.getRemoteDescriptions(), null );
        active = DESCRIPTIONS_REMOTE;
        select( 5 );
    }
    
    /**
     * Defines what happens when the "Remote access" button is pressed.
     */
    public void actionBridges() {
        
        vbridges = gui.getSplitPanel().init( gui.getBridges(), null );
        active = BRIDGES;
        select( 1 );
    }
    
    /**
     * Defines what happens when the "Stat" button is pressed.
     */
    public void actionStat() {
        Vector v = new Vector();
        v.add( FrameworkMain.msdaOutput );
        
        vstat = gui.getSplitPanel().init( v, vstat );
        active = STAT;
        select( 2 );
    }
    
    public void actionDevice() {
        vdevice = gui.getSplitPanel().init( gui.getDevice(), null );
        active = DEVICE;
        select( 3 );
    }
    
    private void select( int index ) {
        
        for ( int i=0; i<items.length; i++ ) {
            if ( i!=index ) 
                items[i].setState( false );
        }
    }
    
    class MenuItemListener implements ItemListener {
        
        public void itemStateChanged(ItemEvent e) {
           Object action = e.getItem();
           if ( PLUGINS.equals( action ) ) {
               actionPlugins();
           } else if ( DESCRIPTIONS.equals( action ) ) {
               actionDescriptionsLocal();
           } else if ( DESCRIPTIONS_REMOTE.equals( action ) ) {
               actionDescriptionsRemote();
           } else if ( BRIDGES.equals( action ) ) {
               actionBridges();
           } else if ( STAT.equals( action ) ) {
               actionStat();
           } else if ( DEVICE.equals( action ) ) {
               actionDevice();
           }
           gui.getToolbarPanel().addTitle( active );
        }
        
    };
    
}
