/*
 * Created on May 16, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.gui.manager;

import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Vector;

import javax.xml.bind.JAXBException;

import org.msda.components.interfaces.PluginInterface;
import org.msda.framework.FrameworkOutput;
import org.msda.gui.ManagerGUI;
import org.msda.msd.DeviceConfig;
import org.msda.msd.InformationComponent;
import org.msda.msd.MSDDescription;
import org.msda.msd.Param;
import org.msda.util.JaxMeUtil;

/**
 * The splitPanel composed of a List and a TextArea.<br>
 * The adding information about the list elements are displayed in the TextArea.
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 */
public class SplitPanel extends Panel {

    private ScrollPane scroll;
    private TextArea textArea;
    private ListItemListener actionList;
    
    private List list;
    private Vector vlist;
    
    private JaxMeUtil jaxmeUtil;
    
    /**
     * Constructor.
     * 
     * @param gui
     */
    public SplitPanel() {
        
        this.jaxmeUtil = new JaxMeUtil();
        
        setBackground( ManagerGUI.COLOR );
        
        textArea = new TextArea(17, 20);
        textArea.setBackground( ManagerGUI.COLOR );
        
        list = new List();
        //list.add( "The manager has been initialized..." );
        actionList = new ListItemListener();
        list.addItemListener( actionList );
        
        scroll = new ScrollPane();
        scroll.add( list );
        scroll.setSize( ManagerGUI.WIDTH-20, 60 );//120 );
        
        // add to the panel
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.fill = GridBagConstraints.HORIZONTAL;
        gridbag.setConstraints( scroll, c);
        
        c.fill = GridBagConstraints.BOTH;
        gridbag.setConstraints( textArea, c);
        
        setLayout(gridbag);
        
        add( scroll );
        add( textArea );
    }
    
    
    /**
     * Initializes the List.
     * @param first
     * @param second
     */
    public Vector init( Vector first, Vector second ) {
        
        if ( first==null ) {
            
            
            list.removeAll();
    	    list.setEnabled(false);
            
    	    list.add( "List null..." );
    	    
    	    list.setEnabled(true);
    	    list.addItemListener( actionList );
            
            return second;
        }
        
        boolean fill = false;
        if ( second==null ) {
            second = new Vector();
            fill = true;
        }
        
        vlist = second;
        
        list.removeAll();
	    list.setEnabled(false);
        
        for( int i=0; i<first.size(); i++) {
            
            Object object = first.get(i);
            
            if ( object instanceof String ) { // to remove
                
                list.add( (String)object );
                if ( fill )
                    vlist.add( object );
                
            } else if ( object instanceof DeviceConfig ) {
                
                DeviceConfig config = (DeviceConfig)object;
                list.add( "Device "+config.getName() );
                
                if ( fill ) {
                    
                    StringBuffer buf = new StringBuffer();
	                buf = buf.append( "Name = ").append( config.getName() );
	                	     
	                if ( !config.getParams().isEmpty() ) {
	                    buf = buf.append( System.getProperty( "line.separator" ) ).append( System.getProperty( "line.separator" ) );
	                    java.util.List list = config.getParams();
	                    for (int j=0;j<list.size(); j++) {
	                        Param param = (Param)list.get(j);
	                        buf = buf.append( param.getType() ).append( " : " ).append( param.getValue() ).append( System.getProperty( "line.separator" ) );
	                    }
	                }
                    vlist.add( buf.toString() );
                }
                
            } else if ( object instanceof PluginInterface ) {
                
                PluginInterface plugin = (PluginInterface)object;
                list.add( plugin.getName() );
                
                if ( fill )
                    vlist.add( plugin.getPluginInfo() );
            
            } else if ( object instanceof MSDDescription ) {
                
                MSDDescription desc = (MSDDescription)object;
                list.add( desc.getSvcDef().getGeneric().getName() );
                
                if ( fill ) {
                    try {
                        vlist.add( jaxmeUtil.toString( desc ) );
                    } catch (JAXBException e) {
                        vlist.add( "Couldn't marshall the description :"+e.getMessage() );
                    }
                }
            } else if ( object instanceof InformationComponent ) {
                
                InformationComponent info = (InformationComponent)object;
                list.add( info.getCompID().getName() );
                if ( fill ) {
	                StringBuffer buf = new StringBuffer();
	                buf = buf.append( "stack ID = ").append( info.getCompID().getName() ).append( System.getProperty( "line.separator" ) )
	                	     .append( "IP address = " ).append( info.getIPAddr() );
	                
	                if ( !info.getParams().isEmpty() ) {
	                    buf = buf.append( System.getProperty( "line.separator" ) ).append( System.getProperty( "line.separator" ) );
	                    java.util.List list = info.getParams();
	                    for (int j=0;j<list.size(); j++) {
	                        Param param = (Param)list.get(j);
	                        buf = buf.append( param.getType() ).append( " : " ).append( param.getValue() ).append( System.getProperty( "line.separator" ) );
	                    }
	                }
	                vlist.add( buf.toString() );
                }

            } else if ( object instanceof FrameworkOutput ) {
                
                FrameworkOutput board = (FrameworkOutput)object;
                if ( board.isCollected() ) {
                    
                    list.add( "Statistical information on "+System.currentTimeMillis() );
                    
                    StringBuffer buf = new StringBuffer();
	                buf = buf.append( "Size of messages in bytes" ).append( System.getProperty( "line.separator" ) )         
	                		 .append( "# messages = " ).append( board.getNbOfReceivedMessages() ).append( System.getProperty( "line.separator" ) )
	                	     .append( "average = " ).append( board.getAvgSizeOfMessages() ).append( System.getProperty( "line.separator" ) ) 
	                		 .append( "min = " ).append( board.getMinSizeOfMessage() ).append( System.getProperty( "line.separator" ) )
	                		 .append( "max = " ).append( board.getMaxSizeOfMessage() );
	                
	                vlist.add( buf.toString() );
	                board.setCollected( false );
                }
            }
        }
        
        list.setEnabled(true);
	    list.addItemListener( actionList );
        
        if ( vlist.size() > 0) {
            textArea.setText( (String)vlist.get( 0 ) );
            list.select( 0 );
        } else {
            textArea.setText( "The list is empty..." );
        }
        
        this.getParent().paintAll( this.getParent().getGraphics() );
        
        return second;
    }
    
   
    /**
     * Class that defines the action when an element of the list is selected. 
     * 
     * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
     */
    class ListItemListener implements ItemListener {
        
        public void itemStateChanged(ItemEvent e) {
            
            // transform into a string
            textArea.setText( (String)vlist.get( list.getSelectedIndex() ) );
            textArea.repaint();
        }
        
    };
}
