/*
 * Created on May 11, 2005
 *
 * @author raverdy
 * Comments, questions: Pierre-Guillaume.Raverdy@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.util;



import java.io.ByteArrayOutputStream;
import javax.xml.soap.SOAPException;

import org.msda.msd.MSDDescription;
import org.msda.msd.UbisecID;
import org.msda.msd.impl.MSDDescriptionImpl;
import org.msda.msd.impl.UbisecIDImpl;


/**
 * All valid types of msd messages are defined in this class.
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 */
public class InfoType {

	public class Component {
		public final static int INVALID = 0;		
		public final static int MANAGER = 1;
		public final static int BRIDGE = 2;
		public final static int DEVICE = 3;
	}

	public class Message {
		public final static int INVALID = 0;		
		public final static int SERVICE_UPDATE 		= 1; // data msg: message returned by remote manager that contains an msd description. can be received by managers/bridges(forward)
		public final static int SERVICE_REQUEST 	= 2; // data msg: message returned by remote manager that contains an msd description. can be received by managers/bridges(forward)
		public final static int REQUEST_RESPONSE 	= 3; // data msg: message returned by remote manager that contains an msd description. can be received by managers/bridges(forward)
		public final static int REMOTE_ACCESS 		= 4; // data msg: contains ServiceAccess content (used for method call and response)
		public final static int REMOTE_ACCESS_OK 	= 5; // data msg: ?? not used, for validating comm. tunnel
		//public final static int REMOTE_ACCESS_ERROR = 6; // data msg: ?? not used, when there is a comm. problem with the communication channel
		public final static int EXIST_MANAGER 		= 7; // ControlMsg: beacon for daemon
		public final static int GET_INFO_COMPONENT 	= 8; // ControlMsg: sent to request info about a device
		public final static int INFO_COMPONENT 		= 9; // ControlMsg: sent info about a device
		public final static int BEACON_MANAGER 		= 10; // ControlMsg: beacon for daemon
		public final static int BEACON_DEVICE 		= 11; // ControlMsg: beacon for daemon
		public final static int BEACON_BRIDGE 		= 12; // ControlMsg: beacon for daemon
		public final static int BRIDGE_ACTIVATION	= 13; // ControlMsg: used by msd manager to activate bridge
		public final static int BRIDGE_DESACTIVATION	= 14; // ControlMsg: used by msd manager to desactivate bridge
		
		public final static int CLIENT_NEW_REQUEST 	= 15; // ClientMsg : client discovery request
		public final static int CLIENT_RUNNING_REQUEST 	= 16; // ClientMsg : client running discovery request
		public final static int CLIENT_RESPONSE 	= 17; // ClientMsg : response to the client
	}
   
	public class Weight 
	{
	    public static final int Energy              = 5;
 	    public static final int TotalMemory         = 1;
	    public static final float StandardTotalMemory = 100000;
	    public static final int FreeMemory          = 2;
	    public static final float StandardFreeMemory  = 100000;
	    public static final int CPUCapacity         = 4;
	    public static final float StandardCPUCapacity = 1000;
	    public static final int CacheSize           = 4;
	    public static final float StandardCacheSize   = 512;
	    public static final int Bandwith            = 3;
	    public static final float StandardBandwith    = 1;
	}

	public class Context {
	    public final static String MAX_USERS = "maxUsers";
	    public final static String NUM_USERS = "numUsers";
	    public final static String NUM_SERVICES = "numServices";
	    
	    public final static int TYPE_STRING = 0;
	    public final static int TYPE_INT = 1;
	    public final static int TYPE_BOOLEAN = 2;
	    public final static int TYPE_TIME = 3;
	}

	public class Config {
	    public final static String GET_SERVICES_TIMEOUT = "getServicesTimeout";
	    public final static String SERVICE_ACCESS_TIMEOUT = "serviceAccessTimeout";
	}
	
	// used to enable testing with partial results (first one)
	public static final boolean TEST_PARTIAL_RESULTS = false;
	// enable/disable the security operations in the comm.
	public static final boolean TEST_SECURITY = false;
	//enable/disable testing for soap call
	public static final boolean TEST_SOAP_CALL = false;
	//enable/disable testing for ctx
	public static final boolean TEST_CTX = true;
	
	
	public static final boolean ACTIVE_SECURITY = false;

	public static final int WAIT_PENDING_REQUEST_CREATION = 20;
	public static final int WAIT_PENDING_REQUEST_FIRST_RESULT = 20;
	public static final int WAIT_TTL_UPDATING = 2000;
	
	// maximum size for incoming multicast messages
	static public final int MSG_BUFFER_SIZE = 30000;

	static public final int DEFAULT_THREADS_MANAGER_POOL = 10;

	static public final int STATE_START = 0;
	static public final int STATE_DEVICE = 1;
	static public final int STATE_MANAGER = 2;

	static public final int STARTUP_TIMEOUT = 3000;
	static public final int BEACON_RECEIVE_TIMEOUT = 12000;
	static public final int BEACON_SEND_TIMEOUT = 5000;

	static public final boolean STACK_PER_DEVICE = false;
	
	static public final long SVC_ACCESS_CACHE_TIMEOUT = 360000;//1H
	

	// static information put in the MSDManager.getServices() to the request...
	static public final int  MAX_HOP = 10;
	
	// Specific MSDDescription to specify the current search
	static public final String RUNNING_REQUEST = "RUNNING_REQUEST";
	static public MSDDescription DUMMY_DESC_RUNNING_REQUEST = new MSDDescriptionImpl();  
	static {
	    UbisecID id = new UbisecIDImpl();
	    id.setName( InfoType.RUNNING_REQUEST );
	    InfoType.DUMMY_DESC_RUNNING_REQUEST.setDescID( id );
	}
	
	// Specific MSDDescription to specify the end of the timeout
	static public final String TIMEOUT_EXPIRED = "TIMEOUT_EXPIRED";
	static public MSDDescription DESC_TIMEOUT_EXPIRED = new MSDDescriptionImpl();  
	static {
	    UbisecID id = new UbisecIDImpl();
	    id.setName( InfoType.TIMEOUT_EXPIRED );
	    InfoType.DESC_TIMEOUT_EXPIRED.setDescID( id );
	}
	
    //
    // Static messages
    //
    
    public static String SOAP_IO_ERROR = "";
    public static String SOAP_PLUGIN_ERROR = "";
    
    static {
	    try {
	        csoap.xml.soap.SOAPMessageImpl soap = new csoap.xml.soap.SOAPMessageImpl();
	        csoap.xml.soap.SOAPBodyImpl body = (csoap.xml.soap.SOAPBodyImpl)soap.getSOAPPart().getEnvelope().getBody();
            csoap.xml.soap.SOAPFaultImpl fault = (csoap.xml.soap.SOAPFaultImpl)body.addFault();
            fault.setFaultString( "I/O Error on the tunnel." ); 
            ByteArrayOutputStream out = new ByteArrayOutputStream(); 
            soap.writeTo( out );
            
            
            SOAP_IO_ERROR = out.toString();
            
        } catch (SOAPException e) {
            //board.info("Couldn't initialize the soap error message...");
            e.printStackTrace();
            System.exit(1);
        }
    }
    
    static {
        try {
            //SOAPMessage soapMessage = csoap.xml.soap.MessageFactoryImpl.newInstance().createMessage();
	        //csoap.xml.soap.SOAPMessageImpl soap = (csoap.xml.soap.SOAPMessageImpl)soapMessage;
            csoap.xml.soap.SOAPMessageImpl soap = new csoap.xml.soap.SOAPMessageImpl();
            csoap.xml.soap.SOAPBodyImpl body = (csoap.xml.soap.SOAPBodyImpl)soap.getSOAPPart().getEnvelope().getBody();
            csoap.xml.soap.SOAPFaultImpl fault = (csoap.xml.soap.SOAPFaultImpl)body.addFault();
            fault.setFaultString( "Plugin not found." ); 
            ByteArrayOutputStream out = new ByteArrayOutputStream(); 
            soap.writeTo( out );
            
            
            SOAP_PLUGIN_ERROR = out.toString();
            
        } catch (SOAPException e) {
            //board.info("Couldn't initialize the soap error message...");
            e.printStackTrace();
            System.exit(1);
        }
    }
//    
//    /**
//     * Error message for a remote service access
//     */
//    public static MSDMessage MESSAGE_RA_ERROR = new MSDMessageImpl();
//    static {
//    	MESSAGE_RA_ERROR.setType( REMOTE_ACCESS_ERROR );
//    }
}
