/*
 * Created on May 16, 2005
 *
 * @author raverdy
 * Comments, questions: Pierre-Guillaume.Raverdy@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import org.msda.msd.MSDDesc;
import org.msda.msd.MSDDescription;
import org.msda.msd.MSDMessage;
import org.msda.msd.impl.MSDDescImpl;
import org.xml.sax.InputSource;

/**
 * Utility class to marshall and unmarshall classes created by JaxMe.
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 */
public class JaxMeUtil {
    private static final String PACKAGE_NAME = "org.msda.msd";
    private JAXBContext context;
	private Marshaller marshaller;
	private Unmarshaller unmarshaller;
	
	/**
	 *  Creates the marshaller and unmarshaller for the default package name. 
	 * 
	 * @param logger	To put traces into a log file.
	 */
	public JaxMeUtil( ) {
	    try {
	    	context = JAXBContext.newInstance( PACKAGE_NAME ,getClass().getClassLoader());
            marshaller = context.createMarshaller();
            unmarshaller = context.createUnmarshaller();
        } catch (JAXBException e) {
        	//board.info( "JaxMeUtil:: Could not create JAXBContext :"+e.getMessage() );
        	e.printStackTrace();
        	System.exit(-1);
        }
	}
	
	/**
	 * Marshalls a MSD Description into a XML string.
	 * 
	 * @param desc	The MSD Description
	 * @return The XML String of the MSD Description 
	 */
	public String toString( MSDDescription desc ) throws JAXBException {
	    // Need to use an element type
	    MSDDesc description = new MSDDescImpl();
	    description.setChecksum( desc.getChecksum() );
	    description.setCtxInfo( desc.getCtxInfo() );
	    description.setCreaInfo( desc.getCreaInfo() );
	    description.setDescID( desc.getDescID() );
	    description.setMetaInfo( desc.getMetaInfo() );
	    description.setPropaInfo( desc.getPropaInfo() );
	    description.setSvcDef( desc.getSvcDef() );
	    
	    StringWriter sw = new StringWriter();
	    marshaller.marshal( description, sw );
	    return sw.toString();
	}
	
	/**
	 * Marshalls a MSD Message into a XML string.
	 * 
	 * @param message	The MSD Message
	 * @return The XML String of the MSD Message
	 */
	public String toString( MSDMessage message ) throws JAXBException {
	    StringWriter sw = new StringWriter();
	    marshaller.marshal( message, sw );
	    return sw.toString();
	}
	
	/**
	 * Unmarshalls a XML string into a MSD Message
	 * 
	 * @param buf
	 * @return
	 * @throws JAXBException
	 */
	public MSDMessage toMSDMessage( byte[] buf ) throws JAXBException {
		StringReader reader = new StringReader( new String(buf) );
		MSDMessage mess = (MSDMessage) unmarshaller.unmarshal( new InputSource( reader ) );
		return mess;
	}
	
	/**
	 * Loads a file containing JAXME xml data.
	 * 
	 * @param filename
	 * @return
	 */
	public Object loadConfigFile (String filename)
	{
		try {
		    File inputFile = new File(filename);
		    InputSource source = new InputSource(new FileInputStream(inputFile));
			source.setSystemId(inputFile.toURL().toString());
			return unmarshaller.unmarshal(source);			
		}
		catch ( IOException e )
		{
			//board.info ("error when reading config file " + e);
			e.printStackTrace();
			return null;
		}
		catch ( JAXBException e )
		{
			//board.info ("error when reading config file " + e);
			e.printStackTrace();
			return null;
		}
	}
}
