/*
 * Created on July 30, 2005
 *
 * @author raverdy
 * Comments, questions: Pierre-Guillaume.Raverdy@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.util;

import java.util.List;

import org.msda.msd.NetworkConfig;
import org.msda.msd.Param;

/**
 * Utility class to manipulate a list of Params.<br>
 * 
 * Usage : ParamsUtil.getInstance(). <br>
 * 
 * TODO - rename the class in ListsUtil
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class ParamsUtil {

    private static ParamsUtil instance = new ParamsUtil();
    
    //	private constructor to prevent from creating instances of this class     
    //private ParamsUtil() {}
    
    /**
     * Returns a static instance of ParamsUtil
     * 
     * @return Returns a static instance of ParamsUtil
     */
    public static ParamsUtil getInstance() {
        return ParamsUtil.instance;
    }
    
	/**
	 * Gets the value corresponding to a type in a list of Params
	 * 
	 * @param type  type name
	 * @param list	list of Params
	 * @return the Value
	 */
    public String getValue( String type, List list ) {
		
		if ( type==null )
			return null;
		
		for( int i=0; i<list.size(); i++) {
			Param param =(Param)list.get(i);
			if ( type.equals( param.getType() ) )
				return param.getValue();
		}
		return null;
	}
    
    /**
     * Updates the value corresponding to a type in a list of Params
     * 
     * @param type		type name
     * @param value		value
     * @param list		list of Params
     */
    public void updateValue( String type, String value, List list ) {
        
        if ( type==null || value==null || list==null )
			return;
        
        for( int i=0; i<list.size(); i++) {
			Param param =(Param)list.get(i);
			if ( type.equals( param.getType() ) ) {
			    param.setValue( value );
			    break;
			}
		}
    }

    /**
     * Gets the NetworkConfig corresponding to a network name in a list of NetworkConfig
     * 
     * @param networkName	the network name
     * @param networks		the list of NetworkConfig
     * @return
     */
    public NetworkConfig findNetworkConfig( String networkName, List networks ) {
        
        for ( int i=0;i<networks.size();i++) {
            NetworkConfig netConfig = (NetworkConfig)networks.get(i);
            if ( networkName.equals( netConfig.getName() ) )
                return netConfig;
        }
        return null;
    }
}
