/*
 * Created on Jun 28, 2005
 *
 * @author raverdy
 * Comments, questions: Pierre-Guillaume.Raverdy@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.util;

import java.util.List;

import org.msda.msd.Hop;
import org.msda.msd.UbisecID;

/**
 * Class to manage the propagation list.
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class PropagationUtil {

    private static final PropagationUtil instance = new PropagationUtil();
    
    //	private constructor to prevent from creating instances of this class  
    private PropagationUtil() {}
    
    /**
     * Returns a static instance of PropagationUtil
     * 
     * @return Returns a static instance of PropagationUtil
     */
    public static PropagationUtil getInstance() {
        return PropagationUtil.instance;
    }
    
    /**
     * Gets the index of the first corresponding hop based on the srcID
     * WARNING : if there is a cycle, maybe a problem ?
     *           no problem only if the hop is removed 
     * 
     * @param srcID
     * @param propagation
     * @return
     */
    public int getNetworkFromSrc( UbisecID srcID, List propagation ) {
     
        for( int i=0; i<propagation.size(); i++) {
            Hop hop = (Hop)propagation.get(i);
            
            if ( MSDUtil.getInstance().compareID( hop.getSrcNetID(), srcID ) ) 
                return i; 
        }
        return -1;
    }
    
    /**
     * Gets the index of the first corresponding hop based on the destID
     * 
     * @param id
     * @param propagation
     * @return
     */
    public int getNetworkFromDest( UbisecID destID, List propagation ) {
        
        for( int i=0; i<propagation.size(); i++) {
            Hop hop = (Hop)propagation.get(i);
            
            if ( MSDUtil.getInstance().compareID( hop.getDestNetID(), destID ) ) 
                return i; 
        }
        return -1;
    }
    
    
    /**
     * Get the destined network of the Hop
     * 
     * @param index
     * @param propagation
     * @return
     */
    public UbisecID getDestNetwork( int index, List propagation ) {
        if ( index<0 )
            return null;
        return ((Hop)propagation.get( index )).getDestNetID();
    }
    
    /**
     * Get the source network of the Hop
     * 
     * @param index
     * @param propagation
     * @return
     */
    public UbisecID getSrcNetwork( int index, List propagation ) {
        if ( index<0 )
            return null;
        return ((Hop)propagation.get( index )).getSrcNetID();
    }
    
    /**
     * Get the last hop of the list 
     * 
     * @param propagation
     * @return
     */
    public Hop getLastHop( List propagation ) {
        if ( propagation.isEmpty() )
            return null;
        return (Hop)propagation.get( propagation.size()-1 );
    }
    
    /**
     * Get the first hop of the list 
     * 
     * @param propagation
     * @return
     */
    public Hop getFirstHop( List propagation ) {
        if ( propagation.isEmpty() )
            return null;
        return (Hop)propagation.get( 0 );
    }
}
