 /*
 * Created on Aug 9, 2004
 *
 * @author raverdy
 * Comments, questions: Pierre-Guillaume.Raverdy@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.util.comm;


import java.net.*;
import java.util.*;
import java.io.*;

import org.msda.framework.FrameworkOutput;
import org.msda.msd.*;
import org.msda.util.InfoType;
import org.msda.util.JaxMeUtil;
import org.msda.util.MSDException;
import org.msda.util.security.SecurityUtil;

import javax.xml.bind.*;



/**
 *
 *	This class implements the group communication used by MSD Bridges and 
 *	MSD Managers to communicate with each others in a network. This class uses
 *	the multicast feature founf in j2se 1.4.2 and above (not okay with j2me for now)
 *	Other implementation should be devised for Java 1.3.1 (for PDA) and/or unicast 
 *	sockets.
 *	<p>
 *	WARNING: problems with (1) static size of messages in contructor; (2) routing issue 229?; 
 *
 * @author raverdy
 * 
 */

public class MulticastGroup extends Thread implements MulticastGroupInterface {

	// addres/port to be used for receiving MSD messages (one socket for all messages)
    protected int groupPort;
    protected String groupAddress;
		
	// only one MSDNetwork component receives the callbacks
	protected MulticastGroupCallback callback;

	// these are the various variables used for the multicast socket
	protected MulticastSocket multicastSock;
	protected InetAddress multicastGroup;
	protected NetworkInterface netif;
	
	protected Listener socketThreads;
		
	protected JaxMeUtil jaxme;

	private boolean doChecksum = false;
	protected FrameworkOutput board = FrameworkOutput.commonBoard();
		
	public static long reveived_message_time = 0;
	public static long sent_message_time = 0;
	public static long marshaling_time = 0;
	public static long marshaling_number = 0;
		
		/**
		 * Creates the multicast group and register the callback interface for messages.
		 * forward them to the callback interface.
		 * <p>
		 * 
		 * @param callback the callback interface where to send incoming messages
		 * @param hostname the hostname to use
		 * @param ifname the name of the network interface to use
		 * @param groupAddress the address to use for the multicast group 
		 * @param groupPort the port to use for the multicast group
		 * @param logger log to print trace
		 * 
		 * @throws MSDException
		 */
		public MulticastGroup ( MulticastGroupCallback callback, String ifname, String groupAddress, int groupPort)  throws MSDException {
			this.doChecksum = false;
			//
			this.groupAddress = groupAddress;
			this.groupPort = groupPort;
			this.callback = callback;

			this.netif = getInterface (ifname);
			if (this.netif == null) {
				Throwable e = new Throwable();
				throw new MSDException ("unkown interface " + ifname, e);
			}
			
			try {
				this.multicastGroup = InetAddress.getByName(groupAddress);

				this.multicastSock = new MulticastSocket(groupPort);
				this.multicastSock.setNetworkInterface(netif);
				// so that the socket can be shared by multiple apps on the same machine
				this.multicastSock.setReuseAddress( true );
				// so that the sender does not receive the message (yes, should be true to be deactivated!)
				// PGR: should make sure this is not a problem when multiple components in the 
				// same process are using the same multicast group
				if (InfoType.STACK_PER_DEVICE)
				if (System.getProperty("java.version").startsWith("1.5"))
				{
					board.info ("JAVA 1.5");
					this.multicastSock.setLoopbackMode(true);
				}
				else
				{
					board.info ("JAVA 1.4?");
					this.multicastSock.setLoopbackMode(false);
				}
					
						
				this.multicastSock.setReceiveBufferSize(65536);
				this.multicastSock.setSendBufferSize(65536);

				this.multicastSock.joinGroup( multicastGroup );

				board.info("Join the multicast group " + multicastGroup + "over " + netif.getDisplayName() + " which has "+ ((InetAddress)netif.getInetAddresses().nextElement()).getHostAddress()+ " port "+groupPort);
			}
			catch (IOException e) {
			    board.notifyError( "MulticastMSDGroup: ERROR with multicast socket " + e.getMessage() );
                board.notifyException(e);
				return;
			}
			catch (Exception e) {
				board.notifyError( "MulticastMSDGroup: ERROR other exception" + e.getMessage() );
                board.notifyException(e);
				return;			
			}
			
			this.jaxme = new JaxMeUtil();
			
			try {
                this.socketThreads = new Listener();
            } catch (JAXBException e) {
                board.notifyError( "MulticastMSDGroup: ERROR while initializing the Listener " + e.getMessage() );
                board.notifyException(e);
            }

		}
	
		public void SetChecksum ( boolean check )
		{
			this.doChecksum = check;
		}
		
		public void SetBoard ( FrameworkOutput board )
		{
		    this.board = board;
		}
       
        
		/**
		 *  Returns the Java address definition based on the interface name provided.
		 * 
		 * @param ifname				name of the network interface to look for		
		 * @return NetworkInterface		interface definition and assigned addresses
		 */
		private NetworkInterface getInterface( String ifname ) {
		    
			NetworkInterface netinterface;
			Enumeration ints;

			try {
				ints = NetworkInterface.getNetworkInterfaces();
			}
			catch (SocketException  e) {		    
                board.notifyException(e);
				return null;
			}
			
			while( ints.hasMoreElements() )
			{
				netinterface =  (NetworkInterface)ints.nextElement();
				if ( ifname.equals(netinterface.getName()) ) {
					return netinterface;
				} 
			}
			return null;
		}
		
		/**
		 * @see java.lang.Object#finalize()
		 **/
		public void finalize () throws Throwable {
			
		    try {
				multicastSock.leaveGroup(this.multicastGroup);
				multicastSock.close();
				socketThreads.terminate();
			} 
			catch (IOException e) {
				board.notifyError( "MulticastGroup::finalize: "+e.getMessage() );
				board.notifyException ( e );
		    } finally {
		        super.finalize();
		    }
		}

		public void terminate () 
		{
		    try {
		    	this.callback = null;
				multicastSock.leaveGroup(this.multicastGroup);
				multicastSock.close();
				multicastSock = null;
				socketThreads.terminate();
			} 
			catch (IOException e) {
				board.notifyError( "MulticastGroup::finalize: "+e.getMessage() );
				board.notifyException ( e );
		    }
		}
		
		
		/**
		 * 
		 * @see org.msda.util.comm.MulticastGroupInterface#sendMSDMessage(org.msda.msd.MSDMessage)
		 */
		public int sendMSDMessage ( MSDMessage mess ) 
		{
			String xml;
			String checksum;
			byte[] msg;
			DatagramPacket beacon;
			
			long start, end;
			
			try {
				// If the MSDMessage has already a checksum, remove it.
				if ( mess.getChecksum()!=null ) 
				    mess.setChecksum( null );
				
				if (InfoType.TEST_SECURITY)
				{
					// ATTNETION: the control of the checksum is not done !! needs to be completed
					// first I marshall the meta description into an XML file
				    xml = this.jaxme.toString( mess );
					// second I generate the checksum, then I "re-marshall" the msd request
				    checksum = SecurityUtil.getInstance( netif.getName() ).createChecksum( xml );
				    mess.setChecksum( checksum );
				}				
	            start = System.currentTimeMillis();
			    xml = this.jaxme.toString( mess );
	            end = System.currentTimeMillis();
	            
				// now I build the beacon from the XML file and send it onto the socket
				msg = xml.getBytes();
				beacon = new DatagramPacket(msg, msg.length, multicastGroup, groupPort);
				multicastSock.send(beacon);
				
	            if ((end-start)>0) {
		            marshaling_number++;
		            marshaling_time += (end-start);
		            //board.info ("info processing current total number average " + (end-start) + " " + marshaling_time + " " + marshaling_number + " " + marshaling_time/marshaling_number);
	            	//board.info ("message sent at time " + System.currentTimeMillis());
	            }
				    
			} catch (IOException e) {
				board.notifyError( "MulticastGroup::sendMSDMessage : IO exception when sending " + e.getMessage() );
				board.notifyException ( e );
			} catch (SecurityException e) {
				board.notifyError( "MulticastGroup::sendMSDMessage : ERROR WHEN GENERATING THE CHECKSUM " + e.getMessage() );
				board.notifyException ( e );
			} catch (JAXBException e) {
				board.notifyError( "MulticastGroup::sendMSDMessage : ERROR WHEN MARSHALLING " + e.getMessage() );
				board.notifyException ( e );
			}
			return 0;
		}
		
		
		/**
		 * NOTE ADLC: previously private and in the Listener class
		 *  
		 * Try to unmarshall the InputSource into a MSDMessage.<br>
		 * TODO If there is a checksum, its value is verified. TO BE DONE
		 * 
		 * @param res
		 * @throws JAXBException - If the checksum is not ok
		 * @throws ClassCastException - If the InputSource is not a MSDMessage
		 */
		void unmarshallMSDMessage( byte[] res ) throws JAXBException {
		    //board.info( new String(res));
			long start, end;
            start = System.currentTimeMillis();
			MSDMessage mess = jaxme.toMSDMessage(res);
            end = System.currentTimeMillis();
            if ((end-start)>0) {
            	marshaling_number++;
            	marshaling_time += (end-start);
            	//board.info ("message received at time " + System.currentTimeMillis());
	            //board.info ("info processing current total number average " + (end-start) + " " + marshaling_time + " " + marshaling_number + " " + marshaling_time/marshaling_number);
            }

	        // forward the message to the communicator
	        if (callback != null) callback.receivedMessage( mess );
	        mess = null;
		}
		

		/**
		 * This is the class that reads msd messages (descriptions, requests, messages) 
		 * from the multicast socket, unmarshall them and send them back the communicator 
		 * using the registered callback interface.
		 * 
		 * @author raverdy
		 */
		class Listener {
			private boolean doSocketLoop;
			private boolean doProcessingLoop;
			private LinkedList receivedMessages;
			private Thread processingThread;
			private Thread socketThread;

			private int currentCounter = 0; // for debug purposes...
			
			
			/**
			 * 
			 *
			 */
			public Listener () throws JAXBException {			
    		    this.receivedMessages = new LinkedList();
        		processingThread = processingLoop();
        		socketThread = socketLoop();
        		processingThread.start();
        		socketThread.start();
        		socketThread.setPriority(Thread.MAX_PRIORITY);
			}

			/**
			 * 
			 *
			 */
			public void terminate () {
				// terminate loops (wake up the processing loop also)
				doProcessingLoop = false;
				doSocketLoop = false;
    			synchronized (receivedMessages) {
    				receivedMessages.notifyAll();	
    			}
			}

			/**
			 * Processing loop for receiving incoming network messages
			 * 
			 * @return
			 */
			public Thread socketLoop() 
			 {
			 	try
				{
			 		Thread socketThread = new Thread( new Runnable() 
					{
					    public void run() 
					    {	
					    	byte[] buf = new byte[InfoType.MSG_BUFFER_SIZE];
					    	byte[] newbuf; 
						    int reqCounter = 0;
						    String str;
						    CharArrayReader reader;
						    doSocketLoop = true;
					    	while (doSocketLoop)
						    {
					    		try {
					    			DatagramPacket recv = new DatagramPacket(buf, buf.length);
					    			multicastSock.receive(recv);
					    			
					    			newbuf = new byte[recv.getLength()];
					    			for (int x=0; x<recv.getLength();x++)
					    			{
					    				newbuf[x] = buf[x];													  
					    			}
					    			synchronized (receivedMessages) {
					    				receivedMessages.addLast(newbuf);
//					    				if ((processingThread.g.getState() == Thread.State.WAITING) || (processingThread.getState() == Thread.State.BLOCKED))
//					    				{
						    				receivedMessages.notify();					    					
//					    				}
					    			}
					    		} catch (IOException e)
								{
					    			board.info ("ERROR when reading socket, exit multicast group");
					    			doSocketLoop = false;
								}
					            
						    }
					    }
					   });	
			 		return socketThread;   
				}
			 	catch (Exception e)
			    {
			 		board.info("Error at create MessageManager.listenProcess Thread");
	                board.notifyException(e);
			    }
			 	return null;
		    }


			 /**
			  * 
			  * Processing loop for incoming MSD messages.
			  * 
			  */
			 public Thread processingLoop() {
		        try
			    {
		        	Thread localProcessingThread = new Thread( new Runnable() 
					{
					    public void run() 
					    {	
					    	int reqCounter = 0;
						    byte newbuf[] = null;
						    boolean gotMessage;
						    doProcessingLoop = true;
					    	while (doProcessingLoop)
						    {
					            synchronized (receivedMessages) {
					            	gotMessage = false;
					            	if (receivedMessages.size()==0)
					            		try {
//							    			board.info ("processing loop waiting for a message");
					            			receivedMessages.wait ();
					            		} catch (InterruptedException e) {
					            		}
					            	else {
//						    			board.info ("taking a message to the list...in cue " + --currentCounter);
						    			newbuf = null;
						    			newbuf = (byte[])receivedMessages.removeFirst();
						            	gotMessage = true;
					            	}
					    		}
					            // now if there was a new message, I unmarshall it and send it to the upper layers...
					            if (gotMessage) {
						            try {
//							            board.info (" request counter " + reqCounter++);
						            	unmarshallMSDMessage( newbuf );
						            }
						        	catch (JAXBException e1)
						            {
						        		if (board!=null) board.notifyError("MulticastGroup::Listener::UpdateLoop : ERROR WHEN UNMARSHALLING " + e1 );
									    e1.printStackTrace();
									    System.exit(-1);
						            }
					            }
//				    			board.info ("exit processing loop");
						    }
				    }
				   });	
				return localProcessingThread;
			    }
		        catch (Exception e)
			    {
		        	board.info("Error at create MessageManager.listenProcess Thread");
	                board.notifyException(e);
				    System.exit(-1);
			    }
		        return null;
		    }
			
		} // listenner class
}
