/*
 * Created on June 16, 2005
 * 
 * @author raverdy
 * Comments, questions: Pierre-Guillaume.Raverdy@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 

package org.msda.util.comm;

import java.net.*;
import java.io.*;

import org.msda.components.plugins.IFUtils;



/**
 * Class to manage unicast network communications.
 *
 * @author Rafik CHIBOUT, delachap
 */

public class NetworkManager 
{ 
    /**
     * 
     * @return
     */
    public String[] getLocalIPAddresses()
    {
        IFUtils ifUtils = new IFUtils();
        return ifUtils.getLocalHostAddresses();
    }
    
    /**
     * Sends a String within a new created socket.
     * 
     * @param message
     * @param address
     * @param port
     * @return
     */
    public Socket sendTo(byte[] message,  InetAddress address, int port ) {
        return sendTo( message, address, port, null );
    }
    
    /**
     * Sends a String within an opened socket.
     * 
     * @param message
     * @param socket
     * @return
     */
    public Socket sendTo(byte[] message, Socket socket) {
        return sendTo( message, null, -1, socket );
    }
    
    /**
     * Sends a String within a socket : <br>
     *   - (1) creates a new Socket with IPAddress and port (socket is null)<br>
     *   - (2) uses an opened Socket
     * 
     * @param message		
     * @param IPAddress		
     * @param port
     * @param socket
     * @return the new Socket (1)
     */
    private Socket sendTo(byte[] message, InetAddress address, int port, Socket socket)
    {
        try
        {
            Socket soc = socket;
            if (soc==null) {
                soc = new Socket(address, port) ;
                //soc.setSoTimeout( TIMEOUT );
            }
            soc.setReuseAddress(true);
            OutputStream outsoc = soc.getOutputStream();
            
            if (message!=null) {
                outsoc.write(message);
                soc.shutdownOutput();
            }
           
            if ( socket!=null ) {
                outsoc.flush();
                outsoc.close();
                soc.close();
            }
            return soc;
        }
        catch (Exception ex)
        {
            //board.info("Error at NetworkManager.sendTo()");
            ex.printStackTrace();
            return null;
        }
    }
    
   
    /**
     * Receives data from an existing socket, then closes the socket. 
     * 
     * @param soc	the socket
     * @return the received byte[]
     */
    public byte[] getReceivedData( Socket socket ) {
        return getReceivedData( socket, true );
    }
    
    /**
     * Receiving data from an open Socket
     * 
     * @param soc	the socket
     * @return the received byte[]
     */
    public byte[] getReceivedData( Socket socket, boolean close )
    {
        byte[] forReturn = new byte[1];
        try
        {
            InputStream socinput = socket.getInputStream();
            socinput.read(forReturn);
            int cp  = 1;
            int num = 0;
            while (socinput.available()>0 || num>0)
            {
                byte[] datatmp = new byte[1024];
                num = socinput.read(datatmp);
                cp = cp+num;
                if (num>0) forReturn = concat(forReturn, datatmp, num);
            }

            if ( close ) {
	            socinput.close();
	            socket.close();
            }
            return forReturn;
        }
        catch(Exception ex)
        {
            //board.info("Error at NetworkManager.getReceivedData");
            ex.printStackTrace();
        }
        return null;
    }
    
    /**
     * Concatenate two byte[]
     * 
     * @param dist
     * @param source
     * @param num
     * @return
     */
    private byte[] concat(byte[] dist , byte[] source, int num)
    {
        if (num > source.length) return dist;
        if (source !=null && dist!=null)
        {
            byte[] result = new byte[dist.length+num];
            for (int i=0; i<dist.length; i++) result[i]= dist[i];
            for (int j=0; j<num; j++) result[j+dist.length]= source[j];
            return result;
        }
        return dist;
    }
}
