/*
 * Created on March 18, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.plugins.ariadne;



import java.util.List;
import java.util.Vector;

import org.msda.components.interfaces.*;
import org.msda.components.plugins.AbstractPlugin;
import org.msda.components.plugins.DiscoveryMode;
import org.msda.components.plugins.Info;
import org.msda.components.plugins.http.HTTPListener;
import org.msda.components.plugins.http.HTTPServer;
import org.msda.components.plugins.msdaservice.MSDService;
import org.msda.msd.*;
import org.msda.msd.impl.MSDRequestImpl;
import org.msda.msd.impl.MethodImpl;
import org.msda.msd.impl.ParamImpl;
import org.msda.msd.impl.ReqDescriptionInformationImpl;
import org.msda.msd.impl.RequestServiceImpl;
import org.msda.plugins.ariadne.SDService.GetServiceData;
import org.msda.plugins.ariadne.SDService.SDBindingStub;

import org.msda.soap.SOAPTools;
import ubisec.util.system.IFUtils;

/**
 * TODO - extends AbstractPlugin !
 * 
 * @author rafik CHIBOUT
 */

public class MSDAriadnePlugin extends AbstractPlugin implements PluginInterface, HTTPListener 
{
	
	org.msda.plugins.ariadne.SDService.SDPortType sd = null;
	private int[] supportedModes = new int[1];
	
	private int currentMode = DiscoveryMode.PASSIVE;
	
	private String[][] parameters = new String[20][2];
	
	private int paramPosition = 0;
	
		
	private TranslatorInterface translator;
	private RequestInterface msdInterface;
	private SOAPTools soapTools = null;
	
	private boolean activate = false;
	
	private MSDService service = null;
	
	private long searchTimeOut = 10000;
		

	/**
	 * 
	 *
	 */
	public MSDAriadnePlugin()
	{
		super();
		translator = new MSDAriadneTranslator();
		
		IFUtils ifUtils = new IFUtils();
		String localIPAddress = ifUtils.getLocalHostAddresses()[0];
		try
		{
			sd = new SDBindingStub("http://"+localIPAddress+":8080/services/SD");
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		supportedModes[0] = DiscoveryMode.PASSIVE;
		info = new Info();
		info.setCreator("Rafik CHIBOUT");
		info.setName("Ariadne");
		info.setProtocolName("Ariadne");
		info.setVersion("0.1");
		info.setDate("18-03-2005");
		info.setDescription("Ariadne plugin for MSD");
		
//		try
//		{
//			soapTools = new SOAPTools("configuration/MSDDeploy.wsdd");
//		}
//		catch(Exception ex)
//		{
//			ex.printStackTrace();
//		}
	}

	
	public static MSDAriadnePlugin getInstance()
	{
		MSDAriadnePlugin forReturn = new MSDAriadnePlugin();
		return forReturn;
	}
	
	public void searchTimeout(int searchID)
	{
	  // not implemented
	}
	
	/**
	 * 
	 * @see org.msda.components.plugins.AbstractPlugin#setTranslator(java.lang.String)
	 */
	public void setTranslator(String className)
	 {
	 	Class translatorClass;
	 	try
		{
	 		translatorClass = Class.forName(className);
	 		translator = (TranslatorInterface)translatorClass.newInstance();
		}
	 	catch(ClassNotFoundException ex)
		{
	 		//logger.error("Class "+className+" not found");
		}
	 	catch(InstantiationException ex)
		{
	 		//logger.error("at "+translatorClass.getName()+" instantiation");
		}
	 	catch(IllegalAccessException ex)
		{
	 		//logger.error("Illegal access to "+translatorClass.getName());
		}
	 }
	 

	/**
	 * 
	 * @see org.msda.components.interfaces.PluginInterface#setMode(int)
	 */
	public void setMode(int mode)
	{
		currentMode = mode;
	}

	
	/**
	 * 
	 * @see org.msda.components.interfaces.PluginInterface#activate()
	 */
	public void activate()
	{
		activate = true;
		try
		{
		//	sd.start();
		//while (!sd.isStarted()) {}
		
		// register the MSDService 
		String dataStr = "";
		
		try
		    {
			java.io.FileInputStream wsdlMSDService = new java.io.FileInputStream("configuration/MSDServiceInterface.wsdl");
						
			while (wsdlMSDService.available() >0)
			    {
					byte[] data = new byte[wsdlMSDService.available()];
					wsdlMSDService.read(data);
					dataStr = dataStr+(new String(data));
			    }
			
		    }
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		
		IFUtils ifUtils = new IFUtils();
		String localIPAddress = ifUtils.getLocalHostAddresses()[0];
		//board.info("Register MSDService at:"+"http://"+localIPAddress+":"+httpServerPort+"/discovery");
		sd.register(dataStr, "http://"+localIPAddress+":"+httpServerPort+"/discovery","","");
		//board.info("MSDService successfully registred");
		
		// 		try to find MSDService
		//		GetServiceData[] res = sd.getServices(dataStr);
		//		if (res != null ) board.info("a service found "+res[0].getServiceEP()) ;
			
		
		HTTPServer server = new HTTPServer();
	    server.setListener(this);
	    server.setHTTPPORT(httpServerPort);
		server.start();
		//logger.info("The Plugin "+this.getInfo().getName() +" is activated");
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	/**
	 * 
	 * @see org.msda.components.interfaces.PluginInterface#terminate()
	 */
	public void terminate()
	{
		activate = false;
		try
		{
		//sd.stop();
		//logger.info("The Plugin "+this.getInfo().getName() +" is stoped");
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	
	/**
	 * 
	 * @see org.msda.components.interfaces.PluginInterface#setNotify(boolean)
	 */
	public boolean setNotify(boolean mode)
	{
		return true;
	}
	
	
	/**
	 * 
	 * @see org.msda.components.interfaces.PluginInterface#doNotify()
	 */
	public boolean doNotify()
	{
		return false;		
	}
	
	
	/**
	 * 
	 * @return
	 */
	public int[] getSupportedModes()
	{
		return supportedModes;
	}
	
	/**
	 * 
	 * @return
	 */
	public int getCurrentMode()
	{
		return currentMode;
	}
	
	
	/**
	 * 
	 * @see org.msda.components.interfaces.PluginInterface#evaluateRequest(org.msda.msd.MSDRequest)
	 */
	public Vector evaluateRequest(MSDRequest request)
	{
		Vector forReturn = new Vector();
		String wsdlDesc = (String)translator.request(request);
		try
		{
						
			// search in Ariadne
			
		
			GetServiceData[] foundServices = sd.getServices(wsdlDesc);
			
			if (foundServices !=null)
			for (int index=0; index<foundServices.length; index++)
			{
				GetServiceData foundService = foundServices[index];
				MSDDescription msdService = (MSDDescription)translator.response(foundService);
				//put this msdService into the msdService Caneva given by the MSD
				MSDDescription msdDescription = callbckInterface.newMSDDescription();
				// WARNING should add conditionnaly only if msdDescription/msdService are not null !!
				//if (msdDescription == null) board.info("msdDescription gived by MSD is null");
				//if (msdService == null) board.info("msdService found is null");
				msdDescription.setSvcDef(msdService.getSvcDef());
				// Add the msdDesc to the forReturn Vector
				forReturn.add(msdDescription);
				// used to test the plugin without MSD
				//forReturn.add(msdService);
			}
			foundServices = null;
		}
		catch(Exception ex)
		{
			//board.notifyException ( ex );
			ex.printStackTrace();
		}
		//logger.info("Ariadne returns  "+forReturn.size()+" services");
		return forReturn;
	}
	
	/**
	 * 
	 * @param args
	 */
	public static void main(String[] args) 
	{
		// NOTE this is a test method for the plugin, should be removed in a final delivery...
		
		MSDAriadnePlugin app = MSDAriadnePlugin.getInstance();
		app.activate();
		
			
		MSDRequest serviceRequest = new MSDRequestImpl();
		ReqDescriptionInformation reqDescription = new ReqDescriptionInformationImpl();
		
		RequestService requestService = new RequestServiceImpl();
		
		requestService.setName("urn:MovieService");
		// Required Methods 
	    List methods = requestService.getMethods();
	    Method method = new MethodImpl();
	    method.setName("getMovies");
	    List paramIn  = method.getIn();
	    List paramOut = method.getOut();
	    
	    /*
	     *  <wsdl:part name="title" type="xsd:string"/>
      <wsdl:part name="actor" type="xsd:string"/>
      <wsdl:part name="director" type="xsd:string"/>
      <wsdl:part name="flag" type="xsd:int"/>
	     */
	    Param param1 = new ParamImpl();
	    param1.setType("title");
	    param1.setValue("string");
	    
	    Param param2 = new ParamImpl();
	    param2.setType("actor");
	    param2.setValue("string");
	    
	    Param param3 = new ParamImpl();
	    param3.setType("director");
	    param3.setValue("string");
	    
	    Param param4 = new ParamImpl();
	    param4.setType("flag");
	    param4.setValue("int");
	    
	    paramIn.add(param1);
	    paramIn.add(param2);
	    paramIn.add(param3);
	    paramIn.add(param4);
	    
	    /*
	     *  <wsdl:part name="movieID" type="xsd:int"/>
      <wsdl:part name="movieName" type="xsd:string"/>
	     */
	    
	    Param param5 = new ParamImpl();
	    param5.setType("movieID");
	    param5.setValue("int");
	    
	    Param param6 = new ParamImpl();
	    param6.setType("movieName");
	    param6.setValue("string");
	    
	    paramOut.add(param5);
	    paramOut.add(param6);
	    
	    methods.add(method);
	        
	    reqDescription.setService(requestService);
	    serviceRequest.setDescInfo(reqDescription);
	    try
		{
	    	Thread.sleep(30000);
	    	//board.info("30s spent");
	    	Thread.sleep(30000);
		}
	    catch (Exception ex)
		{
	    	//board.info("sleep error");
		}
	    
	    //board.info("searching service...");
	    Vector result = null;
	    //Vector result = app.evaluateRequest(serviceRequest);
	    
	    if (result !=null && result.size()>0)
	    {
	    	//board.info("*******msdDescription found out********");
	    	MSDDescription msdService = (MSDDescription)result.elementAt(0);
	    	//board.info("ep="+msdService.getSvcDef().getGeneric().getAddr().getLocation());
	    	//board.info("msdService name="+msdService.getSvcDef().getGeneric().getName());
			methods = msdService.getSvcDef().getGeneric().getMethods();
			int k=0;
			while (k<methods.size())
			{
				method = (Method)methods.get(k); k++;
				//board.info("msdService methodName="+method.getName());
				paramIn = method.getIn();
				paramOut = method.getOut();
				int i=0;
				while (i<paramIn.size())
				{	
					Param param = (Param)paramIn.get(i);i++;
					//board.info("msdService in Param name "+param.getType());
					//board.info("msdService in Param type "+param.getValue());
				}
				i=0;	
				while (i<paramOut.size())
				{
					Param param = (Param)paramOut.get(i);i++;
					//board.info("msdService out Param name "+param.getType());
					//board.info("msdService out Param type "+param.getValue());
				}
			}
			//board.info("*******************************");
	    }
	}

}
