/*
 * Created on March 18, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.plugins.ariadne;

import org.msda.components.interfaces.TranslatorInterface;
import org.msda.components.plugins.Info;
import org.msda.msd.*;
//this removes the ambiguity when building the msd description (import org.msda.msd.Binding;)
import org.msda.msd.Binding;
import org.msda.msd.impl.*;
import org.msda.plugins.ariadne.SDService.GetServiceData;

import org.xml.sax.InputSource;

import java.io.CharArrayReader;
import java.io.CharArrayWriter;
import java.util.Calendar;
import java.util.List;
import java.util.Iterator;

import javax.wsdl.*;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import javax.wsdl.xml.WSDLWriter;
import javax.xml.namespace.QName;

/**
 * @author rafik CHIBOUT
 */

public class MSDAriadneTranslator implements TranslatorInterface{

	Info info = new Info();
	
	public MSDAriadneTranslator()
	{
		info.setName("UbisecMSDOSDP4MTranslator");	
		info.setCreator("Rafik CHIBOUT");
		info.setDate("18/03/2005");
		info.setVersion("0.1");
		info.setDescription("This Translator allows you to translate an OSDP4M Description to a MSD Description and the reverse");
	}
	
	/*
	 * This method returns the creator-given information of the Transaltor  
	 * 
	 */
	public Info getTanslatorInfo()
	{
		return info;
	}
	
	/*
	 * 
	 * this method translates an MSDRequest in a protocole-specific request
	 * 
	 */
	public Object request(Object obj)
	{
		if (obj.getClass().getName().equals(String.class.getName()))
		{
			// XML String processing 
		}
		else
		{
			// Object instance processing
			MSDRequest request = ((MSDRequest)obj);
			ReqDescriptionInformation desc = request.getDescInfo();
			RequestService msdService = desc.getService();
			WSDLFactory wsdlFactory = null;;
			try
			{
				wsdlFactory = WSDLFactory.newInstance();
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}
			Definition def = wsdlFactory.newDefinition();
			
			String msdServiceName=msdService.getName();
			if (msdServiceName.indexOf(":") == -1 ) 
				msdServiceName="urn:"+msdServiceName;
			def.setQName(QName.valueOf(msdServiceName));
			def.setTargetNamespace(msdServiceName);
			def.addNamespace("xsd","http://www.w3.org/2001/XMLSchema");
			def.addNamespace("wsdl","http://schemas.xmlsoap.org/wsdl/");
			def.addNamespace("tns",msdService.getName());
			
			Types types = def.createTypes();
			def.setTypes(types);
			
			PortType portType = def.createPortType();
			
			portType.setQName(QName.valueOf("ServiceInterface"));
										
			List methods = msdService.getMethods();
			Iterator methodsIterator = methods.iterator();
			if (methodsIterator != null)
			while (methodsIterator.hasNext())
			{
				Method method = (Method)methodsIterator.next();
				Operation operation = def.createOperation();
				operation.setName(method.getName());
				Message messageIn = def.createMessage();
				QName messQName = new QName(msdServiceName, operation.getName()+"Req","tns");
				messageIn.setQName(messQName);
				
				Message messageOut = def.createMessage();
				messQName = new QName(msdServiceName, operation.getName()+"Resp","tns");
				messageOut.setQName(messQName);
				List inArguments = method.getIn();
				List outArguments = method.getOut();
				
				Iterator argumentIterator = inArguments.iterator();
				boolean isThereInParam = false;
				boolean isThereOutParam = false;
				while (argumentIterator.hasNext())
				{
					isThereInParam = true;
					Param param = (Param)argumentIterator.next();
					Part part = def.createPart();
					part.setName(param.getType());
					String namespace = def.getNamespace("xsd");
					//board.info("namespace="+namespace);
					QName qName = new QName(namespace, param.getValue(), "xsd");
					part.setTypeName(qName);
					//board.info("part type="+part.getTypeName());
					messageIn.addPart(part);
				}
				
				argumentIterator = outArguments.iterator();
				while(argumentIterator.hasNext())
				{
					isThereOutParam = true;
					Param param = (Param)argumentIterator.next();
					Part part = def.createPart();
					part.setName(param.getType());
					String namespace = def.getNamespace("xsd");
					QName qName = new QName(namespace, param.getValue(), "xsd");
					part.setTypeName(qName);
					//board.info("part type="+part.getTypeName());
					messageOut.addPart(part);
				}
				
				if (isThereInParam) 
				{
					def.addMessage(messageIn);
					Input input = def.createInput();
					input.setMessage(messageIn);
					input.setName(operation.getName()+"Request");
					operation.setInput(input);
				}
				if (isThereOutParam) 
				{
					def.addMessage(messageOut);
					Output output = def.createOutput();
					output.setMessage(messageOut);
					output.setName(operation.getName()+"Response");
					operation.setOutput(output);
				}
				portType.addOperation(operation);
			}
			
			def.addPortType(portType);
			javax.wsdl.Binding binding = def.createBinding();
			def.addBinding(binding);
			
			//wsdl writing 
			StringBuffer wsdlStr = new StringBuffer("<?xml version="+'"'+"1.0"+'"'+" encoding="+'"'+"UTF-8"+'"'+"?>"+'\r');
			// add namespaces
			wsdlStr = wsdlStr.append("<wsdl:definitions name="+'"'+def.getQName()+'"'+" targetNamespace="+'"'+def.getTargetNamespace()+'"'+"\r\n");
			wsdlStr = wsdlStr.append("xmlns:xsd="+'"'+def.getNamespace("xsd")+'"'+"\r\n");
			wsdlStr = wsdlStr.append("xmlns:wsdl="+'"'+def.getNamespace("wsdl")+'"'+"\r\n");
			wsdlStr = wsdlStr.append("xmlns:tns="+'"'+def.getNamespace("tns")+'"'+"\r\n");
			wsdlStr = wsdlStr.append("xmlns="+'"'+"http://schemas.xmlsoap.org/wsdl/"+'"'+">\r\n");
			wsdlStr = wsdlStr.append("\r\n");
			int indexMessages = wsdlStr.length();
			wsdlStr = wsdlStr.append("        \r\n");
			wsdlStr = wsdlStr.append("        \r\n");
			
			portType = (PortType)def.getPortTypes().values().iterator().next();
			
			wsdlStr = wsdlStr.append("\t<wsdl:portType name="+'"'+portType.getQName()+'"'+">\r\n");
			wsdlStr = wsdlStr.append("        \r\n");
			
			Iterator operations = portType.getOperations().iterator();
			if (operations != null)
			while (operations.hasNext())
			{
				Operation operation =(Operation)operations.next();
				wsdlStr = wsdlStr.append("\t\t<wsdl:operation name="+'"'+operation.getName()+'"'+">\r\n");
				Input input = operation.getInput();
				Output output = operation.getOutput();
					
				if (input !=null) 
				{
					Message messageIn = input.getMessage();
					if (messageIn != null)
					{
						wsdlStr = wsdlStr.append("\t\t\t<wsdl:input name="+'"'+input.getName()+'"'+" message="+'"'+messageIn.getQName().getPrefix()+":"+messageIn.getQName().getLocalPart()+'"'+"/>\r\n");
						StringBuffer wsdlMessage = new StringBuffer("\t<wsdl:message name="+'"'+messageIn.getQName().getLocalPart()+'"'+">\r\n");
						Iterator inParts = messageIn.getParts().values().iterator();
						while (inParts.hasNext())
						{
							Part part = (Part)inParts.next();
							wsdlMessage = wsdlMessage.append("\t\t<wsdl:part name="+'"'+part.getName()+'"'+" type="+'"'+part.getTypeName().getPrefix()+":"+part.getTypeName().getLocalPart()+'"'+"/>\r\n");
						}
						wsdlMessage = wsdlMessage.append("\t</wsdl:message>");
						wsdlMessage.append("           \r\n");
						wsdlMessage.append("           \r\n");
						wsdlStr.insert(indexMessages, wsdlMessage.toString());
					}	
				}
				if (output!=null)
				{
					Message messageOut = output.getMessage();	
					if (messageOut !=null)
					{
						wsdlStr = wsdlStr.append("\t\t\t<wsdl:output name="+'"'+output.getName()+'"'+" message="+'"'+messageOut.getQName().getPrefix()+":"+messageOut.getQName().getLocalPart()+'"'+"/>\r\n");
						StringBuffer wsdlMessage = new StringBuffer("\t<wsdl:message name="+'"'+messageOut.getQName().getLocalPart()+'"'+">\r\n");
						Iterator outParts = messageOut.getParts().values().iterator();
						while (outParts.hasNext())
						{
							Part part = (Part)outParts.next();
							wsdlMessage = wsdlMessage.append("\t\t<wsdl:part name="+'"'+part.getName()+'"'+" type="+'"'+part.getTypeName().getPrefix()+":"+part.getTypeName().getLocalPart()+'"'+"/>\r\n");
						}
						wsdlMessage = wsdlMessage.append("\t</wsdl:message>");
						wsdlMessage.append("           \r\n");
						wsdlMessage.append("           \r\n");
						wsdlStr.insert(indexMessages, wsdlMessage.toString());
					}
				}
				wsdlStr = wsdlStr.append("\t\t</wsdl:operation>\r\n");
			}
				
			wsdlStr = wsdlStr.append("\t</wsdl:portType>\r\n");
			wsdlStr = wsdlStr.append("</wsdl:definitions>\r\n");
			/// write into String
			WSDLWriter wsdlWriter = wsdlFactory.newWSDLWriter();
			CharArrayWriter  array = new CharArrayWriter();
						
			try
			{
				wsdlWriter.writeWSDL(def, array);
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}
			
			return wsdlStr.toString();
		}
		return null;
	}
	
	
	/*
	 * this method translates a protocole-specific response in an MSDDescription
	 */
	public Object response(Object obj)
	{
		if (obj.getClass().getName().equals(String.class.getName()))
		{
			// XML String processing 
			// not yet implemented
		}
		else
		{
		if (obj.getClass().getName().equals(org.msda.plugins.ariadne.SDService.GetServiceData.class.getName()))
		{
			// UPnPDevice Casting
			Definition def = null;
			GetServiceData serviceDes = (GetServiceData)obj;
			String wsdlDocument = serviceDes.getWsdlDocument();
			
			
			if (wsdlDocument.indexOf("<ServiceDiscovery>") !=-1 && wsdlDocument.indexOf("</ServiceDiscovery>")!=-1)
			{
				String QoS = wsdlDocument.substring(wsdlDocument.indexOf("<ServiceDiscovery>"), wsdlDocument.indexOf("</ServiceDiscovery>")+19);
				wsdlDocument = wsdlDocument.substring(QoS.length());
			}
			
			try
			{
				WSDLFactory wsdlFactory = WSDLFactory.newInstance();
				WSDLReader wsdlReader = wsdlFactory.newWSDLReader();
				wsdlReader.setFeature("javax.wsdl.verbose", false);
				wsdlReader.setFeature("javax.wsdl.importDocuments", false);
				CharArrayReader wsdlArray = new CharArrayReader(wsdlDocument.toCharArray());
				InputSource wsdlSource = new InputSource(wsdlArray);
				def =  wsdlReader.readWSDL(null, wsdlSource);
			}
			catch(Exception ex)
			{
				ex.printStackTrace();
			}
			
			MSDDescription msdService = new MSDDescriptionImpl();
			ServiceDefinition msdServiceDefinition = new ServiceDefinitionImpl();
			Original msdOriginal = new OriginalImpl();
			msdOriginal.setProtocolName("Ariadne");
			msdOriginal.setDesc("Optimized Service Discovery for MANET");
			Generic msdGeneric = new GenericImpl();
			// set all the infos that are possible
			
			msdGeneric.setName(def.getQName().getLocalPart());
			
            // address
			Address address = new AddressImpl();
			address.setLocation(serviceDes.getServiceEP());
			msdGeneric.setAddr(address);
			
			// binding 
			Binding binding = new BindingImpl();
			binding.setEncoding("XML 1.0");
			binding.setTransport("HTTP/SOAP");
			msdGeneric.setBinding(binding);
				
			// attributes
			Param attribute = new ParamImpl();
			List attributes = msdGeneric.getParams();
			attributes.add(attribute);
			Iterator portTypes = def.getPortTypes().values().iterator();
			if( portTypes.hasNext())
			{
				PortType portType = (PortType)portTypes.next();
				Iterator operations = portType.getOperations().iterator();
				List methods = msdGeneric.getMethods();
								
				while (operations.hasNext())
				{
					Operation operation =(Operation)operations.next();
					Method method = new MethodImpl();
					method.setName(operation.getName());
					
					Input input = operation.getInput();
					Output output = operation.getOutput();
					
					if (input !=null)
					{
					Message messageIn = input.getMessage();
										
					Iterator inParts = messageIn.getParts().values().iterator();
										
					List inArguments = method.getIn();
										
					if (inParts.hasNext())
					while (inParts.hasNext())
					{
						Part part = (Part)inParts.next();
						Param param = new ParamImpl();
						param.setType(part.getName());
						param.setValue(part.getTypeName().getLocalPart());
						inArguments.add(param);
					}
					else 
					{
					    //when the isn't any input parameters
						Param param = new ParamImpl();
						inArguments.add(param);	
					}
					}
					if (output!=null)
					{
					
						Message messageOut = output.getMessage();
						Iterator outParts = messageOut.getParts().values().iterator();
						List outArguments = method.getOut();
						
						if (outParts.hasNext())
						while (outParts.hasNext())
						{
							Part part = (Part)outParts.next();
							Param param = new ParamImpl();
							param.setType(part.getName());
							param.setValue(part.getTypeName().getLocalPart());
							outArguments.add(param);
						}
						else 
						{
							// when the isn't any output parameters
							Param param = new ParamImpl();
							outArguments.add(param);	
						}
					}	
						
					methods.add(method);
				}
				
				// creation time
				Calendar currentTime = Calendar.getInstance();
				currentTime.setTimeInMillis(System.currentTimeMillis());
				msdGeneric.setCreaTime(currentTime);
				
				//Expected life time
				Calendar expectedLifeTime = Calendar.getInstance();
				expectedLifeTime.setTimeInMillis(System.currentTimeMillis()+30000);
				msdGeneric.setExpectedLifetime(expectedLifeTime);
				
				msdServiceDefinition.setOriginal(msdOriginal);
				msdServiceDefinition.setGeneric(msdGeneric);
				msdService.setSvcDef(msdServiceDefinition);
				return msdService;
				}
		}
		}
		return null;
	}
	
	/*
	 * this method translates a protocol-specific response with
	 * ContextInformation in an MSDDescription
	 */
	public Object response(Object response, ContextInformation context){
	    return null;
	}


	
}
