/*
 * Created on July 19, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.plugins.osgi;

import java.math.BigInteger;
import java.util.Vector;

import javax.xml.bind.JAXBContext;

import org.gateway.home.wsSupport.osgi.bundles.axisBundle.WebServicesAvailableInfo;
import org.msda.components.interfaces.TranslatorInterface;
import org.msda.msd.MSDDescription;
import org.msda.msd.MSDMessage;
import org.msda.msd.MSDRequest;
import org.msda.msd.MessageContent;
import org.msda.msd.UbisecID;
import org.msda.msd.impl.MSDMessageImpl;
import org.msda.msd.impl.MessageContentImpl;
import org.msda.msd.impl.UbisecIDImpl;
import org.msda.util.InfoType;
import org.msda.util.comm.MulticastGroup;
import org.msda.util.comm.MulticastGroupCallback;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;


public class MSDOSGIBundle implements MulticastGroupCallback {
	
	TranslatorInterface translator = new MSDOSGITranslator();
	private MulticastGroup mcgroup;
	private BundleContext bundleContext = null;
	JAXBContext ctx;
	
	public MSDOSGIBundle()
	{
		try
		{
			 ctx = JAXBContext.newInstance("org.msda.msd", getClass().getClassLoader());
			 mcgroup = new MulticastGroup(this, "eth0","239.255.255.85", 8585);
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	
	public void receivedMessage(MSDMessage message) {
		if (message.getType().intValue() == InfoType.Message.SERVICE_REQUEST)
		{
         Vector result = evaluateRequest(message.getContent().getRequest());
         if (result != null && result.size()>0)
         {
         	
        	 MSDDescription msdService = (MSDDescription)result.elementAt(0);
        	 MSDMessage messageResponse = new MSDMessageImpl();
        	 MessageContent messageContent = new MessageContentImpl();
        	 messageContent.setDesc(msdService);
        	 messageResponse.setContent(messageContent);
        	 messageResponse.setType(BigInteger.valueOf(InfoType.Message.REQUEST_RESPONSE));
        	 UbisecID ubisecId = new UbisecIDImpl();
        	 ubisecId.setName("MSDOSGIBundle");
        	 messageResponse.setSender(ubisecId);
        	 mcgroup.sendMSDMessage(messageResponse);
         }
		}
		
	}
	
	
	public void setBundleContext(BundleContext context)
	{
		bundleContext = context;
	}
	
	public Vector evaluateRequest(MSDRequest request)
	{
		Vector forReturn = new Vector();
		// not necessary because we cannot use it to search services in AxisBundle, AxisBundle provides only
		// a function that returns all registered web services
		
		OSGIRequest serviceDesc = (OSGIRequest)translator.request(request);
		
		try
		{
			//Search OCDL service in OSGi registry
			
			// search only for web services, by using AxisBundle
			Object[] foundServices = null;
			try
			{
				//find the AxisBundleInfo Reference
				ServiceReference infoRef = bundleContext
				.getServiceReference("org.gateway.home.wsSupport.osgi.bundles.axisBundle.WebServicesAvailableInfo");

				WebServicesAvailableInfo _telventInfo = (WebServicesAvailableInfo) (bundleContext
				.getService(infoRef));
				foundServices = _telventInfo.getPublishServicesList();
				for (int i=0; i<foundServices.length;i++)
				{
					Object servicio = foundServices[i];
					if (servicio.toString().equalsIgnoreCase("urn:control-dispositivos"))
					{
						//board.info("The OSGi/OCDL Service has been found");
						//board.info(servicio.toString());
						
						String wsdl = _telventInfo.getServiceWsdlDescripcion(servicio);
			    		//board.info(wsdl);
			    		
						MSDDescription msdService = (MSDDescription)translator.response(wsdl);
						String serviceName = msdService.getSvcDef().getGeneric().getName();
						if (serviceName==null)
						{
							serviceName = servicio.toString(); 
							msdService.getSvcDef().getGeneric().setName(serviceName);
						}
						//board.info(serviceName);
						if (serviceName != null  && serviceName.equals(request.getDescInfo().getService().getName()))
						{
							forReturn.add(msdService);
						}
						break;
					}
				}
				
				bundleContext.ungetService(infoRef);	
		
			} catch(Exception e) {
				//board.info("Can't retrieve the lis of webservices. "+e.getMessage());
			}

			foundServices = null;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		return forReturn;
	}

	
	public void terminate()
	{
	 mcgroup.terminate();
	}


}
