/*
 * Created on July 19, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.plugins.osgi;


import java.math.BigInteger;
import java.util.*;

import org.msda.components.interfaces.PluginInterface;
import org.msda.components.plugins.AbstractPlugin;
import org.msda.components.plugins.DiscoveryMode;
import org.msda.components.plugins.Info;
import org.msda.msd.*;
import org.msda.msd.impl.CreatorImpl;
import org.msda.msd.impl.MSDMessageImpl;
import org.msda.msd.impl.MSDRequestImpl;
import org.msda.msd.impl.MessageContentImpl;
import org.msda.msd.impl.PropagationInformationImpl;
import org.msda.msd.impl.ReqCreationInformationImpl;
import org.msda.msd.impl.ReqDescriptionInformationImpl;
import org.msda.msd.impl.ReqProcessingInformationImpl;
import org.msda.msd.impl.RequestServiceImpl;
import org.msda.msd.impl.UbisecIDImpl;
import org.msda.util.InfoType;
import org.msda.util.MSDException;
import org.msda.util.comm.MulticastGroup;
import org.msda.util.comm.MulticastGroupCallback;





/*
 * @author 
 */

public class MSDOSGIPlugin extends AbstractPlugin implements PluginInterface,  MulticastGroupCallback
{
	public static String OCDL_SERVICE_URL="http://localhost:8080/axis/services/OCDLService";
	private int[] supportedModes = new int[1];
	private int currentMode      = DiscoveryMode.PASSIVE;
	private Vector responses = new Vector();
	
	public static void main(String[] args)
	{
		MSDOSGIPlugin plugin = new MSDOSGIPlugin();
		
		
		UbisecID id = new UbisecIDImpl();
        id.setName( "ClientOCDL" );
        
        MSDRequestImpl msdRequest = new MSDRequestImpl();
        
        ReqProcessingInformation rpi = new ReqProcessingInformationImpl();
        ReqCreationInformation rci = new ReqCreationInformationImpl();
        Creator creator = new CreatorImpl();
        creator.setMgrID( id );
        creator.setNetID( id );
        Calendar cal = Calendar.getInstance();
        rci.setDate( cal );
	    rci.setTime( cal );
	    rci.setLifetime( BigInteger.valueOf( 5 ) );// in secondes
	    rci.setCreator( creator );
	    rpi.setCrea( rci );
        
        RequestService requestService = new RequestServiceImpl();
        
        //name of the service I am searching
        requestService.setName("urn:control-dispositivos");
        
        ReqDescriptionInformation reqDescription = new 
		ReqDescriptionInformationImpl();
        reqDescription.setService(requestService);
        msdRequest.setDescInfo(reqDescription);
        msdRequest.setProcInfo( rpi );
        
        PropagationInformation pi = new PropagationInformationImpl();
        msdRequest.setPropaInfo( pi );
        
        msdRequest.setClientID( BigInteger.valueOf( -1 ) );
        msdRequest.setReqID( BigInteger.valueOf( -1 ) );
		
		try
		{
			Vector v = plugin.getServicesFromRemoteOSGIGetways(msdRequest);
			if ((v!=null)&&(v.size()>0))
			{
				MSDDescription desc = (MSDDescription)v.get(0);
				ServiceDefinition serv = desc.getSvcDef();
				serv.getGeneric().getName();
				//board.info("OCDL Service: '" + serv.getGeneric().getName()+"'");
				//board.info("Methods:");
				List methods = serv.getGeneric().getMethods();
				for (int i=0;i<methods.size();i++)
				{
					Method metodo = (Method)methods.get(i);
					//board.info("\t" + metodo.getName());
					
				}
			}
			//else board.info("No OCDL services detected");
		}
		catch(MSDException e) {e.printStackTrace();}
		
	}
	
	/**
	 * 
	 *
	 */
	public MSDOSGIPlugin()
	{
		super();
		//translator = new MSDOSGITranslator();

		// create the OCDL/OSGi Instance
		// to complete...

		supportedModes[0]=DiscoveryMode.PASSIVE;
		info = new Info();
		info.setCreator("Pierre-Guilaume Raverdy");
		info.setName("OSGI");
		info.setProtocolName("OSGI");
		info.setVersion("1.0");
		info.setDate("21-07-2005");
		info.setDescription("OSGI plugin for MSD");
		
		
	}
	

	/**
	 * 
	 * @return
	 */
	public static MSDOSGIPlugin getInstance()
	{
		MSDOSGIPlugin forReturn = new MSDOSGIPlugin();
		return forReturn;
	}
	
		 
	/* PluginInterface interface methods */ 
	
	
	 public void searchTimeout(int searchID)
	 {
	
	 }
	
	public void setMode(int mode)
	{
		currentMode = mode;
	}
	
	
	
	public void activate()
	{
		// do any OCDL/OSGi activation....
		
		super.activate();
		activate = true;
	}
	
	public void terminate()
	{
		// do any OCDL/OSGi desactivation....

		activate = false;
	}
	
	public boolean setNotify(boolean mode)
	{
		return true;
	}
	
	public boolean doNotify()
	{
		return false;		
	}
	
	public int[] getSupportedModes()
	{
		return supportedModes;
	}
	
	public int getCurrentMode()
	{
		return currentMode;
	}
	
	
	public void receivedMessage(MSDMessage message) {
		// TODO Auto-generated method stub
		if (message.getType().intValue() == InfoType.Message.REQUEST_RESPONSE)
		{
            responses.add(message.getContent().getDesc());			
		}
		
	}
	
	public Vector getServicesFromRemoteOSGIGetways(MSDRequest request) throws MSDException
	{
		responses = new Vector();
		MulticastGroup mcgroup = new MulticastGroup(this, "eth0","239.255.255.85", 8585);
		MSDMessage message = new MSDMessageImpl();
		MessageContent messageContent = new MessageContentImpl();
		messageContent.setRequest(request);
		message.setContent(messageContent);
		message.setType(BigInteger.valueOf(InfoType.Message.SERVICE_REQUEST));
		UbisecID ubisecId = new UbisecIDImpl();
		ubisecId.setName("MSDOSGIPlugin");
		message.setSender(ubisecId);
		mcgroup.sendMSDMessage(message);
		// wait
		try
		{
			Thread.sleep(2000);
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		// stop mcgroup
		mcgroup.terminate();
		return responses;
	}
	
	public Vector evaluateRequest(MSDRequest request)
	{
		// send Request message to all the OSGI getwaies that have MSDOSGIBundle
		try
		{
			return getServicesFromRemoteOSGIGetways(request);
		}
		catch (MSDException ex)
		{
			ex.printStackTrace();
		}
		return null;
	}


	


}

