/*
 * Created on July 19, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.plugins.osgi;

import org.msda.components.interfaces.TranslatorInterface;
import org.msda.components.plugins.Info;
import org.msda.msd.*;
import org.msda.msd.impl.*;
import org.osgi.framework.ServiceReference;
import org.xml.sax.InputSource;

import java.io.CharArrayReader;
import java.math.BigInteger;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import javax.wsdl.Definition;
import javax.wsdl.Input;
import javax.wsdl.Message;
import javax.wsdl.Operation;
import javax.wsdl.Output;
import javax.wsdl.Part;
import javax.wsdl.PortType;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import javax.xml.namespace.QName;





/*
 * @author 
 */
public class MSDOSGITranslator implements TranslatorInterface{
	Info info = new Info();
	private String name= "OCDL Bundle";
  	private String creator = "TID-INRIA"; 
	
	
	/**
	 * 
	 */
	public MSDOSGITranslator()
	{
		info.setName("UbisecMSDOCDLTranslator");	
		info.setCreator("Pierre-Guillaume Raverdy");
		info.setDate("21/07/2005");
		info.setVersion("0.1");
		info.setDescription("This Translator allows you to translate an OSGi/OCDL Description to a MSD Description and the reverse");
		
	}
	
	/*
	 * This method returns the creator-given information of the Transaltor  
	 * 
	 */
	public Info getTanslatorInfo()
	{
		return info;
	}
	
	/*
	 * 
	 * this method translates an MSDRequest in a protocole-specific request
	 * 
	 */
	public Object request(Object obj)
	{
		if (obj.getClass().getName().equals(String.class.getName()))
		{
			// XML String processing 
		}
		else
		{
			// Object instance processing
			MSDRequest request = (MSDRequest)obj;
			ReqDescriptionInformation desc = request.getDescInfo();
			RequestService msdService = desc.getService();
			
			OSGIRequest ocdlRequest = new OSGIRequest();
			ocdlRequest.setDeviceType(msdService.getName());
			
			// add by Rafik
			
			List params = msdService.getParams();
			if (params!=null)
				for (int i=0; i<params.size(); i++)
				{
				  	Param param = (Param)params.get(i);
				  	ocdlRequest.addParameter(param.getType(),param.getValue());
				}
			return ocdlRequest;
		}
		return null;
	}
	
	/*
	 * this method translates a protocole-specific reponse in an MSDDescription
	 */
	public Object response(Object obj)
	{
		if (obj.getClass().getName().equals(String.class.getName()))
		{
			// WSDL String processing 
			Definition def = null;
			String wsdlDocument = (String)obj;
			if (wsdlDocument.indexOf("<ServiceDiscovery>") !=-1 && wsdlDocument.indexOf("</ServiceDiscovery>")!=-1)
				{
					String QoS = wsdlDocument.substring(wsdlDocument.indexOf("<ServiceDiscovery>"), wsdlDocument.indexOf("</ServiceDiscovery>")+19);
					wsdlDocument = wsdlDocument.substring(QoS.length());
				}
				
			try
			{
				WSDLFactory wsdlFactory = WSDLFactory.newInstance();
				WSDLReader wsdlReader = wsdlFactory.newWSDLReader();
				wsdlReader.setFeature("javax.wsdl.verbose", false);
				wsdlReader.setFeature("javax.wsdl.importDocuments", false);
				CharArrayReader wsdlArray = new CharArrayReader(wsdlDocument.toCharArray());
				InputSource wsdlSource = new InputSource(wsdlArray);
				def =  wsdlReader.readWSDL(null, wsdlSource);
			}
			catch(Exception ex)
			{
				ex.printStackTrace();
			}
				
			MSDDescription msdDescription = newMSDDescription();
			
//         	put this msdService into the given msdServiceCaneva by the MSD
//         	msdDescription = callbckInterface.newMSDDescription();
//        	CreationInformation ci = msdDescription.getCreaInfo(); 
			Plugin plugin = new PluginImpl();
         	plugin.setName( name );
         	plugin.setCreator( creator );
         	CreationInformation ci = msdDescription.getCreaInfo();
         	ci.setPlugin(plugin);
         	msdDescription.setCreaInfo(ci);
         	
     /*    	UbisecIDImpl id = new UbisecIDImpl();
         	id.setName(name);
         	msdDescription.setDescID(id);
*/

			ServiceDefinition msdServiceDefinition = new ServiceDefinitionImpl();
			Original msdOriginal = new OriginalImpl();
			msdOriginal.setProtocolName("OCDL");
			msdOriginal.setDesc("OCDL-OSGI Services");
			Generic msdGeneric = new GenericImpl();
			// set all the infos that are possible
			
			QName qname = def.getQName();
			if (qname!=null) msdGeneric.setName(qname.getLocalPart());
			

			// address
			// Specified in the plugin, Only the plugin know the URL of OCDL  			
			Address address = new AddressImpl();
		 	address.setLocation(MSDOSGIPlugin.OCDL_SERVICE_URL);
		 	msdGeneric.setAddr(address);
		 	
			// binding 
			Binding binding = new BindingImpl();
			binding.setEncoding("OSGI");
			binding.setTransport("HTTP/SOAP");
			
			
			msdGeneric.setBinding(binding);
					
			// attributes
			Param attribute = new ParamImpl();
			List attributes = msdGeneric.getParams();
			attributes.add(attribute);
			Iterator portTypes = def.getPortTypes().values().iterator(); 
			if( portTypes.hasNext())
			{
				PortType portType = (PortType)portTypes.next();
				Iterator operations = portType.getOperations().iterator();
				List methods = msdGeneric.getMethods();
				
				while (operations.hasNext())
				{
					Operation operation =(Operation)operations.next();
					Method method = new MethodImpl();
					method.setName(operation.getName());
					//board.info("\t" + operation.getName());
					
					Input input = operation.getInput();
					Output output = operation.getOutput();
					
					if (input !=null)
					{
						Message messageIn = input.getMessage();
						
						Iterator inParts = messageIn.getParts().values().iterator();
											
						List inArguments = method.getIn();
											
						if (inParts.hasNext())
						while (inParts.hasNext())
						{
							Part part = (Part)inParts.next();
							Param param = new ParamImpl();
							param.setValue(part.getName());
							param.setType(part.getTypeName().getLocalPart());
							inArguments.add(param);
						}
						else 
						{
						    //when the isn't any input parameters
							Param param = new ParamImpl();
							inArguments.add(param);	
						}
						}
						if (output!=null)
						{
						
							Message messageOut = output.getMessage();
							Iterator outParts = messageOut.getParts().values().iterator();
							List outArguments = method.getOut();
							
							if (outParts.hasNext())
							while (outParts.hasNext())
							{
								Part part = (Part)outParts.next();
								Param param = new ParamImpl();
								param.setValue(part.getName());
								param.setType(part.getTypeName().getLocalPart());
								outArguments.add(param);
							}
							else 
							{
								// when the isn't any output parameters
								Param param = new ParamImpl();
								outArguments.add(param);	
							}
						}	
							
						methods.add(method);
					}
					
					// creation time
					Calendar currentTime = Calendar.getInstance();
					currentTime.setTimeInMillis(System.currentTimeMillis());
					msdGeneric.setCreaTime(currentTime);
					
					//Expected life time
					Calendar expectedLifeTime = Calendar.getInstance();
					expectedLifeTime.setTimeInMillis(System.currentTimeMillis()+30000);
					msdGeneric.setExpectedLifetime(expectedLifeTime);
					
					msdServiceDefinition.setOriginal(msdOriginal);
					msdServiceDefinition.setGeneric(msdGeneric);
					msdDescription.setSvcDef(msdServiceDefinition);
					return msdDescription;
					}
		}
		else
		{
			if (obj.getClass().getName().equals(ServiceReference.class.getName()))
			{
				// OSGI Casting
				ServiceReference serviceDes = (ServiceReference)obj;
								
				MSDDescription msdService = new MSDDescriptionImpl();
				ServiceDefinition msdServiceDefinition = new ServiceDefinitionImpl();
				Original msdOriginal = new OriginalImpl();
				msdOriginal.setProtocolName("OCDL");
				msdOriginal.setDesc("OCDL/OSGi service");
				Generic msdGeneric = new GenericImpl();

				// set all the infos that are possible
				msdGeneric.setName(serviceDes.getClass().getName());
				
				// address
				Address address = new AddressImpl();
				// Specified in the plugin, Only the plugin know the URL of OCDL  
			 	address.setLocation(MSDOSGIPlugin.OCDL_SERVICE_URL);
			
				msdGeneric.setAddr(address);
				
				// binding 
				Binding binding = new BindingImpl();
				binding.setEncoding("");
				binding.setTransport("TPC/IP");
				msdGeneric.setBinding(binding);
				
				// attributes
				
				List attributes = msdGeneric.getParams();
				String[] propertykeys = serviceDes.getPropertyKeys();
				if (propertykeys != null)
				for(int j=0;j<propertykeys.length;j++)
				{
					Param attribute = new ParamImpl();
					String prop = (String)serviceDes.getProperty(propertykeys[j]);
					attribute.setType(propertykeys[j]);
					attribute.setValue(prop);
					attributes.add(attribute);
				}
				
				// creation time
				Calendar currentTime = Calendar.getInstance();
				currentTime.setTimeInMillis(System.currentTimeMillis());
				msdGeneric.setCreaTime(currentTime);
				
				//Expected life time
				Calendar expectedLifeTime = Calendar.getInstance();
//				expectedLifeTime.setTimeInMillis(System.currentTimeMillis()+serviceDes.getLifetime());
				msdGeneric.setExpectedLifetime(expectedLifeTime);
				 
				msdServiceDefinition.setOriginal(msdOriginal);
				msdServiceDefinition.setGeneric(msdGeneric);
				msdService.setSvcDef(msdServiceDefinition);
				return msdService;
			}
		}
		
		return null;
	}
	
	/*
	 * this method translates a protocol-specific response with
	 * ContextInformation in an MSDDescription
	 */
	public Object response(Object response, ContextInformation context){
	    return null;
	}
	
	
	public MSDDescription newMSDDescription ()
	{
		CreationInformation newCreation;
		Calendar rightNow;
		Creator newCreator;
		MSDDescription newDescr;
	
		UbisecIDImpl id = new UbisecIDImpl();
		id.setName(name);
	
	//	 create a new empty msd description with a unique id
	//	 WARNING: should create correctly otherwise it may not conform to the schema (all required elements should be there)
		newDescr = new MSDDescriptionImpl();
		newDescr.setDescID(id);
	
		newCreation = new CreationInformationImpl();
		newCreator = new CreatorImpl();
	
		rightNow = Calendar.getInstance();
	
		newCreator.setDvcID(id); 
		newCreator.setNetID(id); 
		newCreator.setMgrID(id); 
		newCreation.setCreator(newCreator);
		newCreation.setCreaDate(rightNow);
		newCreation.setCreaTime(rightNow);
		newCreation.setUpdDate(rightNow);
		newCreation.setUpdTime(rightNow);
		newCreation.setUpdNb( BigInteger.valueOf(0));
		newCreation.setLifetime( BigInteger.valueOf(0));
	
		newDescr.setCreaInfo(newCreation); 
	
		return newDescr;
	}

}
