/*
 * Created on July 19, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.plugins.osgi;

import java.util.Properties;

import org.gateway.home.wsSupport.osgi.bundles.axisBundle.AxisBundleFunctionality;
import org.gateway.home.wsSupport.osgi.bundles.axisBundle.WebServicesAvailableInfo;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.osgi.framework.Filter;
import org.osgi.util.tracker.ServiceTracker;
import org.osgi.util.tracker.ServiceTrackerCustomizer;

public class MSDPluginBundleActivator implements BundleActivator, ServiceTrackerCustomizer {

	final static String MSDPLUGIN_WS_NAME = "MSDOSGIPlugin";
	final static String AXIS_SERVICE = "org.gateway.home.wsSupport.osgi.bundles.axisBundle.AxisBundleFunctionality";

	private BundleContext context;

	private MSDOSGIBundle msdOsgiBundle = null;
	//we need the ServiceRegistration in order to unregister the service
	private ServiceRegistration MSDPluginRegistration = null;

	
	//A service tracker for detect the registration/unregistration of
	// the services of the Axis Bundle
	private ServiceTracker fldTracker = null; 
	


	/*
	*
	*	BundleActivator methods
	*
	*/
	public void start(BundleContext context) throws Exception {

		this.context = context;
		
		msdOsgiBundle = new MSDOSGIBundle();
		Properties props = new Properties();
		MSDPluginRegistration = 
				context.registerService(MSDOSGIBundle.class.getName()
				, msdOsgiBundle
				, props);
		
		//Service Tracker
		String strFiltre = "(" + org.osgi.framework.Constants.OBJECTCLASS
                + '=' + AXIS_SERVICE + ")";
        	Filter filtre = context.createFilter(strFiltre);
        	fldTracker = new ServiceTracker(context, filtre, this);
        	fldTracker.open();
        	msdOsgiBundle.setBundleContext(context);
	}

	public void stop(BundleContext context) throws Exception 
	{
		//close the ServiceTracker
		fldTracker.close();
		//we must to unregister our service
		MSDPluginRegistration.unregister();
		msdOsgiBundle.terminate();
		msdOsgiBundle=null;
	}


	/*
	*
	*	ServiceTrackerCustomizer methods
	*
	*/

	public synchronized Object addingService(ServiceReference reference) {

			Object obj = context.getService(reference);
        		if (obj instanceof AxisBundleFunctionality) {

        		//Register the MSDOSGIPlugin
				//board.info("Registering MSDOSGIPlugin");
				registerServices((AxisBundleFunctionality) obj);

				// Look for all webservices registered and print description
				try {			
					ServiceReference infoRef = context
					.getServiceReference("org.gateway.home.wsSupport.osgi.bundles.axisBundle.WebServicesAvailableInfo");

					WebServicesAvailableInfo _telventInfo = (WebServicesAvailableInfo) (context
					.getService(infoRef));
					Object[] svc = _telventInfo.getPublishServicesList();
					for (int i=0; i<svc.length;i++)
					{
						//board.info("Svc publie : "+svc[i].toString());
						//board.info("desde el activator ???");
			    		//	board.info(_telventInfo.getServiceWsdlDescripcion(svc[0]));
					}
					
					context.ungetService(infoRef);
			
				} catch(Exception e) {
					//board.info("Can't retrieve the lis of webservices. "+e.getMessage());
				}

			} 

		return obj;
	}

	
	public synchronized void modifiedService(ServiceReference reference,
                                             Object service) {
        //Do nothing
   	}


	public synchronized void removedService(ServiceReference reference,
                                            Object service) {

        	if (service instanceof AxisBundleFunctionality) {
			  	//board.info("Unregistering MSDOSGIPlugin services");
				unregisterServices((AxisBundleFunctionality) service);
	    	}	
	}


	
	/*
	*
	*  Private methods
	*
	*/

	private void registerServices(AxisBundleFunctionality _telventService){

		// Register msd osgi service Webservice
				try {

					//Case 4
					_telventService.publishService(MSDPLUGIN_WS_NAME,
								"plugins.osgi.MSDOCDLPlugin",
								"java:RPC",
								null,
								"*",
								null,
								null,
								null);


				} catch (Exception e) {
					//board.info("can't register ws. "+ e.getMessage());
				}
	}

	private void unregisterServices(AxisBundleFunctionality _telventService){
		//Unregister the msd plugin webservices
			try {
				//Case 4
				_telventService.removeService(MSDPLUGIN_WS_NAME);
			} catch (Exception e) {
				//board.info("problems unregistering ws. "+ e.getMessage());
			}
	}
}
