/*
 * Created on March 22, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 



package org.msda.plugins.pdp;


import java.util.Enumeration;
import java.util.Vector;

import org.msda.components.interfaces.PluginInterface;
import org.msda.components.plugins.AbstractPlugin;
import org.msda.components.plugins.DiscoveryMode;
import org.msda.components.plugins.Info;
import org.msda.msd.MSDDescription;
import org.msda.msd.MSDRequest;

import ubisec.util.system.IFUtils;

import com.uc3m.it.pfc.pdp.PDP;
import com.uc3m.it.pfc.pdp.ServiceEntry;

/**
 * @author rafik CHIBOUT
 */
public class MSDPDPPlugin extends AbstractPlugin implements PluginInterface 
{
	private PDP pdp;
	private int[] supportedModes = new int[1];
	private int currentMode      = DiscoveryMode.PASSIVE;
	
	private Enumeration foundServices = null;
	
	private long searchTimeOut = 10000;
		
	public MSDPDPPlugin()
	{
		super();
		translator = new MSDPDPTranslator();
		try
		{
		 // create a pdp Instance
		    pdp = new PDP ( 10, 10, 60000, "" );
		    pdp.setUA    ( true ); // Sets User Agent on
			pdp.setSA    ( true ); // Sets Service Agent on
			pdp.setDereg ( true ); // Sets PDP to inform when leaving the network
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
			System.exit(0);
		}
		
		supportedModes[0]=DiscoveryMode.PASSIVE;
		info = new Info();
		info.setCreator("Rafik CHIBOUT");
		info.setName("PDP");
		info.setProtocolName("PDP");
		info.setVersion("1.1");
		info.setDate("11-07-2005");
		info.setDescription("PDP plugin for MSD");
	}
	
	
	
	public static MSDPDPPlugin getInstance()
	{
		MSDPDPPlugin forReturn = new MSDPDPPlugin();
		return forReturn;
	}
	
		 
	/* PluginInterface interface methods */ 
	
	
	 public void searchTimeout(int searchID)
	 {
	
	 }
	
	public void setMode(int mode)
	{
		currentMode = mode;
	}
	
	
	
	public void activate()
	{
		try
		{
			IFUtils ifUtils = new IFUtils();
			String localIPAddress = ifUtils.getLocalHostAddresses()[0];
			
			boolean done = pdp.addLocal( "msdservice", "http://"+localIPAddress+":"+httpServerPort+"/discovery", 65535);
			//if (done) board.info("MSDService is successfully registered in PDP");
			//else board.info("MSDService cannot be registered in PDP");
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		super.activate();
		activate = true;
	}
	
	public void terminate()
	{
		activate = false;
		pdp.byebye();
	}
	
	public boolean setNotify(boolean mode)
	{
		return true;
	}
	
	public boolean doNotify()
	{
		return false;		
	}
	
	public int[] getSupportedModes()
	{
		return supportedModes;
	}
	
	public int getCurrentMode()
	{
		return currentMode;
	}
	
	public Vector evaluateRequest(MSDRequest request)
	{
		Vector forReturn = new Vector();
		PDPRequest serviceDesc = (PDPRequest)translator.request(request);
		try
		{
			// Search service in PDP
			Object lock = pdp.searchService(serviceDesc.getType(), serviceDesc.isAny());
			
			synchronized ( lock )
			{
				lock.wait ();
			}
			Vector result = pdp.getResponse ( );
		    
			if (result != null) foundServices = result.elements(); 
		
			if (foundServices !=null)
			while (foundServices.hasMoreElements())
			{
				//board.info("An PDP Service has been found");
				ServiceEntry foundService = (ServiceEntry)foundServices.nextElement();
				MSDDescription msdService = (MSDDescription)translator.response(foundService);
				//put this msdService into the msdService Caneva given by the MSD
				MSDDescription msdDescription = callbckInterface.newMSDDescription();
				msdDescription.setSvcDef(msdService.getSvcDef()); 		
				forReturn.add(msdDescription);
				// used to test the plugin without MSD
				//forReturn.add(msdService);
			}
			foundServices = null;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		return forReturn;
	}
	
	
}

