/*
 * Created on March 22, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 



package org.msda.plugins.pdp;

import org.msda.components.interfaces.TranslatorInterface;
import org.msda.components.plugins.Info;
import org.msda.msd.*;
import org.msda.msd.impl.*;

import java.util.Calendar;
import java.util.List;


import com.uc3m.it.pfc.pdp.ServiceEntry;


/**
 * @author rafik CHIBOUT
 */
public class MSDPDPTranslator implements TranslatorInterface{

	Info info = new Info();
	
	/**
	 * 
	 */
	public MSDPDPTranslator()
	{
		info.setName("UbisecMSDPDPTranslator");	
		info.setCreator("Rafik CHIBOUT");
		info.setDate("18/03/2005");
		info.setVersion("0.1");
		info.setDescription("This Translator allows you to translate an OSDP4M Description to a MSD Description and the reverse");
	}
	
	/*
	 * This method returns the creator-given information of the Transaltor  
	 * 
	 */
	public Info getTanslatorInfo()
	{
		return info;
	}
	
	/*
	 * 
	 * this method translates an MSDRequest in a protocole-specific request
	 * 
	 */
	public Object request(Object obj)
	{
		if (obj.getClass().getName().equals(String.class.getName()))
		{
			// XML String processing 
		}
		else
		{
			// Object instance processing
			MSDRequest request = ((MSDRequest)obj);
			ReqDescriptionInformation desc = request.getDescInfo();
			RequestService msdService = desc.getService();
						
			PDPRequest pdpRequest = new PDPRequest();
			pdpRequest.setType(msdService.getName());
			return pdpRequest;
		}
		return null;
	}
	
	/*
	 * this method translates a protocole-specific reponse in an MSDDescription
	 */
	public Object response(Object obj)
	{
		if (obj.getClass().getName().equals(String.class.getName()))
		{
			// XML String processing 
			// not yet implemented
		}
		else
		{
		if (obj.getClass().getName().equals(ServiceEntry.class.getName()))
		{
			// UPnPDevice Casting
			ServiceEntry serviceDes = (ServiceEntry)obj;
						
					
			MSDDescription msdService = new MSDDescriptionImpl();
			ServiceDefinition msdServiceDefinition = new ServiceDefinitionImpl();
			Original msdOriginal = new OriginalImpl();
			msdOriginal.setProtocolName("PDP");
			msdOriginal.setDesc("Pervasive Discovery Protocol");
			Generic msdGeneric = new GenericImpl();
			// set all the infos that are possible
			
					
			msdGeneric.setName(serviceDes.getPDPService().getService());
			
            // address
			Address address = new AddressImpl();
			//address.setLocation(serviceDes.toString());
			address.setLocation(serviceDes.getPDPService().getURL());
			
			msdGeneric.setAddr(address);
			
			// binding 
			Binding binding = new BindingImpl();
			binding.setEncoding("PDP");
			binding.setTransport("TPC/IP");
			msdGeneric.setBinding(binding);
				
			// attributes
			Param attribute = new ParamImpl();
			List attributes = msdGeneric.getParams();
			attributes.add(attribute);
			
			// creation time
			Calendar currentTime = Calendar.getInstance();
			currentTime.setTimeInMillis(System.currentTimeMillis());
			msdGeneric.setCreaTime(currentTime);
			
			//Expected life time
			Calendar expectedLifeTime = Calendar.getInstance();
			expectedLifeTime.setTimeInMillis(System.currentTimeMillis()+serviceDes.getLifetime());
			msdGeneric.setExpectedLifetime(expectedLifeTime);
			
			msdServiceDefinition.setOriginal(msdOriginal);
			msdServiceDefinition.setGeneric(msdGeneric);
			msdService.setSvcDef(msdServiceDefinition);
			return msdService;
		}
		}
		return null;
	}
	
	
}
