/*
 * Created on March 22, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.plugins.slp;


import java.util.Locale;
import java.util.Vector;

import org.msda.components.interfaces.PluginInterface;
import org.msda.components.interfaces.RequestInterface;
import org.msda.components.interfaces.TranslatorInterface;
import org.msda.components.plugins.AbstractPlugin;
import org.msda.components.plugins.DiscoveryMode;
import org.msda.components.plugins.Info;
import org.msda.components.plugins.http.HTTPListener;
import org.msda.components.plugins.msdaservice.MSDService;
import org.msda.msd.MSDDescription;
import org.msda.msd.MSDRequest;
import org.msda.msd.impl.PluginImpl;
import org.msda.util.InfoType;


import org.msda.soap.SOAPTools;
import org.msda.components.plugins.IFUtils;
import org.msda.framework.FrameworkOutput;

import com.solers.slp.*;

/**
 * @author rafik CHIBOUT
 */

public class MSDSLPPlugin extends AbstractPlugin implements PluginInterface, HTTPListener 
{
	private Locator locator = null;
	
	private int[] supportedModes = new int[1];
	private int currentMode      = DiscoveryMode.PASSIVE;
	
	private String[][] parameters = new String[20][2];
	private int paramPosition = 0;
	
	private ServiceURL[] foundServices = null;

	private TranslatorInterface translator;
	private RequestInterface msdInterface;
	private SOAPTools soapTools = null;
	
	private boolean activate = false;
	
	private MSDService service = null ;
	
	private long searchTimeOut = 8000;

	private FrameworkOutput board;

	/**
	 * 
	 *
	 */
	public MSDSLPPlugin()
	{
	    super();
	    translator = new MSDSLPTranslator();
		try {
			locator = ServiceLocationManager.getLocator(Locale.FRANCE);
		}
		catch(ServiceLocationException ex)
		{
			ex.printStackTrace();
			System.exit(0);
		}
		
		supportedModes[0]=DiscoveryMode.PASSIVE;
		info = new Info();
		info.setCreator("Rafik CHIBOUT");
		info.setName("SLP");
		info.setProtocolName("SLP");
		info.setVersion("0.1");
		info.setDate("22-03-2005");
		info.setDescription("SLP plugin for MSD");
		
		//
		board = FrameworkOutput.commonBoard();
	}
	
	
	public static MSDSLPPlugin getInstance()
	{
		MSDSLPPlugin forReturn = new MSDSLPPlugin();
		return forReturn;
	}
	
	public void searchTimeout(int searchID)
	{
	
	}
	
	 
	/* PluginInterface interface methods */ 
	
	public void setMode(int mode)
	{
		currentMode = mode;
	}
	
	
	public void activate()
	{
		if (activate == true)
		{
			board.info("The Plugin is already activated ???");
			return;
		}

		//MSDService Publication in SLP
		try
		{
			Advertiser slpAdvertiser = ServiceLocationManager.getAdvertiser(Locale.FRANCE);
			IFUtils ifUtils = new IFUtils();
			String localIPAddress = ifUtils.getLocalHostAddresses()[0];
			
			ServiceURL serviceURL = new ServiceURL("service:msdservice:http://"+localIPAddress+":"+httpServerPort+"/discovery", ServiceURL.LIFETIME_PERMANENT);
			slpAdvertiser.register(serviceURL, new Vector());

			System.out.println("MSDService is successfully registered in SLP " + serviceURL);
			
			ServiceURL serviceURL2 = new ServiceURL("service:directmsdservice:http://"+localIPAddress+":"+directMSDPort, ServiceURL.LIFETIME_PERMANENT);
			slpAdvertiser.register(serviceURL2, new Vector());
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		
		// completion of activation
		activate = true;
		super.activate();
		board.info("The Plugin "+this.getInfo().getName() +" is activated (1)");
	}
	
	public void terminate()
	{
		activate = false;
	}
	
	public boolean setNotify(boolean mode)
	{
		return true;
	}
	
	public boolean doNotify()
	{
		return false;		
	}
	
	public int[] getSupportedModes()
	{
		return supportedModes;
	}
	
	public int getCurrentMode()
	{
		return currentMode;
	}
	
	/**
	 * 
	 */
	public Vector evaluateRequest(MSDRequest request)
	{
		Vector forReturn = new Vector();
		SLPRequest serviceDesc = (SLPRequest)translator.request(request);
		try
		{
			ServiceLocationEnumeration foundServices = locator.findServices(serviceDesc.getType(),serviceDesc.getScopes(), serviceDesc.getSearchFilter());
			
			if (foundServices !=null) {
			    if ( InfoType.TEST_PARTIAL_RESULTS) {
			        if ( foundServices.hasMoreElements() ) {
			            ServiceURL foundService = (ServiceURL)foundServices.next();
			            MSDDescription msdService = (MSDDescription)translator.response(foundService);
			            //put this msdService into the msdService Caneva given by the MSD
			            MSDDescription msdDescription = callbckInterface.newMSDDescription();
			            msdDescription.setSvcDef(msdService.getSvcDef()); 
			            
			            msdDescription.getCreaInfo().setPlugin( new PluginImpl());
						msdDescription.getCreaInfo().getPlugin().setName(this.name);
						msdDescription.getCreaInfo().getPlugin().setCreator(this.creator);
						msdDescription.getCreaInfo().getPlugin().getTransformers().add("Transformer");
						
			            
			            forReturn.add(msdDescription);
			        }
			        
			    } else {
			        while (foundServices.hasMoreElements())
			        {
			            System.out.println("An SLP Service has been found");
			            ServiceURL foundService = (ServiceURL)foundServices.next();
			            MSDDescription msdService = (MSDDescription)translator.response(foundService);
			            //put this msdService into the msdService Caneva given by the MSD
			            MSDDescription msdDescription = callbckInterface.newMSDDescription();
			            msdDescription.setSvcDef(msdService.getSvcDef()); 
			            
			            msdDescription.getCreaInfo().setPlugin( new PluginImpl());
						msdDescription.getCreaInfo().getPlugin().setName(this.name);
						msdDescription.getCreaInfo().getPlugin().setCreator(this.creator);
						msdDescription.getCreaInfo().getPlugin().getTransformers().add("Transformer");
						
			            forReturn.add(msdDescription);
			            // used to test the plugin without MSD
			            //forReturn.add(msdService);
			        }
			    }
			}
			foundServices.destroy();
			foundServices = null;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		return forReturn;
	}
	
	
	
	public static void main(String[] args) 
	{
		MSDSLPPlugin app = MSDSLPPlugin.getInstance();
		app.activate();
		
		/*	
		MSDRequest serviceRequest = new MSDRequestImpl();
		ReqDescriptionInformation reqDescription = new ReqDescriptionInformationImpl();
		
		RequestService requestService = new RequestServiceImpl();
		requestService.setName("SD");
		       	        
	    reqDescription.setService(requestService);
	    serviceRequest.setDescription(reqDescription);
	   
	    logger.info("searching service...");
	    Vector result = app.evaluateRequest(serviceRequest);
	    
	    if (result !=null) 
	    for (int k=0; k<result.size(); k++)
	    {
	    	logger.info("*******msdDescription found out********");
	    	MSDDescription msdService = (MSDDescription)result.elementAt(k);
	    	logger.info("ep="+msdService.getServiceDefinition().getGeneric().getAddress().getLocation());
			logger.info("msdService name="+msdService.getServiceDefinition().getGeneric().getName());
			logger.info("*******************************");
	    }
	    else logger.info("No Service found");
	    */
	}

}

