/*
 * Created on March 22, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.plugins.slp;

import org.msda.components.interfaces.TranslatorInterface;
import org.msda.components.plugins.Info;
import org.msda.msd.*;
import org.msda.msd.impl.*;

import com.solers.slp.ServiceType;
import com.solers.slp.ServiceURL;

import java.util.Calendar;
import java.util.List;
import java.util.Vector;

/**
 * @author rafik CHIBOUT
 */
public class MSDSLPTranslator implements TranslatorInterface{

	Info info = new Info();
	
	/**
	 * 
	 */
	public MSDSLPTranslator()
	{
		info.setName("UbisecMSDOSDP4MTranslator");	
		info.setCreator("Rafik CHIBOUT");
		info.setDate("18/03/2005");
		info.setVersion("0.1");
		info.setDescription("This Translator allows you to translate an OSDP4M Description to a MSD Description and the reverse");
	}
	
	/*
	 * This method returns the creator-given information of the Transaltor  
	 * 
	 */
	public Info getTanslatorInfo()
	{
		return info;
	}
	
	/*
	 * 
	 * this method translates an MSDRequest in a protocole-specific request
	 * 
	 */
	public Object request(Object obj)
	{
		if (obj.getClass().getName().equals(String.class.getName()))
		{
			// XML String processing 
		}
		else
		{
			// Object instance processing
			MSDRequest request = ((MSDRequest)obj);
			ReqDescriptionInformation desc = request.getDescInfo();
			RequestService msdService = desc.getService();
						
			SLPRequest slpRequest = new SLPRequest();
			ServiceType serviceType = new ServiceType("service:"+msdService.getName());
			slpRequest.setType(serviceType);
			Vector scopes = new Vector();
			scopes.add("default");
			slpRequest.setScopes(scopes);
			slpRequest.setSearchFilter("");
			return slpRequest;
		}
		return null;
	}
	
	/*
	 * this method translates a protocole-specific reponse in an MSDDescription
	 */
	public Object response(Object obj)
	{
		if (obj.getClass().getName().equals(String.class.getName()))
		{
			// XML String processing 
			// not yet implemented
		}
		else
		{
		if (obj.getClass().getName().equals(ServiceURL.class.getName()))
		{
			// UPnPDevice Casting
			ServiceURL serviceDes = (ServiceURL)obj;
						
					
			MSDDescription msdService = new MSDDescriptionImpl();
			ServiceDefinition msdServiceDefinition = new ServiceDefinitionImpl();
			Original msdOriginal = new OriginalImpl();
			msdOriginal.setProtocolName("SLP");
			msdOriginal.setDesc("Open Service Location Protocol");
			Generic msdGeneric = new GenericImpl();
			// set all the infos that are possible
			
					
			msdGeneric.setName(serviceDes.getServiceType().getPrincipleTypeName());
			
            // address
			Address address = new AddressImpl();
			//address.setLocation(serviceDes.toString());
			address.setLocation("http://"+serviceDes.getHost()+":"+serviceDes.getPort()+serviceDes.getURLPath());
			
			msdGeneric.setAddr(address);
			
			// binding 
			Binding binding = new BindingImpl();
			binding.setEncoding("SLP");
			binding.setTransport(serviceDes.getTransport());
			msdGeneric.setBinding(binding);
				
			// attributes
			Param attribute = new ParamImpl();
			List attributes = msdGeneric.getParams();
			attributes.add(attribute);
			
			// creation time
			Calendar currentTime = Calendar.getInstance();
			currentTime.setTimeInMillis(System.currentTimeMillis());
			msdGeneric.setCreaTime(currentTime);
			
			//Expected life time
			Calendar expectedLifeTime = Calendar.getInstance();
			expectedLifeTime.setTimeInMillis(System.currentTimeMillis()+serviceDes.getLifetime());
			msdGeneric.setExpectedLifetime(expectedLifeTime);
			
			msdServiceDefinition.setOriginal(msdOriginal);
			msdServiceDefinition.setGeneric(msdGeneric);
			msdService.setSvcDef(msdServiceDefinition);
			return msdService;
		}
		}
		return null;
	}
	
}

