/*
 * Created on August 4, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.plugins.upnp;

import java.util.Vector;

import org.msda.context.ContextUtil;
import org.msda.msd.ContextElement;
import org.msda.msd.ContextInformationTable;

import com.siemens.upnp.StateVariable;
import com.siemens.upnp.controlpoint.ProxyService;

/**
 * Note : AGNES : for now only String values, and "i4" that stands for Integer values...
 * 				Hack : searching "Ctx" in the name of the context parameters as they are all put in the state variable 
 * 
 * It permits to extract context information from UPnP service descriptions
 * and create a ContextInformation object to be added to the 
 * MSD service description
 * 
 * @author Oriana Riva, INRIA, August 2005
 */
public class MSDUPnPContextTranslator {
    
    
        
    public static void extractContext(ProxyService upnpService, ContextInformationTable serviceCtx){
        
        //board.info("########### MSDUPnP Context Translator - proxyService ############");

    	// default values for mandatory fields
        
        ContextElement elt;
        
//        elt = ContextUtil.getInstance().createCtxElement( ServiceTemplate.SERVICE_STATE, ContextVocabulary.IDLE );
//        serviceCtx.getCITable().add( elt );
//        elt = ContextUtil.getInstance().createCtxElement( ServiceTemplate.COST, 1 );
//        serviceCtx.getCITable().add( elt );
        
		// generic context/profile parameters of the service
        Vector states = upnpService.getStateVariables();
        
        for (int i = 0; i < states.size(); i++) {
            StateVariable var = (StateVariable)states.elementAt(i);
            String name = var.getName();
            //board.info("State Name = "+var.getName());
            //board.info("State Value = "+var.getValue());
            //board.info("State default = "+var.getDefaultValue()); 
            
            // HACK : search for Ctx in the variable name...
            if ( name.indexOf("Ctx") < 0 )
                continue;
            
            // will be always a string...
            if (var.getDefaultValue()!=null && !var.getDefaultValue().equals("")) {
                
                if ( var.getDataType().equals( "i4" ) ) {
                    elt = ContextUtil.getInstance().createCtxElement( name, Integer.parseInt(var.getDefaultValue()) ); 
                } else {
                    elt = ContextUtil.getInstance().createCtxElement( name, var.getDefaultValue() );
                }
	            serviceCtx.getCITable().add( elt );
            }
        }
		   
    }
    
    
    public static void extractContext(UPnPServiceDescription upnpService, ContextInformationTable serviceCtx){
        
        //board.info("########### MSDUPnP Context Translator - UPnPServiceDescription ############");

        ContextElement elt;
        
    	// default values for mandatory fields
//        elt = ContextUtil.getInstance().createCtxElement( ServiceTemplate.SERVICE_STATE, ContextVocabulary.IDLE );
//        serviceCtx.getCITable().add( elt );

        UPnPStateVariable[] list = upnpService.getStateVariableList();
        for (int i = 0; i < list.length; i++) {     
            String variableName = (list[i].getName());
            
            elt = ContextUtil.getInstance().createCtxElement( variableName, list[i].getDefaultValue() );
            serviceCtx.getCITable().add( elt );
        }
    }


}
