/*
 * Created on March 20, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.plugins.upnp;

import org.msda.components.interfaces.*;
import org.msda.components.plugins.AbstractPlugin;
import org.msda.components.plugins.DiscoveryMode;
import org.msda.components.plugins.Info;
import org.msda.components.plugins.http.HTTPListener;
import org.msda.components.plugins.msdaservice.MSDServiceUPnP_Direct;
import org.msda.components.plugins.msdaservice.MSDServiceUPnP_SOAP;
import org.msda.framework.FrameworkOutput;
import org.msda.msd.*;
import org.msda.msd.impl.PluginImpl;
import org.msda.util.InfoType;

import ubisec.util.system.IFUtils;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.*;

import com.siemens.upnp.controlpoint.*;
import com.siemens.upnp.*;


/**
 * This class implements the PluginInterface of UPnP, it listens to the UPnP Device publication and allows MSDManager to
 * find UPnP services by giving an MSDRequest.
 *  
 * @author Rafik CHIBOUT
 */
public class MSDUPnPPlugin extends AbstractPlugin implements PluginInterface, UPnPEventListener, UPnPSearchResponseListener, UPnPStateChangeListener, HTTPListener
{
	private static int MSDA_SERVICE_WEB_PORT = 9999;
	
	private ControlPoint cp;
	private boolean isSearching;
	private boolean doneSearching;
	
	// discovery mode of the plugin (active or passive)
	private int[] supportedModes = new int[2];
	private int currentMode = DiscoveryMode.ACTIVE; //ACTIVE;//PASSIVE
	
	private Vector announcedDeviceID    = new Vector();
	private long[] announcedDeviceTTL   = new long[100];
	private Hashtable serviceList 		= new Hashtable();
	private int[] lastServiceIndex      = new int[100];
	private Vector announcedServiceType = new Vector();
	private int[] announcedServiceTTL   = new int[400];
	
	private ProxyService[] foundServices = null;
	private ProxyDevice[] foundDevices 	= null;
	
	protected int defautlServiceTTL 	= 40000;
	protected long searchTimeOut 		= 10000;
	
	private FrameworkOutput board;
	
	/**
	 * Creates the MSDUPnPPlugin
	 *
	 */
	public MSDUPnPPlugin()
	{
		super();
		//
		translator = new MSDUPnPTranslator();
		supportedModes[0]=DiscoveryMode.PASSIVE;
		supportedModes[1]=DiscoveryMode.ACTIVE;
		// general info for the plugin
		info = new Info();
		info.setCreator("Rafik CHIBOUT");
		info.setName("UPnP");
		info.setProtocolName("UPnP");
		info.setVersion("0.1");
		info.setDate("08-03-2005");
		info.setDescription("UPnP plugin for MSD");
		// initialise the UPnP control point and disable debug output on screen
		cp = ControlPoint.getInstance();
		com.siemens.upnp.util.Tracer.setTraceDirectedToScreen(false);

		//
		isSearching = false;
		doneSearching = false;
		//
		board = FrameworkOutput.commonBoard();
	}
	
	
	/**
	 * Returns a new instance of the UPnP plugin
	 * 
	 * @return
	 */
	public static MSDUPnPPlugin getInstance()
	{
		MSDUPnPPlugin forReturn = new MSDUPnPPlugin();
		return forReturn;
	}
	
	
	/**
	 * 
	 * @see com.siemens.upnp.controlpoint.UPnPEventListener#deviceRemoved(java.lang.String)
	 */
	public void deviceRemoved(java.lang.String uuid)
	{
		//	 	logger.info("the device "+uuid+" is removed");
		//	 	if (announcedDeviceID.contains(uuid))
		//	 	{
		//	 		int deviceIndex = announcedDeviceID.indexOf(uuid);
		//			int firstServiceIndex=0;
		//			int lastServiceIndex=-1;
		//				
		//			if (deviceIndex >0)
		//			{
		//				firstServiceIndex = this.lastServiceIndex[deviceIndex-1]+1;
		//			}
		//			
		//			lastServiceIndex = this.lastServiceIndex[deviceIndex];
		//	
		//			if (currentMode == DiscoveryMode.ACTIVE)
		//		 	{
		//				for (int i=firstServiceIndex; i<= lastServiceIndex; i++)
		//				{
		//					String serviceType = (String)announcedServiceList.elementAt(i);
		//					callbckInterface.endService(uuid+":"+serviceType);
		//				}
		//		 	}
		//			// remove Device and Services
		//			announcedDeviceID.removeElementAt(deviceIndex);
		//			for (int i=firstServiceIndex; i<= lastServiceIndex; i++)
		//			{
		//				announcedServiceList.removeElementAt(i);
		//				announcedServiceType.removeElementAt(i);
		//			}
		//			
		//			int serviceNb = lastServiceIndex-firstServiceIndex+1;
		//			
		//			// update services Index
		//			for (int i=deviceIndex+1; i< this.lastServiceIndex.length; i++)
		//				this.lastServiceIndex[i-1] = this.lastServiceIndex[i]-serviceNb;
		//			
		//			// update services TTL
		//			for (int i=lastServiceIndex+1; i< this.announcedServiceTTL.length; i++ )
		//				announcedServiceTTL[i-serviceNb]=announcedServiceTTL[i];
		//	 	}
	}
	
	/**
	 * 
	 * @see com.siemens.upnp.controlpoint.UPnPEventListener#newDeviceAnnounced(java.lang.String, java.lang.String, java.lang.String)
	 */
	public void newDeviceAnnounced(java.lang.String uuid, java.lang.String type, java.lang.String descriptionURL) 
	{
		board.info("------------------------a new device is connected");
		board.info("------------------------uuid="+uuid);
		board.info("------------------------type="+type);
		board.info("------------------------descriptionURL="+descriptionURL);
		
		URL deviceURL;
		try
		{
			deviceURL = new URL(descriptionURL);
		}
		catch(MalformedURLException ex)
		{
			board.error("Malformed URL: "+descriptionURL);
			return;
		}
		try
		{
			ProxyDevice device = cp.createDevice(deviceURL);
			board.debug("****** the device lease time is "+device.getLeaseTime());
			if (!announcedDeviceID.contains(uuid))
			{
				announcedDeviceID.add(device.getUDN());
				announcedDeviceTTL[announcedDeviceID.size()-1] =  defautlServiceTTL;
				board.debug("****** the Device "+uuid +" is add" );
				
				Vector announcedService = new Vector(); 
				if (device.getServices()!=null)
				{
					Enumeration services = device.getServices().elements();
					while( services.hasMoreElements())
					{
						ProxyService service = (ProxyService)services.nextElement();
						announcedService.add(service);
						
						if (currentMode == DiscoveryMode.ACTIVE)
						{
							// ACTIVE MODE
							MSDDescription msdDescription = callbckInterface.newMSDDescription();
// ATTENTION
// THIS PART IS JUST FOR HACKS/PERFS FOR THE CONTEXT...
//							if ( InfoType.TEST_CTX ) {
//							    String nameMgr = msdDescription.getCreaInfo().getCreator().getMgrID().getName();
//							    if ( "Manager_1".equals(nameMgr))
//							    {
//							        if ( super.directMSDPort==8310 ) {
//							            MSDDescription msdService = (MSDDescription)translator.response(service);
//									    
//									    Plugin plugin = new PluginImpl();
//									    plugin.setName( super.name );
//									    plugin.setCreator( super.creator );
//									    msdDescription.getCreaInfo().setPlugin( plugin );
//									    msdDescription.setSvcDef(msdService.getSvcDef());
//									    
//									    ContextInformation ci = msdService.getCtxInfo();
//									    ContextElement elt = ContextUtil.getInstance().createCtxElement( "CtxSecurity", "Secure"  );
//									    ci.getCISvc().getCITable().add( elt );
//									    elt = ContextUtil.getInstance().createCtxElement( "CtxPayment", "Yes"  );
//									    ci.getCISvc().getCITable().add( elt );
//									    msdDescription.setCtxInfo(ci);
//									    
//									    callbckInterface.newService(msdDescription);
//							        }
//							    }
//							    else if ( "Manager_3".equals(nameMgr)) 
//							    {
//							        if ( super.directMSDPort==8330 ) {
//							            MSDDescription msdService = (MSDDescription)translator.response(service);
//									    
//									    Plugin plugin = new PluginImpl();
//									    plugin.setName( super.name );
//									    plugin.setCreator( super.creator );
//									    msdDescription.getCreaInfo().setPlugin( plugin );
//									    msdDescription.setSvcDef(msdService.getSvcDef());
//									    
//									    ContextInformation ci = msdService.getCtxInfo();
//									    ContextElement elt = ContextUtil.getInstance().createCtxElement( "CtxSecurity", "Open"  );
//									    ci.getCISvc().getCITable().add( elt );
//									    elt = ContextUtil.getInstance().createCtxElement( "CtxPayment", "Yes"  );
//									    ci.getCISvc().getCITable().add( elt );
//									    msdDescription.setCtxInfo(ci);
//									    callbckInterface.newService(msdDescription);
//							        }
//							    }
//							    else if ( "Manager_5".equals(nameMgr) ) 
//							    {
//							        if ( super.directMSDPort==8350) {
//							            MSDDescription msdService = (MSDDescription)translator.response(service);
//									    
//									    Plugin plugin = new PluginImpl();
//									    plugin.setName( super.name );
//									    plugin.setCreator( super.creator );
//									    msdDescription.getCreaInfo().setPlugin( plugin );
//									    msdDescription.setSvcDef(msdService.getSvcDef());
//									    
//									    ContextInformation ci = msdService.getCtxInfo();
//									    ContextElement elt = ContextUtil.getInstance().createCtxElement( "CtxSecurity", "Open"  );
//									    ci.getCISvc().getCITable().add( elt );
//									    elt = ContextUtil.getInstance().createCtxElement( "CtxPayment", "No"  );
//									    ci.getCISvc().getCITable().add( elt );
//									    msdDescription.setCtxInfo(ci);
//									    callbckInterface.newService(msdDescription);
//							        }
//							    }
//							    else if ( "Manager_9".equals(nameMgr) )
//							    {
//							        if (super.directMSDPort==8390) {
//							            MSDDescription msdService = (MSDDescription)translator.response(service);
//									    
//									    Plugin plugin = new PluginImpl();
//									    plugin.setName( super.name );
//									    plugin.setCreator( super.creator );
//									    msdDescription.getCreaInfo().setPlugin( plugin );
//									    msdDescription.setSvcDef(msdService.getSvcDef());
//									    
//									    ContextInformation ci = msdService.getCtxInfo();
//									    ContextElement elt = ContextUtil.getInstance().createCtxElement( "CtxSecurity", "Secure"  );
//									    ci.getCISvc().getCITable().add( elt );
//									    elt = ContextUtil.getInstance().createCtxElement( "CtxPayment", "No"  );
//									    ci.getCISvc().getCITable().add( elt );
//									    msdDescription.setCtxInfo(ci);
//									    callbckInterface.newService(msdDescription);
//							        }
//							    }
//							    
//							} else 
							{
							    // Without test
							    MSDDescription msdService = (MSDDescription)translator.response(service);
							    Plugin plugin = new PluginImpl();
							    plugin.setName( super.name );
							    plugin.setCreator( super.creator );
							    msdDescription.getCreaInfo().setPlugin( plugin );
							    msdDescription.setSvcDef(msdService.getSvcDef());
							    msdDescription.setCtxInfo(msdService.getCtxInfo());
							    callbckInterface.newService(msdDescription);
							}
						}
					}
				}
				serviceList.put(device.getUDN(), announcedService);
			}
			else
			{
				announcedDeviceTTL[announcedDeviceID.indexOf(device.getUDN())] =  defautlServiceTTL;
			}
			
		}
		catch(InvalidDescriptionException ex)
		{
			board.error("Invalid description: "+descriptionURL);
			board.notifyException(ex);
		}
		
		
		
		
		//		board.info("------------------------------- new Device announced-----------------------------");
		//		for (int m=0; m< announcedDeviceID.size(); m++)
		//		{
		//			board.info("    ---------------------------- Device "+m+"------------------------");
		//			board.info(" uuid       = "+(String)announcedDeviceID.elementAt(m));
		//			board.info(" TTL        = "+announcedDeviceTTL[m]);
		//			
		//			board.info("                 ----------------------- Services --------------------------");
		//			Vector services = (Vector)serviceList.get((String)announcedDeviceID.elementAt(m));
		//			for (int w=0; w< services.size(); w++)
		//			{
		//				board.info("                 type services "+w+"       = "+((ProxyService)services.elementAt(w)).getServiceType());
		//				board.info("                 --------------------------------------------------------");
		//			}
		//			
		//			board.info("    -----------------------------------------------------------------");
		//		}
		//		board.info("---------------------------------------------------------------------------------");
		
	}
	
	/**
	 * 
	 * @see com.siemens.upnp.controlpoint.UPnPEventListener#newServiceAnnounced(java.lang.String, java.lang.String, java.lang.String)
	 */
	public void newServiceAnnounced(java.lang.String uuid, java.lang.String type,
			java.lang.String descriptionURL)
	{
		board.debug("a new service is announced");
		board.debug("uuid="+uuid);
		board.debug("type="+type);
		board.debug("descriptionURL="+descriptionURL);
		//		URL deviceURL;
		//		try
		//		{
		//			deviceURL = new URL(descriptionURL);
		//		}
		//		catch(MalformedURLException ex)
		//		{
		//			logger.error("Malformed URL: "+descriptionURL);
		//			return;
		//		}
		//		try
		//		{
		//			ProxyDevice device = cp.createDevice(deviceURL);
		//			logger.debug("****** the device lease time is "+device.getLeaseTime());
		//			ProxyService service = device.getServiceByType(type);
		//			if (!announcedDeviceID.contains(uuid))
		//			{
		//				announcedDeviceID.add(device.getUDN());
		//				logger.debug("****** the device "+uuid+" is add");
		//				if (announcedDeviceID.size()==1)
		//					lastServiceIndex[0] = 0;
		//				else 
		//					lastServiceIndex[announcedDeviceID.size()-1] = 1 + lastServiceIndex[announcedDeviceID.size()-2];
		//			
		//				announcedServiceType.add(service.getServiceType());
		//				announcedServiceList.add(service);
		//				if (device.getLeaseTime()>0) announcedServiceTTL[announcedServiceList.size()-1]=device.getLeaseTime();
		//				else announcedServiceTTL[announcedServiceList.size()-1]= defautlServiceTTL;
		//			
		//				logger.debug("****** the service "+service.getServiceType()+" is add in this device");	
		//				if (currentMode == DiscoveryMode.ACTIVE)
		//				{
		//					// ACTIVE MODE
		//					MSDDescription msdDescription = callbckInterface.newMSDDescription();
		//					ServiceDescription msdService = (ServiceDescription)translator.response(service);
		//					//msdService.getServiceDefinition().getGeneric().setName(uuid+":"+msdService.getServiceDefinition().getGeneric().getName());
		//					msdDescription.setServiceDefinition(msdService.getServiceDefinition());
		//			        msdDescription.setContextInformation(msdService.getContextInformation());
		//					callbckInterface.newService(msdDescription);
		//				}
		//			}
		//			else
		//			{
		//				logger.debug("****** the device "+uuid+" is alreday exist in this plugin");
		//				int deviceIndex  = announcedDeviceID.indexOf(device.getUDN());
		//				int serviceIndex = announcedServiceType.indexOf(service.getServiceType());
		//				logger.debug("the announced service is at the index "+serviceIndex);
		//				logger.debug("lastServiceIndex[deviceIndex]="+lastServiceIndex[deviceIndex]);
		//				if (serviceIndex >-1 && serviceIndex <= lastServiceIndex[deviceIndex] && (deviceIndex==0 || (serviceIndex>lastServiceIndex[deviceIndex-1])))
		//				{
		//					if (device.getLeaseTime()>0) announcedServiceTTL[serviceIndex] = device.getLeaseTime();
		//					else announcedServiceTTL[serviceIndex]= defautlServiceTTL;
		//				}
		//				else
		//				{
		//					// add the service of the existing Device
		//			        announcedServiceType.insertElementAt(service.getServiceType(), lastServiceIndex[deviceIndex]+1);
		//					announcedServiceList.insertElementAt(service.getServiceType(), lastServiceIndex[deviceIndex]+1);
		//					// insert TTL
		//					if (device.getLeaseTime()>0) insertTTL(lastServiceIndex[deviceIndex]+1,device.getLeaseTime());
		//					else  insertTTL(lastServiceIndex[deviceIndex]+1,defautlServiceTTL);
		//					
		//					for (int i=deviceIndex; i<lastServiceIndex.length;i++)
		//			        	if (lastServiceIndex[i]>-1) lastServiceIndex[i]++;
		//			        
		//			        	
		//				    logger.debug("****** the service "+service.getServiceType()+" is inserted in this device");
		//					if (currentMode == DiscoveryMode.ACTIVE)
		//					{
		//						// ACTIVE MODE
		//						MSDDescription msdDescription = callbckInterface.newMSDDescription();
		//						ServiceDescription msdService = (ServiceDescription)translator.response(service);
		//						//msdService.getServiceDefinition().getGeneric().setName(uuid+":"+msdService.getServiceDefinition().getGeneric().getName());
		//						msdDescription.setServiceDefinition(msdService.getServiceDefinition());
		//						msdDescription.setContextInformation(msdService.getContextInformation());
		//						callbckInterface.newService(msdDescription);
		//					}
		//				}
		//			}
		//		}
		//		catch(InvalidDescriptionException ex)
		//		{
		//			logger.error("Invalid description: "+descriptionURL);
		//		}
	}
	
	/**
	 * 
	 * @see com.siemens.upnp.controlpoint.UPnPEventListener#serviceRemoved(java.lang.String, java.lang.String)
	 */
	public void serviceRemoved(java.lang.String uuid, java.lang.String type)
	{
		board.debug("a service is removed");
		board.debug("uuid="+uuid);
		board.debug("type="+type);
		
		if (announcedDeviceID.contains(uuid))
		{
			int deviceIndex = announcedDeviceID.indexOf(uuid);
			int serviceIndex = announcedServiceType.indexOf(type);
			if (serviceIndex >-1 && serviceIndex <= lastServiceIndex[deviceIndex] && (deviceIndex==0 || (serviceIndex>lastServiceIndex[deviceIndex-1])))
			{
				// remove the service
				announcedServiceType.removeElementAt(serviceIndex);
				for (int i=deviceIndex; i<lastServiceIndex.length;i++)
					if (lastServiceIndex[i]>-1) lastServiceIndex[i]--;
					
					// update TTL
				for ( int i= serviceIndex; i< announcedServiceTTL.length-1; i++)
					announcedServiceTTL[i]= announcedServiceTTL[i+1];	
				
				if (currentMode == DiscoveryMode.ACTIVE)
				{
					// ACTIVE MODE
					callbckInterface.endService(uuid+":"+type);
				}
			}			
		}
	}
	
	
	/**
	 * 
	 * @see com.siemens.upnp.controlpoint.UPnPSearchResponseListener#newDeviceSearchResponse(java.lang.String, java.lang.String, java.lang.String)
	 */
	public void newDeviceSearchResponse(java.lang.String uuid, java.lang.String type,java.lang.String descriptionURL)
	{
		URL deviceURL;

		board.info("A Service is found (search response) with uuid="+uuid + " type="+type + "descriptionURL="+descriptionURL);
		try
		{
			deviceURL = new URL(descriptionURL);
		}
		catch(MalformedURLException ex)
		{
			board.error("Malformed URL: "+descriptionURL);
			return;
		}
		try
		{
			ProxyDevice foundDevice = cp.createDevice(deviceURL);
			if (foundDevice != null)
			{
				foundDevices = new ProxyDevice[1];
				foundDevices[0] = foundDevice;
			}
		}
		catch(InvalidDescriptionException ex)
		{
			board.error("Invalid description: "+descriptionURL);
		}
	}
	
	/**
	 * 
	 * @see com.siemens.upnp.controlpoint.UPnPSearchResponseListener#newServiceSearchResponse(java.lang.String, java.lang.String, java.lang.String)
	 */
	public void newServiceSearchResponse(java.lang.String uuid, java.lang.String type,java.lang.String descriptionURL)
	{
		URL deviceURL;
		
		board.info("A Service is found (search response) with uuid="+uuid + " type="+type + "descriptionURL="+descriptionURL);
		try
		{
			deviceURL = new URL(descriptionURL);
		}
		catch(MalformedURLException ex)
		{
			board.debug("Malformed URL: "+descriptionURL);
			return;
		}
		try
		{
			ProxyDevice foundDevice = cp.createDevice(deviceURL);
			ProxyService foundService = foundDevice.getServiceByType(type);
			foundServices = new ProxyService[1];
			foundServices[0] = foundService;
		}
		catch(InvalidDescriptionException ex)
		{
			board.debug("Invalid description: "+descriptionURL);
		}
		
	}	
	
	/**
	 * 
	 * @see com.siemens.upnp.controlpoint.UPnPSearchResponseListener#searchTimeout(int)
	 */
	public void searchTimeout(int searchID)
	{
		board.info("A Search expired with searchID="+searchID);

		isSearching = false;
		doneSearching = true;
	}
	
	
	/**
	 * 
	 * @see com.siemens.upnp.controlpoint.UPnPStateChangeListener#stateChanged(com.siemens.upnp.StateVariable)
	 */
	public void stateChanged(StateVariable variable)
	{
		board.debug("The "+variable.getName()+" variable has been changed its new value is "+variable.getValue());
	}
	
	
	
	
	
	
	/**
	 * 
	 * @see org.msda.components.interfaces.PluginInterface#setMode(int)
	 */
	public void setMode(int mode)
	{
		currentMode = mode;
	}
	
	
	
	/**
	 * 
	 * @see org.msda.components.interfaces.PluginInterface#activate()
	 */
	public void activate()
	{
		if (activate == true)
		{
			board.info("The Plugin is already activated ???");
			return;
		}
		
		//MSDService Publication in UPnP
		UPnPConstants.WEB_PORT = MSDA_SERVICE_WEB_PORT;
		IFUtils ifUtils = new IFUtils();
		String localIPAddress = ifUtils.getLocalHostAddresses()[0];
		
		MSDServiceUPnP_SOAP MSDServicePub = new MSDServiceUPnP_SOAP("http://"+localIPAddress+":"+httpServerPort+"/discovery");
		board.info("MSDServiceUPnP(http://"+localIPAddress+":"+httpServerPort+"/discovery");
		try
		{
			MSDServicePub.announce();
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		String url = new StringBuffer("http://").append(localIPAddress).append(":").append(directMSDPort).toString();
		MSDServiceUPnP_Direct directMSDService = new MSDServiceUPnP_Direct(url);
		try {
            directMSDService.announce();
        } catch (InvalidDescriptionException e) {
            e.printStackTrace();
        }
		
		// completion of activation
		activate = true;
		cp.addUPnPEventListener(this);
		cp.addUPnPSearchListener(this);
		Thread ttlUpdate = ttlUpdating();
		ttlUpdate.start();
		super.activate();
		board.info("The Plugin "+this.getInfo().getName() +" is activated (1)");
	}
	
	/**
	 * 
	 * @see org.msda.components.interfaces.PluginInterface#terminate()
	 */
	public void terminate()
	{
		activate = false;
		cp.removeUPnPEventListener(this);
		cp.removeUPnPSearchListener(this);
		cp.shutDown();
	}
	
	/**
	 * 
	 * @see org.msda.components.interfaces.PluginInterface#setNotify(boolean)
	 */
	public boolean setNotify(boolean mode)
	{
		if (mode) currentMode = DiscoveryMode.ACTIVE;
		else currentMode = DiscoveryMode.PASSIVE;
		return true;
	}
	
	/**
	 * 
	 * @see org.msda.components.interfaces.PluginInterface#doNotify()
	 */
	public boolean doNotify()
	{
		if (currentMode == DiscoveryMode.PASSIVE) return false;
		return true;		
	}
	
	/**
	 * 
	 * @return
	 */
	public int[] getSupportedModes()
	{
		return supportedModes;
	}
	
	/**
	 * 
	 * @return
	 */
	public int getCurrentMode()
	{
		return currentMode;
	}
	
	
	/**
	 * 
	 * @see org.msda.components.interfaces.PluginInterface#evaluateRequest(org.msda.msd.MSDRequest)
	 */
	public Vector evaluateRequest(MSDRequest request)
	{

		synchronized (cp) {
			Vector forReturn = new Vector();
			UPnPServiceDescription upnpService = (UPnPServiceDescription)translator.request(request);
	
			try
			{
				// search in the local registry of services if the plugin is active
				if (currentMode == DiscoveryMode.ACTIVE)
				{
					board.info("search services in the UPnP local cache");
					foundServices = getServiceFromLocalRegistry(upnpService);	
				}
				else
				{
					foundServices = new ProxyService[0];
				}
				
				// if passive or if no response inthe local cache, then perform a discovery request
				if (foundServices.length <=0) 
				{
//					logger.info("asynch search services in the UPnP discovery domain (service type = " + upnpService.getServiceType() + ")" );
					while (isSearching)
					{
//						logger.info("waiting for the previous search to end" );
						Thread.sleep(InfoType.WAIT_PENDING_REQUEST_FIRST_RESULT);
					}
					cp.searchAsync(UPnPConstants.SearchForServiceType(upnpService.getServiceType()));
					isSearching = true;
					doneSearching = false;
				}
				
				// Get the lifetime of the request (convert from sec to millisec)
				long waitingTime = 0;
				ReqCreationInformation creationInformation = request.getProcInfo().getCrea();
				long requestLifetime = searchTimeOut;// milliseconds
				if ( creationInformation.getLifetime()!=null && creationInformation.getLifetime().longValue() > 0 ) 
				{
					requestLifetime = creationInformation.getLifetime().longValue() * 1000;
				}
				
				// now I wait until I get some responses or the timeout expires....
				board.info("wait until the request lifetime expires (" + requestLifetime + " millisec) or results are found");
				while ((foundServices == null || foundServices.length <=0) && waitingTime < requestLifetime)
				{
					// this is currently just for testing purposes, return at the first result...
					// this should be extended so that we support partial results...
					if ( InfoType.TEST_PARTIAL_RESULTS) 
					{ // leave the loop as soon as a response is found.
						if (foundServices.length > 0)
						{
							board.info("MSDUPNPPLuging::evaluateRequest : return after the first result");
							break;
						}
					}
					// we increment the time by 10 millisec...
					//	logger.info("waiting 10 ms - " + waitingTime + " out of " + requestLifetime);
					Thread.sleep(InfoType.WAIT_PENDING_REQUEST_FIRST_RESULT);
					waitingTime = waitingTime + InfoType.WAIT_PENDING_REQUEST_FIRST_RESULT;
				}
				
				// compare the given request with the found service
				board.info("waiting time and request lifetime" + waitingTime + " " + requestLifetime);
				board.info("MSDUPnPPlugin:evaluateRequest found services??? ....nb = " + foundServices.length);
				if (foundServices.length >0 ) 
				{
					ProxyService foundService;
					UPnPAction[] requiredActions;
					Enumeration foundActions;
					int i;
					MSDDescription msdService, msdDescription;
					List methods;
					Method method;
					List paramIn, paramOut;
					Param param;
					for (int index=0; index<foundServices.length; index++)
					{
						foundService = foundServices[index];			
						requiredActions = upnpService.getActionList();
						foundActions = foundService.getActions().elements();
						if (requiredActions!=null)
						{
							i=0;
							while (i<requiredActions.length && contains(foundActions, requiredActions[i]))
								i++;	
							if (i<requiredActions.length)
							{
								board.info("not the good service (missing actions), do not add");
								break;
								//							return forReturn;
							}
						}				
						
						msdService = (MSDDescription)translator.response(foundService);
						
						board.debug("*******msdService found********");
						board.debug("msdService name="+msdService.getSvcDef().getGeneric().getName());
						methods = msdService.getSvcDef().getGeneric().getMethods();
						if (methods.size()>0)
						{
							method = (Method)methods.get(0);
							//logger.debug("msdService methodName="+method.getName());
							paramIn = method.getIn();
							paramOut = method.getOut();
							if (paramIn.size()>0)
							{	
								param = (Param)paramIn.get(0);
								//logger.debug("msdService in Param"+param.getType());
							}
							
							if (paramOut.size()>0)
							{
								param = (Param)paramOut.get(0);
								//logger.debug("msdService out Param"+param.getType());
							}
						}
						
						//put this msdService into the given msdServiceCaneva by the MSD
						msdDescription = callbckInterface.newMSDDescription();
						CreationInformation ci = msdDescription.getCreaInfo();
						
						Plugin plugin = new PluginImpl();
						plugin.setName( super.name );
						plugin.setCreator( super.creator );
						msdDescription.getCreaInfo().setPlugin( plugin );
						
						msdDescription.setSvcDef(msdService.getSvcDef()); 
						msdDescription.setCtxInfo(msdService.getCtxInfo());
						
						board.info("add service for return");
						forReturn.add(msdDescription);
					}
					foundServices = null;
				}
				else {
					board.info("....NO services found");
				}
			}
			catch(Exception ex)
			{
				board.error("at Service search");
				ex.printStackTrace();
			}
			return forReturn;
		}
	}
	
	
	/**
	 * Search services on the local registry
	 * 
	 * @param upnpService
	 * @return
	 */
	public ProxyService[] getServiceFromLocalRegistry(UPnPServiceDescription upnpService)
	{
		Vector services;
		int deviceIndex;
		ProxyService[] forReturn = new ProxyService[0];
		Vector servicesFound = new Vector();
		String type = upnpService.getServiceType();
		String uuid = upnpService.getDeviceId();
		
		if (uuid!=null && !uuid.equals(""))
		{
			if (announcedDeviceID.contains(uuid))
			{
				//logger.debug("getServiceFromLocalRegistry: the device "+uuid+" is contained in this plugin");
				deviceIndex = announcedDeviceID.indexOf(uuid);
				if (deviceIndex>0) 
				{
					services = (Vector)serviceList.get(uuid); 	
					for (int i=0; i<services.size();i++)
					{
						if (((ProxyService)services.elementAt(i)).getServiceType().equals(type))
						{
							// 	return the service
							servicesFound.add((ProxyService)services.elementAt(i));
						}
					}
				}
			}
		}
		else 
		{
			for (int z=0; z<announcedDeviceID.size(); z++)
			{
				//board.info("FindService : local uuid="+(String)announcedDeviceID.elementAt(z));
				services = (Vector)serviceList.get((String)announcedDeviceID.elementAt(z)); 	
				for (int i=0; i<services.size();i++)
				{
					//board.info("FindService : local serviceType="+((ProxyService)services.elementAt(i)).getServiceType());
					if ( ((ProxyService)services.elementAt(i)).getServiceType().indexOf(type)!=-1)
					{
						// 	return the service
						servicesFound.add((ProxyService)services.elementAt(i));
					}
				}
			}
		}
		forReturn = new ProxyService[servicesFound.size()];
		for (int i=0; i< servicesFound.size(); i++)
			forReturn[i] = (ProxyService)servicesFound.elementAt(i);
		return forReturn;
	}
	
	/**
	 * 
	 * @param actions
	 * @param action1
	 * @return
	 */
	public boolean contains(Enumeration actions, UPnPAction action1)
	{
		Action action2;
		
		while (actions.hasMoreElements())
		{
			action2 = (Action)actions.nextElement();
			if (actionsEqual(action1, action2))
				return true;
		}
		return false;
	}
	
	/**
	 * 
	 * @param action1
	 * @param action2
	 * @return
	 */
	public boolean actionsEqual(UPnPAction action1, Action action2)
	{
		
		if (!action1.getName().equals(action2.getName())) return false;
		UPnPArgument[] arguments1 = action1.getArgumentList();
		ArgumentList inArgument2 = action2.getArguments().getInputArguments();
		ArgumentList outArgument2 = action2.getArguments().getOutputArguments();
		
		if (arguments1!= null && arguments1.length>0)
		{
			for (int i=0; i<arguments1.length;i++)
			{
				if (!contains(inArgument2, arguments1[i]) && !contains(outArgument2, arguments1[i])) return false;
			}
		}
		else if ((inArgument2!=null && inArgument2.size()>0) || (outArgument2!=null && outArgument2.size()>0)) 
		{
			return false;
		}
		return true;
	}
	
	/**
	 * 
	 * @param arguments
	 * @param argument1
	 * @return
	 */
	public boolean contains(ArgumentList arguments, UPnPArgument argument1)
	{
		Argument argument2;
		int i =0;
		while (i<arguments.size())
		{
			argument2 = (Argument)arguments.elementAt(i);
			if (argumentsEqual(argument1, argument2))
				return true;
			i++;
		}
		return false;
	}
	
	/**
	 * 
	 * @param argument1
	 * @param argument2
	 * @return
	 */
	public boolean argumentsEqual(UPnPArgument argument1, Argument argument2)
	{
		if (!argument1.getName().equals(argument2.getName())) return false;
		return true;
	}
	
	/**
	 * 
	 * @param index
	 * @param value
	 */
	public void insertTTL( int index, int value)
	{
		int[] ttlTmp = new int[400];
		for (int i=0; i<index; i++)
			ttlTmp[i]= announcedServiceTTL[i];
		ttlTmp[index] = value;
		for (int i=index+1; i< ttlTmp.length; i++)
			ttlTmp[i]= announcedServiceTTL[i-1];
		
		for (int i=0; i< announcedServiceTTL.length ;i++)
			announcedServiceTTL[i]= ttlTmp[i];
	}
	
	
	/**
	 * 
	 * @return
	 */
	public Thread ttlUpdating() 
	{
		try
		{
			Thread ttlUpdatingThread = new Thread(new Runnable(){
				public void run() 
				{
					int m;
					int listSize;
					while (activate)
					{
						try
						{
							Thread.sleep(InfoType.WAIT_TTL_UPDATING);
							
							m=0;
							listSize = announcedDeviceID.size();
							while (m<listSize)
							{
								if (announcedDeviceTTL[m]>=0) announcedDeviceTTL[m] = announcedDeviceTTL[m]-1000;
								
								if (announcedDeviceTTL[m]<0)
								{
									// service will be removed
									//board.info("-----------------------------deviceIndex = "+m+" will be removed");
									serviceList.remove((String)announcedDeviceID.elementAt(m));
									announcedDeviceID.removeElementAt(m);
									
									// update TTL
									for ( int i= m; i< announcedDeviceTTL.length-1; i++)
										announcedDeviceTTL[i]= announcedDeviceTTL[i+1];	
									
								}
								listSize = announcedDeviceID.size();
								m++;
							}
						}
						catch(Exception ex)
						{
							ex.printStackTrace();
						}
					}			
				}			
			});			
			return ttlUpdatingThread; 
		}
		catch (Exception ex)
		{
			board.error("Error at create ttlUpdating Thread");
		}
		return null;
	}
	
}
