/*
 * Created on March 20, 2005
 *
 * Comments, questions: 
 *	Pierre-Guillaume.Raverdy@inria.fr
 *	Rafik.Chibout@inria.fr
 *
 * INRIA, ARLES Project
 * 2004-2005
 */ 


package org.msda.plugins.upnp;

import org.msda.components.interfaces.TranslatorInterface;
import org.msda.components.plugins.Info;
import org.msda.msd.*;
import org.msda.msd.impl.*;

import com.siemens.upnp.Action;
import com.siemens.upnp.Argument;
import com.siemens.upnp.controlpoint.ProxyService;

import java.util.Calendar;
import java.util.Enumeration;
import java.util.List;
import java.util.Iterator;


/**
 * @author rafik CHIBOUT
 *
 */

public class MSDUPnPTranslator implements TranslatorInterface
{
	Info info = new Info();
	
	/**
	 *  
	 */
	public MSDUPnPTranslator()
	{
		info.setName("UbisecMSDUPnPTranslator");	
		info.setCreator("Rafik CHIBOUT");
		info.setDate("01/03/2005");
		info.setVersion("0.1");
		info.setDescription("This Translator allows you to translate an UPnP Description to a MSD Description and the reverse");
	}
	
	
	/*
	 * This method returns the creator-given information of the Transaltor  
	 * 
	 */
	public Info getTanslatorInfo()
	{
		return info;
	}
	
	/*
	 * 
	 * this method translates an MSDRequest in a protocole-specific request
	 * 
	 */
	
	public Object request(Object obj)
	{
		if (obj.getClass().getName().equals(String.class.getName()))
		{
			// XML String processing 
		}
		else
		{
			// Object instance processing
			MSDRequest request = ((MSDRequest)obj);
			ReqDescriptionInformation desc = request.getDescInfo();
			RequestService msdService = desc.getService();
			
			UPnPServiceDescription upnpService = new UPnPServiceDescription();
			
			String serviceID = msdService.getName();
						
			if (serviceID.indexOf("uuid")==0)
			{
				upnpService.setDeviceId(serviceID.substring(0, serviceID.indexOf(":",5)));
				upnpService.setServiceType(serviceID.substring(serviceID.indexOf(":",5)+1));
			}
			else
			{
				upnpService.setDeviceId("");
				upnpService.setServiceType(serviceID);
			}
			
			List methods = msdService.getMethods();
			int methodSize = methods.size();
			UPnPAction[] actionList = new UPnPAction[methodSize];
			int actionListPosition = 0;
			Iterator methodsIterator = methods.iterator();
			while (methodsIterator.hasNext())
			{
				Method method = (Method)methodsIterator.next();
				UPnPAction action = new UPnPAction();
				action.setName(method.getName());
				List inArguments = method.getIn();
				List outArguments = method.getOut();
				UPnPArgument[] argumentList = new UPnPArgument[inArguments.size()+outArguments.size()];
				int argumentListPosition =0;
				Iterator argumentIterator = inArguments.iterator();
				while(argumentIterator.hasNext())
				{
					Param param = (Param)argumentIterator.next();
					UPnPArgument upnpArgument = new UPnPArgument();
					upnpArgument.setName(param.getType());
					upnpArgument.setDirection(UPnPArgument.IN);
					argumentList[argumentListPosition]=upnpArgument;
					argumentListPosition++;
				}
				
				argumentIterator = outArguments.iterator();
				while(argumentIterator.hasNext())
				{
					Param param = (Param)argumentIterator.next();
					UPnPArgument upnpArgument = new UPnPArgument();
					upnpArgument.setName(param.getType());
					upnpArgument.setDirection(UPnPArgument.OUT);
					argumentList[argumentListPosition]=upnpArgument;
					argumentListPosition++;
				}
				action.setArgumentList(argumentList);
				actionList[actionListPosition]= action;
				actionListPosition++;
			}
			upnpService.setActionList(actionList);
			return upnpService;
		}
		return null;
	}
	
	/*
	 * this method translates a protocole-specific response in an MSDDescription
	 */
	
	public Object response(Object obj)
	{
	    print("/////////////////////// response ////////////////////////////////////" + obj.toString());
	    
		if (obj.getClass().getName().equals(String.class.getName()))
		{
			// XML String processing 
			// not yet implemented
		}
		else
		{
		if (obj.getClass().getName().equals(UPnPDeviceDescription.class.getName()))
		{
			// UPnPDevice Casting
			UPnPDeviceDescription upnpDevice = (UPnPDeviceDescription)obj;
			UPnPServiceDescription[] upnpServiceList = upnpDevice.getServiceList();
			// select the requiered service
			if (upnpServiceList!= null && upnpServiceList.length>0)
			{
				UPnPServiceDescription upnpService = upnpServiceList[0];
				MSDDescription msdService = new MSDDescriptionImpl();
				ServiceDefinition msdServiceDefinition = new ServiceDefinitionImpl();
				Original msdOriginal = new OriginalImpl();
				msdOriginal.setProtocolName("UPnP");
				msdOriginal.setDesc("UPnP Siemens Implementation");
				Generic msdGeneric = new GenericImpl();
				// set all the infos that are possible
				String serviceName = upnpService.getDeviceId()+":"+upnpService.getServiceType();
				msdGeneric.setName(serviceName);
				
                // address
				Address address = new AddressImpl();
				address.setLocation(upnpService.getControlURL());
				msdGeneric.setAddr(address);
				// binding 
				Binding binding = new BindingImpl();
				binding.setEncoding("XML 1.0");
				binding.setTransport("HTTP/SOAP");
				msdGeneric.setBinding(binding);
				// attributes
				Param attribute = new ParamImpl();
				List attributes = msdGeneric.getParams();
				attributes.add(attribute);
				
				UPnPAction[] actions = upnpService.getActionList();
				List methods = msdGeneric.getMethods();
								
				for (int i= 0; i< actions.length; i++)
				{
					UPnPAction action = actions[i];
					Method method = new MethodImpl();
					method.setName(action.getName());
					UPnPArgument[] inArgumentsEnum = action.getArgumentList();
					UPnPArgument[] outArgumentsEnum = action.getArgumentList();
					List inArguments = method.getIn();
					List outArguments = method.getOut();
					
					if (inArgumentsEnum!=null && inArgumentsEnum.length>0)
					for (int j=0; j<inArgumentsEnum.length; j++)
					{
						UPnPArgument argument = inArgumentsEnum[j];
						Param param = new ParamImpl();
						param.setType(argument.getName());
						param.setValue("");
						inArguments.add(param);
					}
					else 
					{
					// when the isn't any input parameters
					Param param = new ParamImpl();
					inArguments.add(param);	
					}
					
					
					if (outArgumentsEnum!=null && outArgumentsEnum.length>0)
					for (int j=0; j<outArgumentsEnum.length; j++)
					{
						UPnPArgument argument = outArgumentsEnum[j];
						Param param = new ParamImpl();
						param.setType(argument.getName());
						param.setValue("");
						outArguments.add(param);
					}
					else 
					{
					// when the isn't any output parameters
						Param param = new ParamImpl();
						outArguments.add(param);	
					}
					methods.add(method);
				}
				
				// creation time
				Calendar currentTime = Calendar.getInstance();
				currentTime.setTimeInMillis(System.currentTimeMillis());
				msdGeneric.setCreaTime(currentTime);
				
				//Expected life time
				Calendar expectedLifeTime = Calendar.getInstance();
				expectedLifeTime.setTimeInMillis(System.currentTimeMillis()+30000);
				msdGeneric.setExpectedLifetime(expectedLifeTime);
				
				msdServiceDefinition.setOriginal(msdOriginal);
				msdServiceDefinition.setGeneric(msdGeneric);
				msdService.setSvcDef(msdServiceDefinition);
				
				//Extract context information
				ContextInformation ctx = new ContextInformationImpl();
				ContextInformationTable serviceCtx = new ContextInformationTableImpl();
				MSDUPnPContextTranslator.extractContext(upnpService, serviceCtx);
				//print("contextTranslator has finished");
				ctx.setCISvc( serviceCtx );
				msdService.setCtxInfo(ctx);
				//print("context info has been added to the MSD description");
				//print("context information is: " + msdService.getCtxInfo());
				return msdService;
			}
			return null;
		}
		else
			if (obj.getClass().getName().equals(UPnPServiceDescription.class.getName()))
			{
				//UPnPService Casting
				UPnPServiceDescription upnpService =(UPnPServiceDescription)obj;
				MSDDescription msdService = new MSDDescriptionImpl();
				// will be changed and use the caneva given by MSD
				ServiceDefinition msdServiceDefinition = new ServiceDefinitionImpl();
				Original msdOriginal = new OriginalImpl();
				msdOriginal.setProtocolName("UPnP");
				msdOriginal.setDesc("UPnP Siemens Implementation");
				Generic msdGeneric = new GenericImpl();
				// set all the infos that are possible
				String serviceName = upnpService.getDeviceId()+":"+upnpService.getServiceType();
				msdGeneric.setName(serviceName);
				
				// address
				Address address = new AddressImpl();
				msdGeneric.setAddr(address);
				// binding 
				Binding binding = new BindingImpl();
				binding.setEncoding("XML 1.0");
				binding.setTransport("HTTP/SOAP");
				msdGeneric.setBinding(binding);
				// attributes
				Param attribute = new ParamImpl();
				List attributes = msdGeneric.getParams();
				attributes.add(attribute);
				
				
				UPnPAction[] actions = upnpService.getActionList();
				List methods = msdGeneric.getMethods();
								
				for (int i= 0; i< actions.length; i++)
				{
					UPnPAction action = actions[i];
					Method method = new MethodImpl();
					method.setName(action.getName());
					UPnPArgument[] inArgumentsEnum = action.getArgumentList();
					UPnPArgument[] outArgumentsEnum = action.getArgumentList();
					List inArguments = method.getIn();
					List outArguments = method.getOut();
					
					if (inArgumentsEnum!= null && inArgumentsEnum.length>0)
					for (int j=0; j<inArgumentsEnum.length; j++)
					{
						UPnPArgument argument = inArgumentsEnum[j];
						Param param = new ParamImpl();
						param.setType(argument.getName());
						param.setValue("");
						inArguments.add(param);
					}
					else 
					{
					// when the isn't any input parameters
						Param param = new ParamImpl();
						inArguments.add(param);	
					}
					
					if (outArgumentsEnum!= null && outArgumentsEnum.length>0)
					for (int j=0; j<outArgumentsEnum.length; j++)
					{
						UPnPArgument argument = outArgumentsEnum[j];
						Param param = new ParamImpl();
						param.setType(argument.getName());
						param.setValue("");
						outArguments.add(param);
					}
					else 
					{
					// when the isn't any output parameters
					Param param = new ParamImpl();
					outArguments.add(param);	
					}
					methods.add(method);
				}
				
				//	creation time
				Calendar currentTime = Calendar.getInstance();
				currentTime.setTimeInMillis(System.currentTimeMillis());
				msdGeneric.setCreaTime(currentTime);
				
				//Expected life time
				Calendar expectedLifeTime = Calendar.getInstance();
				expectedLifeTime.setTimeInMillis(System.currentTimeMillis()+30000);
				msdGeneric.setExpectedLifetime(expectedLifeTime);
				
				msdServiceDefinition.setGeneric(msdGeneric);
				msdServiceDefinition.setOriginal(msdOriginal);
				msdService.setSvcDef(msdServiceDefinition);
				
				// Extract context information
				ContextInformation ctx = new ContextInformationImpl();
				ContextInformationTable serviceCtx = new ContextInformationTableImpl();
				MSDUPnPContextTranslator.extractContext(upnpService, serviceCtx);
				//print("contextTranslator has finished");
				ctx.setCISvc( serviceCtx );
				msdService.setCtxInfo(ctx);
				//print("context info has been added to the MSD description");
				//print("context information is : " + msdService.getCtxInfo());
//				print("printer state from serviceCtx " + serviceCtx.getServiceState());
//				print("printer state from msdServiceDescription " + msdService.getCtxInfo().getServiceState());
//				print("printer state from msdServiceDescription " + msdService.getCtxInfo().getJobName());
//				
				return msdService;
			}
			else
				if (obj.getClass().getName().equals(ProxyService.class.getName()))
				{
					//UPnPService Casting
					ProxyService upnpService =(ProxyService)obj;
					MSDDescription msdService = new MSDDescriptionImpl();
					// will be changed and use the caneva given by MSD
					ServiceDefinition msdServiceDefinition = new ServiceDefinitionImpl();
					Original msdOriginal = new OriginalImpl();
					msdOriginal.setProtocolName("UPnP");
					msdOriginal.setDesc("UPnP Siemens Implementation");
					Generic msdGeneric = new GenericImpl();
					// set all the infos that are possible
					String serviceName = upnpService.getDevice().getUDN()+":"+upnpService.getServiceType();
					msdGeneric.setName(serviceName);
					
					// address
					Address address = new AddressImpl();
					msdGeneric.setAddr(address);
					address.setLocation(upnpService.getControlURL());
					// binding 
					Binding binding = new BindingImpl();
					binding.setEncoding("XML 1.0");
					binding.setTransport("HTTP/SOAP");
					msdGeneric.setBinding(binding);
					// attributes
					Param attribute = new ParamImpl();
					List attributes = msdGeneric.getParams();
					attributes.add(attribute);
										
					Enumeration actions = upnpService.getActions().elements();
					List methods = msdGeneric.getMethods();
									
					while (actions.hasMoreElements())
					{
						Action action = (Action)actions.nextElement();
						Method method = new MethodImpl();
						method.setName(action.getName());
						Enumeration inArgumentsEnum = action.getArguments().getInputArguments().elements();
						Enumeration outArgumentsEnum = action.getArguments().getOutputArguments().elements();
						List inArguments = method.getIn();
						List outArguments = method.getOut();
						if (inArgumentsEnum.hasMoreElements())
						while (inArgumentsEnum.hasMoreElements())
						{
							Argument argument = (Argument)inArgumentsEnum.nextElement();
							Param param = new ParamImpl();
							param.setType(argument.getName());
							param.setValue("");
							inArguments.add(param);
						}
						else 
						{
						// when the isn't any input parameters
							Param param = new ParamImpl();
							inArguments.add(param);	
						}
						if (outArgumentsEnum.hasMoreElements())							
						while (outArgumentsEnum.hasMoreElements())
						{
							Argument argument = (Argument)outArgumentsEnum.nextElement();
							Param param = new ParamImpl();
							param.setType(argument.getName());
							param.setValue("");
							outArguments.add(param);
						}
						else 
						{
						// when the isn't any output parameters
							Param param = new ParamImpl();
							outArguments.add(param);	
						}

						methods.add(method);
					}
					
					// creation time
					Calendar currentTime = Calendar.getInstance();
					currentTime.setTimeInMillis(System.currentTimeMillis());
					msdGeneric.setCreaTime(currentTime);
					
					//Expected life time
					Calendar expectedLifeTime = Calendar.getInstance();
					expectedLifeTime.setTimeInMillis(System.currentTimeMillis()+30000);
					msdGeneric.setExpectedLifetime(expectedLifeTime);
					
					msdServiceDefinition.setOriginal(msdOriginal);
					msdServiceDefinition.setGeneric(msdGeneric);
					msdService.setSvcDef(msdServiceDefinition);
					
					// Extract context information
					ContextInformation ctx = new ContextInformationImpl();
					ContextInformationTable serviceCtx = new ContextInformationTableImpl();
					MSDUPnPContextTranslator.extractContext(upnpService, serviceCtx);
					//print("contextTranslator has finished");
					ctx.setCISvc( serviceCtx );
					msdService.setCtxInfo(ctx);
//					print("context info has been added to the MSD description");
//					print("context information is : " + msdService.getContextInformation());
//					print("printer state from serviceCtx " + serviceCtx.getServiceState());
//					print("printer state from msdServiceDescription " + msdService.getContextInformation().getServiceState());
//					print("printer state from msdServiceDescription " + msdService.getContextInformation().getJobName());
					
					return msdService;
				}
		}
		return null;
	}
	
	
	
	/* 
     * to print debug information
    */
	private static boolean debug = true;	
	public static void print(String o){
	    if(debug){
	        //board.info("[MSDUPnPTranslator: ] " + o);
	    }
	}
	
	
}
