/*
 * Created on July 16, 2005
 * 
 * UPC
 * juan
 * 
 * 
 * Preliminary version, please do not distribute
 * 
 * Copyright UPC, INRIA 2005
 */
package org.msda.util.security;

import java.io.IOException;
import java.net.DatagramPacket;
import javax.xml.bind.JAXBException;

import org.msda.msd.MSDMessage;
import org.msda.util.MSDException;
import org.msda.util.comm.MulticastGroup;
import org.msda.util.comm.MulticastGroupCallback;

/** This class extends MulticastGroup adding security to the communication.
 * Every message sent through this multicast group will be signed with the
 * group key to ensure authentication and integrity.
 * @author juan
 */
public class SecureMulticastGroup extends MulticastGroup {
	/** The utility to use for encoding and signing */
	private SecurityUtil secUtil;
	/** The constructor of the multicatr group.
	 * @param callback
	 * @param ifname The name of the interface
	 * @param groupAddress The IP address of the group to use.
	 * @param groupPort The port to listen to remote communications
	 * @throws MSDException If anything goes wrong.
	 */
	public SecureMulticastGroup(MulticastGroupCallback callback, String ifname,
			String groupAddress, int groupPort) throws MSDException {
		super(callback, ifname, groupAddress, groupPort);
		// get an instance of the security util linked to the name of
		// the interface.
		secUtil = SecurityUtil.getInstance(ifname);
	}
	
	/** Send a messato to the multicast group.
	 * @param mess The message to be sent.
	 */
	public int sendMSDMessage ( MSDMessage mess ) 
	{
		//	If the MSDMessage has already a checksum, remove it.
		if ( mess.getChecksum()!=null ) 
			mess.setChecksum( null );
		
		try {
			// first I marshall the meta description into an XML file
			byte[] xml =this.jaxme.toString(mess).getBytes();
			
			// second I generate the checksum
			byte[] checksum = secUtil.createMac(xml);
			
			// now I build the beacon from the XML file and send it onto the socket
			byte[] msg = new byte[3+checksum.length+xml.length];
			msg[0]='M'; msg[1]='A'; msg[2]='C';
			System.arraycopy(checksum,0,msg,3,checksum.length);
			System.arraycopy(xml,0,msg,3+checksum.length,xml.length);
			
			DatagramPacket beacon = new DatagramPacket(msg, msg.length, multicastGroup, groupPort);
			multicastSock.send(beacon);
			
		} catch (IOException e) {
			if (board!=null) board.notifyError( "MulticastGroup::sendMSDMessage : IO exception when sending " + e.getMessage() );
//			ExceptionTool.getInstance().printStackTrace( board.getLogger(), e );
			e.printStackTrace();
		} catch (SecurityException e) {
			if (board!=null) board.notifyError( "MulticastGroup::sendMSDMessage : ERROR WHEN GENERATING THE CHECKSUM " + e.getMessage() );
//			ExceptionTool.getInstance().printStackTrace( board.getLogger(), e );
			e.printStackTrace();
		} catch (JAXBException e) {
			if (board!=null) board.notifyError( "MulticastGroup::sendMSDMessage : ERROR WHEN MARSHALLING " + e.getMessage() );
//			ExceptionTool.getInstance().printStackTrace( board.getLogger(), e );
			e.printStackTrace();
		}
		return 0;
	}
	
	/** Checks the signing of a message.
	 * @param mess The byte array containing the message
	 * @param rvc_mac The expected mac. If null, return false always.
	 * @return Whether the mac is equal or not.
	 */
	private boolean checkMac(byte[] mess, byte[] rvc_mac) {
		if(rvc_mac==null)
			return false;
		// calculate the MAC and check if it is equal
		byte[] calc_mac=secUtil.createMac(mess);
		System.out.println("Received MAC:\t"+secUtil.transformToString(rvc_mac));
		System.out.println("Calculated MAC:\t"+secUtil.transformToString(calc_mac));
		// first check its length
		if(calc_mac.length!=rvc_mac.length)
			return false;
		// and check byte by byte of each MAC
		for(int i=0; i<calc_mac.length; i++)
			if(calc_mac[i]!=rvc_mac[i])
				return false;
		return true;
	}
	
	/** Converts a byte array to a message.
	 * The MSDMessage object created from the byte array is sent to
	 * the MulticastGroupCallback through te receivedMessage method.
	 * 
	 * This method does nothing with the messages not verified. Maybe
	 * we should mark them with an internal code for the upper layers of
	 * the stack protocol.
	 * 
	 * @param res The byte array to convert
	 * @throws JAXBException If the byte array can ot be converted
	 */
	void unmarshallMSDMessage( byte[] res ) throws JAXBException {
		byte[] actualRes=res;
		boolean valid=false;
		// test if the first bytes of the message are a signing.
		if(res.length>3&&res[0]=='M'&&res[1]=='A'&&res[2]=='C'){
			System.out.println("Has MAC");
			// get the mac of the message
			int macLength=secUtil.getMacLength();
			byte[] mac=new byte[macLength];
			System.arraycopy(res,3,mac,0,macLength);
			
			// set the actualRes
			actualRes=new byte[res.length-3-macLength];
			System.arraycopy(res, 3+macLength, actualRes,0,actualRes.length);
			// try to validate the mac
			valid=checkMac(actualRes,mac);
		}
		
		// actions if the message is valid
		if(!valid)
			System.out.println("ALERT! Verification of checksum failed!");
		else
			System.out.println("Beacon verified: MAC is correct");
		
		// create the message and forward it to the receiver
		MSDMessage mess = jaxme.toMSDMessage(actualRes);
		if (callback != null) callback.receivedMessage( mess );
		mess = null;
	}	
	
}
