/*
 * Created on June 13, 2005
 *
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr<br>
 *          Rafik.Chibout@inria.fr
 * 
 * Copyright INRIA 2005
 */
package org.msda.clientsamples.pdp.rfid;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.util.Vector;

import javax.swing.JOptionPane;

import org.apache.log4j.BasicConfigurator;
import org.msda.MSDClientAPIService.MSDClientAPIBindingStub;
import org.msda.msd.Method;
import org.msda.msd.Param;
import org.msda.msd.ReqDescriptionInformation;
import org.msda.msd.RequestService;
import org.msda.msd.impl.MSDDescriptionImpl;
import org.msda.msd.impl.MSDRequestImpl;
import org.msda.msd.impl.ReqDescriptionInformationImpl;
import org.msda.msd.impl.RequestServiceImpl;


import com.uc3m.it.pfc.pdp.PDP;
import com.uc3m.it.pfc.pdp.ServiceEntry;


/**
 * @author Rafik CHIBOUT
 *
 */
public class ClientRFID extends Frame {

    public static final Color COLOR = new Color( 230,230,180 );
    
    public static final int WIDTH = 240;
    public static final int HEIGHT = 240;
    
    private AccessPanel accessPanel;
    private SearchPanel searchPanel;
    
    private RFIDMusicPortType rfidService;
    private PDP pdp = null;
	
    public ClientRFID() {
        
        super( " Client RFID " );
        
        try
		{
		 // create a pdp Instance
			pdp = new PDP ( 10, 10, 60000,"" );
			pdp.setUA    ( true ); // Sets User Agent on
			pdp.setSA    ( true ); // Sets Service Agent on
			pdp.setDereg ( true ); // Sets PDP to inform when leaving the network
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
			System.exit(0);
		}
        
        setSize( WIDTH, HEIGHT );     
        setBackground( ClientRFID.COLOR );
        
        searchPanel = new SearchPanel( this );
        //searchPanel.setSize();
        accessPanel = new AccessPanel( this );
        
        add( searchPanel, BorderLayout.NORTH );
        add( accessPanel, BorderLayout.CENTER );
        
//      To close the application
        addWindowListener(new WindowAdapter() { 
            public void windowClosing(WindowEvent e) { 
                setVisible(false); 
                dispose(); 
                System.exit(0); 
            } 
        }); 
    }
    
    
    
    /**
     * @return Returns the accessPanel.
     */
    public AccessPanel getAccessPanel() {
        return accessPanel;
    }
    /**
     * @return Returns the searchPanel.
     */
    public SearchPanel getSearchPanel() {
        return searchPanel;
    }
    
    
    public void findPDPService()
    {
    	// find URL of MSDService
    	
    	BasicConfigurator.configure();
    	String ctrUrl = "";
    	
    	try
    	{
    		String serviceType = "uc3mwebcamserver";
    		System.out.println("Searching in PDP "+serviceType);
    		
    		Object lock = pdp.searchService(serviceType, true);
    		
    		synchronized (lock)
			{
				lock.wait ();
			}
			
    		Vector result = pdp.getResponse ( );
		    
			if (result != null && result.size()>0)
			{
				ServiceEntry foundService = (ServiceEntry)result.elementAt(0);
				ctrUrl = foundService.getPDPService().getURL();
    			System.out.println("clientPDP: PDP Service URL="+ctrUrl);
    		}
			else 
				System.out.println("PDP Service not found");
     
    	}
    	catch(Exception ex)
    	{
    		ex.printStackTrace();
    	}
    	
    	
    }
    
    public void findService()
    {
    	// find URL of MSDService
    	
    	BasicConfigurator.configure();
    	String ctrUrl = "";
    	
    	
    	try
    	{
    		Object lock = pdp.searchService("msdservice", true);
    		
    		synchronized (lock)
			{
				lock.wait ();
			}
			
    		Vector result = pdp.getResponse ( );
		    
			if (result != null && result.size()>0)
			{
				ServiceEntry foundService = (ServiceEntry)result.elementAt(0);
				ctrUrl = foundService.getPDPService().getURL();
    			System.out.println("clientPDP: MSDService URL="+ctrUrl);
    		}
     
    	}
    	catch(Exception ex)
    	{
    		ex.printStackTrace();
    	}
    	
    	
    	org.msda.MSDClientAPIService.MSDClientAPIPortType service = new MSDClientAPIBindingStub(ctrUrl);

    	
    	MSDRequestImpl msdRequest = new MSDRequestImpl();
    	ReqDescriptionInformation reqDescription = new ReqDescriptionInformationImpl();
    	
    	RequestService requestService = new RequestServiceImpl();

    	requestService.setName("uc3mwebcamserver");
        reqDescription.setService(requestService);
        msdRequest.setDescInfo(reqDescription);
        
        try
    	{
        MSDDescriptionImpl[] msdDescs = service.getServices(msdRequest);
        System.out.println("service invoked");
        if (msdDescs != null)
        {
        	if (msdDescs.length>0)
        	{
        		MSDDescriptionImpl desc = msdDescs[0];
        		
        		String epService = desc.getSvcDef().getGeneric().getAddr().getLocation();
        		System.out.println("epService ="+desc.getSvcDef().getGeneric().getAddr().getLocation());
        		
        		java.util.List methods = desc.getSvcDef().getGeneric().getMethods();
    			for (int d=0; d<methods.size();d++)
    			{
    				Method method = (Method)methods.get(d);
    				System.out.println("msdService methodName="+method.getName());
    				java.util.List paramIn = method.getIn();
    				java.util.List paramOut = method.getOut();
    				
    				for (int k=0; k<paramIn.size(); k++)
    				{	
    					Param param = (Param)paramIn.get(k);
    					System.out.println("msdService in Param"+param.getValue());
    				}
    		
    				for (int u= 0 ;u<paramOut.size(); u++)
    				{
    					Param param = (Param)paramOut.get(u);
    					System.out.println("msdService out Param"+param.getValue());
    				}
    			}
    			System.out.println("*******************************");
        		
    			
    			// getClient ID function
    			//int id = service.getClientID(desc);
    			//System.out.println("id = "+id);

    			//rfidService = new RFIDMusicBindingStub(epService, ctrUrl, id);
    			
    			System.out.println("RFID Service is ready");
    			//if (cpt) 
                 //   JOptionPane.showMessageDialog(this,"Successfully connected to the service","Connection",JOptionPane.INFORMATION_MESSAGE);
                //cpt = true;
                //MovieFind.setEnabled(true);
                //MovieAll.setEnabled(true);
    			
    			
        	}
        	else System.out.println("the result is Empty");
        }
        else System.out.println("the result is null");
    	}
        catch(Exception ex)
    	{
        	ex.printStackTrace();
            JOptionPane.showMessageDialog(this,"Can't connect to Service","Error",JOptionPane.ERROR_MESSAGE);
    	}
    }

    
    public static void main(String[] args) {
        
        ClientRFID client = new ClientRFID();
        client.setVisible(true);
        //client.findService();
    }
}
