/*
 * Created on June 13, 2005
 *
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr<br>
 *          Rafik.Chibout@inria.fr
 * 
 * Copyright INRIA 2005
 */
package org.msda.clientsamples.slp.rfid;


import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;
import org.msda.MSDClientAPIService.MSDClientAPIBindingStub;
import org.msda.context.ContextUtil;
import org.msda.context.ContextVocabulary;
import org.msda.context.NetworkTemplate;
import org.msda.context.ServiceTemplate;
import org.msda.context.UserTemplate;
import org.msda.msd.*;
import org.msda.msd.impl.*;
import org.msda.util.InfoType;
import org.msda.util.JaxMeUtil;

import com.solers.slp.Locator;
import com.solers.slp.ServiceLocationEnumeration;
import com.solers.slp.ServiceLocationManager;
import com.solers.slp.ServiceType;
import com.solers.slp.ServiceURL;



import java.io.IOException;
import java.math.BigInteger;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.*;

import javax.xml.bind.JAXBException;

/**
 * @author rafik CHIBOUT
 */

public class clientSLPSocket {
    
    final static Logger logger = Logger.getLogger(clientSLPSocket.class);
    
    private org.msda.MSDClientAPIService.MSDClientAPIPortType msdService;
    private String ctrUrlMSD;
    private void findMSDService() {
        
        
        try
		{
    		Locator locator = ServiceLocationManager.getLocator(Locale.FRANCE);
    		ServiceType serviceType = new ServiceType("service:msdservice");
    		Vector scopes = new Vector(); 
    		scopes.add("default");
//    		System.out.println("BEFORE foundServices");
    		ServiceLocationEnumeration foundServices = locator.findServices(serviceType, scopes, "");
//    		if ( foundServices==null )
//    			System.out.println("AFTER foundServices is null");
    		if (foundServices !=null)
    		{
				System.out.println("msd service found");
    			if (foundServices.hasMoreElements())
    			{
    				ServiceURL foundService = (ServiceURL)foundServices.next();
    				ctrUrlMSD = "http://"+foundService.getHost()+":"+foundService.getPort()+foundService.getURLPath();
    				System.out.println("clientSLP: MSDService URL="+ctrUrlMSD);
    				
    				msdService = new MSDClientAPIBindingStub(ctrUrlMSD);
    			}
    		}
   			else
   			{
				System.out.println("msd service not found");
   			}    				
    			
		}
    	catch(Exception ex)
		{
    		ex.printStackTrace();
		}
    }
    
    public void callService()
    {
        // find URL Of MSDService
        
        BasicConfigurator.configure();
        String ctrUrl = "";
        
        String address = "";
        int port = -1;
        
        try
        {
            Locator locator = ServiceLocationManager.getLocator(Locale.FRANCE);
            ServiceType serviceType = new ServiceType("service:directmsdservice");
            Vector scopes = new Vector(); 
            scopes.add("default");
            
            ServiceLocationEnumeration foundServices = locator.findServices(serviceType, scopes, "");
            if (foundServices !=null) {
                if (foundServices.hasMoreElements())
                {
                    ServiceURL foundService = (ServiceURL)foundServices.next();
                    ctrUrl = "http://"+foundService.getHost()+":"+foundService.getPort()+foundService.getURLPath();
                    System.out.println("clientSLP: DirectMSDService URL="+ctrUrl);
                    
                    address = foundService.getHost();
                    port = foundService.getPort();
                }
            }
                
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
        }
        

        try {
            
            Socket connection = new Socket(address, port) ;
            
            UbisecID id = new UbisecIDImpl();
            id.setName( "ClientSLPSocket" );
            
            MSDRequestImpl msdRequest = new MSDRequestImpl();
            
            ReqProcessingInformation rpi = new ReqProcessingInformationImpl();
            ReqCreationInformation rci = new ReqCreationInformationImpl();
	        Creator creator = new CreatorImpl();
	        creator.setMgrID( id );
	        creator.setNetID( id );
	        Calendar cal = Calendar.getInstance();
	        rci.setDate( cal );
	        rci.setTime( cal );
	        rci.setLifetime( BigInteger.valueOf( 5 ) );// in secondes
	        rci.setCreator( creator );
	        rpi.setCrea( rci );
            
            RequestService requestService = new RequestServiceImpl();
            requestService.setName("videostreaming");//urn:printbasic1");//RFIDMusicService");
            
            ReqDescriptionInformation reqDescription = new ReqDescriptionInformationImpl();
            reqDescription.setService(requestService);
            msdRequest.setDescInfo(reqDescription);
            msdRequest.setProcInfo( rpi );
            
            PropagationInformation pi = new PropagationInformationImpl();
            msdRequest.setPropaInfo( pi );
            
            msdRequest.setClientID( BigInteger.valueOf( -1 ) );
    	    msdRequest.setReqID( BigInteger.valueOf( -1 ) );
            
    	    // ContextRules   HOP < 10
        	TypeValue compValue = new TypeValueImpl();
        	compValue.setInt( BigInteger.valueOf(10) );    		 	
        	ComparisonNode compNode = new ComparisonNodeImpl();	
        	compNode.setName(NetworkTemplate.HOP);
        	compNode.setOp("L");
        	compNode.setVal(compValue);
        	ContextNode ctxNode = new ContextNodeImpl();
        	ctxNode.setCa(compNode);
        	
        	ContextRulesTree crNet = new ContextRulesTreeImpl();
        	crNet.setRoot(ctxNode);
        	
        	compValue = new TypeValueImpl();
        	compValue.setStr( ContextVocabulary.BUSY );    
        	compNode = new ComparisonNodeImpl();	
        	compNode.setName(ServiceTemplate.SERVICE_STATE);
        	compNode.setOp("!=");
        	compNode.setVal(compValue);
        	ctxNode = new ContextNodeImpl();
        	ctxNode.setCa(compNode);
        	
        	ContextRulesTree crUser = new ContextRulesTreeImpl();
        	crUser.setRoot(ctxNode);
        	
        	ContextRules ctxRule = new ContextRulesImpl();   
        	ctxRule.setCRNet(crNet);       	
        	ctxRule.setCRUser(crUser);
        	
    	    msdRequest.setCtxRules( ctxRule );
    	    
    	    // ContextInformation
    	    ContextInformationTable ciNet = new ContextInformationTableImpl();
    	    ContextElement elt = ContextUtil.getInstance().createCtxElement( NetworkTemplate.HOP, 0 );
            ciNet.getCITable().add( elt );
    	    
    	    ContextInformationTable ciUser = new ContextInformationTableImpl();
    	    elt = ContextUtil.getInstance().createCtxElement( UserTemplate.DEVICE_NAME, "SmartPhoneNum1" );
    	    ciUser.getCITable().add( elt );
    	    
    	    ContextInformation ci = new ContextInformationImpl();
    	    ci.setCINet( ciNet );
    	    ci.setCIUser( ciUser );
    	    
    	    msdRequest.setCtxInfo(ci);
    	    
            MessageContent content = new MessageContentImpl();
            content.setRequest( msdRequest );
            
            MSDMessage mess = new MSDMessageImpl();
            mess.setSender( id );
            mess.setType( BigInteger.valueOf( InfoType.Message.CLIENT_NEW_REQUEST ) );
            mess.setContent( content );
            
            JaxMeUtil jaxme = new JaxMeUtil();
            String message = jaxme.toString( mess );
            
            System.out.println( "Send sth to the manager ..." );
            System.out.println( message );
            connection.getOutputStream().write( message.getBytes() );
            System.out.println( "done" );
            
            byte[] buf = new byte[InfoType.MSG_BUFFER_SIZE];
            long start = System.currentTimeMillis();
            System.out.println( "wait for the response..." );
            int size = connection.getInputStream().read( buf );
            long end = System.currentTimeMillis() - start;
            System.out.println("get a result of "+size +" after "+end);
            if ( size < 0) 
                return;
            
            // Resize the byte[] to keep only the used bytes
            byte[] tmp = buf; 
            buf = new byte[size];
            System.arraycopy(tmp, 0, buf, 0, size);
            
            
            System.out.println( new String(buf));
            
//            findMSDService();
//            if ( msdService!=null ) {
//                System.out.println( "MSD service found" );
//                
//                MSDMessage found = jaxme.toMSDMessage( buf );
//                
//                // take the first result...
//                MSDDescription desc = (MSDDescription)found.getContent().getResults().get(0);
//                
//                if ( desc.getDescriptionID().getName().equals( InfoType.TIMEOUT_EXPIRED )) {
//                    System.out.println("Not found !");
//                    return;
//                }
//                    
//                String epService = desc.getServiceDefinition().getGeneric().getAddress().getLocation();
//                String msdDescId = desc.getDescriptionID().getName();
//                epService = epService + "/" + msdDescId;
//                
//                RFIDMusicBindingStub rfidService = new RFIDMusicBindingStub(epService, ctrUrlMSD);
//                
//                String res = rfidService.getInfo( "4-010165-314153" );
//                
//                System.out.println( res );
//            }
            
        } catch (JAXBException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
            
        } catch (UnknownHostException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        
    }
    
    public static void main(String[] args) {
        
        clientSLPSocket app = new clientSLPSocket();
        app.callService();
    }
}
