/*
 * Created on April 18, 2005
 *
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr<br>
 *          Rafik.Chibout@inria.fr
 * 
 * Copyright INRIA 2005
 */
package org.msda.clientsamples.upnp;


import org.msda.clientsamples.slp.rfid.RFIDMusicBindingStub;
import org.msda.msd.*;
import org.msda.msd.impl.*;
import org.msda.util.InfoType;
import org.msda.util.JaxMeUtil;




import com.siemens.upnp.UPnPConstants;
import com.siemens.upnp.controlpoint.ControlPoint;

import com.siemens.upnp.controlpoint.ProxyDevice;
import com.siemens.upnp.controlpoint.ProxyService;
import com.siemens.upnp.controlpoint.UPnPSearchResponseListener;


import java.io.BufferedInputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.net.HttpURLConnection;
import java.net.Socket;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.*;

import javax.xml.bind.JAXBException;

/**
 * @author delachap
 */

public class ClientUPnPSocket extends AbstractClientUPnP {
	
public void callService()
{
    // DID NOT USE the method AbstractClientUPnP.findUPnPService()
	
	// init the CP
	ControlPoint controlPoint = ControlPoint.getInstance();
	DeviceDesc deviceDesc = new DeviceDesc();
	UPnPSearchResponseListener searchResponseListener = new UPnPSearchResponseListenerImpl(deviceDesc);
	controlPoint.addUPnPSearchListener(searchResponseListener);
	
	// search MSDService
	
	String ctrUrl = "";
	
	try
	{

	controlPoint.search(UPnPConstants.SearchForServiceType("directmsdservice:1:0"));
	while (deviceDesc.getDescriptionURL() == null ) {}
	System.out.println("DescrURL = "+deviceDesc.getDescriptionURL());
	URL url = new URL(deviceDesc.getDescriptionURL());
		// show the Description
		
		HttpURLConnection conn = (HttpURLConnection)url.openConnection();
		BufferedInputStream reader = new BufferedInputStream(conn.getInputStream());
		byte	[] buff = new byte[1000];
		while (reader.available()>0)
		{
			int num= reader.read(buff,0,buff.length);
			System.out.print(new String(buff,0,num));
		}
		System.out.println("");
		System.out.println("**************");
		
		System.out.println("Level 1");
		ProxyDevice device = controlPoint.createDevice(url);
		System.out.println("Level 2: Device created");
		java.util.Vector services = device.getServices();
		
		System.out.println("Level 3: service proxy get");
		if (services != null && services.size()>0)
		{
			ProxyService service = (ProxyService)services.elementAt(0);
			ctrUrl = service.getControlURL();
		}
		System.out.println("Level 5");  
	}
	catch(Exception ex)
	{
		ex.printStackTrace();
	}
	
	
	System.out.println("Found ctrURL="+ctrUrl);
	
	int index = ctrUrl.lastIndexOf(":");
	String address = ctrUrl.substring( ctrUrl.lastIndexOf("/")+1, index );
	int port = Integer.parseInt( ctrUrl.substring( index+1 ) );
	
	try {
	    System.out.println(" try to connect on "+address+" "+port);
        Socket connection = new Socket(address, port) ;
        
        UbisecID id = new UbisecIDImpl();
        id.setName( "ClientUPnPSocket" );
        
        MSDRequestImpl msdRequest = new MSDRequestImpl();
        
        ReqProcessingInformation rpi = new ReqProcessingInformationImpl();
        ReqCreationInformation rci = new ReqCreationInformationImpl();
        Creator creator = new CreatorImpl();
        creator.setMgrID( id );
        creator.setNetID( id );
        Calendar cal = Calendar.getInstance();
        rci.setDate( cal );
        rci.setTime( cal );
        rci.setLifetime( BigInteger.valueOf( 5 ) );// in secondes
        rci.setCreator( creator );
        rpi.setCrea( rci );
        
        RequestService requestService = new RequestServiceImpl();
        requestService.setName("RFIDMusicService");
        
        ReqDescriptionInformation reqDescription = new ReqDescriptionInformationImpl();
        reqDescription.setService(requestService);
        msdRequest.setDescInfo(reqDescription);
        msdRequest.setProcInfo( rpi );
        
        PropagationInformation pi = new PropagationInformationImpl();
        msdRequest.setPropaInfo( pi );
        
        msdRequest.setClientID( BigInteger.valueOf( -1 ) );
	    msdRequest.setReqID( BigInteger.valueOf( -1 ) );
        
        MessageContent content = new MessageContentImpl();
        content.setRequest( msdRequest );
        
        MSDMessage mess = new MSDMessageImpl();
        mess.setSender( id );
        mess.setType( BigInteger.valueOf( InfoType.Message.CLIENT_NEW_REQUEST ) );
        mess.setContent( content );
        
        JaxMeUtil jaxme = new JaxMeUtil();
        String message = jaxme.toString( mess );
        
        //System.out.println( message );
        
        System.out.println( "Send sth to the manager ..." );
        connection.getOutputStream().write( message.getBytes() );
        System.out.println( "done" );
        
        byte[] buf = new byte[InfoType.MSG_BUFFER_SIZE];
        long start = System.currentTimeMillis();
        System.out.println( "wait for the response..." );
        int size = connection.getInputStream().read( buf );
        long end = System.currentTimeMillis() - start;
        System.out.println("get a result of "+size +" after "+end);
        if ( size < 0) 
            return;
        
        // Resize the byte[] to keep only the used bytes
        byte[] tmp = buf; 
        buf = new byte[size];
        System.arraycopy(tmp, 0, buf, 0, size);
        
        
        System.out.println( new String(buf));
        
        try {
            // wait for previous search to end...
            Thread.sleep( 6000 );
        } catch (InterruptedException e2) {
            e2.printStackTrace();
        }
        
        findMSDService();
        
        if ( msdService!=null ) {
            System.out.println( "MSD service found" );
            
            MSDMessage found = jaxme.toMSDMessage( buf );
            
            // take the first result...
            MSDDescription desc = (MSDDescription)found.getContent().getResults().get(0);
            
            if ( desc.getDescID().getName().equals( InfoType.TIMEOUT_EXPIRED )) {
                System.out.println("Not found !");
                return;
            }
                
            String epService = desc.getSvcDef().getGeneric().getAddr().getLocation();
            String msdDescId = desc.getDescID().getName();
            epService = epService + "/" + msdDescId;
            
            RFIDMusicBindingStub rfidService = new RFIDMusicBindingStub(epService, ctrUrl4MSDService);
            
            String res = rfidService.getInfo( "4-010165-314153" );
            
            System.out.println( res );
        }
        
        
    } catch (JAXBException e1) {
        // TODO Auto-generated catch block
        e1.printStackTrace();
        
    } catch (UnknownHostException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
    } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
    }
	
}

	public static void main(String[] args) {
		
		ClientUPnPSocket app = new ClientUPnPSocket();
		app.callService();
	}
}
