/*
 * Created on June 13, 2005
 *
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr<br>
 *          Rafik.Chibout@inria.fr
 * 
 * Copyright INRIA 2005
 */
package org.msda.clientsamples.upnp.printer;


import java.math.BigInteger;
import java.rmi.RemoteException;
import java.util.Calendar;
import java.util.Iterator;
import java.util.Vector;

import org.msda.clientsamples.upnp.AbstractClientUPnP;
import org.msda.context.ContextVocabulary;
import org.msda.context.ServiceTemplate;
import org.msda.msd.*;
import org.msda.msd.impl.*;
import org.msda.util.InfoType;


/**
 * The printer client that can perform a search to a UPnP service using the MSD.
 * 
 * @author delachap, chibout, riva
 *
 */
public class ClientPrinter extends AbstractClientUPnP {

    private MSDDescriptionImpl[] msdDescs;
   
    public ClientPrinter() {
    }

    //
    // Methods to search and access the service using the MSD 
    //
    
    /**
     * First call to getServices
     * 
     * @return
     */
    public MSDDescriptionImpl[]  findService( int timeout, String name ) throws RemoteException {
        return findService( -1, -1, timeout, name );
    }
    
    /**
     * Second or more call to get partial results.
     * 
     * @param clientId
     * @param requestId
     * @return
     */
    public MSDDescriptionImpl[]  findService( int clientId, int requestId, int timeout, String name ) throws RemoteException
    {
    	if (msdService == null)  findMSDService();
    	
    	MSDRequestImpl msdRequest = new MSDRequestImpl();
    	
    	if ( clientId<0 ) {
    	    //First call to findService
    	    msdDescs = null;
    	    
	    	ReqProcessingInformation rpi = new ReqProcessingInformationImpl();
	        ReqCreationInformation rci = new ReqCreationInformationImpl();
	        Creator creator = new CreatorImpl();
	      
	        Calendar cal = Calendar.getInstance();
	        rci.setDate( cal );
	        rci.setTime( cal );
	        rci.setLifetime( BigInteger.valueOf( timeout ) );// in seconds
	        rci.setCreator( creator );
	        rpi.setCrea( rci );
	        
	    	ReqDescriptionInformation reqDescription = new ReqDescriptionInformationImpl();
	    	RequestService requestService = new RequestServiceImpl();
	    	requestService.setName(name);
	    	
//	    	UserContext uCtx = new UserContextImpl();	
//	    	uCtx.setDvcAudioInput(true);
//	    	uCtx.setDvcAudioInputActive(false);
//	    	uCtx.setDvcAudioOutput(true);
//	    	uCtx.setDvcAudioOutputActive(false);
//	    	uCtx.setDvcColor(true);
//	    	uCtx.setDvcConnActive(ContextVocabulary.BLUETOOTH);
//	    	uCtx.setDvcConnQoS(ContextVocabulary.NORMAL);
//	    	uCtx.setDvcGraphics(true);
//	    	uCtx.setDvcCPU("64-bit ARM9-based RISC");
//	    	uCtx.setDvcCPUStatus(ContextVocabulary.IDLE);
//	    	uCtx.setDvcInfo("SmartPhone-ltd");
//	    	uCtx.setDvcMemTotal(new BigInteger("8"));
//	    	uCtx.setDvcMemAvailable(new BigInteger("4"));
//	    	uCtx.setDvcName("SmartPhoneNum1");
//	    	uCtx.setDvcType("NOKIA 6681");
//	    	uCtx.setDvcOS("Windows");
	    	
	    	ContextRules ctxRule = new ContextRulesImpl();	
	    	
	    	//
	    	// Context Rule Ex 1 - simple comparison
	    	//
	    	
	    	// serviceStatus = idle
	    	// serviceStatus != busy 
	    	// copies < 100	    	

//	    	ComparisonValue compValue = new ComparisonValueImpl();
//	    	//compValue.setStringValue(ContextVocabulary.IDLE);
//	    	//compValue.setStringValue(ContextVocabulary.BUSY);
//	    	compValue.setIntegerValue(new BigInteger(Integer.toString(100)));	    	
//	    	System.out.println("comparisonValue is " + compValue.toString());	    	
//	    	
//	    	ComparisonNode compNode = new ComparisonNodeImpl();	
//	    	compNode.setIsMandatory(false);
//	    	//compNode.setName(ServiceTemplate.SERVICE_STATE);
//	    	compNode.setName(PrinterTemplate.COPIES);
//	    	//compNode.setOperation("=");
//	    	//compNode.setOperation("!=");
//	    	compNode.setOperation("<");
//	    	compNode.setValue(compValue);
//	    	
//	    	ContextElementNode ctxElementNode = new ContextElementNodeImpl();
//	    	ctxElementNode.setComparison(compNode);
//	    	
//	    	ContextNode ctxNode = new ContextNodeImpl();
//	    	ctxNode.setIsCombination(false);
//	    	ctxNode.setElement(ctxElementNode);
//	    	
//	    	ctxRule.setRoot(ctxNode);
	    	
	    	//
	    	// Context Rule Example 2    - combination of 2 comparisons	
	    	//
	    	
//	    	ComparisonValue compValue1 = new ComparisonValueImpl();
//	    	compValue1.setStringValue(ContextVocabulary.IDLE);    	
//	    	System.out.println("comparisonValue is " + compValue1.toString());	   	
//	    	ComparisonNode compNode1 = new ComparisonNodeImpl();	
//	    	compNode1.setIsMandatory(false);
//	    	compNode1.setName(ServiceTemplate.SERVICE_STATE);	    	
//	    	compNode1.setOperation("=");	    	
//	    	compNode1.setValue(compValue1);
//	    	ContextElementNode ctxElementNode1 = new ContextElementNodeImpl();
//	    	ctxElementNode1.setComparison(compNode1);
//	    	ContextNode ctxNode1 = new ContextNodeImpl();
//	    	ctxNode1.setIsCombination(false);
//	    	ctxNode1.setElement(ctxElementNode1);	    	
//	    	
//	    	ComparisonValue compValue2 = new ComparisonValueImpl();
//	    	compValue2.setStringValue(ContextVocabulary.BUSY);	    	
//	    	System.out.println("comparisonValue is " + compValue2.toString());	   	
//	    	ComparisonNode compNode2 = new ComparisonNodeImpl();	
//	    	compNode2.setIsMandatory(false);
//	    	compNode2.setName(ServiceTemplate.SERVICE_STATE);	    	
//	    	compNode2.setOperation("!=");	    	
//	    	compNode2.setValue(compValue2);
//	    	ContextElementNode ctxElementNode2 = new ContextElementNodeImpl();
//	    	ctxElementNode2.setComparison(compNode2);
//	    	ContextNode ctxNode2 = new ContextNodeImpl();
//	    	ctxNode2.setIsCombination(false);
//	    	ctxNode2.setElement(ctxElementNode2);
//	    	
//	    	CombinationNode combiNode = new CombinationNodeImpl();
//	    	combiNode.setOperation("||");
//	    	combiNode.getNodes().add(ctxNode1);
//	    	combiNode.getNodes().add(ctxNode2);
//	    	
//	    	ContextElementNode ctxElementNode = new ContextElementNodeImpl();
//	    	ctxElementNode.setCombination(combiNode);
//	    	ContextNode ctxNode = new ContextNodeImpl();
//	    	ctxNode.setIsCombination(true);
//	    	ctxNode.setElement(ctxElementNode);
//	    	
//	    	ctxRule.setRoot(ctxNode);	    	
	    	
	    	//
	    	// Context Rule Example 3    - combination of 2 combinations
	    	//
	    	
//	    	ComparisonValue compValue1 = new ComparisonValueImpl();
//	    	compValue1.setStrVal(ContextVocabulary.IDLE);    	
//	    	System.out.println("comparisonValue is " + compValue1.toString());	   	
//	    	ComparisonNode compNode1 = new ComparisonNodeImpl();	
//	    	compNode1.setRequired(false);
//	    	compNode1.setName(ServiceTemplate.SERVICE_STATE);	    	
//	    	compNode1.setOp("=");	    	
//	    	compNode1.setVal(compValue1);
//	    	ContextNode ctxNode1 = new ContextNodeImpl();
//	    	ctxNode1.setCompa(compNode1); 	
//	    	
//	    	ComparisonValue compValue2 = new ComparisonValueImpl();
//	    	compValue2.setStrVal(ContextVocabulary.BUSY);	    	
//	    	System.out.println("comparisonValue is " + compValue2.toString());	   	
//	    	ComparisonNode compNode2 = new ComparisonNodeImpl();	
//	    	compNode2.setRequired(false);
//	    	compNode2.setName(ServiceTemplate.SERVICE_STATE);	    	
//	    	compNode2.setOp("!=");	    	
//	    	compNode2.setVal(compValue2);
//	    	ContextNode ctxNode2 = new ContextNodeImpl();
//	    	ctxNode2.setCompa(compNode2);
//	    	
//	    	ComparisonValue compValue3 = new ComparisonValueImpl();
//	    	compValue3.setStrVal(ContextVocabulary.NORMAL);	    	
//	    	System.out.println("comparisonValue is " + compValue3.toString());	   	
//	    	ComparisonNode compNode3 = new ComparisonNodeImpl();	
//	    	compNode3.setRequired(false);
//	    	compNode3.setName(ServiceTemplate.SERVICE_QUALITY);	    	
//	    	compNode3.setOp("=");	    	
//	    	compNode3.setVal(compValue3);
//	    	ContextNode ctxNode3 = new ContextNodeImpl();
//	    	ctxNode3.setCompa(compNode3);
//	    	
//			CombinationNode combiNode1 = new CombinationNodeImpl();
//	    	combiNode1.setOp("OR");
//	    	combiNode1.getNodes().add(ctxNode1);
//	    	combiNode1.getNodes().add(ctxNode2);
//	    	ContextNode ctxNode4 = new ContextNodeImpl();
//	    	ctxNode4.setCombi(combiNode1);
//	    	
//	    	CombinationNode combiNode2 = new CombinationNodeImpl();
//	    	combiNode2.setOp("AND");
//	    	combiNode2.getNodes().add(ctxNode3);
//	    	combiNode2.getNodes().add(ctxNode4);
//	    	ContextNode ctxNode5 = new ContextNodeImpl();
//	    	ctxNode5.setCombi(combiNode2);
//
//	    	SelectionRules selection = new SelectionRulesImpl();
//	    	selection.setRoot(ctxNode5);
//	    	ctxRule.setSelectRules( selection );	 
//	    	
//	    	
//	    	System.out.println("the user context is " + uCtx.toString());
//	    	System.out.println("the user rules are " + ctxRule.toString());
//	    	msdRequest.setUserCtx(uCtx);
//	    	msdRequest.setCtxRules(ctxRule);	    	   	
//	        reqDescription.setService(requestService);
//	        msdRequest.setDescInfo(reqDescription);
//	        msdRequest.setProcInfo( rpi );
//	        System.out.println("the msdRequest is " + msdRequest.toString());	 
	        
    	} else {
    	    
    	    if ( requestId < 0 ) {
    	        System.out.println("Invalid values : clientId="+clientId + " requestId="+requestId);
    	        return null;
    	    }
    	    
    	    // get partial results
    	    msdRequest.setClientID( BigInteger.valueOf( clientId ) );
    	    msdRequest.setReqID( BigInteger.valueOf( requestId ) );
    	}
        
    	
    	// save the previous results... 
    	MSDDescriptionImpl[] results = msdDescs;
    	
    	msdDescs = msdService.getServices(msdRequest);
    	System.out.println("service invoked");
    	if (msdDescs == null) {
    	    System.out.println("the result is null");
    	    
    	} else if (msdDescs.length==0) {
    	    System.out.println("the result is Empty");
    	    
    	} else {
    	    MSDDescriptionImpl desc = msdDescs[0];
    	    
    	    String descName = desc.getDescID().getName();
    	    System.out.println("descID.Name="+descName);
    	    
    	    if ( InfoType.RUNNING_REQUEST.equals( descName ) ||
    	            InfoType.TIMEOUT_EXPIRED.equals( descName ) ) {
    	        System.out.println("the result contains only clientId and requestId");
    	        
    	    } else {   
    	        
    	        ServiceDefinition svcDef = desc.getSvcDef();
    	        Generic generic = svcDef.getGeneric();
    	        Original original = svcDef.getOriginal();
    	        
    	        String epService = generic.getAddr().getLocation();
    	        System.out.println("epService ="+ epService );
    	        System.out.println("ProtocoleName ="+original.getProtocolName());
    	        
    	        java.util.List methods = generic.getMethods();
    	        if ( methods!=null ) {
    	            for (int d=0; d<methods.size();d++) {
    	                
    	                Method method = (Method)methods.get(d);
    	                System.out.println("msdService methodName="+method.getName());
    	                java.util.List paramIn = method.getIn();
    	                java.util.List paramOut = method.getOut();
    	                
    	                for (int k=0; k<paramIn.size(); k++) {	
    	                    Param param = (Param)paramIn.get(k);
    	                    System.out.println("msdService in Param"+param.getType());
    	                }
    	                
    	                for (int u= 0 ;u<paramOut.size(); u++) {
    	                    Param param = (Param)paramOut.get(u);
    	                    System.out.println("msdService out Param"+param.getType());
    	                }
    	            }
    	        }
    	    } 
    	    System.out.println("*******************************");
    	}
    	
    	MSDDescriptionImpl[] lastResults = msdDescs;
    	
    	if ( results!=null ) {
    	    // at this point results contains the previous partial results
    	    // and msdDescs contains the latest results 
    	    String descName = lastResults[0].getDescID().getName();
    	    
    	    int size = results.length;
    	    if ( lastResults!=null && 
    	            !InfoType.RUNNING_REQUEST.equals( descName ) &&
    	            !InfoType.TIMEOUT_EXPIRED.equals( descName ) )
    	        size = size + lastResults.length;
    	    
    	    msdDescs = new MSDDescriptionImpl[ size ];
    	    for(int i=0;i<results.length; i++)
    	        msdDescs[i] = results[i];
    	    
    	    if ( lastResults!=null && 
    	            !InfoType.RUNNING_REQUEST.equals( descName ) &&
    	            !InfoType.TIMEOUT_EXPIRED.equals( descName ) )
    	        for(int i=0;i<lastResults.length; i++)
    	            msdDescs[results.length+i] = lastResults[i];
	    } 
        return lastResults;
    }

    /**
     * Returns the MSD description at the specific index
     * @param index		
     * @return MSDDescription
     */
    public MSDDescription getResultDescription( int index ) {
    	
        if ( msdDescs!=null && msdDescs.length>0 && index >= 0)
            return msdDescs[index];
        return null;
    }
    
    
    private Vector orianaResults;
    public void orianaClient() {
        
        orianaResults = new Vector();
        
        Vector v = null;
    	boolean loop = true;
    	MSDDescription[] res = null;
    	int clientId = -1, requestId = -1;
    	
        int timeout = 4;
        String name = "urn:printbasic1";
        
        while( loop ) {
    	    
    	    try {
                if ( v==null ) {
                    v = new Vector();
                    res = findService( timeout, name );
                } else {
                    res = findService( clientId, requestId, timeout, name );
                }
                
            } catch (RemoteException e1) {
                e1.printStackTrace();
                return;
            }
    	    
    	    if ( res!=null ) {
    	        for(int i=0; i<res.length; i++) {
    	            
    	            if ( InfoType.TIMEOUT_EXPIRED.equals( res[i].getDescID().getName() ) ) {
    	                loop = false;
    	                break;
    	            }
    	            
    	            if ( i==0 ) {
    	                CreationInformation creation = res[i].getCreaInfo();
    	                UbisecID deviceId = creation.getCreator().getDvcID();
    	                String deviceName = deviceId.getName(); 
    	                int index = deviceName.indexOf("#");
    	                clientId = Integer.parseInt( deviceName.substring(0, index) ); 
    	                requestId = Integer.parseInt( deviceName.substring( index+1 ) );
    	                System.out.println("Found clientId="+clientId+" requestId="+requestId);
    	                if ( InfoType.RUNNING_REQUEST.equals( res[i].getDescID().getName() ) )
    	                    break;
    	            }
    	            
    	            if ( InfoType.RUNNING_REQUEST.equals( res[i].getDescID().getName() ) )
	                    break;
    	            
    	            ServiceDefinition definition = res[i].getSvcDef();
    	            StringBuffer buf = new StringBuffer();
    	            buf = buf.append( definition.getOriginal().getProtocolName() ).append( "  " );
    	            int n = 7 - definition.getOriginal().getProtocolName().length();
    	            if ( n > 0 )
    	                for( int j=0;j<n;j++)
    	                    buf = buf.append(" ");
    	            buf = buf.append(definition.getGeneric().getName() ) ;
    	            
    	            v.add( buf.toString() );
    	            //orianaResults.add( res[i] );
    	        }
    	    }
    	    
    	    if ( !loop )// leave the loop
    	        break;
    	    
    	    try {
                Thread.sleep( 2000 );
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
    	}
        
        System.out.println("ORIANA the results contains ="+v.size());
        
        for (Iterator iter = v.iterator(); iter.hasNext();) {
            String element = (String) iter.next();
            System.out.println( "" );
            System.out.println( element );
        }
    }
    
    public static void main( String[] args ) {
        
        ClientPrinter client = new ClientPrinter();
        client.orianaClient();
        
    }
}
