/*
 * Created on June 13, 2005
 *
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr<br>
 *          Rafik.Chibout@inria.fr
 * 
 * Copyright INRIA 2005
 */
package org.msda.clientsamples.upnp.printer;


import org.msda.MSDClientAPIService.MSDClientAPIBindingStub;
import org.msda.context.ContextVocabulary;
import org.msda.msd.*;
import org.msda.msd.impl.*;

import com.siemens.upnp.UPnPConstants;
import com.siemens.upnp.controlpoint.ControlPoint;

import com.siemens.upnp.controlpoint.ProxyDevice;
import com.siemens.upnp.controlpoint.ProxyService;
import com.siemens.upnp.controlpoint.UPnPSearchResponseListener;


import java.io.BufferedInputStream;
import java.math.BigInteger;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.*;

/**
 * @author rafik
 * @author Oriana Riva, HIIT - August 2005
 */

public class PrinterClientUPnP {
    
    private PrinterInterface printerService;
    
    public void callService()
    {    
        
        // find URL Of MSDService
        
        // init the CP
        ControlPoint controlPoint = ControlPoint.getInstance();
        DeviceDesc deviceDesc = new DeviceDesc();
        UPnPSearchResponseListener searchResponseListener = new UPnPSearchResponseListenerImpl(deviceDesc);
        controlPoint.addUPnPSearchListener(searchResponseListener);
        
        // search MSDService
        
        String ctrUrl = "";
        
        try
        {
            
            controlPoint.search(UPnPConstants.SearchForServiceType("msdservice:1:0"));
            while (deviceDesc.getDescriptionURL() == null ) {}
            System.out.println("DescrURL = "+deviceDesc.getDescriptionURL());
            URL url = new URL(deviceDesc.getDescriptionURL());
            // show the Description
            
            HttpURLConnection conn = (HttpURLConnection)url.openConnection();
            BufferedInputStream reader = new BufferedInputStream(conn.getInputStream());
            byte	[] buff = new byte[1000];
            while (reader.available()>0)
            {
                int num= reader.read(buff,0,buff.length);
                System.out.print(new String(buff,0,num));
            }
            System.out.println("");
            System.out.println("**************");
            
            System.out.println("Level 1");
            ProxyDevice device = controlPoint.createDevice(url);
            
            
            System.out.println("Level 2: Device created");
            java.util.Vector services = device.getServices();
            
            //ProxyService service = device.getServiceByID("service1");
            //ProxyService service = device.getServiceByType("service1");
            System.out.println("Level 3: service proxy get");
            if (services != null && services.size()>0)
            {
                ProxyService service = (ProxyService)services.elementAt(0);
                ctrUrl = service.getControlURL();
            }
            System.out.println("Level 5");  
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
        }
        
        
        org.msda.MSDClientAPIService.MSDClientAPIPortType service = new MSDClientAPIBindingStub(ctrUrl);
        //MSDClientAPIService.MSDClientAPIPortType service = new MSDClientAPIBindingStub("http://128.93.9.15:8080/discovery/serviceaccess");
        
        MSDRequestImpl msdRequest = new MSDRequestImpl();
        ReqDescriptionInformation reqDescription = new ReqDescriptionInformationImpl();
        
        RequestService requestService = new RequestServiceImpl();
        //requestService.setName("uuid:0000E8699FB4:urn:schemas-upnp-org:service:service1:1:0");
        //requestService.setName("urn:schemas-upnp-org:service:service1:1:0");
        //requestService.setName("uuid:0000E8699FB4:service1");
        //requestService.setName("Movie");
        System.out.println("Search printbasic1");
        requestService.setName("urn:printbasic1");
        
//        UserContext uCtx = new UserContextImpl();	
//        uCtx.setDvcAudioInput(true);
//        uCtx.setDvcAudioInputActive(false);
//        uCtx.setDvcAudioOutput(true);
//        uCtx.setDvcAudioOutputActive(false);
//        uCtx.setDvcColor(true);
//        uCtx.setDvcConnActive(ContextVocabulary.BLUETOOTH);
//        uCtx.setDvcConnQoS(ContextVocabulary.NORMAL);
//        uCtx.setDvcGraphics(true);
//        uCtx.setDvcCPU("64-bit ARM9-based RISC");
//        uCtx.setDvcCPUStatus(ContextVocabulary.BUSY);
//        uCtx.setDvcInfo("SmartPhone-ltd");
//        uCtx.setDvcMemTotal(new BigInteger("8"));
//        uCtx.setDvcMemAvailable(new BigInteger("4"));
//        uCtx.setDvcName("SmartPhone");
//        uCtx.setDvcType("NOKIA 6681");
//        uCtx.setDvcOS("Windows");
        
        // ex: I want a printer whose status is not busy
        // serviceStatus != busy 
        ContextRules ctxRule = new ContextRulesImpl();	
        
//        ComparisonValue compValue = new ComparisonValueImpl();
//        compValue.setStrVal(ContextVocabulary.BUSY);
//        
//        ComparisonNode compNode = new ComparisonNodeImpl();	
//        compNode.setRequired(false);
//        compNode.setName("serviceState");
//        compNode.setOp("notEqual");
//        compNode.setVal(compValue);
//        
//        ContextNode ctxNode = new ContextNodeImpl();
//        ctxNode.setCompa(compNode);
//        
//        SelectionRules selection = new SelectionRulesImpl();
//        selection.setRoot(ctxNode);
//        ctxRule.setSelectRules( selection );
//        
//        System.out.println("the user context is " + uCtx.toString());
//        System.out.println("the user rules are " + ctxRule.toString());
//        msdRequest.setUserCtx(uCtx);
//        msdRequest.setCtxRules(ctxRule);
//        
//        
        reqDescription.setService(requestService);
        msdRequest.setDescInfo(reqDescription);
        
        ReqProcessingInformation rpi = new ReqProcessingInformationImpl();
        ReqCreationInformation rci = new ReqCreationInformationImpl();
        Creator creator = new CreatorImpl();
        
        int timeout = 10;
        
        Calendar cal = Calendar.getInstance();
        rci.setDate( cal );
        rci.setTime( cal );
        rci.setLifetime( BigInteger.valueOf( timeout ) );// en secondes
        rci.setCreator( creator );
        rpi.setCrea( rci );
        
        msdRequest.setProcInfo( rpi );
        
        try
        {
            MSDDescriptionImpl[] msdDescs = service.getServices(msdRequest);
            System.out.println("service invoked");
            if (msdDescs != null)
            {
                if (msdDescs.length>0)
                {
                    MSDDescriptionImpl desc = msdDescs[0];
                    String epService = desc.getSvcDef().getGeneric().getAddr().getLocation();
                    System.out.println("EP service="+desc.getSvcDef().getGeneric().getAddr().getLocation());
                    List methods = desc.getSvcDef().getGeneric().getMethods();
                    for (int d=0; d<methods.size();d++)
                    {
                        Method method = (Method)methods.get(d);
                        System.out.println("msdService methodName="+method.getName());
                        List paramIn = method.getIn();
                        List paramOut = method.getOut();
                        
                        for (int k=0; k<paramIn.size(); k++)
                        {	
                            Param param = (Param)paramIn.get(k);
                            System.out.println("msdService " + k + " in Param " + param.getType());
                        }
                        
                        for (int u= 0 ;u<paramOut.size(); u++)
                        {
                            Param param = (Param)paramOut.get(u);
                            System.out.println("msdService " + u + " out Param" + param.getType());
                        }
                    }
                    System.out.println("*******************************");    		
                    
                    // getClient ID function
//                    int id = service.getClientID(desc);
//                    System.out.println("id = "+id);
                    
                    int id = -1;// ok now
                    if (id < 0) {
                        //System.out.println("******* WARNING ****** initialization not ok ! ");
                    }
                    
                    printerService = new PrinterSOAPBindingStub(epService, ctrUrl, id);			
                    System.out.println("Printer Service is ready");
                    
                }
                else System.out.println("the result is Empty");
            }
            else System.out.println("the result is null");
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
        }
    }
    
    
    public static void main(String[] args) {
        
        PrinterClientUPnP app = new PrinterClientUPnP();
        app.callService();
    }
}
