/*
 * Created on June 13, 2005
 *
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr<br>
 *          Rafik.Chibout@inria.fr
 * 
 * Copyright INRIA 2005
 */
package org.msda.clientsamples.upnp.printer;

import java.awt.Button;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.ScrollPane;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.List;

import java.rmi.RemoteException;
import java.util.Vector;

import javax.swing.JOptionPane;
import javax.xml.bind.JAXBException;

import org.msda.msd.CreationInformation;
import org.msda.msd.MSDDescription;
import org.msda.msd.ServiceDefinition;
import org.msda.msd.UbisecID;
import org.msda.util.InfoType;
import org.msda.util.JaxMeUtil;


/**
 * Class that creates the search panel.
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 */
public class SearchPanel extends Panel {

    private static final String BUTTON1 = "Check UPnP";
    private static final String BUTTON2 = "Check MSD"; 
    
    private static final String BUTTON3 = "Display";
    private static final String BUTTON4 = "Access";
    
    private List list;
    
    private Button button1;
    private Button button2;
    private Button button3;
    private Button button4;
    
    private ScrollPane scroll; 
    private ListItemListener actionList;
    private ClientGUI clientGUI;
    
    private Label labelTimeout;
    private TextField textTimeout;

    private int searchType;
    
    /**
     * Construtor for the search panel.
     * 
     * @param client
     */
    public SearchPanel( ClientGUI client ) {
        
        setSize( ClientGUI.WIDTH, ClientGUI.HEIGHT );
        
        this.clientGUI = client;
        searchType = 0;
        
        // creating and initializing the AWT components
        actionList = new ListItemListener();
        initButton();
        
        labelTimeout = new Label("Timeout in s");
        textTimeout = new TextField("8", 5);
        
        Panel buttonsPanel = new Panel();
        buttonsPanel.add( labelTimeout );
        buttonsPanel.add( textTimeout );
        buttonsPanel.add( button1 );
        buttonsPanel.add( button2 );
        
        list = new List();
        scroll = new ScrollPane();
        scroll.add( list );
        scroll.setSize( ClientGUI.WIDTH-20, ClientGUI.HEIGHT-100 );
        
        Panel buttonsPanel2 = new Panel();
        buttonsPanel2.add( button3 );
        buttonsPanel2.add( button4 );
        
        // adding to the panel
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.fill = GridBagConstraints.HORIZONTAL;
        gridbag.setConstraints( buttonsPanel, c);
        c.fill = GridBagConstraints.BOTH;
        gridbag.setConstraints( scroll, c);
        gridbag.setConstraints( buttonsPanel2, c);
        
        setLayout(gridbag);
        
        add( buttonsPanel );
        add( scroll );
        add( buttonsPanel2 );
    }
    
    /**
     * @return	Returns the name of the searched service
     */
    private String getSearchedServiceName() {
        return "urn:printbasic1";
    }
    
    /**
     * @return	Returns the timeout (milliseconds) to get partial results
     */
    private int getTimeout4PartialResults() {
        return 3000;
    }
    
    /**
	 * @return Returns the searchType.
	 */
	public int getSearchType() {
		return searchType;
	}
    
    /**
     * @return Returns the value of the timeout.
     */
    public String getTimeout() {
        return textTimeout.getText();
    }
    
    /**
     * Creates all the buttons.
     */
    private void initButton() {
        
        ActionListener buttonListener = new ButtonActionListener();
        
        button1 = new Button( BUTTON1 );
        button1.addActionListener( buttonListener );
        
        button2 = new Button( BUTTON2 );
        button2.addActionListener( buttonListener );
        
        button3 = new Button( BUTTON3 );
        button3.addActionListener( buttonListener );
        button3.setEnabled( false );
        
        button4 = new Button( BUTTON4 );
        button4.addActionListener( buttonListener );
        button4.setEnabled( false );
    }
    
    /**
     * Action associated with the "Check UPnP" button.
     *
     */
    private void action1() {
        
        list.removeAll();
	    list.setEnabled(false);
	    
        // invokes the service on UPnP
        searchType = 1;
        System.out.println("Invoke search on UPnP");
        String upnpServiceURL = clientGUI.getClient().findUPnPService(getSearchedServiceName());
        if (upnpServiceURL != null) {
            //
            Vector v = new Vector();
            v.add( upnpServiceURL );
            
            for ( int i=0; i<v.size() ;i++) 
                list.add( (String)v.get(i) );
            
            list.setEnabled(true);
    	    list.addItemListener( actionList );
            
        } else {
            JOptionPane.showMessageDialog(this,"Unsuccessful search...","ERROR",JOptionPane.ERROR_MESSAGE);
        }
    }
    
    /**
     * Action associated with the "Check MSD" button.
     */
    private void action2() 
    {
    	button2.setEnabled( false );
    	
    	// create a loop until the timeout expired.
    	Vector v = null;
    	boolean loop = true;
    	MSDDescription[] res = null;
    	int clientId = -1, requestId = -1;
    	
        int timeout = 0;
        try {
            timeout = Integer.parseInt( getTimeout() );
        } catch (NumberFormatException e) {
        }
    	
        String name = getSearchedServiceName();
        
    	while( loop ) {
    	    
    	    try {
                if ( v==null ) {
                    v = new Vector();
                    res = clientGUI.getClient().findService( timeout, name );
                } else {
                    res = clientGUI.getClient().findService( clientId, requestId, timeout, name );
                }
            } catch (RemoteException e1) {
                e1.printStackTrace();
                JOptionPane.showMessageDialog(this,"Can't connect to "+name,"Error",JOptionPane.ERROR_MESSAGE);
            }
    	    
    	    if ( res!=null ) {
    	        for(int i=0; i<res.length; i++) {
    	            
    	            if ( InfoType.TIMEOUT_EXPIRED.equals( res[i].getDescID().getName() ) ) {
    	                loop = false;
    	                break;
    	            }
    	            
    	            if ( i==0 ) {
    	                CreationInformation creation = res[i].getCreaInfo();
    	                UbisecID deviceId = creation.getCreator().getDvcID();
    	                String deviceName = deviceId.getName(); 
    	                int index = deviceName.indexOf("#");
    	                clientId = Integer.parseInt( deviceName.substring(0, index) ); 
    	                requestId = Integer.parseInt( deviceName.substring( index+1 ) );
    	                System.out.println("Found clientId="+clientId+" requestId="+requestId);
    	                if ( InfoType.RUNNING_REQUEST.equals( res[i].getDescID().getName() ) )
    	                    break;
    	            }
    	            
    	            if ( InfoType.RUNNING_REQUEST.equals( res[i].getDescID().getName() ) )
	                    break;
    	            
    	            ServiceDefinition definition = res[i].getSvcDef();
    	            StringBuffer buf = new StringBuffer();
    	            buf = buf.append( definition.getOriginal().getProtocolName() ).append( "  " );
    	            int n = 7 - definition.getOriginal().getProtocolName().length();
    	            if ( n > 0 )
    	                for( int j=0;j<n;j++)
    	                    buf = buf.append(" ");
    	            buf = buf.append(definition.getGeneric().getName() ) ;
    	            
    	            v.add( buf.toString() );
    	        }
    	    }
    	    
    	    // modify the content of the java.awt.List
    	    list.removeAll();
    	    list.setEnabled(false);
    	    for ( int i=0; i<v.size() ;i++) 
    	        list.add( (String)v.get(i) );
    	    list.setEnabled(true);
    	    list.addItemListener( actionList );
    	    
    	    if ( !loop )// leave the loop
    	        break;
    	    
    	    try {
                Thread.sleep( getTimeout4PartialResults() );
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
    	}
    	
        JOptionPane.showMessageDialog(this,"End of the search","Result",JOptionPane.INFORMATION_MESSAGE);
           
    	button2.setEnabled( true );
        searchType = 2;
    }
    
    /**
     * Action associated with the "Display" button.<br>
     * Displays the UPnP or the MSD description in a window.
     */
    private void action3() {
     
        switch ( searchType ) {
        	case 1: 
        	    // UPnP    
        	    TextArea area = new TextArea( clientGUI.getClient().getUPnPDesc() );
        	    area.setEditable( false );
        	    ScrollPane scroll = new ScrollPane();
        	    scroll.add(area);
        	    scroll.setSize( 510, 500 );
        	    JOptionPane.showMessageDialog(this,scroll,"UPnP description",JOptionPane.INFORMATION_MESSAGE);
                
        	    break;
            case 2:
                // MSD
                MSDDescription desc = clientGUI.getClient().getResultDescription( list.getSelectedIndex() );
                JaxMeUtil jaxme = new JaxMeUtil();
                String res = "";
                try {
                    res = jaxme.toString( desc );
                } catch (JAXBException e) {
                   res = e.getMessage();
                }
                
                TextArea area2 = new TextArea( res );
                area2.setEditable( false );
        	    ScrollPane scroll2 = new ScrollPane();
        	    scroll2.add(area2);
        	    scroll2.setSize( 510, 500 );
                JOptionPane.showMessageDialog(this,scroll2,"MSD description",JOptionPane.INFORMATION_MESSAGE);
                
                break;
            default:
                System.out.println("No search successful");
        }
    }
    
    /**
     * Action associated with the "Access" button.
     */
    private void action4() {
        
//    	switch ( this.searchType ) {
//    		case 1: // case UPnP
//    		    //clientGUI.getClient().initiateUPnPAccess();
//    		    break;
//    		case 2: // case MSD
//    		    //clientGUI.getClient().initiateServiceAccess( list.getSelectedIndex() );
//    		    break;
//    	}
//    	clientGUI.modifyPanel();
    }
	
    /**
     * Class that defines the action when an element of the list is selected. 
     * 
     * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
     */
    class ListItemListener implements ItemListener {
        
        public void itemStateChanged(ItemEvent e) {
            
            button3.setEnabled( true );
            button4.setEnabled( true );
        }
        
    };
    
    /**
     * Class that defines the actions when a button is pressed.
     * 
     * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
     *
     */
    class ButtonActionListener implements ActionListener  {
      
      public void actionPerformed(ActionEvent ae) {
          
          String action = ae.getActionCommand();
          
          if ( BUTTON1.equals( action ) ) {
              action1();
              
          } else if ( BUTTON2.equals( action ) ) {
              action2();
              
          } else if ( BUTTON3.equals( action ) ) {
              action3();
          
          } else if ( BUTTON4.equals( action ) ) {
              action4();
          }
      }
  }
}
