/*
 * Created on June 13, 2005
 *
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr<br>
 *          Rafik.Chibout@inria.fr
 * 
 * Copyright INRIA 2005
 */
package org.msda.clientsamples.upnp.rfid;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * Class that creates a GUI client to search and invoke a UPnP service using UPnP or the MSD protocol.
 * 
 * @author delachap, chibout
 *
 */
public class ClientGUI extends Frame {

    public static final Color COLOR = new Color( 230,230,180 );
    
    public static final int WIDTH = 220;
    public static final int HEIGHT = 320;
    
    private AccessPanel accessPanel;
    private SearchPanel searchPanel;
    
    private ClientRFID client;
    
    public ClientGUI() {
        
        super( " UPnP RFID Client " );
        
        setSize( WIDTH, HEIGHT );     
        setBackground( ClientGUI.COLOR );
        
        searchPanel = new SearchPanel( this );
        accessPanel = new AccessPanel( this );
        
        add( searchPanel, BorderLayout.CENTER );
        
        client = new ClientRFID();
        
        // To close the application
        addWindowListener(new WindowAdapter() { 
            public void windowClosing(WindowEvent e) { 
                setVisible(false); 
                dispose(); 
                System.exit(0); 
            } 
        }); 
    }
    
    /**
     * @return Returns the client.
     */
    public ClientRFID getClient() {
        return client;
    }
    
    /**
     * @return Returns the accessPanel.
     */
    public AccessPanel getAccessPanel() {
        return accessPanel;
    }
    
    /**
     * @return Returns the searchPanel.
     */
    public SearchPanel getSearchPanel() {
        return searchPanel;
    }
    
    /**
     * Displays the access panel instead of the search panel.
     */
    public void modifyPanel() {
        
        this.remove( searchPanel );
        this.add( accessPanel, BorderLayout.CENTER );
        accessPanel.cleanList();

        this.paintAll( this.getGraphics() );
    }
    
    /**
     * Displays back the search panel instead of the access panel.
     */
    public void back() {
        this.remove( accessPanel );
        this.add( searchPanel, BorderLayout.CENTER );
        this.paintAll( this.getGraphics() );
    }
   
	//
	// Main
	//
	
	/**
	 * Creates the UPnP client.
	 */
	public static void main(String[] args)
	{
		ClientGUI client = new ClientGUI();
        client.setVisible(true);
    }
}
