/*
 * Created on June 18, 2005
 *
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr<br>
 *          Rafik.Chibout@inria.fr
 * 
 * Copyright INRIA 2005
 */
package org.msda.clientsamples.upnp.rfid;

import java.math.BigInteger;
import java.rmi.RemoteException;
import java.util.Calendar;

import org.msda.clientsamples.upnp.AbstractClientUPnP;
import org.msda.msd.Creator;
import org.msda.msd.Generic;
import org.msda.msd.MSDDescription;
import org.msda.msd.Method;
import org.msda.msd.Original;
import org.msda.msd.Param;
import org.msda.msd.ReqCreationInformation;
import org.msda.msd.ReqDescriptionInformation;
import org.msda.msd.ReqProcessingInformation;
import org.msda.msd.RequestService;
import org.msda.msd.ServiceDefinition;
import org.msda.msd.impl.CreatorImpl;
import org.msda.msd.impl.MSDDescriptionImpl;
import org.msda.msd.impl.MSDRequestImpl;
import org.msda.msd.impl.ReqCreationInformationImpl;
import org.msda.msd.impl.ReqDescriptionInformationImpl;
import org.msda.msd.impl.ReqProcessingInformationImpl;
import org.msda.msd.impl.RequestServiceImpl;
import org.msda.util.InfoType;

import com.siemens.upnp.controlpoint.ControlPoint;


/**
 * The RFID client that can perform a search and access to a UPnP service using <br> 
 * 		- either UPnP <br>
 * 		- or MSD <br>
 * 
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class ClientRFID extends AbstractClientUPnP {

    private RFIDMusicPortType rfidService;
    private MSDDescriptionImpl[] msdDescs;
    
    
    public ClientRFID() {
        // TODO - really needed ?
        controlPoint = ControlPoint.getInstance();
    }
    
    //
    // Methods to search and access the service using UPnP 
    //

    /**
     * Initializes the service access with UPnP.
     */
    public void initiateUPnPAccess() {    	
        
    	rfidService = new RFIDMusicBindingStub( upnpService.getControlURL() );
    }
    
    //
    // Methods to search and access the service using the MSD 
    //
    
    /**
     * First call to getServices
     * 
     * @return partial results
     */
    protected MSDDescriptionImpl[]  findService( int timeout, String name ) throws RemoteException {
        return findService( -1, -1, timeout, name );
    }
    
    /**
     * Second (or more) call to get partial results.
     * 
     * @param clientId
     * @param requestId
     * @return partial results
     */
    public MSDDescriptionImpl[] findService( int clientId, int requestId, int timeout, String name ) throws RemoteException
    {
    	if (msdService == null)  findMSDService();
    	
    	MSDRequestImpl msdRequest = new MSDRequestImpl();
    	
    	if ( clientId<0 ) {
    	    //First call to findRFIDService
    	    msdDescs = null;
    	    
	    	ReqProcessingInformation rpi = new ReqProcessingInformationImpl();
	        ReqCreationInformation rci = new ReqCreationInformationImpl();
	        Creator creator = new CreatorImpl();
	        
	        Calendar cal = Calendar.getInstance();
	        rci.setDate( cal );
	        rci.setTime( cal );
	        rci.setLifetime( BigInteger.valueOf( timeout ) );// in secondes
	        rci.setCreator( creator );
	        rpi.setCrea( rci );
	        
	    	ReqDescriptionInformation reqDescription = new ReqDescriptionInformationImpl();
	    	RequestService requestService = new RequestServiceImpl();
	    	requestService.setName(name);
	
	        reqDescription.setService(requestService);
	        msdRequest.setDescInfo(reqDescription);
	        msdRequest.setProcInfo( rpi );
	        
    	} else {
    	    
    	    if ( requestId < 0 ) {
    	        System.out.println("Invalid values : clientId="+clientId + " requestId="+requestId);
    	        return null;
    	    }
    	    // get partial results
    	    msdRequest.setClientID( BigInteger.valueOf( clientId ) );
    	    msdRequest.setReqID( BigInteger.valueOf( requestId ) );
    	}
        
    	// save the previous results... 
    	MSDDescriptionImpl[] results = msdDescs;
    	
    	msdDescs = msdService.getServices(msdRequest);
    	System.out.println("service invoked");
    	if (msdDescs == null) {
    	    System.out.println("the result is null");
    	    
    	} else if (msdDescs.length==0) {
    	    System.out.println("the result is Empty");
    	    
    	} else {
    	    MSDDescriptionImpl desc = msdDescs[0];
    	    String descName = desc.getDescID().getName();
    	    System.out.println("descID.Name="+descName);
    	    
    	    if ( InfoType.RUNNING_REQUEST.equals( descName ) ||
    	            InfoType.TIMEOUT_EXPIRED.equals( descName ) ) {
    	        System.out.println("the result contains only clientId and requestId");
    	        
    	    } else {
    	        
    	        ServiceDefinition svcDef = desc.getSvcDef();
    	        Generic generic = svcDef.getGeneric();
    	        Original original = svcDef.getOriginal();
    	        
    	        String epService = generic.getAddr().getLocation();
    	        System.out.println("epService ="+ epService );
    	        System.out.println("ProtocoleName ="+original.getProtocolName());
    	        
    	        java.util.List methods = generic.getMethods();
    	        if ( methods!=null ) {
    	            for (int d=0; d<methods.size();d++)
    	            {
    	                Method method = (Method)methods.get(d);
    	                System.out.println("msdService methodName="+method.getName());
    	                java.util.List paramIn = method.getIn();
    	                java.util.List paramOut = method.getOut();
    	                
    	                for (int k=0; k<paramIn.size(); k++) {
    	                    Param param = (Param)paramIn.get(k);
    	                    System.out.println("msdService in Param"+param.getType());
    	                }
    	                
    	                for (int u= 0 ;u<paramOut.size(); u++) {
    	                    Param param = (Param)paramOut.get(u);
    	                    System.out.println("msdService out Param"+param.getType());
    	                }
    	            }
    	        }
    	    } 
    	    System.out.println("*******************************");
    	}
    	
    	MSDDescriptionImpl[] lastResults = msdDescs;
    	
    	if ( results!=null ) {
    	    // at this point, results contains the previous partial results
    	    // and msdDescs contains the latest results 
    	    String descName = lastResults[0].getDescID().getName();
    	    
    	    int size = results.length;
    	    if ( lastResults!=null && 
    	            !InfoType.RUNNING_REQUEST.equals( descName ) &&
    	            !InfoType.TIMEOUT_EXPIRED.equals( descName ) )
    	        size = size + lastResults.length;
    	    
    	    msdDescs = new MSDDescriptionImpl[ size ];
    	    for(int i=0;i<results.length; i++)
    	        msdDescs[i] = results[i];
    	    
    	    if ( lastResults!=null && 
    	            !InfoType.RUNNING_REQUEST.equals( descName ) &&
    	            !InfoType.TIMEOUT_EXPIRED.equals( descName ) )
    	        for(int i=0;i<lastResults.length; i++)
    	            msdDescs[results.length+i] = lastResults[i];
	    } 
    	
        return lastResults;
    }
    
    /**
     * Returns the MSD description at the specific index
     * @param index		
     * @return MSDDescription
     */
    public MSDDescription getResultDescription( int index ) {
    	
        if ( msdDescs!=null && msdDescs.length>0 && index >= 0)
            return msdDescs[index];
        return null;
    }
    
    /**
     * Initializes the service access with the MSD.
     * 
     * @param index
     */
    public void initiateServiceAccess( int index ) {
     
		    MSDDescriptionImpl desc = msdDescs[ index ];

            String epService = desc.getSvcDef().getGeneric().getAddr().getLocation();
            String msdDescId = desc.getDescID().getName();
            epService = epService + "/" + msdDescId;
            
            rfidService = new RFIDMusicBindingStub(epService, ctrUrl4MSDService);
            System.out.println("RFID Service is ready");
    }
    
    /**
	 * @return Returns the rfidService.
	 */
	public RFIDMusicPortType getRfidService() {
		return rfidService;
	}
	
}
