/*
 * Created on Jun 27, 2005
 * @author raverdy
 * Comments, questions: Pierre-Guillaume.Raverdy@inria.fr
 *
 * Copyright INRIA, 2004-2005
 * UBISEC
 * This code should not be distributed
 */ 
package org.msda.tests;

import org.msda.framework.FrameworkOutput;
import org.msda.msd.*;
import org.msda.msd.impl.*;
import org.msda.util.*;
import org.msda.util.comm.MulticastGroup;
import org.msda.util.comm.MulticastGroupCallback;

import java.io.*;
import java.math.BigInteger;

/*
 * @author raverdy
 */
public class TestMulticastGroup {


	public void startTest (String[] args, FrameworkOutput board )
	{
		BufferedReader standard = new BufferedReader(new InputStreamReader(System.in));
		String ifname, groupAddress, devName;
		int groupPort, testNumber;
		MulticastGroup mgroup;

		testNumber = Integer.parseInt(args[1]);
		ifname = args[2];
		groupAddress = args[3];
		groupPort = Integer.parseInt(args[4]);
		devName = args[5];

		CallBackTestGroup client1, client2, client3;
		
		switch (testNumber)
		{
		case 0:
		{
			System.out.println (">> starting test 0");
			// In the first test, a CallBackTestGroup client is created and sends a message. 
			// It should receive it.
			// The thread then waits for a keyboard entry before terminating all objects. 
			System.out.println (" - creating client1");
			client1 = new CallBackTestGroup ( devName + "_client1", ifname, groupAddress, groupPort);
			client1.SendTestMessage (devName + "_client1_1", 1);

			System.out.println ("Enter a line to exit");
			try {
				String inline = standard.readLine();
			}
			catch (Exception e)	{
			}

			System.out.println (" - terminating client1");
			client1.terminate();
			System.out.println (">> terminating test 0");
		}
		break;
		
		case 1:
		{
			System.out.println (">> starting test 1");
			// In the second test, a CallBackTestGroup client is created and sends 1000 messages.
			// It should receive them.
			System.out.println (" - creating client2 (sends 1000 messages as fast as possible)");
			client1 = new CallBackTestGroup ( devName + "_client2", ifname, groupAddress, groupPort);

			for (int x=0; x<1000; x++) {
				client1.SendTestMessage (devName + "_client2_" + x, 1);
			}			

			System.out.println (" - terminating client2");
			client1.terminate();
			System.out.println (">> terminating test 1");
		}
		break;
			case 2:
			{
				System.out.println (">> starting test 2");
				// In the first test, 3 CallBackTestGroup clients are created with the same group
				// and one client sends a message. It should be received by all 3 (including sender)
				// a client is then terminated, a message is sent, then another client is terminated 
				// and a meesage send. and finally a client is created again and a message sent
				System.out.println (" - creating clientA, clientB, clientC");
				client1 = new CallBackTestGroup ( devName + "_client_A", ifname, groupAddress, groupPort);
				client2 = new CallBackTestGroup ( devName + "_client_B", ifname, groupAddress, groupPort);
				client3 = new CallBackTestGroup ( devName + "_client_C", ifname, groupAddress, groupPort);
				client1.SendTestMessage (devName + "_client_A_1", 1);
				client2.terminate();
				client1.SendTestMessage (devName + "_client_A_2", 2);
				client3.terminate();				
				client3 = null;
				client1.SendTestMessage (devName + "_client_A_3", 3);
				client3 = new CallBackTestGroup ( devName + "_clientC_bis", ifname, groupAddress, groupPort);
				client1.SendTestMessage (devName + "_client_A_4", 4);
				try {
					Thread.sleep(1000);
				}
				catch (InterruptedException e)
				{
				}
				client3.terminate();
				client1.terminate();
				System.out.println (">> terminating test 2");
			}
			break;
		}
		System.out.println (" - exit TestMulticastGroup");
		System.exit(0);
	}
	
	
	public class CallBackTestGroup implements MulticastGroupCallback {
		String clientName;
		MulticastGroup mgroup;
		FrameworkOutput board;
		
		public CallBackTestGroup (String clientName, String ifname, String groupAddress, int groupPort)
		{
			this.clientName = clientName;
			try {
				mgroup = new MulticastGroup( this, clientName, ifname, groupAddress, groupPort);
			}
			catch (MSDException e)
			{
				System.out.println ("Could not create multicast group (" + ifname +":"+ groupAddress +":"+ groupPort +")"+ e);
				e.printStackTrace();
				System.exit(-1);
			}
		}

		public void terminate ()
		{
			mgroup.terminate();
		}

		public void SendTestMessage (String string, int val)
		{
			System.out.println (" - client " + clientName + " sends message with " + string + "-" + val );

			MSDDescription descr = new MSDDescriptionImpl ();
			descr.setMSDDescriptionID( string + "-" + val);
			MSDMessage mess = new MSDMessageImpl();
			mess.setType(BigInteger.valueOf(InfoType.Message.SERVICE_UPDATE));
			mess.setContent(new MessageContentImpl());
			mess.getContent().setDescription(descr);
			mgroup.sendMSDMessage(mess);
		}
//		/**
//		 * 
//		 * @see org.ubisec.util.MulticastGroupCallback#receivedDescription(org.ubisec.msd.MSDDescription)
//		 */
//		public void receivedDescription ( MSDDescription description )
//		{
//			System.out.println ( clientName + ": received a description with ID " + description.getMSDDescriptionID() );
//		}
//
//		/**
//		 * 
//		 * @see org.ubisec.util.MulticastGroupCallback#receivedRequest(org.ubisec.msd.MSDRequest)
//		 */
//		public void receivedRequest ( MSDRequest request )
//		{
//			
//		}
		
		/**
		 * 
		 * @see org.msda.util.comm.MulticastGroupCallback#receivedMessage(org.ubisec.msd.MSDMessage)
		 */
		public void receivedMessage ( MSDMessage message )
		{
			System.out.println ( clientName + ": received a description with ID " + message.getContent().getDescription().getMSDDescriptionID() );
		}
		
	}
}
