/*
 * Created on Aug 22, 2005
 * 
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr
 * Preliminary version, please do not distribute
 * 
 * Copyright INRIA 2005
 */
package org.msda.tests.slp;


import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.math.BigInteger;
import java.rmi.RemoteException;
import java.text.FieldPosition;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;
import java.util.Vector;

import org.apache.log4j.Appender;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.RollingFileAppender;
import org.msda.MSDClientAPIService.MSDClientAPIBindingStub;
import org.msda.msd.Creator;
import org.msda.msd.ReqCreationInformation;
import org.msda.msd.ReqDescriptionInformation;
import org.msda.msd.ReqProcessingInformation;
import org.msda.msd.RequestService;
import org.msda.msd.impl.CreatorImpl;
import org.msda.msd.impl.MSDDescriptionImpl;
import org.msda.msd.impl.MSDRequestImpl;
import org.msda.msd.impl.ReqCreationInformationImpl;
import org.msda.msd.impl.ReqDescriptionInformationImpl;
import org.msda.msd.impl.ReqProcessingInformationImpl;
import org.msda.msd.impl.RequestServiceImpl;
import org.msda.plugins.slp.SLPRequest;
import org.msda.util.InfoType;

import ubisec.util.system.IFUtils;

import com.solers.slp.Advertiser;
import com.solers.slp.Locator;
import com.solers.slp.ServiceLocationEnumeration;
import com.solers.slp.ServiceLocationException;
import com.solers.slp.ServiceLocationManager;
import com.solers.slp.ServiceType;
import com.solers.slp.ServiceURL;

/**
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class ServiceAccessMSD {
	
	private Logger log;

	private static final String MSD_SERVICE_NAME = "msdservice:1:0";
	private String ctrUrl4MSDService;
	private org.msda.MSDClientAPIService.MSDClientAPIPortType msdService;
	private RFIDMusicPortType rfidService;
	private MSDDescriptionImpl[] msdDescs;
	
	private Locator locator = null;
	private int httpServerPort = 8089;
	
	
	/**
	 * 
	 * @param log
	 */
	public void setLogger( Logger log ) {
		this.log = log;
	}
	
	
	/**
	 * 
	 *
	 */
	private void init() {
        try {
	        locator = ServiceLocationManager.getLocator(Locale.FRANCE);
	    } catch(ServiceLocationException ex) {
	        ex.printStackTrace();
	        System.exit(0);
	    }
    }
	
	public void registerWantedServices() {
        try {
            Advertiser slpAdvertiser = ServiceLocationManager.getAdvertiser(Locale.FRANCE);
            IFUtils ifUtils = new IFUtils();
            String localIPAddress = ifUtils.getLocalHostAddresses()[0];
System.out.println ("register service RFIDMusicService")     ;       
            ServiceURL serviceURL2 = new ServiceURL("service:RFIDMusicService:http://"+localIPAddress+":"+httpServerPort+"/RFIDMusicService", ServiceURL.LIFETIME_PERMANENT);
            slpAdvertiser.register(serviceURL2, new Vector());

//            ServiceURL serviceURL2 = new ServiceURL("service:RFIDMusicService:http://"+"192.168.0.2"+":"+httpServerPort+"/RFIDMusicService", ServiceURL.LIFETIME_PERMANENT);
//            slpAdvertiser.register(serviceURL2, new Vector());
//            serviceURL2 = new ServiceURL("service:msdservice:http://"+"192.168.0.2"+":"+"9510"+"/discovery", ServiceURL.LIFETIME_PERMANENT);
//            slpAdvertiser.register(serviceURL2, new Vector());
            
        } catch (ServiceLocationException e) {
            e.printStackTrace();
        }
    }
    
 
    
    public String find(String name)
	{
		String serviceName = "service:"+name;
		
		SLPRequest slpRequest = new SLPRequest();
		ServiceType serviceType = new ServiceType(serviceName);
		slpRequest.setType(serviceType);
		Vector scopes = new Vector();
		scopes.add("default");
		slpRequest.setScopes(scopes);
		slpRequest.setSearchFilter("");
		
		String url = null;
		
		try
		{
		    ServiceLocationEnumeration foundServices = locator.findServices(slpRequest.getType(),slpRequest.getScopes(), slpRequest.getSearchFilter());
		    
		    if (foundServices !=null && foundServices.hasMoreElements() ) {
		        ServiceURL foundService = (ServiceURL)foundServices.next();
		        url = "http://"+foundService.getHost()+":"+foundService.getPort()+foundService.getURLPath();
		    }
		    
		    foundServices.destroy();
		    foundServices = null;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		//System.out.println ("found service " + url );
		return url;
	}
    
    
    private void findMSDService() {
        String ctrUrl = find("msdservice");
        ctrUrl4MSDService = ctrUrl;
        System.out.println ("found the MSD service at " + ctrUrl);
        msdService = new MSDClientAPIBindingStub(ctrUrl);
    }
	
	
	
	/**
	 * First call to getServices
	 * 
	 * @return partial results
	 */
	protected MSDDescriptionImpl[]  findService( int timeout, String name ) throws RemoteException {
		return findService( -1, -1, timeout, name );
	}
	
	/**
	 * Second (or more) call to get partial results.
	 * 
	 * @param clientId
	 * @param requestId
	 * @return partial results
	 */
	public MSDDescriptionImpl[] findService( int clientId, int requestId, int timeout, String name ) throws RemoteException
	{
		if (msdService == null)  findMSDService();
		
		MSDRequestImpl msdRequest = new MSDRequestImpl();
		
		if ( clientId<0 ) {
			//First call to findRFIDService
			msdDescs = null;
			
			ReqProcessingInformation rpi = new ReqProcessingInformationImpl();
			ReqCreationInformation rci = new ReqCreationInformationImpl();
			Creator creator = new CreatorImpl();
			
			Calendar cal = Calendar.getInstance();
			rci.setDate( cal );
			rci.setTime( cal );
			rci.setLifetime( BigInteger.valueOf( timeout ) );// in secondes
			rci.setCreator( creator );
			rpi.setCrea( rci );
			
			ReqDescriptionInformation reqDescription = new ReqDescriptionInformationImpl();
			RequestService requestService = new RequestServiceImpl();
			requestService.setName(name);
			
			reqDescription.setService(requestService);
			msdRequest.setDescInfo(reqDescription);
			msdRequest.setProcInfo( rpi );
			
		} else {
			
			if ( requestId < 0 ) {
				System.out.println("Invalid values : clientId="+clientId + " requestId="+requestId);
				return null;
			}
			// get partial results
			msdRequest.setClientID( BigInteger.valueOf( clientId ) );
			msdRequest.setReqID( BigInteger.valueOf( requestId ) );
		}
		
		// save the previous results... 
		MSDDescriptionImpl[] results = msdDescs;
		
		msdDescs = msdService.getServices(msdRequest);
		//System.out.println("service invoked");
		if (msdDescs == null) {
			System.out.println("the result is null");
			return null;
		} else if (msdDescs.length<=0) {
			System.out.println("the result is Empty");
			return null;
		} 
		
		MSDDescriptionImpl[] lastResults = msdDescs;
		
		if ( results!=null ) {
			// at this point, results contains the previous partial results
			// and msdDescs contains the latest results 
			String descName = lastResults[0].getDescID().getName();
			
			int size = results.length;
			if ( lastResults!=null && 
					!InfoType.RUNNING_REQUEST.equals( descName ) &&
					!InfoType.TIMEOUT_EXPIRED.equals( descName ) )
				size = size + lastResults.length;
			
			msdDescs = new MSDDescriptionImpl[ size ];
			for(int i=0;i<results.length; i++)
				msdDescs[i] = results[i];
			
			if ( lastResults!=null && 
					!InfoType.RUNNING_REQUEST.equals( descName ) &&
					!InfoType.TIMEOUT_EXPIRED.equals( descName ) )
				for(int i=0;i<lastResults.length; i++)
					msdDescs[results.length+i] = lastResults[i];
		} 
		
		return lastResults;
	}
	
	/**
	 * 
	 * @param description
	 */
	private long accessService(MSDDescriptionImpl description) {
		int clientid;
		String serviceURL ;
		String[] RFIDs;
		String[] MusicInfos;
		long tstart, tend, res;
		boolean ok;
		res = -1;
		try
		{
			//clientid = msdService.getClientID(description);
			serviceURL = description.getSvcDef().getGeneric().getAddr().getLocation();
			System.out.println("Final service Found at "+serviceURL + " create an MSD stub to it");
			rfidService = new RFIDMusicBindingStub(serviceURL,ctrUrl4MSDService);
			
			RFIDs = new String[1];
			MusicInfos = new String[1];
			RFIDs[0] = "x-010165-314101#";
			MusicInfos[0] = "unkown Singer+1#";
			tstart = System.currentTimeMillis();
			ok = rfidService.setInfos(RFIDs, MusicInfos);
			tend = System.currentTimeMillis();
			res = tend-tstart;
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
			res = -1;
		}
		return res;
		
	}

	/**
	 * 
	 * @param description
	 * @throws IOException
	 */	
	private long accessServiceVariable(MSDDescriptionImpl description) throws IOException 
	{
		long res = 0;
		FileWriter fileW = null;
		try
		{
			
			java.io.File file = new java.io.File("log"+File.separator+"upnp.msd.access.log");
			if (!file.exists()) file.createNewFile();
			else file.delete();
			fileW = new FileWriter(file);
			
			
			String serviceURL = description.getSvcDef().getGeneric().getAddr().getLocation();
			System.out.println("Service Found at "+serviceURL);
			rfidService = new RFIDMusicBindingStub(serviceURL,ctrUrl4MSDService);
			for (int i=1; i<=100; i++)
			{
				String RFID ="";
				String MusicInfo="";
				for (int k=0; k<=i*100; k++)
				{
					RFID = RFID+"x-010165-31410"+k+"#";
					MusicInfo = MusicInfo + "unkown Singer+"+k+"#";
				}			
				
				System.out.println("Send "+i*100+" elements to Service");
				long tstart = System.currentTimeMillis();
				boolean resp = rfidService.setInfo(RFID, MusicInfo);
				long tend = System.currentTimeMillis();
				long spenttime = tend-tstart;
				System.out.println("The response value is "+ resp);
				rfidService.removeAll();
				// write log file
				fileW.write("\r\nAccess Time for "+i*100+" elements : "+spenttime);
				fileW.flush();
				Thread.sleep(200);
			}
			fileW.close();
		}
		catch (Exception ex)
		{
			fileW.close();
			ex.printStackTrace();
		}
		return res;
	}
	
	/*
	 * 
	 * MAIN entry point for testing service access via MSD
	 * 
	 */
	
	/**
	 * 
	 * @param args
	 */
	public static void main( String[] args ) {
		String name = "RFIDMusicService";
		int timeout = 3;
		MSDDescriptionImpl[] res = null;
		long start, end, diff, accesstime;
		ServiceAccessMSD local ;
		final boolean STATIC_SIZE_ACCESS_DATA = true;

//		BasicConfigurator.configure();
		String pattern = "%m%n";//"%d{HH:mm:ss,SSS} %p [%t] %m%n";
		PatternLayout patternLayout = new PatternLayout( pattern );
		SimpleDateFormat format = new SimpleDateFormat( "yyyyMMdd");
		StringBuffer buffer = format.format( Calendar.getInstance().getTime(), new StringBuffer(), new FieldPosition( NumberFormat.INTEGER_FIELD ) );
		String date = buffer.toString();
		String filename = date +"_msd_slp_svca.log";
		Logger log = Logger.getLogger( filename );
		Appender appender;
		try {
			appender = new RollingFileAppender(patternLayout, filename  );
			log.addAppender( appender );
		} catch (IOException e1) {
			e1.printStackTrace();
		} 
		log.info( "Service access time" );
		
		// create an instance of the test
		local = new ServiceAccessMSD();
		local.init();
		local.setLogger( log );
		
		local.registerWantedServices();
		
		// First search for the MSD service
		//String ctrUrl = local.find(name);
		
		local.findMSDService();
		
		try {
			start = System.currentTimeMillis();
			res = local.findService( timeout, name );
			end = System.currentTimeMillis();
			if (res == null || res.length<=0) {
				System.out.println ("no music service found");
				return;
			}
			for (int x=0; x<50; x++)
			{
				try 
				{
					Thread.sleep (3000);
					
					start = System.currentTimeMillis();
					System.out.println("ServiceAccess VIA MSD..");
					if (STATIC_SIZE_ACCESS_DATA) {
						accesstime = local.accessService(res[0]);
					} else {
						accesstime = local.accessServiceVariable(res[0]);
					}
					end = System.currentTimeMillis();

					diff = end - start;
					if (accesstime >0) {
						log.info( Long.toString(diff) + ";" + Long.toString(accesstime) );
					}
					else 
					{
						System.out.println ("ERROR, could not access service");
						log.info( "access error" );
					}
				}
				catch (IOException e)
				{
				}
				catch (InterruptedException e)
				{
				}
			}	
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		System.exit( 1 );
	}
}

