/*
 * Created on Aug 23, 2005
 * 
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr
 * Preliminary version, please do not distribute
 * 
 * Copyright INRIA 2005
 */
package org.msda.tests.slp;

import java.io.IOException;
import java.math.BigInteger;
import java.rmi.RemoteException;
import java.text.FieldPosition;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;
import java.util.Vector;

import org.apache.log4j.Appender;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.RollingFileAppender;
import org.msda.MSDClientAPIService.MSDClientAPIBindingStub;
import org.msda.msd.Creator;
import org.msda.msd.MSDDescription;
import org.msda.msd.ReqCreationInformation;
import org.msda.msd.ReqDescriptionInformation;
import org.msda.msd.ReqProcessingInformation;
import org.msda.msd.RequestService;
import org.msda.msd.impl.CreatorImpl;
import org.msda.msd.impl.MSDDescriptionImpl;
import org.msda.msd.impl.MSDRequestImpl;
import org.msda.msd.impl.ReqCreationInformationImpl;
import org.msda.msd.impl.ReqDescriptionInformationImpl;
import org.msda.msd.impl.ReqProcessingInformationImpl;
import org.msda.msd.impl.RequestServiceImpl;
import org.msda.plugins.slp.SLPRequest;
import org.msda.tests.upnp.RFIDMusicPortType;
import org.msda.tests.upnp.UPnPListener;
import org.msda.util.InfoType;

import ubisec.util.system.IFUtils;

import com.siemens.upnp.controlpoint.ControlPoint;
import com.solers.slp.Advertiser;
import com.solers.slp.Locator;
import com.solers.slp.ServiceLocationEnumeration;
import com.solers.slp.ServiceLocationException;
import com.solers.slp.ServiceLocationManager;
import com.solers.slp.ServiceType;
import com.solers.slp.ServiceURL;

/**
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class TestMSD {
	// generic test
    private Logger log;
    private org.msda.MSDClientAPIService.MSDClientAPIPortType msdService;
    private RFIDMusicPortType rfidService;
    private MSDDescriptionImpl[] msdDescs;
    // specific to slp
    private Locator locator = null;
    private int httpServerPort = 8089;
	
	/**
	 * 
	 * @param log
	 */
	public void setLogger( Logger log ) {
		this.log = log;
	}
	
	
	/**
	 * 
	 *
	 */
	private void init() {
        try {
	        locator = ServiceLocationManager.getLocator(Locale.FRANCE);
	    } catch(ServiceLocationException ex) {
	        ex.printStackTrace();
	        System.exit(0);
	    }
    }

    private void findMSDService() {
        String ctrUrl = find("msdservice");
        System.out.println ("found the MSD service at " + ctrUrl);
        msdService = new MSDClientAPIBindingStub(ctrUrl);
    }
   
    public void registerWantedServices() {
        try {
            Advertiser slpAdvertiser = ServiceLocationManager.getAdvertiser(Locale.FRANCE);
            IFUtils ifUtils = new IFUtils();
            String localIPAddress = ifUtils.getLocalHostAddresses()[0];
System.out.println ("register service RFIDMusicService")     ;       
            ServiceURL serviceURL2 = new ServiceURL("service:RFIDMusicService:http://"+localIPAddress+":"+httpServerPort+"/RFIDMusicService", ServiceURL.LIFETIME_PERMANENT);
            slpAdvertiser.register(serviceURL2, new Vector());

//            ServiceURL serviceURL2 = new ServiceURL("service:RFIDMusicService:http://"+"192.168.0.2"+":"+httpServerPort+"/RFIDMusicService", ServiceURL.LIFETIME_PERMANENT);
//            slpAdvertiser.register(serviceURL2, new Vector());
//            serviceURL2 = new ServiceURL("service:msdservice:http://"+"192.168.0.2"+":"+"9510"+"/discovery", ServiceURL.LIFETIME_PERMANENT);
//            slpAdvertiser.register(serviceURL2, new Vector());
            
        } catch (ServiceLocationException e) {
            e.printStackTrace();
        }
    }
    
 
    
    public String find(String name)
	{
		String serviceName = "service:"+name;
		
		SLPRequest slpRequest = new SLPRequest();
		ServiceType serviceType = new ServiceType(serviceName);
		slpRequest.setType(serviceType);
		Vector scopes = new Vector();
		scopes.add("default");
		slpRequest.setScopes(scopes);
		slpRequest.setSearchFilter("");
		
		String url = null;
		
		try
		{
		    ServiceLocationEnumeration foundServices = locator.findServices(slpRequest.getType(),slpRequest.getScopes(), slpRequest.getSearchFilter());
		    
		    if (foundServices !=null && foundServices.hasMoreElements() ) {
		        ServiceURL foundService = (ServiceURL)foundServices.next();
		        url = "http://"+foundService.getHost()+":"+foundService.getPort()+foundService.getURLPath();
		    }
		    
		    foundServices.destroy();
		    foundServices = null;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		//System.out.println ("found service " + url );
		return url;
	}
    
    
 
    /**
     * First call to getServices
     * 
     * @return partial results
     */
    protected MSDDescriptionImpl[]  findService( int timeout, String name ) throws RemoteException {
        return findService( -1, -1, timeout, name );
    }
    
    /**
     * Second (or more) call to get partial results.
     * 
     * @param clientId
     * @param requestId
     * @return partial results
     */
    public MSDDescriptionImpl[] findService( int clientId, int requestId, int timeout, String name ) throws RemoteException
    {
    	if (msdService == null)  findMSDService();
    	
    	MSDRequestImpl msdRequest = new MSDRequestImpl();
    	
    	if ( clientId<0 ) {
    	    //First call to findRFIDService
    	    msdDescs = null;
    	    
	    	ReqProcessingInformation rpi = new ReqProcessingInformationImpl();
	        ReqCreationInformation rci = new ReqCreationInformationImpl();
	        Creator creator = new CreatorImpl();
	        
	        Calendar cal = Calendar.getInstance();
	        rci.setDate( cal );
	        rci.setTime( cal );
	        rci.setLifetime( BigInteger.valueOf( timeout ) );// in secondes
	        rci.setCreator( creator );
	        rpi.setCrea( rci );
	        
	    	ReqDescriptionInformation reqDescription = new ReqDescriptionInformationImpl();
	    	RequestService requestService = new RequestServiceImpl();
	    	requestService.setName(name);
	
	        reqDescription.setService(requestService);
	        msdRequest.setDescInfo(reqDescription);
	        msdRequest.setProcInfo( rpi );
	        
    	} else {
    	    
    	    if ( requestId < 0 ) {
    	        //System.out.println("Invalid values : clientId="+clientId + " requestId="+requestId);
    	        return null;
    	    }
    	    // get partial results
    	    msdRequest.setClientID( BigInteger.valueOf( clientId ) );
    	    msdRequest.setReqID( BigInteger.valueOf( requestId ) );
    	}
        
    	// save the previous results... 
    	MSDDescriptionImpl[] results = msdDescs;
    	
    	//System.out.println("service invoked");
    	msdDescs = msdService.getServices(msdRequest);
    	if (msdDescs == null) {
    	    //System.out.println("the result is null");
    	    return null;
    	    
    	} else if (msdDescs.length<=0) {
    	    //System.out.println("the result is Empty");
    	    return null;
    	    
    	} 
    	
    	MSDDescriptionImpl[] lastResults = msdDescs;
    	
    	if ( results!=null ) {
    	    // at this point, results contains the previous partial results
    	    // and msdDescs contains the latest results 
    	    String descName = lastResults[0].getDescID().getName();
    	    
    	    int size = results.length;
    	    if ( lastResults!=null && 
    	            !InfoType.RUNNING_REQUEST.equals( descName ) &&
    	            !InfoType.TIMEOUT_EXPIRED.equals( descName ) )
    	        size = size + lastResults.length;
    	    
    	    msdDescs = new MSDDescriptionImpl[ size ];
    	    for(int i=0;i<results.length; i++)
    	        msdDescs[i] = results[i];
    	    
    	    if ( lastResults!=null && 
    	            !InfoType.RUNNING_REQUEST.equals( descName ) &&
    	            !InfoType.TIMEOUT_EXPIRED.equals( descName ) )
    	        for(int i=0;i<lastResults.length; i++)
    	            msdDescs[results.length+i] = lastResults[i];
	    } 
    	
        return lastResults;
    }
    
    public static void main( String[] args ) {
		String name = "RFIDMusicService"; // "urn:printbasic1";//
		int timeout = 4;
		MSDDescription[] res = null;
		long start, end, diff;
		TestMSD test ;

		
//		BasicConfigurator.configure();
		String pattern = "%m%n";//"%d{HH:mm:ss,SSS} %p [%t] %m%n";
		PatternLayout patternLayout = new PatternLayout( pattern );
		SimpleDateFormat format = new SimpleDateFormat( "yyyyMMdd");
		StringBuffer buffer = format.format( Calendar.getInstance().getTime(), new StringBuffer(), new FieldPosition( NumberFormat.INTEGER_FIELD ) );
		String date = buffer.toString();
		String filename = date +"_msd.local.slp.log";
		Logger log = Logger.getLogger( filename );
		log.removeAllAppenders();
		Appender appender;
		try {
			appender = new RollingFileAppender(patternLayout, filename  );
			log.addAppender( appender );
		} catch (IOException e1) {
			e1.printStackTrace();
		} 
		log.info( "Total SD time" );

		//
        test = new TestMSD();
        test.init();
        test.setLogger( log );

        test.registerWantedServices();
        
		//String ctrUrl = test.find(name);
		//System.out.println ("found the " + name + " service at " + ctrUrl);

        
		// First search for the MSD service
        test.findMSDService();
		
		// MSD search for the service
		log.info( "Will look for service " + name );
		for (int x=0; x<50; x++)
		{
			try {
				Thread.sleep (6000);
			}
			catch (InterruptedException e)
			{
				
			}
			start = System.currentTimeMillis();
			try {
				res = test.findService( timeout, name );
			} catch (RemoteException e) {
				e.printStackTrace();
			}
			end = System.currentTimeMillis();
			
			// now write the discovery time in the output file
			diff = end - start;
			if( res==null || res.length==0 )
			{
				log.info( "Not found " + Long.toString(diff) );
			}
			else 
			{
				if ( res[0].getDescID().getName().equals( InfoType.TIMEOUT_EXPIRED ) )
				{
					log.info( "Not found " + Long.toString(diff) );
				}
				else if ( res[0].getDescID().getName().equals( InfoType.RUNNING_REQUEST ) ) {
				    log.info( "Not found " + Long.toString(diff) );
				}
				else 
				{
				   log.info( Long.toString(diff) );
//					System.out.println ("service found in network " + res[0].getCreationInformation().getCreator().getNetworkID().getName());
				}
			}
		}
        System.exit( 1 );
    }
}
