/*
 * Created on June 13, 2005
 *
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr<br>
 *          Rafik.Chibout@inria.fr
 * 
 * Copyright INRIA 2005
 */
package org.msda.tests.slp;




import org.apache.log4j.Appender;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.RollingFileAppender;
import org.msda.MSDClientAPIService.MSDClientAPIBindingStub;
import org.msda.msd.*;
import org.msda.msd.impl.*;
import org.msda.util.InfoType;
import org.msda.util.JaxMeUtil;

import com.solers.slp.Locator;
import com.solers.slp.ServiceLocationEnumeration;
import com.solers.slp.ServiceLocationManager;
import com.solers.slp.ServiceType;
import com.solers.slp.ServiceURL;



import java.io.IOException;
import java.math.BigInteger;
import java.net.DatagramPacket;
import java.net.Socket;
import java.net.UnknownHostException;
import java.rmi.RemoteException;
import java.text.FieldPosition;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.*;

import javax.xml.bind.JAXBException;

/**
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 */

public class TestMSDSocket {
    
    //final static Logger logger = Logger.getLogger(TestMSDSocket.class);
    
    Logger log;
    private void setLogger( Logger log ) {
        this.log = log;
    }
    
    private org.msda.MSDClientAPIService.MSDClientAPIPortType msdService;
    private String ctrUrlMSD;
    private void findMSDService() {
        
        
        try
		{
    		Locator locator = ServiceLocationManager.getLocator(Locale.FRANCE);
    		ServiceType serviceType = new ServiceType("service:msdservice");
    		Vector scopes = new Vector(); 
    		scopes.add("default");
//    		System.out.println("BEFORE foundServices");
    		ServiceLocationEnumeration foundServices = locator.findServices(serviceType, scopes, "");
//    		if ( foundServices==null )
//    			System.out.println("AFTER foundServices is null");
    		if (foundServices !=null)
    		{
				System.out.println("msd service found");
    			if (foundServices.hasMoreElements())
    			{
    				ServiceURL foundService = (ServiceURL)foundServices.next();
    				ctrUrlMSD = "http://"+foundService.getHost()+":"+foundService.getPort()+foundService.getURLPath();
    				System.out.println("clientSLP: MSDService URL="+ctrUrlMSD);
    				
    				msdService = new MSDClientAPIBindingStub(ctrUrlMSD);
    			}
    		}
   			else
   			{
				System.out.println("msd service not found");
   			}    				
    			
		}
    	catch(Exception ex)
		{
    		ex.printStackTrace();
		}
    }
    
    String address = "";
    int port = -1;
    
    private void findDirectMSDService() {
        String ctrUrl = "";
        try
        {
            Locator locator = ServiceLocationManager.getLocator(Locale.FRANCE);
            ServiceType serviceType = new ServiceType("service:directmsdservice");
            Vector scopes = new Vector(); 
            scopes.add("default");
            
            ServiceLocationEnumeration foundServices = locator.findServices(serviceType, scopes, "");
            if (foundServices !=null) {
                if (foundServices.hasMoreElements())
                {
                    ServiceURL foundService = (ServiceURL)foundServices.next();
                    ctrUrl = "http://"+foundService.getHost()+":"+foundService.getPort()+foundService.getURLPath();
                    System.out.println("clientSLP: DirectMSDService URL="+ctrUrl);
                    
                    address = foundService.getHost();
                    port = foundService.getPort();
                }
            }
                
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
        }
        
    }
    
    public MSDDescription findService( String name, int timeout )
    {
        // find URL Of MSDService
        try {
            Socket connection = new Socket(address, port) ;
            
            UbisecID id = new UbisecIDImpl();
            id.setName( "ClientSLPSocket" );
            
            MSDRequestImpl msdRequest = new MSDRequestImpl();
            
            ReqProcessingInformation rpi = new ReqProcessingInformationImpl();
            ReqCreationInformation rci = new ReqCreationInformationImpl();
	        Creator creator = new CreatorImpl();
	        creator.setMgrID( id );
	        creator.setNetID( id );
	        Calendar cal = Calendar.getInstance();
	        rci.setDate( cal );
	        rci.setTime( cal );
	        rci.setLifetime( BigInteger.valueOf( timeout ) );// in secondes
	        rci.setCreator( creator );
	        rpi.setCrea( rci );
            
            RequestService requestService = new RequestServiceImpl();
            requestService.setName( name );
            
            ReqDescriptionInformation reqDescription = new ReqDescriptionInformationImpl();
            reqDescription.setService(requestService);
            msdRequest.setDescInfo(reqDescription);
            msdRequest.setProcInfo( rpi );
            
            PropagationInformation pi = new PropagationInformationImpl();
            msdRequest.setPropaInfo( pi );
            
            msdRequest.setClientID( BigInteger.valueOf( -1 ) );
    	    msdRequest.setReqID( BigInteger.valueOf( -1 ) );
            
            MessageContent content = new MessageContentImpl();
            content.setRequest( msdRequest );
            
            MSDMessage mess = new MSDMessageImpl();
            mess.setSender( id );
            mess.setType( BigInteger.valueOf( InfoType.Message.CLIENT_NEW_REQUEST ) );
            mess.setContent( content );
            
            JaxMeUtil jaxme = new JaxMeUtil();
            String message = jaxme.toString( mess );
            connection.getOutputStream().write( message.getBytes() );
            
            byte[] buf = new byte[InfoType.MSG_BUFFER_SIZE];
            //long start = System.currentTimeMillis();
            //System.out.println( "wait for the response..." );
            int size = connection.getInputStream().read( buf );
            //long end = System.currentTimeMillis() - start;
            //System.out.println("get a result of "+size +" after "+end);
            if ( size < 0) 
                return null;
            
           // log.info( Long.toString( end ) );
            
            // Resize the byte[] to keep only the used bytes
            byte[] tmp = buf; 
            buf = new byte[size];
            System.arraycopy(tmp, 0, buf, 0, size);
            
            
            //System.out.println( new String(buf));
            MSDMessage found = jaxme.toMSDMessage( buf );
//          take the first result...
          MSDDescription desc = (MSDDescription)found.getContent().getResults().get(0);
          
          return desc;
            
            
//            findMSDService();
//            if ( msdService!=null ) {
//                System.out.println( "MSD service found" );
//                
//                MSDMessage found = jaxme.toMSDMessage( buf );
//                
//                // take the first result...
//                MSDDescription desc = (MSDDescription)found.getContent().getResults().get(0);
//                
//                if ( desc.getDescriptionID().getName().equals( InfoType.TIMEOUT_EXPIRED )) {
//                    System.out.println("Not found !");
//                    return;
//                }
//                    
//                String epService = desc.getServiceDefinition().getGeneric().getAddress().getLocation();
//                String msdDescId = desc.getDescriptionID().getName();
//                epService = epService + "/" + msdDescId;
//                
//                RFIDMusicBindingStub rfidService = new RFIDMusicBindingStub(epService, ctrUrlMSD);
//                
//                String res = rfidService.getInfo( "4-010165-314153" );
//                
//                System.out.println( res );
//            }
            
        } catch (JAXBException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
            
        } catch (UnknownHostException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return null;
    }
    
    public static void main(String[] args) {
        
        TestMSDSocket test = new TestMSDSocket();
        
		String name = "RFIDMusicService"; // "urn:printbasic1";//
		int timeout = 4;
		MSDDescription res = null;
		long start, end, diff;
		//TestMSD test ;

		
//		BasicConfigurator.configure();
		String pattern = "%m%n";//"%d{HH:mm:ss,SSS} %p [%t] %m%n";
		PatternLayout patternLayout = new PatternLayout( pattern );
		SimpleDateFormat format = new SimpleDateFormat( "yyyyMMdd");
		StringBuffer buffer = format.format( Calendar.getInstance().getTime(), new StringBuffer(), new FieldPosition( NumberFormat.INTEGER_FIELD ) );
		String date = buffer.toString();
		String filename = date +"_msd.socket1.slp.log";
		Logger log = Logger.getLogger( filename );
		Appender appender;
		try {
			appender = new RollingFileAppender(patternLayout, filename  );
			log.addAppender( appender );
		} catch (IOException e1) {
			e1.printStackTrace();
		} 
//		
//		String filename2 = date +"_msd.socket2.slp.log";
//		Logger log2 = Logger.getLogger( filename2 );
//		Appender appender2;
//		try {
//			appender2 = new RollingFileAppender(patternLayout, filename2  );
//			log2.addAppender( appender2 );
//		} catch (IOException e1) {
//			e1.printStackTrace();
//		} 
//		
		log.info( "Total SD time" );
		//log2.info("Total SD time" );
		
       // test.setLogger( log2 );

        //test.registerWantedServices();
        
		//String ctrUrl = test.find(name);
		//System.out.println ("found the " + name + " service at " + ctrUrl);

		test.findDirectMSDService();
		
		// MSD search for the service
		log.info( "Will look for service " + name );
		for (int x=0; x<50; x++)
		{
			try {
				Thread.sleep (1000);
			}
			catch (InterruptedException e)
			{
				
			}
			start = System.currentTimeMillis();
			res = test.findService(name, timeout);
			end = System.currentTimeMillis();
			
			// now write the discovery time in the output file
			diff = end - start;
			if( res==null ) //|| res.length==0 )
			{
				log.info( "Not found " + Long.toString(diff) );
			}
			else 
			{
				if ( res.getDescID().getName().equals( InfoType.TIMEOUT_EXPIRED ) )
				{
					log.info( "Not found " + Long.toString(diff) );
				}
				else if ( res.getDescID().getName().equals( InfoType.RUNNING_REQUEST ) ) {
				    log.info( "Not found " + Long.toString(diff) );
				}
				else 
				{
				   log.info( Long.toString(diff) );
//					System.out.println ("service found in network " + res[0].getCreationInformation().getCreator().getNetworkID().getName());
				}
			}
		}
        System.exit( 1 );
    }
}
