/*
 * Created on June 13, 2005
 *
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr<br>
 *          Rafik.Chibout@inria.fr
 * 
 * Copyright INRIA 2005
 */
package org.msda.tests.slp;




import org.apache.log4j.Appender;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.RollingFileAppender;
import org.msda.MSDClientAPIService.MSDClientAPIBindingStub;
import org.msda.context.ContextUtil;
import org.msda.context.NetworkTemplate;
import org.msda.context.UserTemplate;
import org.msda.msd.*;
import org.msda.msd.impl.*;
import org.msda.util.InfoType;
import org.msda.util.JaxMeUtil;

import com.solers.slp.Locator;
import com.solers.slp.ServiceLocationEnumeration;
import com.solers.slp.ServiceLocationManager;
import com.solers.slp.ServiceType;
import com.solers.slp.ServiceURL;



import java.io.IOException;
import java.math.BigInteger;
import java.net.Socket;
import java.net.UnknownHostException;
import java.text.FieldPosition;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.*;

import javax.xml.bind.JAXBException;

/**
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 */

public class TestMSDSocketContext {
    
    //final static Logger logger = Logger.getLogger(TestMSDSocket.class);
    
    private static Random random;
    
    Logger log;
    private void setLogger( Logger log ) {
        this.log = log;
    }
    
    private org.msda.MSDClientAPIService.MSDClientAPIPortType msdService;
    private String ctrUrlMSD;
    private void findMSDService() {
        
        
        try
		{
    		Locator locator = ServiceLocationManager.getLocator(Locale.FRANCE);
    		ServiceType serviceType = new ServiceType("service:msdservice");
    		Vector scopes = new Vector(); 
    		scopes.add("default");
//    		System.out.println("BEFORE foundServices");
    		ServiceLocationEnumeration foundServices = locator.findServices(serviceType, scopes, "");
//    		if ( foundServices==null )
//    			System.out.println("AFTER foundServices is null");
    		if (foundServices !=null)
    		{
				System.out.println("msd service found");
    			if (foundServices.hasMoreElements())
    			{
    				ServiceURL foundService = (ServiceURL)foundServices.next();
    				ctrUrlMSD = "http://"+foundService.getHost()+":"+foundService.getPort()+foundService.getURLPath();
    				System.out.println("clientSLP: MSDService URL="+ctrUrlMSD);
    				
    				msdService = new MSDClientAPIBindingStub(ctrUrlMSD);
    			}
    		}
   			else
   			{
				System.out.println("msd service not found");
   			}    				
    			
		}
    	catch(Exception ex)
		{
    		ex.printStackTrace();
		}
    }
    
    String address = "";
    int port = -1;
    
    private void findDirectMSDService() {
        String ctrUrl = "";
        try
        {
            Locator locator = ServiceLocationManager.getLocator(Locale.FRANCE);
            ServiceType serviceType = new ServiceType("service:directmsdservice");
            Vector scopes = new Vector(); 
            scopes.add("default");
            
            ServiceLocationEnumeration foundServices = locator.findServices(serviceType, scopes, "");
            if (foundServices !=null) {
                if (foundServices.hasMoreElements())
                {
                    ServiceURL foundService = (ServiceURL)foundServices.next();
                    ctrUrl = "http://"+foundService.getHost()+":"+foundService.getPort()+foundService.getURLPath();
                    System.out.println("clientSLP: DirectMSDService URL="+ctrUrl);
                    
                    address = foundService.getHost();
                    port = foundService.getPort();
                }
            }
                
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
        }
        
    }
    
    private ContextNode create5Nodes(int from) {
        
        TypeValue[] compValues = new TypeValueImpl[5];
    	  
    	// create the leafs
        ContextNode[] ctxN = new ContextNodeImpl[5];
    	ComparisonNode[] compNodes = new ComparisonNodeImpl[5];
    	for( int i=0;i<compNodes.length;i++) {
    	    
    	    compValues[i] = new TypeValueImpl();
    	    int ra = random.nextInt();
    	    if ( ra < 0 || ra > 2000000000 )
    	        ra = 1813984761;
    	    compValues[i].setInt( BigInteger.valueOf(ra) );   
    	    
    	    compNodes[i] = new ComparisonNodeImpl();
    	    int index = from+i+1;
    	    compNodes[i].setName( "CtxT"+index );
    	    compNodes[i].setOp("L");
    	    compNodes[i].setVal(compValues[i]);
    	    
    	    ctxN[i] = new ContextNodeImpl();
    	    ctxN[i].setCa( compNodes[i] );
    	}
    
    	ContextNode[] ctxNodes = new ContextNodeImpl[4];
    	CombinationNode[] combNodes = new CombinationNodeImpl[4];
    	for( int i=0;i<combNodes.length;i++) {
    	    combNodes[i] = new CombinationNodeImpl();
    	    combNodes[i].setOp( "OR" );  
    	    
    	    ctxNodes[i] = new ContextNodeImpl();
    	    ctxNodes[i].setCi( combNodes[i] );
    	}
    	
    	ctxNodes[3].getCi().setCN1( ctxN[3] );
    	ctxNodes[3].getCi().setCN2( ctxN[4] );
    	
    	ctxNodes[1].getCi().setCN1( ctxN[0] );
    	ctxNodes[1].getCi().setCN2( ctxN[1] );
    	
    	ctxNodes[2].getCi().setCN1( ctxN[2] );
    	ctxNodes[2].getCi().setCN2( ctxNodes[3] );
    	
    	ctxNodes[0].getCi().setCN1( ctxNodes[1] );
    	ctxNodes[0].getCi().setCN2( ctxNodes[2] );
    	
    	return ctxNodes[0];
    }
    
    private ContextNode create10Nodes(int from) {
        
        TypeValue[] compValues = new TypeValueImpl[10];
        
        // create the leafs
        ContextNode[] ctxN = new ContextNodeImpl[10];
        ComparisonNode[] compNodes = new ComparisonNodeImpl[10];
        for( int i=0;i<compNodes.length;i++) {
            
            compValues[i] = new TypeValueImpl();
            int ra = random.nextInt();
    	    if ( ra < 0 || ra > 2000000000 )
    	        ra = 1813984761;
    	    compValues[i].setInt( BigInteger.valueOf(ra) );  
            
            compNodes[i] = new ComparisonNodeImpl();
            int index = from + i+1;
            compNodes[i].setName( "CtxT"+index );
            compNodes[i].setOp("L");
            compNodes[i].setVal(compValues[i]);
            
            ctxN[i] = new ContextNodeImpl();
    	    ctxN[i].setCa( compNodes[i] );
        }
        
        ContextNode[] ctxNodes = new ContextNodeImpl[9];
        CombinationNode[] combNodes = new CombinationNodeImpl[9];
        for( int i=0;i<combNodes.length;i++) {
            combNodes[i] = new CombinationNodeImpl();
            combNodes[i].setOp( "OR" );   
            
            ctxNodes[i] = new ContextNodeImpl();
    	    ctxNodes[i].setCi( combNodes[i] );
        }
        
        // right part
        ctxNodes[7].getCi().setCN1( ctxN[6] );
        ctxNodes[7].getCi().setCN2( ctxN[7] );
        
        ctxNodes[8].getCi().setCN1( ctxN[8] );
        ctxNodes[8].getCi().setCN2( ctxN[9] );
        
        ctxNodes[6].getCi().setCN1( ctxNodes[7] );
        ctxNodes[6].getCi().setCN2( ctxNodes[8] );
        
        ctxNodes[5].getCi().setCN1( ctxN[4] );
        ctxNodes[5].getCi().setCN2( ctxN[5] );
        
        ctxNodes[2].getCi().setCN1( ctxNodes[5] );
        ctxNodes[2].getCi().setCN2( ctxNodes[6] );
        
        // left part
        ctxNodes[3].getCi().setCN1( ctxN[0] );
        ctxNodes[3].getCi().setCN2( ctxN[1] );
        
        ctxNodes[4].getCi().setCN1( ctxN[2] );
        ctxNodes[4].getCi().setCN2( ctxN[3] );
        
        ctxNodes[1].getCi().setCN1( ctxNodes[3] );
        ctxNodes[1].getCi().setCN2( ctxNodes[4] );
        
        // last
        ctxNodes[0].getCi().setCN1( ctxNodes[1] );
        ctxNodes[0].getCi().setCN2( ctxNodes[2] );
        
        return ctxNodes[0];
        
    }
    
    private ContextNode create20Nodes(int from) {
       
        TypeValue[] compValues = new TypeValueImpl[20];
        
        // create the leafs
        ContextNode[] ctxN = new ContextNodeImpl[20];
        ComparisonNode[] compNodes = new ComparisonNodeImpl[20];
        for( int i=0;i<compNodes.length;i++) {
            
            compValues[i] = new TypeValueImpl();
            int ra = random.nextInt();
    	    if ( ra < 0 || ra > 2000000000 )
    	        ra = 1813984761;
    	    compValues[i].setInt( BigInteger.valueOf(ra) );    
            
            compNodes[i] = new ComparisonNodeImpl();
            int index = i+from+1;
            compNodes[i].setName( "CtxT"+index );
            compNodes[i].setOp("L");
            compNodes[i].setVal(compValues[i]);
            
            ctxN[i] = new ContextNodeImpl();
    	    ctxN[i].setCa( compNodes[i] );
        }
        
        ContextNode[] ctxNodes = new ContextNodeImpl[19];
        CombinationNode[] combNodes = new CombinationNodeImpl[19];
        for( int i=0;i<combNodes.length;i++) {
            combNodes[i] = new CombinationNodeImpl();
            combNodes[i].setOp( "OR" );   
            
            ctxNodes[i] = new ContextNodeImpl();
    	    ctxNodes[i].setCi( combNodes[i] );
        }
        
        //  left part
        ctxNodes[7].getCi().setCN1( ctxN[0] );
        ctxNodes[7].getCi().setCN2( ctxN[1] );
        
        ctxNodes[8].getCi().setCN1( ctxN[2] );
        ctxNodes[8].getCi().setCN2( ctxN[3] );
        
        ctxNodes[3].getCi().setCN1( ctxNodes[7] );
        ctxNodes[3].getCi().setCN2( ctxNodes[8] );
        
        ctxNodes[9].getCi().setCN1( ctxN[4] );
        ctxNodes[9].getCi().setCN2( ctxN[5] );
        
        ctxNodes[10].getCi().setCN1( ctxN[6] );
        ctxNodes[10].getCi().setCN2( ctxN[7] );
        
        ctxNodes[4].getCi().setCN1( ctxNodes[9] );
        ctxNodes[4].getCi().setCN2( ctxNodes[10] );
        
        ctxNodes[1].getCi().setCN1( ctxNodes[3] );
        ctxNodes[1].getCi().setCN2( ctxNodes[4] );
        
        // right part
        ctxNodes[11].getCi().setCN1( ctxN[8] );
        ctxNodes[11].getCi().setCN2( ctxN[9] );
        
        ctxNodes[12].getCi().setCN1( ctxN[10] );
        ctxNodes[12].getCi().setCN2( ctxN[11] );
        
        ctxNodes[5].getCi().setCN1( ctxNodes[11] );
        ctxNodes[5].getCi().setCN2( ctxNodes[12] );
        
        ctxNodes[15].getCi().setCN1( ctxN[12] );
        ctxNodes[15].getCi().setCN2( ctxN[13] );
        
        ctxNodes[16].getCi().setCN1( ctxN[14] );
        ctxNodes[16].getCi().setCN2( ctxN[15] );
        
        ctxNodes[13].getCi().setCN1( ctxNodes[15] );
        ctxNodes[13].getCi().setCN2( ctxNodes[16] );
        
        ctxNodes[17].getCi().setCN1( ctxN[16] );
        ctxNodes[17].getCi().setCN2( ctxN[17] );
        
        ctxNodes[18].getCi().setCN1( ctxN[18] );
        ctxNodes[18].getCi().setCN2( ctxN[19] );
        
        ctxNodes[14].getCi().setCN1( ctxNodes[17] );
        ctxNodes[14].getCi().setCN2( ctxNodes[18] );
        
        ctxNodes[6].getCi().setCN1( ctxNodes[13] );
        ctxNodes[6].getCi().setCN2( ctxNodes[14] );
        
        ctxNodes[2].getCi().setCN1( ctxNodes[5] );
        ctxNodes[2].getCi().setCN2( ctxNodes[6] );
        
        // last
        ctxNodes[0].getCi().setCN1( ctxNodes[1] );
        ctxNodes[0].getCi().setCN2( ctxNodes[2] );
        
        return ctxNodes[0];
    }
    
    
    private ContextNode createRules( int n ) {
        
        // enable all cases
        switch(n) {
        case 0:
            TypeValue compValue = new TypeValueImpl();
            compValue.setStr( "Low" );    
            ComparisonNode compNode = new ComparisonNodeImpl();	
            compNode.setName( "CtxBandwidth" );
            compNode.setOp("=");
            compNode.setVal(compValue);
            
            ContextNode n1 = new ContextNodeImpl();
            n1.setCa(compNode);
            
            compValue = new TypeValueImpl();
            compValue.setStr( "Medium" );    
            compNode = new ComparisonNodeImpl();	
            compNode.setName( "CtxBandwidth" );
            compNode.setOp("=");
            compNode.setVal(compValue);
            
            ContextNode n2 = new ContextNodeImpl();
            n2.setCa(compNode);
            
            compValue = new TypeValueImpl();
            compValue.setStr( "High" );    
            compNode = new ComparisonNodeImpl();	
            compNode.setName( "CtxBandwidth" );
            compNode.setOp("=");
            compNode.setVal(compValue);
            
            ContextNode n3 = new ContextNodeImpl();
            n3.setCa(compNode);
            
            compValue = new TypeValueImpl();
            compValue.setStr( "Open" );    
            compNode = new ComparisonNodeImpl();	
            compNode.setName( "CtxSecurity" );
            compNode.setOp("=");
            compNode.setVal(compValue);
            
            ContextNode n4 = new ContextNodeImpl();
            n4.setCa(compNode);
            
            compValue = new TypeValueImpl();
            compValue.setStr( "Secure" );    
            compNode = new ComparisonNodeImpl();	
            compNode.setName( "CtxSecurity" );
            compNode.setOp("=");
            compNode.setVal(compValue);
            
            ContextNode n5 = new ContextNodeImpl();
            n5.setCa(compNode);
            
            
            ContextNode[] ctxN = new ContextNodeImpl[4];
            CombinationNode[] combNodes3 = new CombinationNodeImpl[4];
            for( int i=0;i<combNodes3.length;i++) {
                combNodes3[i] = new CombinationNodeImpl();
                combNodes3[i].setOp( "OR" );   
                
                ctxN[i] = new ContextNodeImpl();
                ctxN[i].setCi( combNodes3[i] );
            }
            
            combNodes3[0].setOp( "AND" );
            
            ctxN[2].getCi().setCN1( n2 );
            ctxN[2].getCi().setCN2( n3 );
            
            ctxN[1].getCi().setCN1( n1 );
            ctxN[1].getCi().setCN2( ctxN[2] );
            
            ctxN[3].getCi().setCN1( n4 );
            ctxN[3].getCi().setCN2( n5 );
            
            ctxN[0].getCi().setCN1( ctxN[1] );
            ctxN[0].getCi().setCN2( ctxN[3] );
            
            return ctxN[0];
            
        case 2:
            compValue = new TypeValueImpl();
            compValue.setStr( "Low" );    
            compNode = new ComparisonNodeImpl();	
            compNode.setName( "CtxBandwidth" );
            compNode.setOp("!=");
            compNode.setVal(compValue);
            
            ContextNode ctxNode = new ContextNodeImpl();
            ctxNode.setCa( compNode );
            return ctxNode;
        case 3:
            compValue = new TypeValueImpl();
            compValue.setStr( "Secure" );    
            compNode = new ComparisonNodeImpl();	
            compNode.setName( "CtxSecurity" );
            compNode.setOp("=");
            compNode.setVal(compValue);
            
            ctxNode = new ContextNodeImpl();
            ctxNode.setCa( compNode );
            return ctxNode;
        case 4:
            compValue = new TypeValueImpl();
            compValue.setStr( "No" );    
            compNode = new ComparisonNodeImpl();	
            compNode.setName( "CtxPayment" );
            compNode.setOp("=");
            compNode.setVal(compValue);
            
            TypeValue compValue2 = new TypeValueImpl();
            compValue2.setStr( "Secure" );    
            ComparisonNode compNode2 = new ComparisonNodeImpl();	
            compNode2.setName( "CtxSecurity" );
            compNode2.setOp("=");
            compNode2.setVal(compValue2);
            
            ContextNode[] nodes = new ContextNodeImpl[3];
            for (int i=0;i<nodes.length;i++) {
            
                nodes[i] = new ContextNodeImpl();
            }
            
            nodes[0].setCa( compNode );
            nodes[1].setCa( compNode2 );
            
            CombinationNode node = new CombinationNodeImpl();
            node.setOp( "AND" );
            node.setCN1( nodes[0] );
            node.setCN2( nodes[1] );
            
            nodes[2].setCi( node );
            return nodes[2];
        case 5:
            compValue = new TypeValueImpl();
            compValue.setStr( "Yes" );    
            compNode = new ComparisonNodeImpl();	
            compNode.setName( "CtxPayment" );
            compNode.setOp("=");
            compNode.setVal(compValue);
            
            compValue2 = new TypeValueImpl();
            compValue2.setStr( "Secure" );    
            compNode2 = new ComparisonNodeImpl();	
            compNode2.setName( "CtxSecurity" );
            compNode2.setOp("=");
            compNode2.setVal(compValue2);
            
            nodes = new ContextNodeImpl[3];
            for (int i=0;i<nodes.length;i++) {
            
                nodes[i] = new ContextNodeImpl();
            }
            
            nodes[0].setCa( compNode );
            nodes[1].setCa( compNode2 );
            
            node = new CombinationNodeImpl();
            node.setOp( "AND" );
            node.setCN1( nodes[0] );
            node.setCN2( nodes[1] );
            
            nodes[2].setCi( node );
            return nodes[2];
        }
        
        return null;
    }
    
    private MSDRequest generateContext( MSDRequest msdRequest, int n ) {
        
//      Network ContextRules   HOP < 10
    	TypeValue compValue = new TypeValueImpl();
    	compValue.setInt( BigInteger.valueOf(5) );    		 	
    	ComparisonNode compNode = new ComparisonNodeImpl();	
    	compNode.setName(NetworkTemplate.HOP);
    	compNode.setOp("L");
    	compNode.setVal(compValue);
    	ContextNode ctxNode = new ContextNodeImpl();
    	ctxNode.setCa(compNode);
    	
    	// rules for bandwidth and security
    	ContextNode ctxNode2 = createRules(3);
    	
    	CombinationNode rootCbi = new CombinationNodeImpl();
    	rootCbi.setOp("AND");
    	rootCbi.setCN1( ctxNode );
    	rootCbi.setCN2( ctxNode2 );
    	
    	ContextNode rootNode = new ContextNodeImpl();
    	rootNode.setCi( rootCbi );
    	
    	ContextRulesTree crNet = new ContextRulesTreeImpl();
    	crNet.setRoot(rootNode);
    	
    	//ContextRulesTree crNet = new ContextRulesTreeImpl();
    	//crNet.setRoot(ctxNode);
    	
    	// User Context rule (variable part 1, 5, 10, 20)
    	//System.out.println( BigInteger.valueOf(random.nextInt()) );
    	
    	switch(n) {
    	case 0:
    	    ctxNode = null;
    	    break;
    	case 1: 
    	    compValue = new TypeValueImpl();
    	    int ra = random.nextInt();
    	    if ( ra < 0 )
    	        ra = 1813984761;
	    	compValue.setInt( BigInteger.valueOf(ra) );    
	    	compNode = new ComparisonNodeImpl();	
	    	compNode.setName( "CtxTest1" );
	    	compNode.setOp("L");
	    	compNode.setVal(compValue);
	    	ctxNode = new ContextNodeImpl();
	    	ctxNode.setCa(compNode);
	    	break;
	    case 2: // 9 networks with specific rules
	        ctxNode = createRules(5);
	        break;
	    case 5:
	        ctxNode = create5Nodes(0);
	        break;
	    case 10:
	        ctxNode = create10Nodes(0);
	        break;
	    case 20:
	        ctxNode = create20Nodes(0);
	        break;
	    case 40: // 40
	        ContextNode nod1 = create20Nodes(0);
	        ContextNode nod2 = create20Nodes(20);
	        
	        ContextNode[] ctxN3 = new ContextNodeImpl[2];
	        CombinationNode[] combNodes3 = new CombinationNodeImpl[2];
	    	for( int i=0;i<combNodes3.length;i++) {
	    	    combNodes3[i] = new CombinationNodeImpl();
	    	    combNodes3[i].setOp( "OR" );   
	    	    
	    	    ctxN3[i] = new ContextNodeImpl();
	    	    ctxN3[i].setCi( combNodes3[i] );
	    	}
	    	
	    	ctxN3[0].getCi().setCN1( nod1 );
	    	ctxN3[0].getCi().setCN2( nod2 );
	        
	    	ctxNode = ctxN3[0];
	        break;
	        
	    case 25: // 25 CR
	        ContextNode node1 = create10Nodes(0);
	        ContextNode node2 = create10Nodes(10);
	        ContextNode node3 = create5Nodes(20);
	        
	        ContextNode[] ctxN2 = new ContextNodeImpl[2];
	        CombinationNode[] combNodes2 = new CombinationNodeImpl[2];
	    	for( int i=0;i<combNodes2.length;i++) {
	    	    combNodes2[i] = new CombinationNodeImpl();
	    	    combNodes2[i].setOp( "OR" );   
	    	    
	    	    ctxN2[i] = new ContextNodeImpl();
	    	    ctxN2[i].setCi( combNodes2[i] );
	    	}
	    	
	    	ctxN2[1].getCi().setCN1( node2 );
	    	ctxN2[1].getCi().setCN2( node3 );
	    	
	    	ctxN2[0].getCi().setCN1( node1 );
	    	ctxN2[0].getCi().setCN2( ctxN2[1] );
	        
	    	ctxNode = ctxN2[0];
	        break;
	        
	       
	    case 45: // max going through the socket is 45 CR
	        ContextNode n1 = create10Nodes(0);
	        ContextNode n2 = create10Nodes(10);
	        ContextNode n3 = create10Nodes(20);
	        ContextNode n4 = create10Nodes(30);
	        ContextNode n5 = create5Nodes(40);
	        
	        ContextNode[] ctxN = new ContextNodeImpl[4];
	        CombinationNode[] combNodes = new CombinationNodeImpl[4];
	    	for( int i=0;i<combNodes.length;i++) {
	    	    combNodes[i] = new CombinationNodeImpl();
	    	    combNodes[i].setOp( "OR" );   
	    	    
	    	    ctxN[i] = new ContextNodeImpl();
	    	    ctxN[i].setCi( combNodes[i] );
	    	}
	        
	    	ctxN[1].getCi().setCN1( n1 );
	    	ctxN[1].getCi().setCN2( n2 );
	        
	    	ctxN[3].getCi().setCN1( n4 );
	    	ctxN[3].getCi().setCN2( n5 );
	        
	    	ctxN[2].getCi().setCN1( n3 );
	    	ctxN[2].getCi().setCN2( ctxN[3] );
	    	
	    	ctxN[0].getCi().setCN1( ctxN[1] );
	    	ctxN[0].getCi().setCN2( ctxN[2] );
	    	
	    	ctxNode = ctxN[0];
	        break;
    	}
    	
    	ContextRules ctxRule = new ContextRulesImpl();   
	    ctxRule.setCRNet(crNet); 
    	
    	if ( ctxNode != null ) {
    	    
    	    ContextRulesTree crUser = new ContextRulesTreeImpl();
    	    crUser.setRoot(ctxNode);
    	    ctxRule.setCRSvc(crUser);
    	}
    	
    	msdRequest.setCtxRules( ctxRule );
//	  ContextInformation
	    ContextInformationTable ciNet = new ContextInformationTableImpl();
	    ContextElement elt = ContextUtil.getInstance().createCtxElement( NetworkTemplate.HOP, 0 );
        ciNet.getCITable().add( elt );
	    
	    ContextInformationTable ciUser = new ContextInformationTableImpl();
	    elt = ContextUtil.getInstance().createCtxElement( UserTemplate.DEVICE_NAME, "TestNum1" );
	    ciUser.getCITable().add( elt );
	    
	    ContextInformation ci = new ContextInformationImpl();
	    ci.setCINet( ciNet );
	    ci.setCISvc( ciUser );
	    
	    msdRequest.setCtxInfo(ci);
	    
	    return msdRequest;
    }
    
    public MSDDescription findService( String name, int timeout, Logger log )
    {
        // find URL Of MSDService
        try {
            Socket connection = new Socket(address, port) ;
            
            UbisecID id = new UbisecIDImpl();
            id.setName( "ClientSLPSocket" );
            
            MSDRequest msdRequest = new MSDRequestImpl();
            
            ReqProcessingInformation rpi = new ReqProcessingInformationImpl();
            ReqCreationInformation rci = new ReqCreationInformationImpl();
	        Creator creator = new CreatorImpl();
	        creator.setMgrID( id );
	        creator.setNetID( id );
	        Calendar cal = Calendar.getInstance();
	        rci.setDate( cal );
	        rci.setTime( cal );
	        rci.setLifetime( BigInteger.valueOf( timeout ) );// in secondes
	        rci.setCreator( creator );
	        rpi.setCrea( rci );
            
            RequestService requestService = new RequestServiceImpl();
            requestService.setName( name );
            
            ReqDescriptionInformation reqDescription = new ReqDescriptionInformationImpl();
            reqDescription.setService(requestService);
            msdRequest.setDescInfo(reqDescription);
            msdRequest.setProcInfo( rpi );
            
            PropagationInformation pi = new PropagationInformationImpl();
            msdRequest.setPropaInfo( pi );
            
            msdRequest.setClientID( BigInteger.valueOf( -1 ) );
    	    msdRequest.setReqID( BigInteger.valueOf( -1 ) );
            
    	    // context information
    	    
    	    msdRequest = generateContext( msdRequest, 0 );
    	    
            MessageContent content = new MessageContentImpl();
            content.setRequest( msdRequest );
            
            MSDMessage mess = new MSDMessageImpl();
            mess.setSender( id );
            mess.setType( BigInteger.valueOf( InfoType.Message.CLIENT_NEW_REQUEST ) );
            mess.setContent( content );
            
            JaxMeUtil jaxme = new JaxMeUtil();
            
            String message = jaxme.toString( mess );
//            long time = System.currentTimeMillis();
//            jaxme.toMSDMessage( message.getBytes() );
//            long end = System.currentTimeMillis() - time;
//            log.info( Long.toString(end) );
            
          //  System.out.println( message );
            
            connection.setSendBufferSize( InfoType.MSG_BUFFER_SIZE );
            connection.setReceiveBufferSize( InfoType.MSG_BUFFER_SIZE );
            connection.getOutputStream().write( message.getBytes() );
           // System.out.println( "done" );
            
            byte[] buf = new byte[InfoType.MSG_BUFFER_SIZE];
            //long start = System.currentTimeMillis();
            //System.out.println( "wait for the response..." );
            int size = connection.getInputStream().read( buf );
            //long end = System.currentTimeMillis() - start;
            //System.out.println("get a result of "+size +" after "+end);
            if ( size < 0) 
                return null;
            
           // log.info( Long.toString( end ) );
            
            // Resize the byte[] to keep only the used bytes
            byte[] tmp = buf; 
            buf = new byte[size];
            System.arraycopy(tmp, 0, buf, 0, size);
            
          
         //   System.out.println( new String(buf));
            
           // long start = System.currentTimeMillis();
            MSDMessage found = jaxme.toMSDMessage( buf );
           // long end = System.currentTimeMillis() - start;
           // log.info( Long.toString(end) );
//            
//          take the first result...
            if ( found.getContent()!=null ) {
                MSDDescription desc = (MSDDescription)found.getContent().getResults().get(0);
                
                return desc;
            }
            return null;
            
        } catch (JAXBException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
            
        } catch (UnknownHostException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return null;
    }
    
    public static void main(String[] args) {
        
        TestMSDSocketContext test = new TestMSDSocketContext();
        
		String name = "videostreaming";//"RFIDMusicService"; // "urn:printbasic1";//
		int timeout = 8;
		MSDDescription res = null;
		long start, end, diff;
		//TestMSD test ;		
		
		
//		long time = System.currentTimeMillis();
//		random = new Random( time );
//		int r = random.nextInt();
//		long ra = BigInteger.valueOf(time).mod( BigInteger.valueOf(r)).longValue();
//		
		random = new Random( System.currentTimeMillis() );
		
//		BasicConfigurator.configure();
		String pattern = "%m%n";//"%d{HH:mm:ss,SSS} %p [%t] %m%n";
		PatternLayout patternLayout = new PatternLayout( pattern );
		SimpleDateFormat format = new SimpleDateFormat( "yyyyMMdd");
		StringBuffer buffer = format.format( Calendar.getInstance().getTime(), new StringBuffer(), new FieldPosition( NumberFormat.INTEGER_FIELD ) );
		String date = buffer.toString();
		String filename = date +"_msd.socket1.slp.log";
		Logger log = Logger.getLogger( filename );
		Appender appender;
		try {
			appender = new RollingFileAppender(patternLayout, filename  );
			log.addAppender( appender );
		} catch (IOException e1) {
			e1.printStackTrace();
		} 
//		
//		String filename2 = date +"_msd.socket2.slp.log";
//		Logger log2 = Logger.getLogger( filename2 );
//		Appender appender2;
//		try {
//			appender2 = new RollingFileAppender(patternLayout, filename2  );
//			log2.addAppender( appender2 );
//		} catch (IOException e1) {
//			e1.printStackTrace();
//		} 
//		
		log.info( "Total SD time" );
		//log2.info("Total SD time" );
		
       // test.setLogger( log2 );

        //test.registerWantedServices();
        
		//String ctrUrl = test.find(name);
		//System.out.println ("found the " + name + " service at " + ctrUrl);

		test.findDirectMSDService();
		
		// MSD search for the service
		log.info( "Will look for service " + name );
		for (int x=0; x<20; x++)
		{
			try {
				Thread.sleep(1000);
			}
			catch (InterruptedException e)
			{
				
			}
			start = System.currentTimeMillis();
			res = test.findService(name, timeout, log);
			end = System.currentTimeMillis();
			
			// now write the discovery time in the output file
			diff = end - start;
			if( res==null ) //|| res.length==0 )
			{
				log.info( "Not found " + Long.toString(diff) );
			}
			else 
			{
				if ( res.getDescID().getName().equals( InfoType.TIMEOUT_EXPIRED ) )
				{
					log.info( "Not found " + Long.toString(diff) );
				}
				else if ( res.getDescID().getName().equals( InfoType.RUNNING_REQUEST ) ) {
				    log.info( "Not found " + Long.toString(diff) );
				}
				else 
				{
				   log.info( Long.toString(diff) );
//					System.out.println ("service found in network " + res[0].getCreationInformation().getCreator().getNetworkID().getName());
				}
			}
		}
        System.exit( 1 );
    }
}
