/*
 * Created on Aug 23, 2005
 * 
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr
 * Preliminary version, please do not distribute
 * 
 * Copyright INRIA 2005
 */
package org.msda.tests.slp;

import java.io.IOException;
import java.text.FieldPosition;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;
import java.util.Vector;

import org.apache.log4j.Appender;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.RollingFileAppender;
import org.msda.plugins.slp.SLPRequest;

import ubisec.util.system.IFUtils;

import com.solers.slp.Advertiser;
import com.solers.slp.Locator;
import com.solers.slp.ServiceLocationEnumeration;
import com.solers.slp.ServiceLocationException;
import com.solers.slp.ServiceLocationManager;
import com.solers.slp.ServiceType;
import com.solers.slp.ServiceURL;

/**
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class TestSLP {

    private Locator locator = null;
    private int httpServerPort = 8089;
    private Logger log;
    
    public void setLogger( Logger log ) {
        this.log = log;
    }
    
    private void init() {
        try
	    {
	        locator = ServiceLocationManager.getLocator(Locale.FRANCE);
	    }
	    catch(ServiceLocationException ex)
	    {
	        ex.printStackTrace();
	        System.exit(0);
	    }
    }
    
    public void registerServices() {
        
        try {
            Advertiser slpAdvertiser = ServiceLocationManager.getAdvertiser(Locale.FRANCE);
            IFUtils ifUtils = new IFUtils();
            String localIPAddress = ifUtils.getLocalHostAddresses()[0];
            
            ServiceURL serviceURL2 = new ServiceURL("service:RFIDMusicService:http://"+localIPAddress+":"+httpServerPort+"/RFIDMusicService", ServiceURL.LIFETIME_PERMANENT);
            slpAdvertiser.register(serviceURL2, new Vector());
            
        } catch (ServiceLocationException e) {
            e.printStackTrace();
        }
    }
    
    public void find(String name)
	{
		long start, end, diff;

		String serviceName = "service:"+name;
		
		SLPRequest slpRequest = new SLPRequest();
		ServiceType serviceType = new ServiceType(serviceName);
		slpRequest.setType(serviceType);
		Vector scopes = new Vector();
		scopes.add("default");
		slpRequest.setScopes(scopes);
		slpRequest.setSearchFilter("");
		
		try
		{
		    start = System.nanoTime();
		    ServiceLocationEnumeration foundServices = locator.findServices(slpRequest.getType(),slpRequest.getScopes(), slpRequest.getSearchFilter());
		    end = (System.nanoTime() - start)/1000000;
		    if (foundServices !=null && foundServices.hasMoreElements() ) {
		        log.info( Long.toString(end) );
		        ServiceURL foundService = (ServiceURL)foundServices.next();
//		        System.out.println( foundService );
		    }
		    else {
		    	log.info( "not found" );
		    }
		    foundServices.destroy();
		    foundServices = null;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}
    
    public static void Nano () {
    	long time = 0;
    	long newTime;
    	long smaller = 9999999999999L;
    	
    	for (int i = 0; i < 100000; i++) {
    		time = System.nanoTime();
    		newTime = System.nanoTime();
    		
    		smaller = Math.min(smaller, newTime - time);
    	}
    	System.out.println("Smallest nano interval measured: " + smaller);
    	System.out.println("Current time millis: " + System.currentTimeMillis());
    	System.out.println("Nano time: " + System.nanoTime());
    	return;
    }
    
    public static void main( String[] args ) {
        
        String pattern = "%m%n";//"%d{HH:mm:ss,SSS} %p [%t] %m%n";
	    PatternLayout patternLayout = new PatternLayout( pattern );
	    SimpleDateFormat format = new SimpleDateFormat( "yyyyMMdd");
	    StringBuffer buffer = format.format( Calendar.getInstance().getTime(), new StringBuffer(), new FieldPosition( NumberFormat.INTEGER_FIELD ) );
	    String date = buffer.toString();

	    String filename = date +"_slp.local.log";
	    Logger log = Logger.getLogger( filename );
    	Appender appender;
    	try {
    	    appender = new RollingFileAppender(patternLayout, filename  );
    		log.addAppender( appender );
    	} catch (IOException e1) {
    		e1.printStackTrace();
    	} 

    	Nano();

        log.info( "Test SDLP SD local" );

    	TestSLP test = new TestSLP();
    	test.init();
    	try {
            Thread.sleep(100);
        } catch (InterruptedException e2) {
            e2.printStackTrace();
        }
        
    	test.registerServices();
    	test.setLogger( log );
        try {
            Thread.sleep(100);
        } catch (InterruptedException e2) {
            e2.printStackTrace();
        }
        
//        while(true)
//            try {
//                Thread.sleep(1000);
//            } catch (InterruptedException e2) {
//                e2.printStackTrace();
//            }
        
        String name = "RFIDMusicService";
        
//		for (int x=0; x<50; x++)
//			try {
//				System.out.println ("search ...");
//				Thread.sleep(1000);
//		        test.find( name );
//			}
//			catch (InterruptedException e)
//			{
//			}
//        

        
        
        
        System.exit( 1 );
    }
}
