/*
 * Created on Aug 22, 2005
 * 
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr
 * Preliminary version, please do not distribute
 * 
 * Copyright INRIA 2005
 */
package org.msda.tests.upnp;


import java.io.*;
import java.math.BigInteger;
import java.net.*;
import java.util.*;

import org.msda.MSDClientAPIService.*;
import org.msda.msd.*;
import org.msda.msd.impl.*;
import org.msda.util.*;

import com.siemens.upnp.*;
import com.siemens.upnp.controlpoint.*;

/**
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class ServiceAccessMSD {
	private UPnPListener listener;
	private ControlPoint controlPoint;
	private static final String MSD_SERVICE_NAME = "msdservice:1:0";
	private String ctrUrl4MSDService;
	private org.msda.MSDClientAPIService.MSDClientAPIPortType msdService;
	private RFIDMusicPortType rfidService;
	private MSDDescriptionImpl[] msdDescs;
	
	
	/**
	 * 
	 *
	 */
	private void init() {
		controlPoint = ControlPoint.getInstance();
		listener = new UPnPListener();
		controlPoint.addUPnPSearchListener(listener);
		controlPoint.addUPnPEventListener(listener);
	}
	
	
	/**
	 * 
	 *
	 */
	protected void findMSDService() {
		
		ctrUrl4MSDService = "";
		
		// search MSDService
		try {
			HttpURLConnection conn = getConnection( MSD_SERVICE_NAME );
			if ( conn==null )
				throw new IOException("Can't connect to the service");
			
			URL url = conn.getURL();
			
			BufferedInputStream reader = new BufferedInputStream(conn.getInputStream());
			byte[] buff = new byte[1000];
			while (reader.available()>0)
			{
				int num= reader.read(buff,0,buff.length);
				//System.out.print(new String(buff,0,num));
			}
			
			ProxyDevice device = controlPoint.createDevice(url);
			
			java.util.Vector services = device.getServices();
			
			
			// Here, takes the first
			if (services != null && services.size()>0) {
				ProxyService service = (ProxyService)services.elementAt(0);
				ctrUrl4MSDService = service.getControlURL();
				
				//System.out.println("MSDService Found at "+ctrUrl4MSDService);
				msdService = new MSDClientAPIBindingStub( ctrUrl4MSDService );
			}
			
		} catch (IOException e) {
			e.printStackTrace();
		} catch (InvalidDescriptionException e) {
			e.printStackTrace();
		}
		
		controlPoint.shutDown();
	}
	
	
	/**
	 * 
	 * @param serviceType
	 * @return
	 */
	private HttpURLConnection getConnection( String serviceType ) 
	{
		int searchId;
		
		try {
			listener.startSearch();
			// do the search 
			searchId = controlPoint.searchAsync(UPnPConstants.SearchForServiceType(serviceType));
			// wait for the response (only one)
			while (!listener.doneSearch()) 
			{
				try {
					Thread.sleep (50);
				}
				catch (InterruptedException e)
				{
				}
			}
			// end time
			String deviceDescURL = listener.device.getDescriptionURL();
			URL url = new URL( deviceDescURL );
			return (HttpURLConnection)url.openConnection();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * First call to getServices
	 * 
	 * @return partial results
	 */
	protected MSDDescriptionImpl[]  findService( int timeout, String name ) {
		return findService( -1, -1, timeout, name );
	}
	
	/**
	 * Second (or more) call to get partial results.
	 * 
	 * @param clientId
	 * @param requestId
	 * @return partial results
	 */
	public MSDDescriptionImpl[] findService( int clientId, int requestId, int timeout, String name ) 
	{
		if (msdService == null)  findMSDService();
		
		MSDRequestImpl msdRequest = new MSDRequestImpl();
		
		if ( clientId<0 ) {
			//First call to findRFIDService
			msdDescs = null;
			
			ReqProcessingInformation rpi = new ReqProcessingInformationImpl();
			ReqCreationInformation rci = new ReqCreationInformationImpl();
			Creator creator = new CreatorImpl();
			
			Calendar cal = Calendar.getInstance();
			rci.setDate( cal );
			rci.setTime( cal );
			rci.setLifetime( BigInteger.valueOf( timeout ) );// in secondes
			rci.setCreator( creator );
			rpi.setCrea( rci );
			
			ReqDescriptionInformation reqDescription = new ReqDescriptionInformationImpl();
			RequestService requestService = new RequestServiceImpl();
			requestService.setName(name);
			reqDescription.setService(requestService);
			msdRequest.setDescInfo(reqDescription);

			PropagationInformation propag = new PropagationInformationImpl();
			msdRequest.setPropaInfo(propag);

			msdRequest.setProcInfo( rpi );
			
		} else {
			
			if ( requestId < 0 ) {
				System.out.println("Invalid values : clientId="+clientId + " requestId="+requestId);
				return null;
			}
			// get partial results
			msdRequest.setClientID( BigInteger.valueOf( clientId ) );
			msdRequest.setReqID( BigInteger.valueOf( requestId ) );
		}
		
		// save the previous results... 
		MSDDescriptionImpl[] results = msdDescs;
		
		msdDescs = msdService.getServices(msdRequest);
		//System.out.println("service invoked");
		if (msdDescs == null) {
			System.out.println("the result is null");
			return null;
		} else if (msdDescs.length<=0) {
			System.out.println("the result is Empty");
			return null;
		} 
		
		MSDDescriptionImpl[] lastResults = msdDescs;
		
		if ( results!=null ) {
			// at this point, results contains the previous partial results
			// and msdDescs contains the latest results 
			String descName = lastResults[0].getDescID().getName();
			
			int size = results.length;
			if ( lastResults!=null && 
					!InfoType.RUNNING_REQUEST.equals( descName ) &&
					!InfoType.TIMEOUT_EXPIRED.equals( descName ) )
				size = size + lastResults.length;
			
			msdDescs = new MSDDescriptionImpl[ size ];
			for(int i=0;i<results.length; i++)
				msdDescs[i] = results[i];
			
			if ( lastResults!=null && 
					!InfoType.RUNNING_REQUEST.equals( descName ) &&
					!InfoType.TIMEOUT_EXPIRED.equals( descName ) )
				for(int i=0;i<lastResults.length; i++)
					msdDescs[results.length+i] = lastResults[i];
		} 
		
		return lastResults;
	}
	
	/**
	 * 
	 * @param description
	 */
	private long accessService(MSDDescriptionImpl description) {
		String serviceURL ;
		String descId ;
		String[] RFIDs;
		String[] MusicInfos;
		long tstart, tend, res;
		boolean ok;

		res = -1;
		MusicInfos = new String[1];
		RFIDs = new String[1];

		try
		{
			System.out.println("ServiceAccess : MSDA service Found at " + ctrUrl4MSDService );

			serviceURL = description.getSvcDef().getGeneric().getAddr().getLocation();
			descId = description.getDescID().getName();
			System.out.println("ServiceAccess : Final service Found at " + serviceURL + " create an MSD stub to it");
			rfidService = new RFIDMusicBindingStub( serviceURL, ctrUrl4MSDService, descId );


			RFIDs[0] = "x-010165-314101#";
			MusicInfos[0] = "unkown Singer+1#";
			
			tstart = System.currentTimeMillis();
			System.out.println("ServiceAccess : sending access message");
			ok = rfidService.setInfos(RFIDs, MusicInfos);
			System.out.println("ServiceAccess : access returned");
			tend = System.currentTimeMillis();
			//
			res = tend-tstart;

		}
		catch (Exception ex)
		{
			ex.printStackTrace();
			res = -1;
		}
		return res;
		
	}

	/**
	 * 
	 * @param description
	 * @throws IOException
	 */	
	private long accessServiceVariable(MSDDescriptionImpl description) throws IOException 
	{
		String serviceURL ;
		String descId ;
		long res = 0;
		FileWriter fileW = null;
		try
		{
			
			java.io.File file = new java.io.File("log"+File.separator+"upnp.msd.access.log");
			if (!file.exists()) file.createNewFile();
			else file.delete();
			fileW = new FileWriter(file);
			
			
			//int clientid = msdService.getClientID(description);
			serviceURL = description.getSvcDef().getGeneric().getAddr().getLocation();
			descId = description.getDescID().getName();
			System.out.println("ServiceAccess : Final service Found at " + serviceURL + " create an MSD stub to it");
			rfidService = new RFIDMusicBindingStub( serviceURL, ctrUrl4MSDService, descId );

			for (int i=1; i<=100; i++)
			{
				String RFID ="";
				String MusicInfo="";
				for (int k=0; k<=i*100; k++)
				{
					RFID = RFID+"x-010165-31410"+k+"#";
					MusicInfo = MusicInfo + "unkown Singer+"+k+"#";
				}			
				
				System.out.println("Send "+i*100+" elements to Service");
				long tstart = System.currentTimeMillis();
				boolean resp = rfidService.setInfo(RFID, MusicInfo);
				long tend = System.currentTimeMillis();
				long spenttime = tend-tstart;
				System.out.println("The response value is "+ resp);
				rfidService.removeAll();
				// write log file
				fileW.write("\r\nAccess Time for "+i*100+" elements : "+spenttime);
				fileW.flush();
				Thread.sleep(200);
			}
			fileW.close();
		}
		catch (Exception ex)
		{
			fileW.close();
			ex.printStackTrace();
		}
		return res;
	}
	
	/*
	 * 
	 * MAIN entry point for testing service access via MSD
	 * 
	 */
	
	/**
	 * 
	 * @param args
	 */
	public static void main( String[] args ) {
		String name = "RFIDMusicService";
		int timeout = 3;
		MSDDescriptionImpl[] res = null;
		long start, end, diff, accesstime;
		ServiceAccessMSD local ;
		final boolean STATIC_SIZE_ACCESS_DATA = true;

		
		// create an instance of the test
		local = new ServiceAccessMSD();
		local.init();
		local.findMSDService();

		//
		start = System.currentTimeMillis();
		res = local.findService( timeout, name );
		end = System.currentTimeMillis();
		if (res == null || res.length<=0) {
			System.out.println ("no music service found");
			return;
		}
		for (int x=0; x<50; x++)
		{
			try 
			{
				//
				Thread.sleep (3000);
				
				//
				//
				start = System.currentTimeMillis();
				System.out.println("ServiceAccess VIA MSD..");
				if (STATIC_SIZE_ACCESS_DATA) {
					accesstime = local.accessService(res[0]);
				} else {
					accesstime = local.accessServiceVariable(res[0]);
				}
				end = System.currentTimeMillis();
				diff = end - start;
				if (accesstime >=0) {
					System.out.println (Long.toString(diff) + "	" + Long.toString(accesstime));
					}
				else 
				{
					System.out.println ("ERROR, could not access service");
				}
			}
			catch (IOException e)
			{
			}
			catch (InterruptedException e)
			{
			}
		}	
		System.exit( 1 );
	}
}

