/* Created on Aug 22, 2005
 * 
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr
 * Preliminary version, please do not distribute
 * 
 * Copyright INRIA 2005
 */
package org.msda.tests.upnp;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.FieldPosition;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import org.apache.log4j.Appender;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.RollingFileAppender;

import com.siemens.upnp.Action;
import com.siemens.upnp.Argument;
import com.siemens.upnp.ArgumentList;
import com.siemens.upnp.InvalidDescriptionException;
import com.siemens.upnp.SOAPException;
import com.siemens.upnp.UPnPConstants;
import com.siemens.upnp.controlpoint.ControlPoint;
import com.siemens.upnp.controlpoint.EventSubscription;
import com.siemens.upnp.controlpoint.ProxyDevice;
import com.siemens.upnp.controlpoint.ProxyService;

/**
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class ServiceAccessUPnP {
	private ControlPoint controlPoint;
	private Logger log;
	private UPnPListener listener;
	
	
	public synchronized boolean isSearching ()
	{
		return listener.isSearching;
	}
	/**
	 * 
	 * @param log
	 */
	public void setLogger( Logger log ) {
		this.log = log;
	}
	
	/**
	 * 
	 */
	private void init() {
		controlPoint = ControlPoint.getInstance();
		listener = new UPnPListener();
		controlPoint.addUPnPSearchListener(listener);
		controlPoint.addUPnPEventListener(listener);
	}
	
	/**
	 * 
	 */
	private void terminate() {
		controlPoint.removeUPnPSearchListener( listener );
		controlPoint.removeUPnPEventListener( listener );
	}
	
	/**
	 * 
	 * @param serviceType
	 */
	
	private synchronized int searchService( String serviceType ) {
		int searchId = -1;
		try {
			// initi the space for the response
			// starting time
			listener.startSearch();
			// do the search 
			searchId = controlPoint.searchAsync(UPnPConstants.SearchForServiceType(serviceType));
			// wait for the response (only one)
			while (!listener.doneSearch()) 
			{
				try {
					Thread.sleep (10);
				}
				catch (InterruptedException e)
				{
				}
			}
			
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return searchId;
	}
	
	
	
	
	public long accessService(DeviceDesc deviceP, String serviceName)
	{
		long tstart, tend, spenttime;
		spenttime = 0;
		try {
			// create the device and service proxy
			URL url = new URL(deviceP.getDescriptionURL());
			ProxyDevice device = controlPoint.createDevice(url);
			java.util.Vector services = device.getServices();
			if (services != null && services.size()>0)
			{
				ProxyService service = (ProxyService)services.elementAt(0);
				String RFID ="x-010165-314101#";
				String MusicInfo="unkown Singer 1#";
				Action action3 = new Action("setInfos", service);
				Argument arg4 = new Argument("RFID",Argument.IN ,false, "", RFID);
				Argument arg5 = new Argument("MusicInfo",Argument.IN ,false, "", MusicInfo);
				Argument argReturn3 = new Argument("return",Argument.OUT ,true, "return");
				action3.addArgument(arg4);
				action3.addArgument(arg5);
				action3.addArgument(argReturn3);
				//
				tstart = System.currentTimeMillis();
				service.submitActionRequest(action3);
				tend = System.currentTimeMillis();
				spenttime = tend-tstart;
				//
				ArgumentList argList3 = action3.getArguments();
				Argument argRet3 = argList3.getArgumentByName("return");
				if (argRet3 != null )
				{
					System.out.println("The result of the action invokation is :"+argRet3.getValue());
				}
			}
		}
		catch (InvalidDescriptionException e)
		{
			e.printStackTrace();
			return -1;
		}
		catch (MalformedURLException e)
		{
			e.printStackTrace();
			return -1;
		}
		catch (SOAPException e)
		{
			e.printStackTrace();
			return -1;
		}
		return spenttime;
	}
	
	
	
	public 	long accessServiceVariable(DeviceDesc deviceP, String serviceName) throws IOException
	{
		long res = 0;
		FileWriter fileW = null;
		try
		{
			java.io.File file = new java.io.File("log"+File.separator+"upnp.native.access.log");
			if (!file.exists()) file.createNewFile();
			else file.delete();
			
			fileW = new FileWriter(file);
			
			URL url = new URL(deviceP.getDescriptionURL());
			// show the Description
			ProxyDevice device = controlPoint.createDevice(url);
			
			System.out.println("Level 2: Device created");
			java.util.Vector services = device.getServices();
			
			System.out.println("Level 3: service proxy get");
			if (services != null && services.size()>0)
			{
				ProxyService service = (ProxyService)services.elementAt(0);
				//			EventSubscription eventSubscription = controlPoint.requestSubscription(service, stateChangeListener);
				//			System.out.println("The duration of the Subscription is ="+eventSubscription.getSubscriptionDuration());
				//			System.out.println("Level 4: service proxy isn't null");
				
				
				//setInfos(java.lang.String[] RFID, java.lang.String[] MusicInfo);
				//			Action action2 = new Action("setInfos", service);
				//			Argument arg2 = new Argument("RFID",Argument.IN ,false, "", "x-010165-314100");
				//			Argument arg3 = new Argument("MusicInfo",Argument.IN ,false, "", "unkown Singer");
				//			Argument argReturn2 = new Argument("return",Argument.OUT ,true, "return");
				//			action2.addArgument(arg2);
				//			action2.addArgument(arg3);
				//			action2.addArgument(argReturn2);
				//			System.out.println("I will invok an action :setInfo");
				//			service.submitActionRequest(action2);
				//			ArgumentList argList2 = action2.getArguments();
				//			Argument argRet2 = argList2.getArgumentByName("return");
				//			if (argRet2 != null )
				//			{
				//				System.out.println("The result of the action invokation is :"+argRet2.getValue());
				//			}
				
				for (int i=1; i<=100; i++)
				{
					String RFID ="";
					String MusicInfo="";
					for (int k=0; k<=i*100; k++)
					{
						RFID = RFID+"x-010165-31410"+k+"#";
						MusicInfo = MusicInfo + "unkown Singer+"+k+"#";
					}						
					Action action3 = new Action("setInfos", service);
					Argument arg4 = new Argument("RFID",Argument.IN ,false, "", RFID);
					Argument arg5 = new Argument("MusicInfo",Argument.IN ,false, "", MusicInfo);
					Argument argReturn3 = new Argument("return",Argument.OUT ,true, "return");
					action3.addArgument(arg4);
					action3.addArgument(arg5);
					action3.addArgument(argReturn3);
					System.out.println("I will invok an action :setInfos");
					
					long tstart = System.currentTimeMillis();
					service.submitActionRequest(action3);
					long tend = System.currentTimeMillis();
					
					long spenttime = tend-tstart;
					
					//System.out.println("Access Time for "+i+" elements : "+spenttime);
					fileW.write("\r\nAccess Time for "+i+" elements : "+spenttime);
					
					ArgumentList argList3 = action3.getArguments();
					Argument argRet3 = argList3.getArgumentByName("return");
					if (argRet3 != null )
					{
						System.out.println("The result of the action invokation is :"+argRet3.getValue());
					}
					
					
					
					Action actionDelete = new Action("removeAll", service);
					//			actionDelete.addArgument(new Argument("all", Argument.IN, false, "", ""));
					service.submitActionRequest(actionDelete);
					System.out.println("removeAll function is invoked");
					
				}
				//			Action action = new Action("getInfo", service);
				//			Argument arg = new Argument("RFID",Argument.IN ,false, "", "x-010165-314102");
				//			Argument argReturn = new Argument("return",Argument.OUT ,true, "return");
				//			action.addArgument(arg);
				//			action.addArgument(argReturn);
				//			System.out.println("I will invok an action :getInfo");
				//			service.submitActionRequest(action);
				//			ArgumentList argList = action.getArguments();
				//			Argument argRet = argList.getArgumentByName("return");
				//			if (argRet != null )
				//			{
				//				System.out.println("The result of the action invokation is :"+argRet.getValue());
				//			}
				//eventSubscription = cp.requestSubscription(service, stateChangeListener);
			}
			fileW.close();
			System.out.println("Level 5");  
		}
		catch(Exception ex)
		{
			fileW.close();
			ex.printStackTrace();
		}
		return res;
	}
	
	
	
	
	/**
	 * 
	 * @param args
	 */
	public static void main( String[] args ) {
		final boolean STATIC_SIZE_ACCESS_DATA = true;
		long start, end, diff, res;
		ServiceAccessUPnP local;
		String name = "RFIDMusicService";
		DeviceDesc device;
		String serviceName="";
		
		BasicConfigurator.configure();
		String pattern = "%m%n";//"%d{HH:mm:ss,SSS} %p [%t] %m%n";
		PatternLayout patternLayout = new PatternLayout( pattern );
		//SimpleDateFormat format = new SimpleDateFormat( "yyyyMMdd_HHmmss");
		SimpleDateFormat format = new SimpleDateFormat( "yyyyMMdd");
		StringBuffer buffer = format.format( Calendar.getInstance().getTime(), new StringBuffer(), new FieldPosition( NumberFormat.INTEGER_FIELD ) );
		String date = buffer.toString();
		
		String filename = date +"_upnp.local.siemens.log";
		Logger log = Logger.getLogger( filename );
		Appender appender;
		try {
			appender = new RollingFileAppender(patternLayout, filename  );
			log.addAppender( appender );
		} catch (IOException e1) {
			e1.printStackTrace();
		} 
		log.info( "UPnP Service access time" );

		// starts the UPnP test
		local = new ServiceAccessUPnP();
		local.setLogger( log );
		local.init();
		
		log.info( "test service access direct UPnP" );
		
		// first look for the target service
		try {
			System.out.println ("search ...");
			while (local.isSearching()) {
				Thread.sleep(100);
			}
			local.searchService(name);
			device = local.listener.getFoundDevice();
			for (int x=0; x<50; x++)
				try {
					System.out.println ("Access...");
					Thread.sleep(2000);
					start = System.currentTimeMillis();
					// now access the service
					if (STATIC_SIZE_ACCESS_DATA) {
						res = local.accessService(device,serviceName);
					} else {
						res = local.accessServiceVariable(device,serviceName);
					}
					end = System.currentTimeMillis();
					diff = end - start;
					log.info( Long.toString(diff) + " " + Long.toString(res) );
				}
			catch (InterruptedException e)
			{
			}
			catch (IOException e)
			{
			}
			
		}
		catch (InterruptedException e)
		{
		}
		
		
		
		
		local.terminate();
		System.exit( 1 );
	}
}
