/*
 * Created on Aug 22, 2005
 * 
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr
 * Preliminary version, please do not distribute
 * 
 * Copyright INRIA 2005
 */
package org.msda.tests.upnp;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import java.math.BigInteger;
//import java.rmi.RemoteException;

//import org.apache.log4j.*;
import org.msda.MSDClientAPIService.*;
import org.msda.msd.*;
import org.msda.msd.impl.*;
import org.msda.util.*;

import com.siemens.upnp.*;
import com.siemens.upnp.controlpoint.*;

/**
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class TestMSD {
	// generic to test
	private String ctrUrl4MSDService;
	private MSDClientAPIPortType msdService;
	private RFIDMusicPortType rfidService;
	private MSDDescriptionImpl[] msdDescs;	
	private UPnPListener listener;
	private ControlPoint controlPoint;
	private static final String MSD_SERVICE_NAME = "msdservice:1:0";
	
	
	
	/**
	 * Initialize the control point
	 *
	 */
	private void init() {
		controlPoint = ControlPoint.getInstance();
		listener = new UPnPListener();
		controlPoint.addUPnPSearchListener(listener);
		controlPoint.addUPnPEventListener(listener);
	}
	
	
	/**
	 * Initialise the connection to the MSDA service found in UPnP
	 *
	 */
	protected void findMSDService() {
		URL url;
		ProxyDevice device;
		Vector services;
		String deviceDescURL;
		// init the global variable
		ctrUrl4MSDService = "";
		//

		try {
			// search for MSDA service and select the first one
			deviceDescURL = getServiceURL( MSD_SERVICE_NAME );
			System.out.println("MSDA <Device> Found at " + deviceDescURL);
			url = new URL( deviceDescURL );
			// check that the service exist...
//			HttpURLConnection conn = (HttpURLConnection)url.openConnection();
//			if ( conn==null ) {
//				throw new IOException("Can't connect to the service");
//			}
//			else 
			{
				// then take the first service of the MSDA device
				device = controlPoint.createDevice(url);
				services = device.getServices();
				if (services != null && services.size()>0) {
					ProxyService service = (ProxyService)services.elementAt(0);
					ctrUrl4MSDService = service.getControlURL();
					System.out.println("MSDA <Service> Found at " + ctrUrl4MSDService);
					msdService = new MSDClientAPIBindingStub( ctrUrl4MSDService );
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (InvalidDescriptionException e) {
			e.printStackTrace();
		}
		controlPoint.shutDown();
	}
	
	
	/**
	 * 
	 * @param serviceType
	 * @return
	 */
	private String getServiceURL( String serviceType ) 
	{
		int searchId;
		String deviceDescURL;
		URL url;
		
		try {
			// do the UPnP search and get the first response
			System.out.println ("looking for the MSDA service");
			listener.startSearch();
			searchId = controlPoint.searchAsync(UPnPConstants.SearchForServiceType(serviceType));
			while (!listener.doneSearch()) 
			{
				try {
					Thread.sleep (50);
				}
				catch (InterruptedException e)
				{
				}
			}
			// end time
			deviceDescURL = listener.device.getDescriptionURL();
			System.out.println ("found the MSDA service at URL " + deviceDescURL);
			return deviceDescURL;
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * First call to getServices
	 * 
	 * @return partial results
	 */
	protected MSDDescriptionImpl[]  findService( int timeout, String name ) {
		return findService( -1, -1, timeout, name );
	}
	
	/**
	 * Second (or more) call to get partial results.
	 * 
	 * @param clientId
	 * @param requestId
	 * @return partial results
	 */
	public MSDDescriptionImpl[] findService( int clientId, int requestId, int timeout, String name ) 
	{
		if (msdService == null)  findMSDService();
		System.out.println ("looking for service " + name);
		MSDRequestImpl msdRequest = new MSDRequestImpl();
		
		if ( clientId<0 ) {
			//First call to findRFIDService
			msdDescs = null;
			
			ReqProcessingInformation rpi = new ReqProcessingInformationImpl();
			ReqCreationInformation rci = new ReqCreationInformationImpl();
			Creator creator = new CreatorImpl();
			
			Calendar cal = Calendar.getInstance();
			rci.setDate( cal );
			rci.setTime( cal );
			rci.setLifetime( BigInteger.valueOf( timeout ) );// in secondes
			rci.setCreator( creator );
			rpi.setCrea( rci );
			
			ReqDescriptionInformation reqDescription = new ReqDescriptionInformationImpl();
			RequestService requestService = new RequestServiceImpl();
			requestService.setName(name);
			reqDescription.setService(requestService);
			msdRequest.setDescInfo(reqDescription);
			msdRequest.setProcInfo( rpi );
			
		} else {
			
			if ( requestId < 0 ) {
				System.out.println("Invalid values : clientId="+clientId + " requestId="+requestId);
				return null;
			}
			// get partial results
			msdRequest.setClientID( BigInteger.valueOf( clientId ) );
			msdRequest.setReqID( BigInteger.valueOf( requestId ) );
		}
		
		// save the previous results... 
		MSDDescriptionImpl[] results = msdDescs;
		
		msdDescs = msdService.getServices(msdRequest);
		//System.out.println("service invoked");
		if (msdDescs == null) {
			System.out.println("the result is null");
			return null;
		} else if (msdDescs.length<=0) {
			System.out.println("the result is Empty");
			return null;
		} 
		
		MSDDescriptionImpl[] lastResults = msdDescs;
		
		if ( results!=null ) {
			// at this point, results contains the previous partial results
			// and msdDescs contains the latest results 
			String descName = lastResults[0].getDescID().getName();
			
			int size = results.length;
			if ( lastResults!=null && 
					!InfoType.RUNNING_REQUEST.equals( descName ) &&
					!InfoType.TIMEOUT_EXPIRED.equals( descName ) )
				size = size + lastResults.length;
			
			msdDescs = new MSDDescriptionImpl[ size ];
			for(int i=0;i<results.length; i++)
				msdDescs[i] = results[i];
			
			if ( lastResults!=null && 
					!InfoType.RUNNING_REQUEST.equals( descName ) &&
					!InfoType.TIMEOUT_EXPIRED.equals( descName ) )
				for(int i=0;i<lastResults.length; i++)
					msdDescs[results.length+i] = lastResults[i];
		} 
		
		return lastResults;
	}
	
	
	
	/**
	 * 
	 * @param args
	 */
	public static void main( String[] args ) {
		String name = "RFIDMusicService";
		int timeout = 3;
		MSDDescription[] res = null;
		long start, end, diff;
		TestMSD local ;
		
		System.out.println( "MSD/UPnP Service discovery time" );
		com.siemens.upnp.util.Tracer.setTraceDirectedToScreen(false);

		// create an instance of the test
		local = new TestMSD();
		local.init();
		// First search for the MSD service
		local.findMSDService();
		
		// call MSD search for the service (50 times)
		System.out.println( "Do the search loop" );
		for (int x=0; x<1; x++)
		{
			try {
				// wait 2 seconds between each instance
				Thread.sleep (2000);
			}
			catch (InterruptedException e)
			{
				
			}
			start = System.currentTimeMillis();
			res = local.findService( timeout, name );
			end = System.currentTimeMillis();
			
			// now write the discovery time in the output file
			diff = end - start;
			if( res==null || res.length==0 ) {
				System.out.println( "Not found " + Long.toString(diff) );
			}
			else 
			{
				if ( res[0].getDescID().getName().equals( InfoType.TIMEOUT_EXPIRED ) ) {
					System.out.println ( "Not found " + Long.toString(diff) );
				}
				else if ( res[0].getDescID().getName().equals( InfoType.RUNNING_REQUEST ) ) {
					System.out.println( "Not found " + Long.toString(diff) ); 
				}
				else {
					System.out.println( "found in " + Long.toString(diff)+ " milliseconds" );
				}
			}
		}
		System.exit( 0 );
	}
}
