/*
 * Created on Aug 22, 2005
 * 
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr
 * Preliminary version, please do not distribute
 * 
 * Copyright INRIA 2005
 */
package org.msda.tests.upnp;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.URL;
import java.net.UnknownHostException;
import java.rmi.RemoteException;
import java.text.FieldPosition;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import javax.xml.bind.JAXBException;

import org.msda.MSDClientAPIService.MSDClientAPIBindingStub;
import org.msda.msd.Creator;
import org.msda.msd.MSDDescription;
import org.msda.msd.MSDMessage;
import org.msda.msd.MessageContent;
import org.msda.msd.PropagationInformation;
import org.msda.msd.ReqCreationInformation;
import org.msda.msd.ReqDescriptionInformation;
import org.msda.msd.ReqProcessingInformation;
import org.msda.msd.RequestService;
import org.msda.msd.UbisecID;
import org.msda.msd.impl.CreatorImpl;
import org.msda.msd.impl.MSDDescriptionImpl;
import org.msda.msd.impl.MSDMessageImpl;
import org.msda.msd.impl.MSDRequestImpl;
import org.msda.msd.impl.MessageContentImpl;
import org.msda.msd.impl.PropagationInformationImpl;
import org.msda.msd.impl.ReqCreationInformationImpl;
import org.msda.msd.impl.ReqDescriptionInformationImpl;
import org.msda.msd.impl.ReqProcessingInformationImpl;
import org.msda.msd.impl.RequestServiceImpl;
import org.msda.msd.impl.UbisecIDImpl;
import org.msda.util.InfoType;
import org.msda.util.JaxMeUtil;

import com.siemens.upnp.InvalidDescriptionException;
import com.siemens.upnp.UPnPConstants;
import com.siemens.upnp.controlpoint.ControlPoint;
import com.siemens.upnp.controlpoint.ProxyDevice;
import com.siemens.upnp.controlpoint.ProxyService;
import com.siemens.upnp.controlpoint.UPnPSearchResponseListener;

/**
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class TestMSDSocket {
	private org.msda.MSDClientAPIService.MSDClientAPIPortType msdService;
	private RFIDMusicPortType rfidService;
	private MSDDescriptionImpl[] msdDescs;	
	private String ctrUrl4MSDService;
	private UPnPListener listener;
	private ControlPoint controlPoint;
	private static final String MSD_SERVICE_NAME = "msdservice:1:0";
	
	private String address;
	private int port;
	

	/**
	 * 
	 *
	 */
	private void init() {
		controlPoint = ControlPoint.getInstance();
		listener = new UPnPListener();
		controlPoint.addUPnPSearchListener(listener);
		controlPoint.addUPnPEventListener(listener);
	}
	
	
	protected void findDirectMSDService() {
	    
	    
	    try {
            String ctrUrl = "";
            HttpURLConnection conn = getConnection( "directmsdservice:1:0" );
            if ( conn==null )
                throw new IOException("Can't connect to the service");
            
            URL url = conn.getURL();
            
            BufferedInputStream reader = new BufferedInputStream(conn.getInputStream());
            byte[] buff = new byte[1000];
            while (reader.available()>0)
            {
                int num= reader.read(buff,0,buff.length);
                //System.out.print(new String(buff,0,num));
            }
            
            ProxyDevice device = controlPoint.createDevice(url);
            java.util.Vector services = device.getServices();
            
            // Here, takes the first
            if (services != null && services.size()>0) {
                ProxyService service = (ProxyService)services.elementAt(0);
                ctrUrl = service.getControlURL();
            }
            
            int index = ctrUrl.lastIndexOf(":");
            address = ctrUrl.substring( ctrUrl.lastIndexOf("/")+1, index );
            port = Integer.parseInt( ctrUrl.substring( index+1 ) );
            
            
        } catch (NumberFormatException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (InvalidDescriptionException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
	}
	
	/**
	 * 
	 *
	 */
	protected void findMSDService() {
		
		ctrUrl4MSDService = "";
		
		// search MSDService
		try {
			HttpURLConnection conn = getConnection( MSD_SERVICE_NAME );
			if ( conn==null )
				throw new IOException("Can't connect to the service");
			
			URL url = conn.getURL();
			
			BufferedInputStream reader = new BufferedInputStream(conn.getInputStream());
			byte[] buff = new byte[1000];
			while (reader.available()>0)
			{
				int num= reader.read(buff,0,buff.length);
				//System.out.print(new String(buff,0,num));
			}
			
			ProxyDevice device = controlPoint.createDevice(url);
			
			java.util.Vector services = device.getServices();
			
			
			// Here, takes the first
			if (services != null && services.size()>0) {
				ProxyService service = (ProxyService)services.elementAt(0);
				ctrUrl4MSDService = service.getControlURL();
				
				//System.out.println("MSDService Found at "+ctrUrl4MSDService);
				msdService = new MSDClientAPIBindingStub( ctrUrl4MSDService );
			}
			
		} catch (IOException e) {
			e.printStackTrace();
		} catch (InvalidDescriptionException e) {
			e.printStackTrace();
		}
		
		controlPoint.shutDown();
	}
	
	
	/**
	 * Opens an HTTP connection with a specific UPnP service
	 * 
	 * @param serviceType
	 * @return HTTP connection
	 */
	private HttpURLConnection getConnection( String serviceType ) 
	{
		int searchId;
		
		try {
			listener.startSearch();
			// do the search 
			searchId = controlPoint.searchAsync(UPnPConstants.SearchForServiceType(serviceType));
			// wait for the response (only one)
			while (!listener.doneSearch()) 
			{
				try {
					Thread.sleep (50);
				}
				catch (InterruptedException e)
				{
					// service not found, wait another 50ms
				}
			}
			// end time
			String deviceDescURL = listener.device.getDescriptionURL();
			URL url = new URL( deviceDescURL );
			return (HttpURLConnection)url.openConnection();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public MSDDescription findService( int timeout, String name ) {
	    
		try {
		  //  System.out.println(" try to connect on "+address+" "+port);
	        Socket connection = new Socket(address, port) ;
	        
	        UbisecID id = new UbisecIDImpl();
	        id.setName( "ClientUPnPSocket" );
	        
	        MSDRequestImpl msdRequest = new MSDRequestImpl();
	        
	        ReqProcessingInformation rpi = new ReqProcessingInformationImpl();
	        ReqCreationInformation rci = new ReqCreationInformationImpl();
	        Creator creator = new CreatorImpl();
	        creator.setMgrID( id );
	        creator.setNetID( id );
	        Calendar cal = Calendar.getInstance();
	        rci.setDate( cal );
	        rci.setTime( cal );
	        rci.setLifetime( BigInteger.valueOf( timeout ) );// in secondes
	        rci.setCreator( creator );
	        rpi.setCrea( rci );
	        
	        RequestService requestService = new RequestServiceImpl();
	        requestService.setName( name );
	        
	        ReqDescriptionInformation reqDescription = new ReqDescriptionInformationImpl();
	        reqDescription.setService(requestService);
	        msdRequest.setDescInfo(reqDescription);
	        msdRequest.setProcInfo( rpi );
	        
	        PropagationInformation pi = new PropagationInformationImpl();
	        msdRequest.setPropaInfo( pi );
	        
	        msdRequest.setClientID( BigInteger.valueOf( -1 ) );
		    msdRequest.setReqID( BigInteger.valueOf( -1 ) );
	        
	        MessageContent content = new MessageContentImpl();
	        content.setRequest( msdRequest );
	        
	        MSDMessage mess = new MSDMessageImpl();
	        mess.setSender( id );
	        mess.setType( BigInteger.valueOf( InfoType.Message.CLIENT_NEW_REQUEST ) );
	        mess.setContent( content );
	        
	        JaxMeUtil jaxme = new JaxMeUtil();
	        String message = jaxme.toString( mess );
	        
	        //System.out.println( message );
	        
	       // System.out.println( "Send sth to the manager ..." );
	        connection.getOutputStream().write( message.getBytes() );
	       // System.out.println( "done" );
	        
	        byte[] buf = new byte[InfoType.MSG_BUFFER_SIZE];
	       // long start = System.currentTimeMillis();
	        //System.out.println( "wait for the response..." );
	        int size = connection.getInputStream().read( buf );
	        //long end = System.currentTimeMillis() - start;
	      //  System.out.println("get a result of "+size +" after "+end);
	        if ( size < 0) 
	            return null;
	        
	        // Resize the byte[] to keep only the used bytes
	        byte[] tmp = buf; 
	        buf = new byte[size];
	        System.arraycopy(tmp, 0, buf, 0, size);
	        
	        
	       // System.out.println( new String(buf));
	        MSDMessage found = jaxme.toMSDMessage( buf );
//          take the first result...
          MSDDescription desc = (MSDDescription)found.getContent().getResults().get(0);
	        return desc;
	        
	    } catch (JAXBException e1) {
	        // TODO Auto-generated catch block
	        e1.printStackTrace();
	        
	    } catch (UnknownHostException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	    } catch (IOException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	    }
		return null;
	}
	
	
	/**
	 * 
	 * @param args
	 */
	public static void main( String[] args ) {
		String name = "RFIDMusicService";
		int timeout = 3;
		MSDDescription res = null;
		long start, end, diff;
		TestMSDSocket local ;
		
		
		// create an instance of the test
		local = new TestMSDSocket();
		local.init();
		
		// First search for the MSD service
		local.findDirectMSDService();
		try {
			Thread.sleep (6000);
		}
		catch (InterruptedException e)
		{
			
		}
		
		
		// MSD search for the service
		for (int x=0; x<50; x++)
		{
			try {
				Thread.sleep (2000);
			}
			catch (InterruptedException e)
			{
				
			}
			start = System.currentTimeMillis();
//			try {
				res = local.findService( timeout, name );
//			} catch (RemoteException e) {
//				e.printStackTrace();
//			}
			end = System.currentTimeMillis();
			
			// now write the discovery time in the output file
			diff = end - start;
			if( res==null )
			{
				System.out.println( "Not found " + Long.toString(diff) );
			}
			else 
			{
				if ( res.getDescID().getName().equals( InfoType.TIMEOUT_EXPIRED ) )
				{
					System.out.println( "Not found " + Long.toString(diff) );
				}
				else if ( res.getDescID().getName().equals( InfoType.RUNNING_REQUEST ) ) {
					System.out.println( "Not found " + Long.toString(diff) );
				}
				else 
				{
					System.out.println( Long.toString(diff) );
//					System.out.println ("service found in network " + res[0].getCreationInformation().getCreator().getNetworkID().getName());
				}
			}
		}
		System.exit( 1 );
	}
}
