/*
 * Created on Aug 22, 2005
 * 
 * INRIA, ARLES Project
 * 
 * contact: Pierre-Guillaume.Raverdy@inria.fr
 * Preliminary version, please do not distribute
 * 
 * Copyright INRIA 2005
 */
package org.msda.tests.upnp;

import java.io.IOException;
import java.net.MalformedURLException;
import java.text.FieldPosition;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import org.apache.log4j.Appender;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.RollingFileAppender;

import com.siemens.upnp.UPnPConstants;
import com.siemens.upnp.controlpoint.ControlPoint;

/**
 * @author agnes.de_la_chapelle@inria.fr - INRIA ARLES
 *
 */
public class TestUPnP {
	private ControlPoint controlPoint;
	private Logger log;
	private UPnPListener listener;
	
	
	public synchronized boolean isSearching ()
	{
		return listener.isSearching;
	}
	/**
	 * 
	 * @param log
	 */
	public void setLogger( Logger log ) {
		this.log = log;
	}
	
	/**
	 * 
	 */
	private void init() {
		controlPoint = ControlPoint.getInstance();
		listener = new UPnPListener();
		controlPoint.addUPnPSearchListener(listener);
		controlPoint.addUPnPEventListener(listener);
	}
	
	/**
	 * 
	 */
	private void terminate() {
		controlPoint.removeUPnPSearchListener( listener );
		controlPoint.removeUPnPEventListener( listener );
	}
	
	/**
	 * 
	 * @param serviceType
	 */
	private synchronized int search( String serviceType ) {
		long start, end, diff;
		int searchId = -1;
		try {
			// initi the space for the response
			// starting time
			start = System.currentTimeMillis();
			listener.startSearch();
			// do the search 
			searchId = controlPoint.searchAsync(UPnPConstants.SearchForServiceType(serviceType));
			// wait for the response (only one)
			while (!listener.doneSearch()) 
			{
				try {
					Thread.sleep (10);
				}
				catch (InterruptedException e)
				{
				}
			}
			// end time
			end = System.currentTimeMillis();
			
			// now write diff in output file
			diff = end - start;
			log.info( Long.toString(diff) );
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return searchId;
	}
	
	
	
	
	
	/**
	 * 
	 * @param args
	 */
	public static void main( String[] args ) {
		TestUPnP local;
		String name = "RFIDMusicService";
		
		BasicConfigurator.configure();
		String pattern = "%m%n";//"%d{HH:mm:ss,SSS} %p [%t] %m%n";
		PatternLayout patternLayout = new PatternLayout( pattern );
		//SimpleDateFormat format = new SimpleDateFormat( "yyyyMMdd_HHmmss");
		SimpleDateFormat format = new SimpleDateFormat( "yyyyMMdd");
		StringBuffer buffer = format.format( Calendar.getInstance().getTime(), new StringBuffer(), new FieldPosition( NumberFormat.INTEGER_FIELD ) );
		String date = buffer.toString();
		
		String filename = date +"_upnp.local.siemens.log";
		Logger log = Logger.getLogger( filename );
		Appender appender;
		try {
			appender = new RollingFileAppender(patternLayout, filename  );
			log.addAppender( appender );
		} catch (IOException e1) {
			e1.printStackTrace();
		} 
		log.info( "UPnP Service discovery time" );
		
		// starts the UPnP test
		local = new TestUPnP();
		local.setLogger( log );
		local.init();
		
		for (int x=0; x<50; x++)
		try {
			System.out.println ("search ...");
			while (local.isSearching()) {
				Thread.sleep(100);
			}
			Thread.sleep(1000);
			local.search(name);
		}
		catch (InterruptedException e)
		{
		}
		
		// UPnP search
		
		local.terminate();

		System.exit( 1 );
	}
}
